<?php
/*
Name: bloo.install.php
Purpose: Installs a Bloo blog
Author: Bill Roberts
Created: March 1, 2006
$Revision: 1.16 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/* definitions */
/* This is the version number for this version */
define( "BLOO_VERSION", "1.00" );

/* Set to true to send a ping to the out of the bloo domain when upgrade done */
define( "SEND_PING", true ); 

/* Leave these next two alone for pinging */
define( "PING_HOST", "outofthebloo.com" ); 
define( "PING_SHARE", "" );
 
require_once( "phoo.base.php" );

/**************************************************************/
/* Initialize                                                 */
/**************************************************************/
/* create the session */
$_session = new PhooSession( );

/* setup the bloo_version variable */
$_session->setVar( "bloo_version", BLOO_VERSION, "APPLICATION" );

/**************************************************************/
/* Include needed snapOns and Overlays                        */
/**************************************************************/
$_session->loadOverlaysInXML( "core/error.template.overlays.xml", "core" );
$_session->loadOverlaysInXML( "core/bloo_pages.template.overlays.xml", "core" );
$_session->loadOverlaysInXML( "install/bloo.install.command.overlays.xml", "core" );
$_session->loadOverlaysInXML( "install/bloo.install.template.overlays.xml", "core" );
$_session->loadOverlaysInXML( "bloo_session.template.overlays.xml", "core" );

class BlooTimeZoneDropDown extends TimeZoneDropDown {
   var $name = "BlooTimeZoneDropDown";
   var $securityRole = ROLE_EVERYONE;
   var $templateHeader = "=|TimeZone.dropdown.header|=";
   var $template = "=|TimeZone.dropdown.item|=";
   var $templateFooter = "=|TimeZone.dropdown.footer|=";

   function setup( ) {
      parent::setup( );
      
      /* get current server time, determine difference from GMT, 
         and take a good guess at the current time zone */

      /* figure out hours different from GMT */
      $serverHourDiffFromGmt = round( ( gmmktime( ) - mktime( ) ) / 3600 );

      /* determine the timezone ID that best fits this */
      $i = 0;
      $found = false;
      while ( !$found && $i < count( $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ] ) ) {
         if ( $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ $i ][ "GMT-OFFSET" ] == $serverHourDiffFromGmt ) {
            $timeZoneId = $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ $i ][ "ID" ];
            $found = true;
         }
         else {
            $i++;
         }
      }

      if (!$found ) {
         /* weird, just set to the first one */
         $timeZoneId = $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ 0 ][ "ID" ];
      }
      $this->setVar( "time_zone_id", $timeZoneId, "POST" );

      parent::setup( );
   }
}

/* write out the page header */
$s =& $_session->snapOnFactory( "PhooDisplay", $_session );
$s->securityRole = ROLE_EVERYONE;
$s->template = "=|Install.header|=";
$s->process( );
$s->display( );

$promptChar = "&#187; ";


/* check to see if bloo.config.php exists - if not, a friendly message and out */
if ( !file_exists( "bloo.config.php" ) ) {
   echo $promptChar . "<b>Not Ready to Install:</b> The installation program cannot find the file ";
   echo " <strong><font color=blue>bloo.config.php</font></strong>. It may be that you have ";
   echo "not yet copied or renamed the release file bloo.config_template.php to bloo.config.php ";
   echo "and edited it to include your database connection information.</p><p>Please see the install instructions ";
   echo "in install.txt for more information.<br><br>";
   echo "The Installation process cancelled for now. Please try again when bloo.config.php has been created . . . <br>";
   
   return;

}

require_once( "bloo.config.php" );

echo  $promptChar . "Checking database parameters from the configuration file. . .<br>";

/* Connect to the database */
echo $promptChar . "Connecting to the database . . .<br>";
echo "<hr>";
$_session->connect( $_server, $_database, $_userId, $_password );
if ( $_session->isError( ) ) {
   echo $promptChar . "<b>Error:</b> Connection unsuccessful: " . $_session->getError( ) . "<br><br>";
   echo "Please ensure that the database connection information you edited in ";
   echo "bloo.config.php is correct and try again.<br><br>";
   echo "Stopping the installation . . . <br>";

   return;
}

/* check to see if the settings table exists. If so, let them know the install has 
   already happened */
if ( $_session->connection->tableExists( $_settingsTable ) ) {
   echo $promptChar . "<b>Note:</b> It appears that an installation of Bloo already exists in the ";
   echo $_database . " database. Please clear out any bloo tables in this database, ";
   echo "or change the database specifications in bloo.config.php.<br>";  
   echo "<br>Now terminating the installation process with extreme prejudice . . . <br>";
   return;
}

/* get information - administrator userid, password, table prefix */
class InstallQuestions extends PhooForm {

   var $name = "InstallQuestions";
   var $package = "install";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Install.questions|=";
   var $successPage = ""; /* don't want to redirect after the form is completed */
   var $field = array( "blog_name"           => array( "required" => "Y"
                                               ,"formalName" => "Blog Name" )
                      ,"site_url"            => array( "required" => "Y"
                                               ,"formalName" => "Site URL" )
                      ,"table_prefix"        => array( "required" => "Y"
                                               ,"formalName" => "Table Prefix" )
                      ,"admin_first_name"    => array( "required" => "Y"
                                               ,"formalName" => "First Name" )
                      ,"admin_last_name"     => array( "required" => "Y"
                                               ,"formalName" => "Last Name" )
                      ,"admin_email"         => array( "required" => "Y"
                                               ,"formalName" => "Email" )
                     );
   var $trigger = array( "btnSubmit" => array( "process" => "submit" ) );
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-02" );
      $this->setPurpose( "Gathers user information during the install" );
   }

   function validateForm( $aArray = null ) {
      parent::validateForm( $aArray );

      if ( !$this->isError( ) ) {
         $pwd1 = $this->getVar( "password1", "POST" );
         $pwd2 = $this->getVar( "password2", "POST" );

         if ( $pwd1 != $pwd2 ) {
            $this->setError( "Passwords must match", MSG_WARNING );
            return;
         }

         if ( strlen( $pwd1 ) < 5 ) {
            $this->setError( "Passwords should be at least five characters long, preferably with a mix of letters and numbers", MSG_WARNING );
            return;
         }
            
         /* convert the password to md5 */
         $this->setVar( "enc_password", md5( $pwd1 ), "POST" );
      }
   }   
   
}   

/* initialize the form */
/* auto-detect site url */
$siteUrl = $_session->getVar( "site_url", "POST" );
if ( !$siteUrl ) {
   $domain = $_session->getVar( "HTTP_HOST", "SERVER" );
   /* note: using REQUEST_URI rather than PHP_SELF so that this
      will work on servers with apache mod-rewrites */
   $path = $_session->getVar( "REQUEST_URI", "SERVER" );
    /* locate final slash in the variable and remove the script name */
   $pos = strrpos( $path, "/" );
   $path = substr( $path, 0, $pos+1 );

   /* remove the final slash, if there is one */
   if ( substr( $path, strlen( $path )-1, 1 ) == "/" ) { /* trim the slash */
      $path = substr( $path, 0, strlen( $path ) - 1 ); 
   }
   
   $siteUrl = $domain . $path;
   $_session->setVar( "site_url", "http://" . $siteUrl, "POST" );
}

/* determine default table prefix based on the prefix of the settings table */
$tablePrefix = $_session->getVar( "table_prefix", "POST" );
if ( !$tablePrefix ) {
   /* parse out the table prefix from the settings table name */
   $pos = strpos( $_settingsTable, "_" );
   if ( $pos !== false ) {
      $tablePrefix = substr( $_settingsTable, 0, $pos+1 );
   }
   $_session->setVar( "table_prefix", $tablePrefix, "POST" );
}

if ( $tablePrefix == "" ) {
   $tablePrefix = "bloo_"; /* default, just in case */
}

/* determine default userid, if none specified */
$adminUserId = $_session->getVar( "admin_user_id", "POST" );
if ( !$adminUserId ) {
   $_session->setVar( "admin_user_id", "admin", "POST" ); /* this is the default */  
}

$s =& $_session->snapOnFactory( "InstallQuestions", $_session );
$s->process( );

$submitButton = $_session->getVar( "btnSubmit", "POST" );
if ( $s->isError( ) || !$submitButton ) {
   $s->display( );
   return;
}

/* Form was successfully submitted.
   re-perform the check on the table (make that a function in a class somewhere)
   so that someone doesn't just skip to step 2 and wipe everything out */
if ( $_session->connection->tableExists( $_settingsTable ) ) {
   echo $promptChar . "<b>Error:</b> It appears that an installation of Bloo already exists in the ";
   echo $_database . " database. Please clear out any bloo tables in this database, ";
   echo "or change the database specifications in bloo.config.php.<br>";  
   echo "<br>Bailing out . . . <br>";
   return;
}

echo $promptChar . "Information collected, we're ready to create the database. . .<br><br>";

/* set application variables for all the tables */
$tablePrefix = $_session->getVar( "table_prefix", "POST" );

$_session->setVar( "list_table", $_settingsTable, "APPLICATION" );
$_session->setVar( "post_table", $tablePrefix . "post", "APPLICATION" );
$_session->setVar( "comment_table", $tablePrefix . "comment", "APPLICATION" );
$_session->setVar( "user_table", $tablePrefix . "user", "APPLICATION" );
$_session->setVar( "trackback_table", $tablePrefix . "trackback", "APPLICATION" );
$_session->setVar( "log_table", $tablePrefix . "log", "APPLICATION" );
$_session->setVar( "role_table", $tablePrefix . "role", "APPLICATION" );
$_session->setVar( "list_type_table", $tablePrefix . "list_type", "APPLICATION" );
$_session->setVar( "overlay_table", $tablePrefix . "overlay", "APPLICATION" ); 
$_session->setVar( "snap_on_table", $tablePrefix . "snap_on", "APPLICATION" ); 
$_session->setVar( "post_meta_table", $tablePrefix . "post_meta", "APPLICATION" ); 
$_session->setVar( "queue_table", $tablePrefix . "queue", "APPLICATION" );
$_session->setVar( "static_page_table", $tablePrefix . "static_page", "APPLICATION" );

$password = $_session->getVar( "password1", "POST" );
$_session->setVar( "enc_password", md5( $password ), "POST" );

/* note: name on blog is generally just the first name */
$_session->setVar( "admin_name_on_blog", $_session->getVar( "admin_first_name", "POST" ), "POST" );

/* set the hour diff from server variable */
$hourDiffFromGmt = $_session->getVar( "hour_diff_from_gmt", "POST" );
$hourDiffFromServer = $hourDiffFromGmt - $serverHourDiffFromGmt;
$_session->setVar( "hour_diff_from_server", $hourDiffFromServer, "POST" );

echo $promptChar . "Creating tables . . .<br>";
/* create the SnapOn that will handle all the database processes */
$p =& $_session->snapOnFactory( "PhooMultiProcess", $_session );
$p->securityRole = ROLE_EVERYONE;
$p->command = "=|Tables.create|=";
$p->process( );

if ( $p->isError( ) ) {
   echo $promptChar . "<b>Error:</b> An error occurred during table creation: " . $p->getError( ) . " . . . <br>";
   echo "<br>Back to the drawing board . . . <br>";
   return;
}

echo $promptChar . "Tables created . . . <br>";

echo $promptChar . "Loading data . . .<br>";
/* create the SnapOn that will handle all the data load processes */
$p =& $_session->snapOnFactory( "PhooMultiProcess", $_session );
$p->securityRole = ROLE_EVERYONE;
$p->command = "=|Tables.load|=";
$p->process( );

if ( $p->isError( ) ) {
   echo $promptChar . "<b>Error:</b> An error occurred during initial data loading: " . $p->getError( ) . " . . . <br>";
   echo "<br>Back to the drawing board . . . <br>";
   return;
}

echo $promptChar . "Data loaded . . . <br>";

/* send an XML ping to the out of the bloo domain, if desired */
if ( SEND_PING ) {
   echo $promptChar . "Sending a BlooInstall ping to " . PING_HOST . " . . .<br>";

   /* open the socket */
   $fp = @fsockopen( PING_HOST, 80 );
   if ( $fp ) { /* don't display errors if this doesn't work */
      /* build the ping xml */
      $data = "<?xml version=\"1.0\"?>";
      $data .= "<methodCall>";
      $data .= "<methodName>blooPing</methodName>";
      $data .= "<params>";
      $data .= "<param>";
      $data .= "<value>BlooInstall</value>";
      $data .= "</param>";
      $data .= "<param>";
      $data .= "<value>" . $_session->getVar( "blog_name", "POST" ) . "</value>";
      $data .= "</param>";
      $data .= "<param>";
      $data .= "<value>" . $_session->getVar( "site_url", "POST" ) . "</value>";
      $data .= "</param>";
      $data .= "<param>";
      $data .= "<value>" . BLOO_VERSION . "</value>";
      $data .= "</param>";
      $data .= "</params>";
      $data .= "</methodCall>";

      /* build the header */
      $header = "POST " . PING_SHARE . "/bloo.ops.php HTTP/1.1\r\n";
      $header .= "User-Agent: PHPTest\r\n";
      $header .= "Host: " . PING_HOST . "\r\n";
      $header .= "Content-type: text/xml\r\n";
      $header .= "Content-Length: " . strlen( $data ) . "\r\n";
      $header .= "Connection: Close\r\n\r\n";

      $content = $header . $data;   

      /* send the ping */
      fwrite($fp, $content );
   
      /* get the results, but don't display */
      while (!feof($fp)) {
          $s = fgets($fp, 128);
      }

      /* close the socket */
      fclose($fp);
   }
}

$siteUrl = $_session->getVar( "site_url", "POST" );
echo $promptChar . "Install completed! Your userid is <b>" . $_session->getVar( "admin_user_id", "POST" ) . "</b>" .  
                   ". Click <a href=\"" . $siteUrl . "\">here</a> to go to the blog. Once there, you can click the " .
                   "<a href=\"bloo.ops.php\">login</a> link to login and begin creating posts.<br>";

/* write out the page footer */
$s =& $_session->snapOnFactory( "PhooDisplay", $_session );
$s->securityRole = ROLE_EVERYONE;
$s->template = "=|Install.footer|=";
$s->process( );
$s->display( );


?>
