<?php
/*
Name: bloo.upgrade.php
Purpose: Upgrades a Bloo blog from any previous release to version 1.00  
Author: Bill Roberts
Created: March 20, 2006
$Revision: 1.27 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/* definitions */
/* This is the version number for this version */
define( "BLOO_VERSION", "1.00" );

/* Set to true to send a ping to the out of the bloo domain when upgrade done */
define( "SEND_PING", true ); 

/* Leave these next two alone for pinging */
define( "PING_HOST", "outofthebloo.com" ); 
define( "PING_SHARE", "" );
   
require_once( "phoo.base.php" );
require_once( "bloo.config.php" );
require_once( "core/security.classes.php" );
/**************************************************************/
/* Initialize                                                 */
/**************************************************************/
/* create the session */
$_session = new PhooSession( );

$_session->loadOverlaysInXML( "bloo_session.template.overlays.xml", "core" );

class BlooTimeZoneDropDown extends TimeZoneDropDown {
   var $name = "BlooTimeZoneDropDown";
   var $securityRole = ROLE_EVERYONE;
   var $templateHeader = "=|TimeZone.dropdown.header|=";
   var $template = "=|TimeZone.dropdown.item|=";
   var $templateFooter = "=|TimeZone.dropdown.footer|=";

   function setup( ) {
      parent::setup( );

      /* get current server time, determine difference from GMT, 
         and take a good guess at the current time zone */

      /* figure out hours different from GMT */
      $serverHourDiffFromGmt = round( ( gmmktime( ) - mktime( ) ) / 3600 );

      /* determine the timezone ID that best fits this */
      $i = 0;
      $found = false;
      while ( !$found && $i < count( $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ] ) ) {
         if ( $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ $i ][ "GMT-OFFSET" ] == $serverHourDiffFromGmt ) {
            $timeZoneId = $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ $i ][ "ID" ];
            $found = true;
         }
         else {
            $i++;
         }
      }

      if (!$found ) {
         /* weird, just set to the first one */
         $timeZoneId = $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ 0 ][ "ID" ];
      }
      $this->setVar( "time_zone_id", $timeZoneId, "POST" );

      parent::setup( );
   }
}

class BlooTimeZoneChooseForm extends PhooForm {
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "Please select your timezone:<br />
                    <form method=post>_{BlooTimeZoneDropDown}_<br />
                    <input name=btn_select type=submit value='Select Timezone' />
                    <input name=user_login_id type=hidden value='_@user_login_id@_' />
                    <input name=user_password type=hidden value='_@user_password@_' />
                    </form>";
}

/**************************************************************/
/* Include needed snapOns and Overlays                        */
/**************************************************************/
$_session->loadOverlaysInXML( "core/error.template.overlays.xml", "core" );
$_session->loadOverlaysInXML( "install/bloo.upgrade.command.overlays.xml", "core" );
$_session->loadOverlaysInXML( "install/bloo.upgrade.template.overlays.xml", "core" );
$_session->loadOverlaysInXML( "core/security.command.overlays.xml", "core" );
$_session->loadOverlaysInXML( "core/security.template.overlays.xml", "core" );

/* some initial setup */

/* parse out the table prefix from the settings table name */
$pos = strpos( $_settingsTable, "_" );
if ( $pos !== false ) {
   $tablePrefix = substr( $_settingsTable, 0, $pos+1 );
}

$_session->setVar( "overlay_table", $tablePrefix . "overlay", "POST" );
$_session->setVar( "snap_on_table", $tablePrefix . "snap_on", "POST" );


/* initialize for reading settings */
$_session->setVar( "settings_table", $_settingsTable, "APPLICATION" );

$promptChar = "&#187; ";

$noProblems = true; /* assume no problems */
$upgradeStarted = false;
$upgradeComplete = false;
/* Connect to the database */
$_session->connect( $_server, $_database, $_userId, $_password );
if ( $_session->isError( ) ) {
   echo $promptChar . "<b>Error:</b> Connection unsuccessful: " . $_session->getError( ) . "<br />";
   echo "Please ensure that the database connection information you edited in ";
   echo "bloo.config.php is correct and try again.<br />";
   echo "<br />Stopping the upgrade . . . <br />";
   $noProblems = false;
}

/* check to see if the settings table exists. If not, let them know that an install is needed */ 
if ( !$_session->connection->tableExists( $_settingsTable ) ) {
   echo $promptChar . "<b>Error:</b> It appears that there is no installation of Bloo on this server. ";
   echo " Please verify that you have installed Bloo. You can run the bloo install program ";
   echo "by clicking <a href=\"bloo.install.php\">here.</a>.<br />";  
   echo "<br />Now terminating the upgrade process . . . <br />";
   $noProblems = false;
}

/* add bootstrap overlay to read settings */
$_session->addOverlay( "Settings.list.get", "command",
      "select l2.item_name as setting_category
             ,l1.list_item_id
             ,l1.item_name AS setting_name
             ,l1.meta_data_01 AS setting_caption
             ,l1.meta_data_02 AS setting_value
         from _@settings_table@_ l1
             ,_@settings_table@_ l2
        where l2.type = 'settings.category'
          and l1.type = 'settings'
          and l1.meta_data_03 = l2.list_item_id"
      ,"Retrieves all settings", "bootstrap", "core" );

/* add bootstrap overlay to read security roles */
$_session->addOverlay( "Role.information.get", "command",
     "select role_id
            ,role_name
            ,contained_roles 
        from _@role_table@_ 
       where role_id = _@role_id@_ 
         and deleted_flag = 'N'"
      ,"Retrieves role information", "bootstrap", "core" );

/* load settings */      
/* Create a snapOnQuery to load the settings */
$s =& $_session->snapOnFactory( "PhooQuery", $_session );
$s->securityRole = ROLE_EVERYONE;
$s->command = "=|Settings.list.get|=";
$s->process( );

/* for each resulting row, set the application variable */
while ( $row = $s->getResultRow( ) ) {
   /* trap for a final slash in the site_url variable - if it's there, remove */  
   if ( $row[ "setting_name" ] == "site_url" ) {
      $value = $row[ "setting_value" ];
      if ( substr( $value, strlen( $value )-1, 1 ) == "/" ) { /* trim the slash */
         $row[ "setting_value" ] = substr( $value, 0, strlen( $value ) - 1 ); 
      }
   }
   $_session->setVar( $row[ "setting_name" ], $row[ "setting_value" ], "APPLICATION" );
}

$continueProcessing = true;

$_session->setVar( "bloo_version", BLOO_VERSION, "APPLICATION" ); /* setup a variable */      

if ( !$_session->isLoggedIn( ) ) {
   /* attempt a login */
   $loginId = $_session->getVar( "user_login_id", "POST" );
   $password = md5( $_session->getVar( "user_password", "POST" ) );
   if ( isset( $loginId ) && isset( $password ) ) {
      $_session->logIn( $loginId, $password, false );
      unset( $loginId );
      unset( $password );
   }
}

/* write out the page header */
$s =& $_session->snapOnFactory( "PhooDisplay", $_session );
$s->securityRole = ROLE_EVERYONE;
$s->template = "=|Upgrade.header|=";
$s->process( );
$s->display( );

if ( !$_session->isLoggedIn( ) ) {
   $s =& $_session->snapOnFactory( "PhooDisplay", $_session );
   $s->securityRole = ROLE_EVERYONE;
   $s->template = "=|Upgrade.login.form|=";
   $s->process( );
   $s->display( );
   $continueProcessing = false;
}
   
if ( $continueProcessing ) {
   $upgradeStarted = true;
   /* create the SnapOn that will handle all the database processes */
   $p =& $_session->snapOnFactory( "PhooMultiProcess", $_session );
   $p->securityRole = ROLE_EVERYONE;
   
   /* create the SnapOn that will handle all the database queries*/
   $q =& $_session->snapOnFactory( "PhooQuery", $_session );
   $q->securityRole = ROLE_EVERYONE;
}

if ( $continueProcessing ) { 
   /* check to see if the version setting exists. If it doesn't, create it. If it does, set it to the
      current version */
   $q->command = "=|BlooVersion.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the Bloo version settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Creating the Bloo version setting . . .<br />";
         
         $p->command = "=|BlooVersion.create|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not insert the Bloo version setting record in the settings table: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }
      else {
         /* update the setting */
         $p->command = "=|BlooVersion.update|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not update the Bloo version setting record in the settings table: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }         
   }
}

if ( $continueProcessing ) { 
   /* modify the tables to reflect the change in directory name from "snapons" to "core" */
   echo $promptChar . "Adjusting for renaming of snapons directory to core (if necessary) . . .<br />";
   $p->command = "=|CorePath.rename|=";
   $p->process( );
   if ( $p->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not adjust for the renaming of the snapons path to core: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
}

if ( $continueProcessing ) { 
   /* Change the "Everyone" role name to "Guest" */
   echo $promptChar . "Renaming 'Everyone' role to 'Guest' (if necessary) . . .<br />";
   $p->command = "=|Role.everyone.rename|=";
   $p->process( );
   if ( $p->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not rename Everyone role to Guest: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
}

if ( $continueProcessing ) { 
   /* If the bloo_trackback table doesn't exist, rename bloo_ping as bloo_trackback. Also set the name of the
      trackback table in the settings table */
   $newTrackbackTableName = $tablePrefix . "trackback";      
   if ( !$_session->connection->tableExists( $newTrackbackTableName ) ) {
      echo $promptChar . "Renaming the trackback table from " . $tablePrefix . "ping to " . 
                         $newTrackbackTableName . "<br />";
      $_session->setVar( "list_table", $tablePrefix . "list", "POST" );
      $_session->setVar( "trackback_table", $tablePrefix . "ping", "POST" );
      $_session->setVar( "new_trackback_table", $tablePrefix . "trackback", "POST" );
      /* renames the table and changes its name setting in the list table */
      $p->command = "=|Table.rename.ping|=";
      $p->process( );
      /* now set the trackback table name to the new name, if used in later processing */
      $_session->setVar( "list_table", $tablePrefix . "list", "POST" );
      
      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during table rename: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
   }
}

if ( $continueProcessing ) {
   /* Check to see if time zone functionality exists. If it doesn't
      and the user hasn't already been prompted, prompt the user with 
      a time zone drop-down */
   $q->command = "=|Settings.time_zone_id.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the time zone id settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         $timeZoneId = $_session->getVar( "time_zone_id", "POST" );
         if ( isset( $timeZoneId ) ) {
            echo $promptChar . "Storing the time zone id in the settings table <br />";
        
            $q->command = "=|Settings.NextId.get|=";
            $q->process( );
            if ( $q->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
            else {
               while ( $row = $q->getResultRow( ) ) {
                  $nextId = $row[ "nextId" ];
               }
               if ( !is_numeric( $nextId ) ) {
                  $nextId = 1;
               }
               $p->setVar( "id", $nextId, "PRIVATE" );
               $p->command = "=|Settings.time_zone_id.set|=";
               $p->process( );
               if ( $p->isError( ) ) {
                  echo $promptChar . "<b>Error:</b> Could not insert a record for the time zone id in the settings table: " . $q->getError( ) . " . . . <br />";
                  echo "<br />For help please email support@outofthebloo.com . . . <br />";
                  $noProblems = false;
               }
            }

            /* time zone set, set flag */
            echo $promptChar . "Setting the time zone adjustment needed pending flag . . . <br />";
            $p->command = "=|Upgrade.time_zone_adjustment.setpending|=";
            $p->process( );

            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> An error occurred during flag set: " . $p->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
         else {
            $dd = $_session->snapOnFactory( "BlooTimeZoneChooseForm", $_session );
            $dd->process( );
            $dd->display( );
            $continueProcessing = false;
         }
      }
      else {
         echo $promptChar . "Checked for a record for the time zone id in the settings table. It already exists . . . <br />";
      }         
   }
}
   
if ( $continueProcessing ) {
   echo $promptChar . "Checking for time zone adjustment pending<br />";

   $q->command = "=|Upgrade.time_zone_adjustment.pending|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if a time zone adjustment is pending: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      while ( $row = $q->getResultRow( ) ) {
         $timeZoneAdjustmentPending = true;
      }
   
      /* if upgrading time zone, will adjust all the dates in the main tables back to GMT */
      if ( $timeZoneAdjustmentPending ) {
         /* this section will create a range of dates with hour adjustments to GMT */
         
         /* setup table list */
         $tzUpgradeTable = array( "list", "log", "overlay", "trackback", "snap_on", "role", "user", "post", "comment" );
      
         /* get earliest date on the blog in local time */
         $hourDifferenceFromServer = $_session->getVar( "hour_difference_from_server", "APPLICATION" );
         
         /* select earliest date from the list table */
         $_session->setVar( "list_table", $tablePrefix . "list", "POST" );
         
         $q->command = "=|Settings.time_zone_id.upgrade_min_date|=";
         $q->process( );
         if ( $q->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not determine starting date for time zone upgrade: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
         else {
            while ( $row = $q->getResultRow( ) ) {
               $minDate = $row[ "min_date" ];
            }
   
            if ( !isset( $minDate ) ) { 
               /* just go back to when Bloo basically started */
               $minDate = '2006-03-01 00:00:00';
            }
           
            /* create a time zone object */
            $timeZoneId = $_session->getVar( "time_zone_id" );
            $tz = new PhooTimeZone( $timeZoneId );
            
            /* convert to GMT date - note, truncate to hour */
            $gmtDate = $tz->getGMTTime( "Y-m-d H", $minDate ) . ":00:00";
   
            /* convert to a GMT timestamp (may make this a function in tz */
            $gmtTimeStamp = strtotime( $gmtDate . "+0000" );
   
            $looping = true;
            $start = true;
      
            $prevGmDate = 0;
   
            $dateRange = array( );
   
            while ( $looping ) {
               $dstStatus = $tz->isDST( $tz->getLocalStandardTimeStamp( $gmtTimeStamp ) );
         
               if ( $start ) {
                  $dstStatusChanged = true;
               }
               elseif ( $dstStatus != $prevDstStatus ) {
                  $dstStatusChanged = true;
               }
          
               if ( $gmtTimeStamp > time( ) + 3600*24*365 ) {
                  $looping = false;
                  $dstStatusChanged = true;
               }
            
               if ( $dstStatusChanged ) {
                  if ( $prevGmDate > 0 ) {
                     $hourAdj = $tz->gmtOffset;
                     if ( $prevDstStatus ) {
                        $hourAdj += $tz->dstOffset;
                     }
   
                     /* subtract difference from server */
                     $hourAdj -= $hourDifferenceFromServer;
   
                     array_push( $dateRange, array( $prevGmDate, $gmtTimeStamp-1, -$hourAdj, $prevDstStatus ) );
                  }
   
                  $prevGmDate = $gmtTimeStamp;
                  $prevDstStatus = $dstStatus;
            
                  /* if dst just turned off, add the dst offset back in to avoid overlaps */
                  if ( !$prevDstStatus && !$start ) {
                     $prevGmDate += $tz->dstOffset * 3600;
                  }
   
            
                  $dstStatusChanged = false;
               }
      
               $gmtTimeStamp += 3600; /* add one hour */
   
               if ( $start ) {
                  $start = false;
               }
            }
         
            /* adjust the time for each range */
            echo $promptChar . "Adjusting times in main tables to GMT . . . <br />";
            
            $p->command = "=|Tables.time.update|=";
            foreach( $dateRange as $ar ) {
               $time = $tz->getTimeInZone( BASE_TIME_FORMAT, gmdate( BASE_TIME_FORMAT, $ar[0] ) );
               $p->setVar( "start_date", $time[ 0 ], "PRIVATE" );
               $time = $tz->getTimeInZone( BASE_TIME_FORMAT, gmdate( BASE_TIME_FORMAT, $ar[1] ) );
               $p->setVar( "end_date", $time[ 0 ], "PRIVATE" );
               $p->setVar( "hour_adj", $ar[ 2 ], "PRIVATE" );
   
               $p->process( );
               if ( $p->isError( ) ) {
                  echo $promptChar . "<b>Error:</b> An error occurred during table time update: " . $p->getError( ) . " . . . <br />";
                  echo "<br />For help please email support@outofthebloo.com . . . <br />";
                  $noProblems = false;
                  break;
               }
            }   
         }
      }
      else {
         echo $promptChar . "Time zone adjustments have already been made . . . <br />";
      }
   }
   
   $overlayTable = $_session->getVar( "overlay_table" );
   
   /* check to see if the overlay table exists. If not, create it */
   if ( !$_session->connection->tableExists( $overlayTable ) ) {
      echo $promptChar . "Creating the overlay table . . . <br />";
      $p->command = "=|Table.create.overlay|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during table creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
   }
   else {
      echo $promptChar . "Checked for overlay table - it already exists . . . <br />";
   }

   echo $promptChar . "Adding new event handlers . . . <br />";
   /* See if the events exist - for each one, if it doesn't, 
      retrieve the max id for it and insert */
   $event = array( "CommentCookieBaker", "CommentCookieEater", "CommentMailer", "PostMoreConverter", "TrackbackMailer" );
   for ( $i=0; $i < count( $event ); $i++ ) {   
      $eventName = $event[ $i ];      
      $q->command = "=|Event." . $eventName . ".exists|=";
      $q->process( );
      if ( $q->isError( ) ) {
         echo $promptChar . "<b>Error:</b> Could not determine if event handler exists: " . $q->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         $exists = false;
         while ( $row = $q->getResultRow( ) ) {
            $exists = true;
         }
   
         if ( $exists ) {
            echo $promptChar . $eventName . " event already exists . . . <br />"; 
         }
         else {
            echo $promptChar . "Installing the " . $eventName . " event handler . . . <br />";
            
            $q->command = "=|Event.NextId.get|=";
            $q->process( );
            if ( $q->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not determine next event id: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
            else {
               while ( $row = $q->getResultRow( ) ) {
                  $nextId = $row[ "nextId" ];
               }
               if ( !is_numeric( $nextId ) ) {
                  $nextId = 1;
               }
               $p->setVar( "id", $nextId, "PRIVATE" );
               $p->command = "=|Event." . $eventName . ".set|=";
               $p->process( );
               if ( $p->isError( ) ) {
                  echo $promptChar . "<b>Error:</b> Could not insert event handler: " . $p->getError( ) . " . . . <br />";
                  echo "<br />For help please email support@outofthebloo.com . . . <br />";
                  $noProblems = false;
               }
               else {
                  echo $promptChar . $eventName . " event installed . . . <br />";
               }               
            }
         }
      }
   }
            
   /* see if the overlay table exists in the settings table. If it doesn't,
      insert the row */
   $q->command = "=|Settings.overlaytablerecord.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the overlay table settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Placing a record for the overlay table name in the settings table <br />";
         
         $q->command = "=|Settings.NextId.get|=";
         $q->process( );
         if ( $q->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
         else {
            while ( $row = $q->getResultRow( ) ) {
               $nextId = $row[ "nextId" ];
            }
            if ( !is_numeric( $nextId ) ) {
               $nextId = 1;
            }
            $p->setVar( "id", $nextId, "PRIVATE" );
            $p->command = "=|Settings.overlaytablerecord.set|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not insert a record for the overlay table name in the settings table: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
      }
      else {
               echo $promptChar . "Checked for a record for the overlay table name in the settings table. It already exists . . . <br />";
      }         
   }

   /* fix the group type on the extension table setting */
   echo $promptChar . "Fixing the group indicator on the extension table name in the settings table if necessary . . . <br />";   
   $p->command = "=|Settings.fixextensiontablegroup|=";
   $p->process( );

   if ( $p->isError( ) ) {
      echo $promptChar . "<b>Error:</b> An error occurred on the fix: " . $p->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }

   /* disable any custom overlays that reference Sidebar.right or Sidebar.left, or that are named
      either of those two */
   echo $promptChar . "Deactivating deprecated Sidebar.right or Sidebar.left custom overlays if necessary . . . <br />";   
   echo $promptChar . "<b>Note</b>: If you re-activate them, be sure to change all references to Sidebar.right and Sidebar.left ";   
   echo "To &quot;Sidebar&quot; and change css styles sidebar_right and sidebar_left to &quot;sidebar&quot;.";
   echo "You can do this in the Look and Feel section of the administrative page, under Custom overlays.<br>";
   
   $p->command = "=|Overlay.deactivateSidebarLR|=";
   $p->process( );

   if ( $p->isError( ) ) {
      echo $promptChar . "<b>Error:</b> An error occurred: " . $p->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }

   /* add overlay.favorite settings */
   $fav = array( "Page.banner", "Page.sidebar",  "CSS.extension" );
   for ( $i=0; $i < count( $fav ); $i++ ) {   
      $favName = $fav[ $i ];      
      $q->setVar( "item_name", $favName, "PRIVATE" );
      $q->command = "=|Overlay.favorite.exists|=";
      $q->process( );
      if ( $q->isError( ) ) {
         echo $promptChar . "<b>Error:</b> Could not determine if overlay favorite exists: " . $q->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         $exists = false;
         while ( $row = $q->getResultRow( ) ) {
            $exists = true;
         }
   
         if ( $exists ) {
            echo $promptChar . $favName . " favorite already exists . . . <br />"; 
         }
         else {
            echo $promptChar . "Setting up " . $favName . " in the overlay favorites . . . <br />";
            
            $q->command = "=|Overlay.favorite.NextId.get|=";
            $q->process( );
            if ( $q->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not determine next overlay favorite id: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
            else {
               while ( $row = $q->getResultRow( ) ) {
                  $nextId = $row[ "nextId" ];
               }
               if ( !is_numeric( $nextId ) ) {
                  $nextId = 1;
               }
               $p->setVar( "id", $nextId, "PRIVATE" );
               $p->setVar( "item_name", $favName, "PRIVATE" );
               $p->command = "=|Overlay.favorite.set|=";
               $p->process( );
               if ( $p->isError( ) ) {
                  echo $promptChar . "<b>Error:</b> Could not insert Overlay favorite: " . $p->getError( ) . " . . . <br />";
                  echo "<br />For help please email support@outofthebloo.com . . . <br />";
                  $noProblems = false;
               }
               else {
                  echo $promptChar . $favName . " Overlay favorite installed . . . <br />";
               }               
            }
         }
      }
   }
   
   /* check to see if the approved_flag column exists on the comment table. If not, create it and
      related spam-fighting columns */
   $_session->setVar( "comment_table", $tablePrefix . "comment", "POST" );
      
   $commentTable = $_session->getVar( "comment_table", "POST" );
   
   echo $promptChar . "Checking for spam-fighting columns in the comment table . . . <br />";
   if ( !$_session->connection->columnExists( $commentTable, "approved_flag" ) ) {
      $p->command = "=|Columns.create.comment_spam|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Created the spam-fighting columns in the comment table . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for spam-fighting columns - they already exist . . . <br />";
   }

   /* check to see if the approved_flag column exists on the comment table. If not, create it and
      related spam-fighting columns */
   $_session->setVar( "trackback_table", $tablePrefix . "trackback", "POST" );
      
   $trackbackTable = $_session->getVar( "trackback_table", "POST" );
   
   if ( !$_session->connection->columnExists( $trackbackTable, "approved_flag" ) ) {
      echo $promptChar . "Checking for spam-fighting columns in the trackback table . . . <br />";
      $p->command = "=|Columns.create.trackback_spam|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Created the spam-fighting columns in the trackback table . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for spam-fighting columns - they already exist . . . <br />";
   }
   
   /* check for the trackback IP field */
   if ( !$_session->connection->columnExists( $trackbackTable, "trackback_ip" ) ) {
      echo $promptChar . "Checking for the trackback_ip field in the trackback table . . . <br />";
      $p->command = "=|Columns.create.trackback_ip|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Created the trackback_ip field in the trackback table . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for trackback_ip field - it already exists . . . <br />";
   }
   
   /* check for the comment and trackback mark fields */
   if ( !$_session->connection->columnExists( $trackbackTable, "trackback_mark" ) ) {
      echo $promptChar . "Checking for the comment_mark, trackback_mark fields . . . <br />";
      $p->command = "=|Columns.create.marks|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Created the comment_mark, trackback_mark fields . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for comment_mark, trackback_mark fields - they already exists . . . <br />";
   }

   echo $promptChar . "Checking for Comment guest edit key in the comment table . . . <br />";
   if ( !$_session->connection->columnExists( $commentTable, "guest_edit_key" ) ) {
      echo $promptChar . "Creating the guest edit key column in the comment table . . . <br />";
      $p->command = "=|Column.create.comment_guest_edit_key|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Created the guest edit key column in the comment table . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for the guest edit key column - it already exists . . . <br />";
   }
   

   /* Check for need to add Comment and Trackback allow settings */
   /* doing these four in a loop just to limit the coding involved */
   echo $promptChar . "Checking for Comment and Trackback allow settings . . . <br />";
   
   $examine = array( "=|Settings.commentsalloweddefault.exists|=", "=|Settings.trackbacksalloweddefault.exists|=",
                     "=|Settings.commentsallowed.exists|=", "=|Settings.trackbacksallowed.exists|=",
                     "=|Settings.guestcommenteditallowed.exists|=", "=|Settings.guestcommentedittimelimit.exists|=" );
   $command = array( "=|Settings.commentsalloweddefault.set|=", "=|Settings.trackbacksalloweddefault.set|=",
                     "=|Settings.commentsallowed.set|=", "=|Settings.trackbacksallowed.set|=",
                     "=|Settings.guestcommenteditallowed.set|=", "=|Settings.guestcommentedittimelimit.set|=" );   
   
   for ( $i = 0; $i < count( $examine ); $i++ ) {
      $q->command = $examine[ $i ];
   
      $q->process( );
      if ( $q->isError( ) ) {
         echo $promptChar . "<b>Error:</b> Could not check for Comment and Trackback allow settings: " . $q->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         $exists = false;

         while ( $row = $q->getResultRow( ) ) {
            $exists = true;
         }

         if ( !$exists ) {
            echo $promptChar . "Adding Comments and Trackback allow setting #" . $i . "<br />";
            $q->command = "=|Settings.NextId.get|=";
            $q->process( );
            if ( $q->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
            else {
               while ( $row = $q->getResultRow( ) ) {
                  $nextId = $row[ "nextId" ];
                }
                if ( !is_numeric( $nextId ) ) {
                   $nextId = 1;
               }
               $p->setVar( "id", $nextId, "PRIVATE" );
               $p->command = $command[ $i ];
               $p->process( );
               if ( $p->isError( ) ) {
                  echo $promptChar . "<b>Error:</b> Could not insert a record for Comments and Trackbacks allow setting #" . $i . ": " . $p->getError( ) . " . . . <br />";
                  echo "<br />For help please email support@outofthebloo.com . . . <br />";
                  $noProblems = false;
               }
            }
         }
         else {
            echo $promptChar . "Comments and Trackbacks allow setting #" . $i . " already exists . . . <br />";
         }         
      }
   }               

   /* check to see if the comments_allowed_flag column exists on the post table. If not, create it and
      the trackbacks_allowed column also */
   $_session->setVar( "post_table", $tablePrefix . "post", "POST" );
      
   $postTable = $_session->getVar( "post_table", "POST" );
   
   echo $promptChar . "Checking for Comment and Trackback allow flags in the post table . . . <br />";
   if ( !$_session->connection->columnExists( $postTable, "comments_allowed_flag" ) ) {
      echo $promptChar . "Creating the Comment and Trackback allow flags in the post table . . . <br />";
      $p->command = "=|Columns.create.comment_and_trackback.allowed|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Created the Comment and Trackback allow flag columns in the post table . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for Comment and Trackback allow flag columns - they already exist . . . <br />";
   }

   /* change event type codes */   
   echo $promptChar . "Renaming event type codes if necessary . . . <br />";   
   $eventType = array( "1" => "eventPreRender"
                      ,"2" => "eventPostRender" 
                      ,"3" => "eventPreProcess" 
                      ,"4" => "eventPostProcess" 
                      ,"5" => "eventPreSave" 
                      ,"6" => "eventPostSave" 
                      ,"7" => "eventPostRowRetrieve" );

   $p->command = "=|Event.changeType|=";
   $renameSuccessful = true;
   foreach ( $eventType as $oldEventType => $newEventType ) {
      $p->setVar( "old_event_type", $oldEventType, "PRIVATE" );
      $p->setVar( "new_event_type", $newEventType, "PRIVATE" );
   
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred event type rename: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
         $renameSuccessful = false;
         break;
      }
   }

   $snapOnTable = $_session->getVar( "snap_on_table" );
   
   /* check to see if the overlay table exists. If not, create it */
   if ( !$_session->connection->tableExists( $snapOnTable ) ) {
      echo $promptChar . "Creating the snap_on table . . . <br />";
      $p->command = "=|Table.create.snap_on|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during table creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else { 
         /* populate the snap_on table */
         echo $promptChar . "Populating the snap_on table . . . <br />";
         $p->command = "=|Table.populate.snap_on|=";
         $p->process( );

         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> An error occurred during table population: " . $p->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }

         /* port over old records from the theme table, if it exists */
         $_session->setVar( "theme_table", $tablePrefix . "theme", "POST" );
         $_session->setVar( "user_table", $tablePrefix . "user", "POST" );
         $themeTable = $_session->getVar( "theme_table", "POST" );
   
         /* check to see if the theme table exists. If it does, port over its records */
         if ( $_session->connection->tableExists( $themeTable ) ) {
            /* read the records from the theme table */
            $q->command = "=|Theme.get_rows|=";
            $q->process( );
            if ( $q->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not read from the old themes table: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
            else {
               $p->command = "=|Table.port_themes|=";
      
               $themeRow = array( );
               
               while ( ( $row = $q->getResultRow( ) ) && $noProblems ) {
                  array_push( $themeRow, $row );
               }   
               
               foreach ( $themeRow as $row ) {
                  /* based on the row data, create a descriptor */
                  $_session->setVar( "theme_tag", $row[ "theme_tag" ], "POST" );
                  $_session->setVar( "create_date", $row[ "create_date" ], "POST" );
                  $descriptor = "<?xml version=\"1.0\"?>" .
                    "  <snapon>" .
                    "    <name>" . $row[ "theme_tag" ] . "</name>" .
                    "    <display_name>" . $row[ "theme_name" ] . "</display_name>" .
                    "    <version>1.0</version>" .
                    "    <description></description>" .
                    "    <type>theme</type>" .
                    "    <author>" . $row[ "theme_author" ] . "</author>" .
                    "    <create_date>" . $row[ "create_date" ] . "</create_date>" .
                    "    <cascades_from_snap_on>" . 
                           $row[ "theme_cascades_from" ] . "</cascades_from_snap_on>" .
                    "    <style_sheet></style_sheet>" .
                    "  </snapon>";

                  $_session->setVar( "descriptor", addslashes( $descriptor ), "POST" );

                  /* run the insert command */
                  $p->process( );
                  if ( $p->isError( ) ) {
                     echo $promptChar . "<b>Error:</b> Could not port themes over from the old theme table: " . $p->getError( ) . " . . . <br />";
                     echo "<br />For help please email support@outofthebloo.com . . . <br />";
                     $noProblems = false;
                  }
               }
            }
         }

         /* drop the theme table */
         if ( $noProblems ) {
            $p->command = "=|Table.theme_table.drop|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not drop the old theme table: " . $p->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
            
         /* drop the extension table */
         if ( $noProblems ) {
            $_session->setVar( "extension_table", $tablePrefix . "extension", "POST" );
            $p->command = "=|Table.extension_table.drop|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not drop the old extension table: " . $p->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
      }
   }
   else {
      echo $promptChar . "Checked for the snap_on table - it already exists . . . <br />";
   }
   
   /* populate a record for the bloo_queue core snapon in the SnapOn table, if it doesn't exist */
   echo $promptChar . "Checking for bloo_queue core snap_on entry . . . <br />";
   $q->command = "=|SnapOns.bloo_queue_entry.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if bloo_queue entry exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Adding new bloo_queue entry . . . <br />";
         
         $p->command = "=|SnapOns.bloo_queue_entry.create|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not create bloo_queue entry: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }
      else {
         echo $promptChar . "Bloo_queue entry already exists . . . <br />";
      }         
   }

   /* see if the snap_on table exists in the settings table. If it doesn't,
      insert the row */
   $q->command = "=|Settings.snap_ontablerecord.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the snap_on table settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Placing a record for the snap_on table name in the settings table <br />";
         
         $q->command = "=|Settings.NextId.get|=";
         $q->process( );
         if ( $q->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
         else {
            while ( $row = $q->getResultRow( ) ) {
               $nextId = $row[ "nextId" ];
            }
            if ( !is_numeric( $nextId ) ) {
               $nextId = 1;
            }
            $p->setVar( "id", $nextId, "PRIVATE" );
            $p->command = "=|Settings.snap_ontablerecord.set|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not insert a record for the snap_on table name in the settings table: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
      }
      else {
               echo $promptChar . "Checked for a record for the snap_on table name in the settings table. It already exists . . . <br />";
      }         
   }

   /* check to see if the additive_flag column exists on the overlay table. If it does, remove it */
   $overlayTable = $_session->getVar( "overlay_table", "POST" );
   if ( $_session->connection->columnExists( $overlayTable, "additive_flag" ) ) {
      echo $promptChar . "Checking for the additive_flag column in the overlay table . . . <br />";
      $p->command = "=|Columns.remove.additive_flag|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column removal: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Removed additive_flag column in the overlay table . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for additive_flag column - already removed . . . <br />";
   }

   
   /* check to see if the pend_to column exists on the overlay table. If not, create it and
      the pend_type */
   if ( !$_session->connection->columnExists( $overlayTable, "pend_to" ) ) {
      echo $promptChar . "Checking for pend_to and pend_type columns in the overlay table . . . <br />";
      $p->command = "=|Columns.create.pend_fields|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Created the pend_to, pend_type columns in the overlay table . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for pend_to, pend_type columns - they already exist . . . <br />";
   }
      
   echo $promptChar . "Modifying path variables, if necessary . . . <br />";
   $p->command = "=|Path.modify|=";
   $p->process( );

   if ( $p->isError( ) ) {
      echo $promptChar . "<b>Error:</b> An error occurred during path modification: " . $p->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
  
   /* see if the blog image url exists in the settings table. If it doesn't,
      insert the row */
   $q->command = "=|Settings.blog_image_url.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the blog image url settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Placing a record for the blog image url in the settings table <br />";
         
         $q->command = "=|Settings.NextId.get|=";
         $q->process( );
         if ( $q->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
         else {
            while ( $row = $q->getResultRow( ) ) {
               $nextId = $row[ "nextId" ];
            }
            if ( !is_numeric( $nextId ) ) {
               $nextId = 1;
            }
            $p->setVar( "id", $nextId, "PRIVATE" );
            $p->command = "=|Settings.blog_image_url.set|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not insert a record for the blog image url in the settings table: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
      }
      else {
               echo $promptChar . "Checked for a record for the blog image url in the settings table. It already exists . . . <br />";
      }         
   }

   /* update the blog image url if it is currently set to the puzzle piece */
   echo $promptChar . "Updating the blog image, if necessary . . . <br />";
   $p->command = "=|Settings.blog_image_url.update|=";
   $p->process( );
   if ( $p->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not update the blog image url in the settings table: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   
   echo $promptChar . "Removing unnecessary settings category variables, if necessary . . . <br />";
   $p->command = "=|SettingsCategory.cleanup|=";
   $p->process( );

   if ( $p->isError( ) ) {
      echo $promptChar . "<b>Error:</b> An error occurred during settings category cleanup: " . $p->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   
   echo $promptChar . "Recategorizing settings, if necessary . . . <br />";
   $p->command = "=|SettingsCategory.fix|=";
   $p->process( );

   if ( $p->isError( ) ) {
      echo $promptChar . "<b>Error:</b> An error occurred during settings recategorization: " . $p->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
      

   /* see if new display settings exist */
   $q->command = "=|SettingsDisplay.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the display settings records exist: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Creating display settings records . . .<br />";
         
         $q->command = "=|Settings.NextId.get|=";
         $q->process( );
         if ( $q->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
         else {
            while ( $row = $q->getResultRow( ) ) {
               $nextId = $row[ "nextId" ];
            }
            if ( !is_numeric( $nextId ) ) {
               $nextId = 1;
            }
            $p->setVar( "id", $nextId, "PRIVATE" );
            $p->command = "=|SettingsDisplay.create|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not insert display settings records in the settings table: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
      }
      else {
               echo $promptChar . "Checked for display settings - they already exist . . . <br />";
      }         
   }

   /* see if the new settings categories exist in the settings table. If they doesn't,
      insert the row */

   echo $promptChar . "Checking for new settings categories . . . <br />";
   $q->command = "=|Settings.NewCategories.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if new settings categories exist: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Adding new settings categories . . . <br />";
         
         $p->command = "=|Settings.NewCategories.create|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not create new settings categories: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }
      else {
         echo $promptChar . "New settings categories already exist . . . <br />";
      }         
   }

   /* port over old records from the theme table, if it exists */
   $_session->setVar( "list_type_table", $tablePrefix . "list_type", "POST" );
   $listTypeTable = $_session->getVar( "list_type_table", "POST" );
   
   /* check to see if the list type table exists. If it does, drop it */
   if ( $_session->connection->tableExists( $listTypeTable ) ) {
      echo $promptChar . "Dropping the list type table . . . <br />";
      /* drop the theme table */
      $p->command = "=|Table.list_type_table.drop|=";
      $p->process( );
      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> Could not drop the old list type table: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
   }

   /* Remove any unnecessary table specifications */
   /* only do this if there are no problems */
   if ( $noProblems ) {
      echo $promptChar . "Removing unnecessary table specifications in the settings table if necessary . . . <br />";   
      $p->command = "=|Settings.table_specs.remove|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred on the table spec removal: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
   }
   
   /* Remove any unnecessary hour specifications */
   /* only do this if there are no problems */
   if ( $noProblems ) {
      echo $promptChar . "Removing unnecessary hour specifications in the settings table if necessary . . . <br />";   
      $p->command = "=|Settings.hour_specs.remove|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred on the hour spec removal: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
   }

   $_session->setVar( "post_meta_table", $tablePrefix . "post_meta", "POST" );
   $postMetaTable = $_session->getVar( "post_meta_table" );
   
   /* check to see if the post meta table exists. If not, create it */
   if ( !$_session->connection->tableExists( $postMetaTable ) ) {
      echo $promptChar . "Creating the post_meta table . . . <br />";
      $p->command = "=|Table.create.post_meta|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during table creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }

      if ( !$p->isError( ) ) {
         /* move categories from the post table to the post_meta table */
         echo $promptChar . "Populating the post_meta table . . . <br />";
         $p->command = "=|PostMeta.populate|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> An error occurred during post meta table population: " . $p->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }
      if ( !$p->isError( ) ) {
         /* Drop the post category id field */
         echo $promptChar . "Dropping the post_category_id field . . . <br />";
         $p->command = "=|PostCategory.drop|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> An error occurred during the drop of the post_category_id field: " . $p->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }
   }
   else {
      echo $promptChar . "Checked for the post_meta table - it already exists . . . <br />";
   }
   
   /* see if the post_meta table exists in the settings table. If it doesn't,
      insert the row */
   $q->command = "=|Settings.post_metatablerecord.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the post_meta table settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Placing a record for the post_meta table name in the settings table <br />";
         
         $q->command = "=|Settings.NextId.get|=";
         $q->process( );
         if ( $q->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
         else {
            while ( $row = $q->getResultRow( ) ) {
               $nextId = $row[ "nextId" ];
            }
            if ( !is_numeric( $nextId ) ) {
               $nextId = 1;
            }
            $p->setVar( "id", $nextId, "PRIVATE" );
            $p->command = "=|Settings.post_metatablerecord.set|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not insert a record for the post_meta table name in the settings table: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
      }
      else {
               echo $promptChar . "Checked for a record for the post_meta table name in the settings table. It already exists . . . <br />";
      }         
   }

   /* check for create/modify userid fields */
   echo $promptChar . "Checking for create and modify user id columns in the post and comment tables . . . <br />";
   /* assume if not in post, not anywhere */
   if ( !$_session->connection->columnExists( $postTable, "create_user_id" ) ) {
      $p->command = "=|Columns.create.post_comment.create_and_modify|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during column creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
      else {
         echo $promptChar . "Created the create and modify user id columns in the comment table . . . <br />";
      }
   }
   else {
      echo $promptChar . "Checked for create and modify userid columns - they already exist . . . <br />";
   }
   
   /* see if the Private post type exists in the settings table. If it doesn't,
      insert the row */
   $q->command = "=|Settings.post_type_private.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the private post type settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Creating the private post type setting . . .<br />";
         $p->command = "=|Settings.post_type_private.insert|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not create the private post type in the settings table: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }
      else {
         echo $promptChar . "Checked for the private post type in the settings table. It already exists . . . <br />";
      }         
   }

   $_session->setVar( "queue_table", $tablePrefix . "queue", "POST" );
   $queueTable = $_session->getVar( "queue_table" );
   
   /* check to see if the queue table exists. If not, create it */
   if ( !$_session->connection->tableExists( $queueTable ) ) {
      echo $promptChar . "Creating the queue table . . . <br />";
      $p->command = "=|Table.create.queue|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during table creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
   }
   else {
      echo $promptChar . "Checked for the queue table - it already exists . . . <br />";
   }
   
   /* see if the queue table exists in the settings table. If it doesn't,
      insert the row */
   $q->command = "=|Settings.queuetablerecord.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the queue table settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Placing a record for the queue table name in the settings table <br />";
         
         $q->command = "=|Settings.NextId.get|=";
         $q->process( );
         if ( $q->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
         else {
            while ( $row = $q->getResultRow( ) ) {
               $nextId = $row[ "nextId" ];
            }
            if ( !is_numeric( $nextId ) ) {
               $nextId = 1;
            }
            $p->setVar( "id", $nextId, "PRIVATE" );
            $p->command = "=|Settings.queuetablerecord.set|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not insert a record for the queue table name in the settings table: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
      }
      else {
               echo $promptChar . "Checked for a record for the queue table name in the settings table. It already exists . . . <br />";
      }         
   }
   
   /* populate a record for the static_page core snapon in the SnapOn table, if it doesn't exist */
   echo $promptChar . "Checking for static_page core snap_on entry . . . <br />";
   $q->command = "=|SnapOns.static_page_entry.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if static_page entry exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Adding new static_page entry . . . <br />";
         
         $p->command = "=|SnapOns.static_page_entry.create|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not create static_page entry: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }
      else {
         echo $promptChar . "Page entry already exists . . . <br />";
      }         
   }

   /* create the page table if it doesn't already exist */
   $_session->setVar( "static_page_table", $tablePrefix . "static_page", "POST" );
   $staticPageTable = $_session->getVar( "static_page_table" );
   
   if ( !$_session->connection->tableExists( $staticPageTable ) ) {
      echo $promptChar . "Creating the static_page table . . . <br />";
      $p->command = "=|Table.create.static_page|=";
      $p->process( );

      if ( $p->isError( ) ) {
         echo $promptChar . "<b>Error:</b> An error occurred during table creation: " . $p->getError( ) . " . . . <br />";
         echo "<br />For help please email support@outofthebloo.com . . . <br />";
         $noProblems = false;
      }
   }
   else {
      echo $promptChar . "Checked for the static_page table - it already exists . . . <br />";
   }
   
   /* see if the static_page table exists in the settings table. If it doesn't,
      insert the row */
   $q->command = "=|Settings.staticpagetablerecord.exists|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the static_page table settings record exists: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Placing a record for the static_page table name in the settings table <br />";
         
         $q->command = "=|Settings.NextId.get|=";
         $q->process( );
         if ( $q->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not determine next settings id: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
         else {
            while ( $row = $q->getResultRow( ) ) {
               $nextId = $row[ "nextId" ];
            }
            if ( !is_numeric( $nextId ) ) {
               $nextId = 1;
            }
            $p->setVar( "id", $nextId, "PRIVATE" );
            $p->command = "=|Settings.staticpagetablerecord.set|=";
            $p->process( );
            if ( $p->isError( ) ) {
               echo $promptChar . "<b>Error:</b> Could not insert a record for the static_page table name in the settings table: " . $q->getError( ) . " . . . <br />";
               echo "<br />For help please email support@outofthebloo.com . . . <br />";
               $noProblems = false;
            }
         }
      }
      else {
               echo $promptChar . "Checked for a record for the static_page table name in the settings table. It already exists . . . <br />";
      }         
   }

   /* see if the static page types exists in the settings table. If not, create them */
   $q->command = "=|Settings.static_page_types.exist|=";
   $q->process( );
   if ( $q->isError( ) ) {
      echo $promptChar . "<b>Error:</b> Could not determine if the static page type settings records exist: " . $q->getError( ) . " . . . <br />";
      echo "<br />For help please email support@outofthebloo.com . . . <br />";
      $noProblems = false;
   }
   else {
      $exists = false;
      while ( $row = $q->getResultRow( ) ) {
         $exists = true;
      }

      if ( !$exists ) {
         echo $promptChar . "Creating the static page type settings . . .<br />";
         $p->command = "=|Settings.static_page_types.insert|=";
         $p->process( );
         if ( $p->isError( ) ) {
            echo $promptChar . "<b>Error:</b> Could not create the static page types in the settings table: " . $q->getError( ) . " . . . <br />";
            echo "<br />For help please email support@outofthebloo.com . . . <br />";
            $noProblems = false;
         }
      }
      else {
         echo $promptChar . "Checked for the static page types in the settings table. They already exists . . . <br />";
      }         
   }
   
   $upgradeComplete = true;
   
   /* send an XML ping to the out of the bloo domain, if desired */
   if ( SEND_PING ) {
      echo $promptChar . "Sending a ping to " . PING_HOST;

      /* open the socket */
      $fp = @fsockopen( PING_HOST, 80 );
      if ( $fp ) { /* don't display errors if this doesn't work */
         /* build the ping xml */
         $data = "<?xml version=\"1.0\"?>";
         $data .= "<methodCall>";
         $data .= "<methodName>blooPing</methodName>";
         $data .= "<params>";
         $data .= "<param>";
         $data .= "<value>BlooUpgrade</value>";
         $data .= "</param>";
         $data .= "<param>";
         $data .= "<value>" . $_session->getVar( "blog_name", "APPLICATION" ) . "</value>";
         $data .= "</param>";
         $data .= "<param>";
         $data .= "<value>" . $_session->getVar( "site_url", "APPLICATION" ) . "</value>";
         $data .= "</param>";
         $data .= "<param>";
         $data .= "<value>" . BLOO_VERSION . "</value>";
         $data .= "</param>";
         $data .= "</params>";
         $data .= "</methodCall>";

         /* build the header */
         $header = "POST " . PING_SHARE . "/bloo.ops.php HTTP/1.1\r\n";
         $header .= "User-Agent: PHPTest\r\n";
         $header .= "Host: " . PING_HOST . "\r\n";
         $header .= "Content-type: text/xml\r\n";
         $header .= "Content-Length: " . strlen( $data ) . "\r\n";
         $header .= "Connection: Close\r\n\r\n";

         $content = $header . $data;   

         /* send the ping */
         fwrite($fp, $content );

         /* get the results, but don't display */
         while (!feof($fp)) {
            $s = fgets($fp, 128);
         }

         /* close the socket */
         fclose($fp);
      }
   }
}


/* write out the page footer */
if ( $upgradeStarted ) {
   if ( $noProblems && $upgradeComplete ) {
      echo "<br /><br />" . $promptChar . "<b>Upgrade successful!</b><br />";
      echo "<br /><br />" . $promptChar . "You can get to your upgraded blog by clicking <a href=\"" . $_session->getVar( "site_url" ) . "\">here</a><br />";   
   }
   else {
      if ( !$noProblems ) {
         echo "<br /><br />" . $promptChar . "<b>There were problems with the upgrade.</b><br />";
      }
   }
}
   
$s =& $_session->snapOnFactory( "PhooDisplay", $_session );
$s->securityRole = ROLE_EVERYONE;
$s->template = "=|Upgrade.footer|=";
$s->process( );
$s->display( );


?>
