<?php
/* 
   Name: bloo_events.classes.php
   Author: Bill Roberts
   Created: 2/11/2006
   Purpose: Event Handling SnapOn classes related to the display and processing
   of blog data elements
   $Revision: 1.3 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/****************************************************************/
/*** BRToLineFeedConverter - converts BR to Line Feeds        ***/
/*** on form display. This is abstract and will be inherited  ***/
/*** to specialize for Posts, Comments, etc                   ***/
/*** this is a PreRender event handler                        ***/
/****************************************************************/
class BRToLineFeedConverter extends PreRenderEventHandler {
   var $package = "bloo_events";
   var $name = "BRToLineFeedConverter";
   var $securityRole = ROLE_EVERYONE;
   var $fieldName;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-11" );
      $this->setPurpose( "converts <BR> tags to linefeeds" );
   }

   function doAction( ) {
      $content = $this->container->getVar( $this->fieldName );

      if ( isset( $content ) ) {
         $content = str_replace( "<br />", "\n", $content );
         $this->container->setVar( $this->fieldName, $content, "PRIVATE" );
      }
   }
}

class PostBRToLineFeedConverter extends BRToLineFeedConverter {
   var $package = "bloo_events";
   var $name = "PostBRToLineFeedConverter";
   var $securityRole = ROLE_POSTER;
   var $fieldName = "post_content";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-11" );
      $this->setPurpose( "converts <BR> tags to linefeeds in posts" );
   }
}

class CommentBRToLineFeedConverter extends BRToLineFeedConverter {
   var $package = "bloo_events";
   var $name = "CommentBRToLineFeedConverter";
   var $securityRole = ROLE_EVERYONE;
   var $fieldName = "comment_content";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-12" );
      $this->setPurpose( "converts <BR> tags to linefeeds in comments" );
   }
}

/****************************************************************/
/*** LineFeedToBRConverter - converts line feeds to br tags   ***/
/*** on save. This is abstract and will be inherited          ***/
/*** to specialize for Posts, Comments, etc                   ***/
/*** this is a pre-save event handler                         ***/
/****************************************************************/
class LineFeedToBRConverter extends PreSaveEventHandler {
   var $package = "bloo_events";
   var $name = "LineFeedToBRConverter";
   var $securityRole = ROLE_EVERYONE;
   var $fieldName;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-12" );
      $this->setPurpose( "converts linefeeds to BR tags prior to saving a post" );
   }

   function doAction( ) {
      $content = $this->container->getVar( $this->fieldName );

      if ( isset( $content ) ) {
         $content = str_replace( "\n", "<br />", $content );
         $this->container->setVar( $this->fieldName, $content, "PRIVATE" );
      }
   }
}

class PostLineFeedToBRConverter extends LineFeedToBRConverter {
   var $package = "bloo_events";
   var $name = "PostLineFeedToBRConverter";
   var $securityRole = ROLE_POSTER;
   var $fieldName = "post_content";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-12" );
      $this->setPurpose( "converts linefeeds to <BR> tags in posts" );
   }
}

class CommentLineFeedToBRConverter extends LineFeedToBRConverter {
   var $package = "bloo_events";
   var $name = "CommentLineFeedToBRConverter";
   var $securityRole = ROLE_EVERYONE;
   var $fieldName = "comment_content";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-12" );
      $this->setPurpose( "converts linefeeds to <BR> tags in comments" );
   }
}

/*************************************************************************/
/*** PostMoreConverter - converts a <!--more--> tag to a link to the   ***/
/*** permalink of the post (if this is not a single post display       ***/
/*************************************************************************/
class PostMoreConverter extends PostRowRetrieveEventHandler {
   var $package = "bloo_events";
   var $name = "PostMoreConverter";
   var $securityRole = ROLE_EVERYONE;
   var $fieldName = "post_content";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-10-10" );
      $this->setPurpose( "converts the <!--more--> tag to a 'Read the rest of this post' permalink" );
   }

   function doAction( ) {
      $content = $this->container->getVar( $this->fieldName );
      $pos = strpos( $content, "<!--more-->" );
      if ( $pos > 0 ) {
         if ( $this->container->singlePost ) {
            $replacement = $this->expandOverlays( "=|Post.anchor.more|=" );
            $content = str_replace( "<!--more-->", $replacement, $content );
         }
         else { /* cut off everything after the more and replace a link */
            $additional = $this->expandOverlays( "=|Post.link.more|=" );
            $content = substr( $content, 0, $pos ) . $additional;
         }
         $this->container->setVar( $this->fieldName, $content, "PRIVATE" );
      }
   }
}


/****************************************************************/
/*** TagBalancer - balances tags in a post or comment         ***/
/*** This is abstract and will be inherited                   ***/
/*** to specialize for Posts, Comments, etc                   ***/
/*** This is a pre-save event handler                         ***/
/****************************************************************/
class TagBalancer extends PreSaveEventHandler {
   var $package = "bloo_events";
   var $name = "TagBalancer";
   var $securityRole = ROLE_EVERYONE;
   var $fieldName = array( ); /* fields to balance */
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-12" );
      $this->setPurpose( "balance html tags in content" );
   }
   
   function getNextTag( $aHtml, &$aStartPos ) {
      /* locate opening tag, if any */
      
      /* since strpos will remove a zero on the first char if it's a tag character,
         check it manually if starting position is 0 */
      if ( $aStartPos == 0 && substr( $aHtml, 0, 1 ) == "<" ) {
         $sPos = 0; 
      }
      else {
         $sPos = strpos( $aHtml, "<", $aStartPos );
      
         if ( $sPos == 0 ) {
            return null;
         }
      }

      /* find the end of the opening tag */
      $ePos = strpos( $aHtml, ">", $sPos );
      if ( $ePos == 0 ) {
         return null;
      }

      $aStartPos = $ePos + 1;
      return substr( $aHtml, $sPos, $ePos - $sPos + 1 );
   }

   function balanceTags( $aHtml ) {
      $singleTags = array( "br", "img", "hr" ); /* don't balance these tags */
      
      $html = $aHtml;
      $nodeStack = array( );
      
      /* loop through, locating tags */
      $looping = true;

      $pos = 0; 
      while ( $looping ) {
         $tag = $this->getNextTag( $html, $pos );
         if ( !isset( $tag ) ) {
            $looping = false;
         }
         else {
            /* strip the tag down to its node name and whether it's an end or a beginning tag */
            eregi( "<([a-zA-Z]+)", $tag, $match );
            if ( isset( $match[1] ) ) {
               $nodeType = TAG_START;
               $nodeName = $match[1];
               unset( $match[1]);
               if ( !in_array( $nodeName, $singleTags ) ) { 
                  /* new element - push it on the stack */
                  array_push( $nodeStack, $nodeName );
               }
            }
            else {
               eregi( "</([a-zA-Z]+)", $tag, $match );
               if ( isset( $match[1] ) ) {
                  $nodeType = TAG_END;
                  $nodeName = $match[1];
                  unset( $match[1]);
                  
                  /* pop off the top element on the stack IF it 
                     matches this one */
                  if ( !in_array( $nodeName, $singleTags ) ) { 
                     $topNodeName = array_pop( $nodeStack );
                     if ( $topNodeName != $nodeName ) {
                        /* push it back on - no match */
                        array_push( $nodeStack, $topNodeName );
                     }
                  }
               }
            }
         }
      }
      while ( $orphanNode = array_pop( $nodeStack ) ) {
         $html .= "</" . $orphanNode . ">"; 
      }
      return $html;
   }

   function doAction( ) {
      if ( is_array( $this->fieldName ) ) {
         foreach ( $this->fieldName as $f ) {
            $html = $this->container->getVar( $f );
            if ( isset( $html ) ) {
               $html = $this->balanceTags( $html );
               $this->container->setVar( $f, $html, "PRIVATE" );
            }
         }
      }
   }
}

class PostTagBalancer extends TagBalancer {
   var $package = "bloo_events";
   var $name = "PostTagBalancer";
   var $securityRole = ROLE_POSTER;
   var $fieldName = array( "post_content", "post_title" );

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-12" );
      $this->setPurpose( "balances tags in post content and title" );
   }
}

class CommentTagBalancer extends TagBalancer {
   var $package = "bloo_events";
   var $name = "CommentTagBalancer";
   var $securityRole = ROLE_EVERYONE;
   var $fieldName = array( "comment_content", "comment_author", "comment_author_email", "comment_author_url" );

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-12" );
      $this->setPurpose( "balances tags in comment fields" );
   }
}

/*******************************************************************/
/*** DraftDateSwitcher - if a post is converted from Draft       ***/
/*** to public, set the post date to now                         ***/
/*** this is a pre-save event handler                            ***/
/*******************************************************************/
class DraftDateSwitcher extends PreSaveEventHandler {
   var $package = "bloo_events";
   var $name = "DraftDateSwitcher";
   var $securityRole = ROLE_POSTER;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-12" );
      $this->setPurpose( "sets the post date to now anytime a post is moved from draft to public" );
   }

   function doAction( ) {
      $postId =  $this->container->getVar( "post_id" );

      if ( !isset( $postId ) ) {
         /* this is probably an insert - do nothing */
         return;
      }
      
      $newPostTypeId = $this->container->getVar( "post_type_id" );
      if ( $newPostTypeId != POST_DRAFT ) {
         /* check to see if the post is currently a draft */
         $s = $this->session->snapOnFactory( "PhooQuery", $this );
         $s->securityRole = $this->securityRole;
         $s->command = "=|Post.type.get|=";
         $s->process( );
         if ( $s->isError( ) ) {
            return;
         }

         $row = $s->getResultRow( );
         if ( is_array( $row ) ) {
            $postTypeId = $row[ "post_type_id" ];
            if ( $postTypeId == POST_DRAFT ) {
               /* update the post's date */
               $s = $this->session->snapOnFactory( "PhooQuery", $this );
               $s->securityRole = $this->securityRole;
               $s->command = "=|Post.date.touch|=";
               $s->process( );
               if ( $s->isError( ) ) {
                  return;
               }
            }   
         }
      }
   }
}

/****************************************************************/
/*** Comment Flood Query - helps prevent comment flooding by  ***/
/*** preventing the same IP address from sending a multitude  ***/
/*** of comments in a very short time                         ***/
/*** This is a pre-save event handler                         ***/
/****************************************************************/
class CommentFloodPreventer extends PreSaveEventHandler {
   var $package = "bloo_events";
   var $name = "CommentFloodPreventer";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-23" );
      $this->setPurpose( "Helps prevent comment flooding" );
   }

   function doAction( ) { /* on insert only */
      $action = $this->container->getVar( FORM_ACTION );
      if ( $action == "insert" ) {   
         /* run the jittery post same comment twice query. This will catch comments that 
            posted - same exact content, etc - within mere seconds of eachother. */
         $jitteryQuery =& $this->session->snapOnFactory( "PhooQuery", $this );
         $jitteryQuery->securityRole = $this->securityRole;
         $jitteryQuery->command = "=|Comment.jitteryquery|=";
         /* get the comment author IP */
         $jitteryQuery->setVar( "comment_author_ip", 
                              $this->getVar( "REMOTE_ADDR", "SERVER" ), "PRIVATE" );
         $jitteryQuery->setVar( "comment_content", 
                              $this->container->getVar( "comment_content", "PRIVATE" ), "PRIVATE" );
         
         $jitteryQuery->process( );
         if ( $this->isError( ) ) {
            return;
         }

         $row = $jitteryQuery->getResultRow( );
         if ( is_array( $row ) ) {
            $this->container->dontProcessCommand = true; /* will just quietly not process the command */
         }   
         else {
            /* run the flood query */
            $floodQuery =& $this->session->snapOnFactory( "PhooQuery", $this );
            $floodQuery->securityRole = $this->securityRole;
            $floodQuery->command = "=|Comment.floodquery|=";
            /* get the comment author IP */
            $floodQuery->setVar( "comment_author_ip", 
                                 $this->getVar( "REMOTE_ADDR", "SERVER" ), "PRIVATE" );
         
            $floodQuery->process( );
            if ( $this->isError( ) ) {
               return;
            }

            $row = $floodQuery->getResultRow( );
            if ( is_array( $row ) ) {
               $commentFloodSeconds = $this->getVar( "comment_flood_seconds", "APPLICATION" );  
               $this->setError( "You may only leave a comment once every " . $commentFloodSeconds . " seconds." .
                                " Wait a few more seconds and submit the comment again",
                                MSG_WARNING );
            }
         }

      }

      /* clear successPage so that a redirect won't happen */
      /*** What is this? ***
      $this->spStore = $this->successPage;
      $this->setSuccessPage( "" );
      parent::triggerProcess( $process );
      
      if ( $this->isError( ) ) {
         return;
      }

      $this->setSuccessPage( $this->spStore );
      ***/
   }
}

/*******************************************************************/
/*** Comment Cookie Baker - Stores information on the commenter  ***/
/*** in a cookie.                                                ***/
/*** This is a post-save event handler                           ***/
/*******************************************************************/
class CommentCookieBaker extends PostSaveEventHandler {
   var $package = "bloo_events";
   var $name = "CommentCookieBaker";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-05" );
      $this->setPurpose( "Stores comment information (author url, email, name) in a cookie - on insert only" );
   }

   function doAction( ) {
      $action = $this->container->getVar( FORM_ACTION );

      if ( $action == "insert" ) { /* inserting, store the comment author, url, email, and guest comment edit key in a cookie */
         $this->setVar( "comment_author", 
         $this->container->getVar( "comment_author" ), "COOKIE", 
                              time()+60*60*24*30 );

         $this->setVar( "comment_author_url", 
            $this->container->getVar( "comment_author_url", "POST" ), "COOKIE", 
                              time()+60*60*24*30 );

         $this->setVar( "comment_author_email", 
            $this->container->getVar( "comment_author_email", "POST" ), "COOKIE", 
                              time()+60*60*24*30 );

         $this->setVar( "comment_guest_edit_key", 
            $this->container->getVar( "comment_guest_edit_key", "POST" ), "COOKIE", 
                              time()+60*60*24*30 );
                              
      }
   }
}

/*******************************************************************/
/*** Comment Cookie Eater - Loads information on the commenter   ***/
/*** from a cookie.                                              ***/
/*** This is a pre-render event handler                          ***/
/*******************************************************************/
class CommentCookieEater extends PreRenderEventHandler {
   var $package = "bloo_events";
   var $name = "CommentCookieEater";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-05" );
      $this->setPurpose( "Retrieves comment information (author url, email, name) from a cookie - on insert only" );
   }

   function doAction( ) {
      $action = $this->container->getVar( FORM_ACTION );

      if ( $action == "insert" ) { /* inserting, store the comment author and url in a cookie */
         $commentAuthor = $this->container->getVar( "comment_author" );
	 
         if ( !$commentAuthor ) {
            $this->container->setVar( "comment_author", $this->getVar( "comment_author", "COOKIE" ) );
         }

         $commenAuthortUrl = $this->container->getVar( "comment_author_url" );
	 if ( !$commentAuthorUrl ) {
            $this->container->setVar( "comment_author_url", $this->getVar( "comment_author_url", "COOKIE" ) );
	 }

         $commentAuthorEmail = $this->container->getVar( "comment_author_email" );
	 if ( !$commentAuthorEmail ) {
            $this->container->setVar( "comment_author_email", $this->getVar( "comment_author_email", "COOKIE" ) );
	 }
	 
      }
   }
}

/*******************************************************************/
/*** Comment Mailer - Mails comments to the post author          ***/
/*** This is a post-save event handler                           ***/
/*******************************************************************/
class CommentMailer extends PostSaveEventHandler {
   var $package = "bloo_events";
   var $name = "CommentMailer";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-28" );
      $this->setPurpose( "Sends an email to the post author on a comment insert" );
   }

   function setup( ) {
      parent::setup( );
      /* if this is a test blog, just disable */
      $testBlog = $this->session->getSetting( "test_blog" );
      if ( $testBlog == "Y" ) {
         $this->disabled = true;
      }
   }

   function doAction( ) {
      $action = $this->container->getVar( FORM_ACTION );
      if ( $action == "insert" ) { /* send the email */
         /* on insert, email the post author */
         $s =& $this->session->snapOnFactory( "PostAuthorEmail", $this );
         $s->process( );
         $authorEmail = $s->render( );
         
         $s =& $this->session->snapOnFactory( "PostTitle", $this );
         $s->process( );
         $title = $s->render( );

         /* use the commenter's email as the mail sender, if it exists. Otherwise use their name */
         $fromEmail = $this->getVar( "comment_author_email" );
         if ( $fromEmail == "" ) {
            /* no email - will need to make up one */
            $host = $this->getVar( "HTTP_HOST", "SERVER" );
            $fromEmail = "comments@" . $host;
         }           
         
         $s =& $this->session->snapOnFactory( "MailSender", $this );
         $commentContent = $this->getVar( "comment_content", "POST" );
         /* replace any <br /> tags with linefeeds */
         $commentContent = stripslashes( str_replace( "<br />", "\n", $commentContent ) );
         
         $commentAuthor = $this->getVar( "comment_author", "POST" );

         $from = $commentAuthor . " <" . $fromEmail . ">";

         $blogTitle = $this->getVar( "blog_name", "APPLICATION" );
         
         $s->setMail( $authorEmail, $blogTitle . ": Comment on post \"" . $title . 
                      "\"", $commentContent . "\n" . "- " . $commentAuthor, $from );
         $s->process( );
      }
   }
}


/*******************************************************************/
/*** Trackback Mailer - Mails trackbacks to the post author      ***/
/*** This is a post-save event handler                           ***/
/*******************************************************************/
class TrackbackMailer extends PostSaveEventHandler {
   var $securityRole = ROLE_EVERYONE;

   function setup( ) {
      parent::setup( );
      /* if this is a test blog, just disable */
      $testBlog = $this->session->getSetting( "test_blog" );
      if ( $testBlog == "Y" ) {
         $this->disabled = true;
      }
   }

   function doAction( ) {
      $this->setVar( "post_id", $this->container->getVar( "trackback_post_id" ), "POST" );
      $s =& $this->session->snapOnFactory( "PostAuthorEmail", $this );
      $s->process( );
      $authorEmail = $s->render( );
         
      $s =& $this->session->snapOnFactory( "PostTitle", $this );
      $s->process( );
      $title = $s->render( );

      /* Use the trackback blog name as the email sender */
      $host = $this->getVar( "HTTP_HOST", "SERVER" );
      $trackbackBlog = $this->container->getVar( "trackback_blog_name" );
      $fromEmail = "trackbacks@" . $host;
         
      $s =& $this->session->snapOnFactory( "MailSender", $this );
      $trackbackContent = $this->container->getVar( "trackback_excerpt" );
      /* replace any <br /> tags with linefeeds */
      $trackbackContent = stripslashes( str_replace( "<br />", "\n", $trackbackContent ) );
      $trackbackLink = "<a href=\"" . $this->container->getVar( "trackback_blog_url" ) . "\">" . $trackbackBlog .
                       "</a>";
         
      $from = $trackbackBlog . " <" . $fromEmail . ">";

      $blogTitle = $this->getVar( "blog_name", "APPLICATION" );
         
      $s->setMail( $authorEmail, $blogTitle . ": Trackback on post \"" . $title . 
                                 "\"", $trackbackContent . "\n" . "- " . $trackbackLink, $from );

      $s->process( );
   }
}

?>
