<?php
/* 
   Name: bloo_pages.classes.php
   Author: Bill Roberts
   Created: 2/14/2006
   $Revision: 1.3 $

   Purpose: Contains the page snapon classes for the major pages 
            within bloo

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/************************************************************/
/* BlooPage - ancestor for all bloo pages                   */
/************************************************************/
class BlooPage extends PhooPage {
   var $package = "page";
   var $name = "BlooPage";
   var $securityRole = ROLE_EVERYONE;
   var $templateHeader = "=|Page.header|=";
   var $templateMenu = "";
   var $template = "";
   var $templateFooter = "=|Page.footer|=";
}

/************************************************************/
/* BlooMainPage - this is the main/front page of the blog   */
/************************************************************/
class BlooMainPage extends BlooPage {
   var $package = "bloo_pages";
   var $name = "BlooMainPage";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Page.main|=";
}

/************************************************************/
/* BlooAdminPage - ancestor for all bloo administrative     */
/* pages                                                    */
/************************************************************/
class BlooAdminPage extends BlooPage {
   var $package = "bloo_pages";
   var $name = "BlooAdminPage";
   var $securityRole = ROLE_EVERYONE; /* note: security will be checked on whatever page this spawns */
   var $templateHeader = "=|Page.admin.header|=";
   var $templateFooter = "=|Page.admin.footer|=";
   var $templateMenu = "=|Menu.admin|=";

   function setup( ) {
      parent::setup( );
      /* This is a gatekeeper - you can't do an admin page
         unless you are logged in */

      if ( !$this->session->isLoggedIn( ) ) {
         $this->template = "_{LoginForm}_";
         $this->templateMenu = ""; /* clear the menu - we just want the login form to display */
         $this->setupComplete = true;
      }
      else { /* we're logged in - we can proceed */
         /* determine the main template for this page, based on the 
            page variable passed */
         $page = $this->getVar( "page" );
         switch ( $page ) {
            case "settingsAdmin" :
               $pageSnapOn = "AdminSettingsPageSection";
               break;
            
            case "userAdmin" :
               $pageSnapOn = "AdminUsersPageSection";
               break;
      
            case "profileAdmin" :
               $pageSnapOn = "AdminProfilePageSection";
               break;
            
            case "profilePasswordAdmin" :
               $pageSnapOn = "AdminProfilePasswordPageSection";
               break;
      
            case "log" :
               $pageSnapOn = "LogPageSection";
               break;

            case "extensionAdmin" :
               $pageSnapOn = "AdminSnapOnPageSection";
               break;
     
            case "lookAndFeelAdmin" :
               /* check for a help action */
               $action = $this->getVar( "action" );
               $subPage = $this->getVar( "subpage" );
               if ( $action == "help" && $subPage == "display" ) { /* display the date format help */
                  $pageSnapOn = "HelpPageSection";
                  $this->setVar( "help_overlay", "=|DateFormat.help|=", "POST" );
               }
               else {
                  $pageSnapOn = "AdminLookAndFeelPageSection";
               }
               break;
     
            case "moderationAdmin" :
               $pageSnapOn = "AdminModerationPageSection";
               break;
               
            case "pageAdmin" :
               $pageSnapOn = "AdminPagePageSection";
               break;
               
            default :
               /* get user role(s) */
               $userRole = $this->session->getUserRoles( );
               /* set the default page based on the role */
               if ( in_array( ROLE_POSTER, $userRole ) || in_array( ROLE_ADMINISTRATOR, $userRole ) ) {
                  $pageSnapOn = "AdminPostPageSection";
               }
               else { /* assume a low-securitied person. Put them in profile admin */
                  $pageSnapOn = "AdminProfilePageSection";
                  $this->setVar( "page", "profileAdmin", "GET" );
               }
               break;
         }
         $this->template = EMBEDDEDSNAPON_PREFIX . $pageSnapOn . EMBEDDEDSNAPON_SUFFIX;
      }
   }
}

/*************************************************************/
/* Page Sections                                             */
/*************************************************************/

/****************************************************************/
/* PostPageSection - creates the snapon/overlay configuration   */
/* for the portion of the page that displays posts, based       */
/* on the parameters passed                                     */
/****************************************************************/
class PostPageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "PostPageSection";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-14" );
      $this->setPurpose( "renders the post section of the main page of the blog" );
   }

   function setup( ) {
      parent::setup( );
      /* if recent commentary GET var set, show recent commentary */
      $recentCommentaryFlag = $this->getVar( "recent_commentary", "GET" );
      if ( isset( $recentCommentaryFlag ) ) {
         $this->template = "=|Comment.recent.list|=";
      }
      else {
         $postId = $this->getVar( "post_id" );

         if ( isset( $postId ) ) { /* display a single post */
            $this->template = "=|Post.single|=";
         }
         else {
            /* check for a static page request */
            $staticPageId = $this->getVar( "static_page_id" );
            if ( isset( $staticPageId ) ) {
               $this->session->setSetting( "allowed_page_type_list", "1" );
               $this->template = "_{StaticPageDisplay}_";
            }
            else {
               $this->template = "=|Post.list|=";
            }
         }
      }
   }

   function doAction( ) {
      /* check for a delete command. If one was passed,
         perform the delete action instead of the default
         action */
      $action = $this->getVar( FORM_ACTION, "GET" );
      $postId = $this->getVar( "post_id" );
      $commentId = $this->getVar( "comment_id" );
      $trackbackId = $this->getVar( "trackback_id" );
      /* note: if comment id is set, this is a request to delete a comment, not a post, so ignore */
      if ( $action == "delete" && isset( $postId ) && !isset( $commentId ) && !isset( $trackbackId ) ) {
         /* process a post deleter */
         $s =& $this->session->snapOnFactory( "PostDeleter", $this );
         /* set the success page to take you back to the calling page, with no parameters
         */
         $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
         $s->process( );
         if ( $s->isError( ) ) {
            /* load the template up with the error syntax, etc */
            $this->template .= $s->render( );
            return;
         }
      }
      else {
         /* check for a delete of a static page */
         $pageId = $this->getVar( "static_page_id" );
         if ( $action == "delete" && isset( $pageId ) ) {
            /* process a static page deleter */
            $s =& $this->session->snapOnFactory( "StaticPageDeleter", $this );
            /* set the success page to take you back to the calling page */
            $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
            $s->process( );
            if ( $s->isError( ) ) {
               /* load the template up with the error syntax, etc */
               $this->template .= $s->render( );
               return;
            }
         }
         else {   
            
            /* continue on */
            parent::doAction( );
         }
      }
   }
}

/*************************************************************/
/* AdminPostPageSection                                      */
/* This controls the section of the administration page      */
/* for creating and editing posts                            */
/*************************************************************/
class AdminPostPageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "AdminPostPageSection";
   var $securityRole = ROLE_POSTER; 
   var $template = "=|PageSection.admin.post|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.3" );
      $this->setCreateDate( "2005-05-27" );
      $this->setPurpose( "Page for post administration" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-31" );
      $this->addModificationHistory( "refactored per code review", "2006-02-17" );
   }
   
   function setup(  ) {
      parent::setup( );

      /* check to see if a post id was passed. If so, display the post */   
      $postId = $this->getVar( "post_id" );
      if ( isset( $postId ) ) {
         $this->template .= "=|Post.admin.post.display|="; 
         /* modify the allowed post types to include draft */
         $allowedPostTypeList = $this->session->getSetting( "allowed_post_type_list" );
         $this->session->setSetting( "allowed_post_type_list", $allowedPostTypeList . ", 2" );
         
      }
   }

   function doAction( ) {
      /* check for a delete command. If one was passed,
         perform the delete action instead of the default
         action */
      $action = $this->getVar( FORM_ACTION, "GET" );
      $postId = $this->getVar( "post_id" );
      $commentId = $this->getVar( "comment_id" );
      /* note: if comment id is set, this is a request to delete a comment, not a post, so ignore */
      if ( $action == "delete" && isset( $postId ) && !isset( $commentId ) ) {
         /* process a post deleter */
         $s =& $this->session->snapOnFactory( "PostDeleter", $this );
         /* set the success page to take you back to the calling page */
         $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
         $s->process( );
         if ( $s->isError( ) ) {
            /* load the template up with the error syntax, etc */
            $this->template .= $s->render( );
            return;
         }
      }
      else { /* otherwise, continue on */
         parent::doAction( );
      }
   }
}

/*************************************************************/
/* AdminPagePageSection                                      */
/* This controls the section of the administration page      */
/* for creating and editing pages                            */
/*************************************************************/
class AdminPagePageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "AdminPagePageSection";
   var $securityRole = ROLE_POSTER; 
   var $template = "=|PageSection.admin.staticpage|=";
   
   function setup(  ) {
      parent::setup( );

      /* can display both public and draft pages */
      $this->session->setSetting( "allowed_page_type_list", "1, 2" );
      /* check to see if a page id was passed. If so, display the page */   
      $pageId = $this->getVar( "page_id" );
      if ( isset( $pageId ) ) {
         $this->template .= "=|Page.admin.page.display|="; 
      }
   }

   function doAction( ) {
      /* check for a delete command. If one was passed,
         perform the delete action instead of the default
         action */
      $action = $this->getVar( FORM_ACTION, "GET" );
      $pageId = $this->getVar( "static_page_id" );
      
      if ( $action == "delete" && isset( $pageId ) ) {
         /* process a static page deleter */
         $s =& $this->session->snapOnFactory( "StaticPageDeleter", $this );
         /* set the success page to take you back to the calling page */
         $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
         $s->process( );
         if ( $s->isError( ) ) {
            /* load the template up with the error syntax, etc */
            $this->template .= $s->render( );
            return;
         }
      }
      else { /* otherwise, continue on */
         parent::doAction( );
      }
   }
}

/*************************************************************/
/* AdminUsersPageSection                                     */
/* This controls the section of the administration page      */
/* for administering users                                   */
/*************************************************************/
class AdminUsersPageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "AdminUsersPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;

   function setup(  ) {
      parent::setup( );

      $action = $this->getVar( FORM_ACTION, "GET" );
      switch ( $action ) {
         case "insert" :
         case "update" :
            $this->template .= "_{UserForm}_<br><br>";
            break;

         case "reset_password" :
            $this->template .= "_{ResetPasswordForm}_<br><br>";
            break;
         
         default :
            $this->template .= "_{UserList}_<br><br>";
            break;
      }
   }

   function doAction( ) {
      /* check for a delete command. If one was passed,
         perform the delete action instead of the default
         action */
      $action = $this->getVar( FORM_ACTION, "GET" );
      if ( $action == "delete" ) {
               
         $s =& $this->session->snapOnFactory( "UserDeleter", $this );
         /* set the success page to take you back to the calling page */
         $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
         $s->process( );
         if ( $s->isError( ) ) {
            /* load the template up with the error syntax, etc */
            $this->template .= $s->render( );
            return;
         }
      }
      else { /* otherwise, continue on */
         parent::doAction( );
      }
   }
}

/*************************************************************/
/* AdminProfilePageSection, AdminProfilePasswordPageSection  */
/* This controls the section of the administration page      */
/* for administering a users' profile                        */
/*************************************************************/
class AdminProfilePageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "AdminProfilePageSection";
   var $securityRole = ROLE_EVERYONE; /* Everyone can manage their profile */
   var $template = "_{UserProfileForm}_";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-12-09" );
      $this->setPurpose( "Page for user profile administration" );
      $this->addModificationHistory( "refactored per code review", "2006-02-17" );
   }
}

class AdminProfilePasswordPageSection extends PhooClass {
   var $package = "bloo_pages";
   var $name = "AdminProfilePasswordPageSection";
   var $securityRole = ROLE_EVERYONE; /* everyone can manage their password */
   var $template = "_{UserProfilePasswordForm}_";
}

/*************************************************************/
/* LogPageSection                                            */
/* Displays contents of the system log                       */
/*************************************************************/
class LogPageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "LogPageSection";
   var $securityRole = array( ROLE_POSTER, ROLE_ADMINISTRATOR );
   var $template = "_{SystemLogDisplay}_";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-08-11" );
      $this->setPurpose( "Page for log viewing" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-31" );
      $this->addModificationHistory( "refactored per code review", "2006-02-17" );
   }
}

/*************************************************************/
/* AdminSnapOnPageSection                                 */
/* This controls the administration and activation of        */
/* extensions                                                */
/*************************************************************/
class AdminSnapOnPageSection extends PhooDisplay { 
   var $package = "bloo_pages";
   var $name = "AdminSnapOnPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;  
   var $template = "_{ListSnapOnList}_"; 

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-25" );
      $this->setPurpose( "Page for extension administration" );
   }

   function doAction( ) {
      $action = $this->getVar( FORM_ACTION, "GET" );
      
      /* process based on the action */
      switch ( $action ) {
         case "activate" :
         case "deactivate" :
            /* activate or deactivate an extension */
            $s =& $this->session->snapOnFactory( "SnapOnActivator", $this );
            $s->process( ); /* process the activation/deactivation */
            if ( $s->isError( ) ) {
               return;
            }
            break;
      }

      /* and now back to our regularly scheduled action */      
      parent::doAction( );
   }  

   function renderSnapOn( ) { /* determine the snapon or overlay to use in the case of special actions */
      $action = $this->getVar( FORM_ACTION );

      switch ( $action ) {
         case "administer" :
         case "insert" :
         case "update" :
         case "delete" : 
         case "set"    : /* all actions to take place on the extension administration */
            $extensionName = $this->getVar( "extension_name" );
            $descriptor = $this->session->getSnapOnDescriptor( $extensionName );
            if ( !is_array( $descriptor ) ) {
               $this->setError( "Illegal Extension SnapOn Name: $extensionName", MSG_FATAL );
               return;
            }
            $administrationClass = $descriptor[ "administration_class" ];
            if ( $administrationClass > "" ) {
               $s =& $this->session->snapOnFactory( $administrationClass, $this );
               $s->process( );

               return $s->renderSnapOn( );
            }
            break;

         case "help" : /* display a help page */
            $extensionName = $this->getVar( "extension_name" );
            $descriptor = $this->session->getSnapOnDescriptor( $extensionName );
            if ( !is_array( $descriptor ) ) {
               $this->setError( "Illegal Extension SnapOn Name: $extensionName", MSG_FATAL );
               return;
            }
            $helpOverlay = $descriptor[ "help_overlay" ];
            if ( $helpOverlay > "" ) {
               $this->setVar( "help_overlay", $helpOverlay, "POST" );
               
               $s =& $this->session->snapOnFactory( "HelpPageSection", $this );
               $s->process( );

               return $s->renderSnapOn( );
            }
            break;

         default :
            return parent::renderSnapOn( );
            break;
      }
   }
}

/*************************************************************/
/* HelpPageSection                                           */
/* Displays a page of help, based on the help_snapon variable*/
/*************************************************************/
class HelpPageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "HelpPageSection";
   var $securityRole = ROLE_EVERYONE;  

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-28" );
      $this->addExpects( "help_overlay", "POST" );
      $this->setPurpose( "Page for displaying help" );
   }

   function setup( ) {
      parent::setup( );
      $this->template = $this->getVar( "help_overlay", "POST" );
   }
}

/*************************************************************/
/* AdminLookAndFeelPageSection                               */
/* Displays administration page for the blog's look and feel */
/*************************************************************/
class AdminLookAndFeelPageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "AdminLookAndFeelPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;  

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-09" );
      $this->setPurpose( "Page for displaying template information" );
   }

   function setup( ) {
      /* determine the subpage */
      $subPage = $this->getVar( "subpage" );
      if ( $subPage == "" ) { /* default */
         $subPage = "overlays";
      }

      $action = $this->getVar( FORM_ACTION );

      switch ( $subPage ) {
         case "themes" :
            $this->template = "=|PageSection.admin.look_and_feel.themes|=";
            if ( $action == "insert" || $this->getVar( "btnInsert" ) > "" 
              || $action == "update" || $this->getVar( "btnUpdate" ) > "" ) {
               /* template is the themes form */
               $this->template .= "_{ThemeForm}_";

               /* if action not set, go ahead and set it */
               if ( !isset( $action ) ) {
                  if ( $this->getVar( "btnInsert" ) ) { 
                     $this->setVar( FORM_ACTION, "insert", "PRIVATE" );
                  }
                  else {
                     $this->setVar( FORM_ACTION, "update", "PRIVATE" );
                  }
               }                     
            }
            else {
               $this->template .= "_{ThemeList}_";
            }
            
            break;

         case "overlays" :
            $this->template = "=|PageSection.admin.look_and_feel.overlays|=";
            if ( $action == "insert" || $this->getVar( "btnInsert" ) > "" 
              || $action == "update" || $this->getVar( "btnUpdate" ) > "" ) {
               /* template is the themes form */
               $this->template .= "_{TemplateForm}_";

               /* if action not set, go ahead and set it */
               if ( !isset( $action ) ) {
                  if ( $this->getVar( "btnInsert" ) ) { 
                     $this->setVar( FORM_ACTION, "insert", "PRIVATE" );
                  }
                  else {
                     $this->setVar( FORM_ACTION, "update", "PRIVATE" );
                  }
               }                     
            }
            else {
               $this->template .= "_{TemplateList}_";
            }
            
            break;
            
         case "display" :
            $this->template .= "_{DisplaySettingsAdminForm}_";
            break; 
      }
   }      

   function doAction( ) {
      $action = $this->getVar( FORM_ACTION, "GET" );
      $subPage = $this->getVar( "subpage" );
      /* perform special processes based on the action */
      switch ( $subPage ) {
         case "themes" :
            switch ( $action ) {
               case "activate" :
                  $s =& $this->session->snapOnFactory( "ThemeDefaultSetter", $this );
                  $s->process( ); /* process the default-setting */
                  if ( $s->isError( ) ) {
                     return;
                  }
                  break;
                  
               case "delete" :
                  $s =& $this->session->snapOnFactory( "ThemeDeleter", $this );
                  $s->process( );
                  if ( $s->isError( ) ) {
                     return;
                  }
                  break;
                  
            }
            break;

         case "overlays" :
            switch ( $action ) {
               case "activate" :
               case "deactivate" :
                  /* activate or deactivate a template */
                  $s =& $this->session->snapOnFactory( "TemplateActivator", $this );
                  $s->process( ); /* process the activation/deactivation */
                  if ( $s->isError( ) ) {
                     return;
                  }

                  break;
               case "delete" :
                  $s =& $this->session->snapOnFactory( "TemplateDeleter", $this );
                  $s->process( );
                  if ( $s->isError( ) ) {
                     return;
                  }
                  break;
            }
            break;
            
      }
      /* and now back to our regularly scheduled action */      
      parent::doAction( );
   }  
}

/*************************************************************/
/* AdminModerationPageSection                                */
/* Displays comment and trackback moderation pages           */
/*************************************************************/
class AdminModerationPageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "AdminModerationPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;  

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-10-30" );
      $this->setPurpose( "Page for displaying and managing moderated comments and trackbacks information" );
   }

   function setup( ) {
      /* determine the subpage */
      $subPage = $this->getVar( "subpage" );
      if ( $subPage == "" ) { /* default */
         $subPage = "approved_comments";
      }

      $action = $this->getVar( FORM_ACTION );

      switch ( $subPage ) {
         case "approved_comments" :
            $this->template = "=|PageSection.admin.approved.comments|=";
            
            break;
            
         case "unapproved_comments" :
            $this->template = "=|PageSection.admin.unapproved.comments|=";
            
            break;

         case "approved_trackbacks" :
            $this->template = "=|PageSection.admin.approved.trackbacks|=";
            
            break;
            
         case "unapproved_trackbacks" :
            $this->template = "=|PageSection.admin.unapproved.trackbacks|=";
            
            break;
      }
   }      
}

/*************************************************************/
/* LoginLink                                                 */
/* Displays a link, either "Login" or "Administration"       */
/*************************************************************/
class LoginLink extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "LoginLink";
   var $securityRole = ROLE_EVERYONE;  

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-02" );
      $this->setPurpose( "displays a link for logging in" );
   }

   function setup( ) {
      parent::setup( );
      if ( $this->session->isLoggedIn( ) ) {
         $this->template = "=|Link.administer|=";
      }
      else {
         $this->template = "=|Link.login|=";
      }
   }
}

/*************************************************************/
/* Session Message                                           */
/* Displays a System Message, stored in a session variable   */
/*************************************************************/
class SessionMessage extends PhooDisplay {
   var $package = "bloo_pages";
   var $name = "SessionMessage";
   var $securityRole = ROLE_EVERYONE;  
   var $template = "=|Session.message|=";
   
   function setup( ) {
      parent::setup( );
      $msgAr = $this->session->getSessionMessage( );

      $sessionMessage = $msgAr[ "session_message" ];

      if ( isset( $sessionMessage ) ) {
         
         switch ( $msgAr[ "session_message_type" ] ) {
            case MSG_FATAL :
               $class = "error";
               break;
               
            case MSG_WARNING :
               $class = "warning";
               break;
               
            default :
            case MSG_INFO :
               $class = "information";
               break;
         }

         $this->setVar( "session_message_class", $class, "PRIVATE" );
         $this->setVar( "session_message", $sessionMessage, "PRIVATE" );
         $this->session->clearSessionMessage( );
      }
      else {
         $this->disabled = true;
      }
   }
}


?>
