<?php
/*
Name: bloo_queue.classes.php
Purpose: Classes related to the processing of bloo queue tasks
Author: Bill Roberts
Created: 9/10/2007
$Revision: 1.2 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/**************************************************************/
/* BlooQueue Event Handler                                    */
/* Checks for Bloo Queue GET parameters and processes them    */
/* replaces the content section of the Bloo Admin page        */
/**************************************************************/
class BlooQueueProcessor extends PreProcessEventHandler {
   var $name = "BlooQueueProcessor";
   var $securityRole = ROLE_EVERYONE;
   var $command;
   var $metaData;

   function setup( ) {
      /* see if a queue_key was passed */
      $queueKey = $this->getVar( "queue_key" );
      if ( $queueKey > "" ) {
         /* look it up in the queue table */
         $rows = $this->getRowsFromQueryCommand( "=|BlooQueue.getActiveQueueRow|=" );
         if ( is_array( $rows ) ) {
            $this->queueKey = $queueKey;
            $this->command = $rows[ 0 ][ "queue_command" ];
            $this->metaData = $rows[ 0 ][ "queue_meta_data" ];
         }
      }

      parent::setup( );
   }

   /* note: features that use BlooQueue should attach a preProcess event to the BlooQueueProcessor */
}

/* this class will generate a blooQueue Key, verifying that the key generated has never been used before */
class BlooQueueKey extends PhooClass {
   var $name = "BlooQueueKey";
   var $securityRole = ROLE_EVERYONE;
   var $key;
   var $length = 64;
   
   /* note: this code based on code written by thebomb-hq [AT] gmx [DOT] de from the php help on the rand 
      function */
   function createKey( $aCommand, $aSecondsToExpiry, $aMetaData ) {
      $pattern = "1234567890abcdefghijklmnopqrstuvwxyz";
      $key = "";
      
      $keyNotCreated = true;

      while ( $keyNotCreated ) {
         for ( $i=0; $i < $this->length; $i++ ) {
           $key .= $pattern{ rand( 0, 35 ) };
         }

         /* check for key already in use */
         $this->setVar( "queue_key", $key, "PRIVATE" );
         $rows = $this->getRowsFromQueryCommand( "=|BlooQueue.getKeyInfo|=" );
         if ( !is_array( $rows ) || count( $rows ) == 0 ) {
            /* if not in use, reset the flag to exit this loop */
            $keyNotCreated = false;
         }
      }

      /* insert the key in the bloo_queue */
      $this->setVar( "queue_command", $aCommand, "PRIVATE" );
      $this->setVar( "queue_meta_data", $aMetaData, "PRIVATE" );
      $this->setVar( "queue_seconds_to_expiry", $aSecondsToExpiry, "PRIVATE" );
      $this->runModificationCommand( "=|BlooQueue.insertKey|=" );

      if ( $this->isError( ) ) {
         return;
      }

      return $key;
   }

   function keyValid( $aKey ) {
      /* verifies that the key passed is valid and unexpired */
      $this->setVar( "queue_key", $key, "PRIVATE" );
      $rows = $this->getRowsFromQueryCommand( "=|BlooQueue.getActiveQueueRow|=" );
      if ( !is_array( $rows ) || count( $rows ) == 0 ) {
         return false;
      }
      else {
         return true;
      }
   }

   function deactivateKey( $aKey ) {
      /* deactivates the key passed */
      $this->setVar( "queue_key", $key, "PRIVATE" );
      $this->runModificationCommand( "=|BlooQueue.deactivateKey|=" );
   }
}
?>
