<?php
/*
Name: bloogroll.classes.php
Purpose: SnapOns related to the display and administration of bloogrolls
Author: Bill Roberts
Created: 4/11/2005
$Revision: 1.2 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

class Bloogroll extends PhooQuery {
   var $package = "bloogroll";
   var $name = "Bloogroll";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Bloogroll.list.get|=";
   var $templateHeader = "=|Bloogroll.header|=";
   var $template = "_{BloogrollCategoryDisplay}_\n=|Bloogroll|=";
   var $templateFooter = "=|Bloogroll.footer|=";

   function setup( ) {
      /* if the bloogroll_category_list application variable is set
         limit this to just the bloogroll categories in that list */
      $bloogrollCategoryList = $this->getVar( "bloogroll_category_list", "APPLICATION" );
      if ( $bloogrollCategoryList > "" ) {
         /* explode out each individual list item, if there is more than one */
         $cat = explode( ",", $bloogrollCategoryList );
         if ( is_array( $cat ) ) {
            $addedWhereClause = "and l2.item_name in (";
            $first = true;
            foreach ( $cat as $c ) {
               if ( $first ) {
                  $first = false;
               }
               else {
                  $addedWhereClause .= ",";
               }
               $addedWhereClause .= "'" . trim( $c ) . "'";
            }
            $addedWhereClause .= ")";
            $this->setVar( "added_where_clause", $addedWhereClause, "POST" );
               
         }
      }
      parent::setup( );
   }
} 

class BloogrollCategoryDropDown extends PhooDropDown {
   var $package = "bloogroll";
   var $name = "BloogrollCategoryDropDown";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Bloogroll.category.list.get|=";
   var $templateHeader = "=|Bloogroll.category.dropdown.header|=";
   var $template = "=|Bloogroll.category.dropdown.item|=";
   var $templateFooter = "=|Bloogroll.category.dropdown.footer|=";
   var $dropDownKeyName = "bloogroll_category_id";
}

class BloogrollCategoryDisplay extends PhooDisplay {
   var $name = "BloogrollCategoryDisplay";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Bloogroll.category|=";
   var $noRepeat = true; /* don't want to repeat the category for every item. just when it changes */                                            
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-04" );
      $this->setPurpose( "Displays the bloogroll category" );
      $this->addExpects( "bloogroll_category", "CONTAINER" );
   }
}

class BloogrollDeleter extends PhooProcess {
   var $package = "bloogroll";
   var $name = "BloogrollDeleter";
   var $command = "=|Bloogroll.item.delete|=";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot delete a bloogroll item. you do not have security privileges";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-15" );
      $this->addExpects( "list_type", "CONTAINER" );
      $this->addExpects( "bloogroll_id", "CONTAINER" );
      $this->setPurpose( "Controls deletion of bloogroll items" );
   }
}

class BloogrollCategoryDeleter extends PhooProcess {
   var $package = "bloogroll";
   var $name = "BloogrollCategoryDeleter";
   var $command = "=|Bloogroll.category.delete|=";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete a bloogroll category. you do not have security privileges";
   var $constraint = array( "=|Bloogroll.category.inuse.constraint|=" );

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-02" );
      $this->addExpects( "list_type", "CONTAINER" );
      $this->addExpects( "bloogroll_category_id", "CONTAINER" );
      $this->setPurpose( "Controls deletion of bloogroll categories" );
   }
}

class ListBloogrollDisplay extends PhooQuery {
   var $package = "bloogroll";
   var $name = "ListBloogrollDisplay";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Bloogroll.list.get|=";
   var $templateHeader = "=|Bloogroll.list.header|=";
   var $template = "=|Bloogroll.list|=";
   var $templateFooter = "=|Bloogroll.list.footer|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-12-19" );
      $this->setPurpose( "Displays the bloogroll administration list" );
   }
}

class ListBloogrollForm extends PhooForm {
   var $package = "bloogroll";
   var $name = "ListBloogrollForm";
   var $securityRole = ROLE_POSTER;
   var $loadCommand = "=|Bloogroll.item.get|=";
   var $updateForm = "=|Bloogroll.form.update|=";
   var $updateCommand = "=|Bloogroll.item.update|=";
   var $insertForm = "=|Bloogroll.form.insert|=";
   var $insertCommand = "=|Bloogroll.item.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-12-20" );
      $this->setPurpose( "Displays and processes the bloogroll administration form" );
   }

   function setup( ) {
      parent::setup( );

      /* set the list type here */
      $this->setVar( "list_type", "bloogroll", "POST" ); /* set in post vars to make it accessible to 
                                                            the new_id command */

      /* set the success page, in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=settingsAdmin&subpage=bloogroll" );
   }
}

class ListBloogrollCategoryDisplay extends PhooQuery {
   var $package = "bloogroll";
   var $name = "ListBloogrollCategoryDisplay";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Bloogroll.category.list.get|=";
   var $templateHeader = "=|Bloogroll.category.list.header|=";
   var $template = "=|Bloogroll.category.list|=";
   var $templateFooter = "=|Bloogroll.category.list.footer|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-01-09" );
      $this->setPurpose( "Displays the bloogroll category list" );
   }
}

class ListBloogrollCategoryForm extends PhooForm {
   var $package = "bloogroll";
   var $name = "ListBloogrollCategoryForm";
   var $securityRole = ROLE_POSTER;
   var $loadCommand = "=|Bloogroll.category.get|=";
   var $updateForm = "=|Bloogroll.category.form.update|=";
   var $updateCommand = "=|Bloogroll.category.update|=";
   var $insertForm = "=|Bloogroll.category.form.insert|=";
   var $insertCommand = "=|Bloogroll.category.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   
   function setup( ) {
      parent::setup( );

      /* set the list type here */
      $this->setVar( "list_type", "bloogroll.category", "POST" ); /* set in post vars to make it accessible to 
                                                            the new_id command */

      /* set the success page, in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=settingsAdmin&subpage=bloogroll.category" );
   }

   /* on an insert, will need to populate the list_order */
   function doAction( ) {
      if ( $this->action == "insert" ) {
         $s =& $this->session->snapOnFactory( "PhooQuery", $this );
         $s->securityRole = $this->securityRole; /* set the security to same as containter */
         $s->command = "=|List.nextorder.get|=";
         $s->process( );
         if ( $s->isError( ) ) {
            return;
         }
         $row = $s->getResultRow( );
         
         if ( isset( $row[ "next_order" ] ) ) {
            if ( $row[ "next_order" ] == 0 ) {  /* this is the first list item */
               $row[ "next_order" ] = 10;
            }
         
            $this->setVar( "list_order", $row[ "next_order" ], "PRIVATE" );
         }
      }

      parent::doAction( );
   }
}  
?>
