<?php  
/*
   Name: comment.classes.php
   Author: Bill Roberts
   Created: 4/19/2005
   Purpose: Handles all comment processing
   $Revision: 1.2 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
  
/****************************************************************/
/* CommentPageSection - creates the snapon/overlay configuration*/
/* for the portion of the page that displays comments and       */
/* comment forms, based on the parameters passed                */
/****************************************************************/
class CommentPageSection extends PhooDisplay {
   var $package = "comment";
   var $name = "CommentPageSection";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-20" );
      $this->setPurpose( "renders the comment section of the main page of the blog" );
   }
   
   function setup( ) {
      parent::setup( );
      /* if recent commentary GET var set, show recent commentary */
      $recentCommentaryFlag = $this->getVar( "recent_commentary", "GET" );
      if ( isset( $recentCommentaryFlag ) ) {
         $this->template = "=|Comment.recent.list|=";
      }
      else {
         $action = $this->getVar( FORM_ACTION, "GET" );
         $commentId = $this->getVar( "comment_id" );

         if ( $action == "update" && isset( $commentId ) ) { /* display a single comment in a form for updating */
            $this->template = "=|Comment.single.update|=";
         }
         else {                    /* display a comments list and comment form */
            $this->template = "=|Comment.and.form|=";
         }
      }
   }

   function doAction( ) {
      /* check for a delete command. If one was passed,
         perform the delete action instead of the default
         action */
      $action = $this->getVar( FORM_ACTION, "GET" );
      $commentId = $this->getVar( "comment_id" );

      if ( $commentId > "" ) {
         switch ( $action ) {
            case "delete" :
               /* process a comment deleter */
               $s =& $this->session->snapOnFactory( "CommentDeleter", $this );
               /* set the success page to take you back to the calling page, with no parameters
               */
               $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
               $s->process( );
               if ( $s->isError( ) ) {
                  /* load the template up with the error syntax, etc */
                  $this->template .= $s->render( );
                  return;
               }

               break;

            case "markspam" :
            case "marktroll" :
               /* process a comment marker */
               $this->setVar( "comment_mark_action", $action, "PRIVATE" );
               $s =& $this->session->snapOnFactory( "CommentMarker", $this );
               /* set the success page to take you back to the calling page, with no parameters
               */
               $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
               $s->process( );
               if ( $s->isError( ) ) {
                  /* load the template up with the error syntax, etc */
                  $this->template .= $s->render( );
                  return;
               }

               break;
            
            default : /* otherwise, continue on */
               parent::doAction( );
         }
      }
      else {
         parent::doAction( );
      }         
   }
}

/****************************************************************/
/* CommentPopupPage - Renders the comment page in a popup window*/
/****************************************************************/
class CommentPopupPage extends BlooPage {
   var $package = "comment";
   var $name = "CommentPopupPage";
   var $securityRole = ROLE_EVERYONE;
   var $templateHeader = "=|Page.comment.header|=";
   var $template = "_{CommentPageSection}_";
   var $templateFooter = "=|Page.comment.footer|=";
}

class CommentsDisplay extends PhooQuery {
   var $package = "comment";
   var $name = "CommentsDisplay";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Comment.get.bypost|=";
   var $templateHeader = "=|Comment.list.header|=";
   var $template = "=|Comment|=";
   var $templateFooter = "=|Comment.list.footer|=";
   var $showTimeZone = false;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Displays comments for a post, in ascending order by date" );
      $this->addExpects( "comment_table", "APPLICATION" );
      $this->addExpects( "post_table", "APPLICATION" );
      $this->addExpects( "post_id", "CONTAINER" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-28" );
   }

   function setup( ) {
      parent::setup( );
      /* set date format */
      $dateFormat = $this->session->getSetting( "comment_date_format" );

      /* note: since PHP 4 doesn't recognize the time zone format indicator, and since we're
         using a Phoo based, rather than PHP native, time zone, modify the format here */
      if ( strpos( $dateFormat, "e" ) ) { /* check for time zone indicator */
         $dateFormat = str_replace( "e", "", $dateFormat );
         $this->showTimeZone = true;
      }
      

      $this->field = array( "comment_date" => array( "format" => $dateFormat
                                            ,"type" => "date" ) );
   }      

   function doAction( ) {
      /* if a post id is specified, grab the post title, so that it will display */
      $postId = $this->getVar( "post_id" );
      if ( isset( $postId ) ) {
         $rows = $this->getRowsFromQueryCommand( "=|Post.get|=" );
         if ( $this->isError( ) ) {
            return;
         }

         if ( is_array( $rows ) ) {
            $this->setVar( "post_title", $rows[ 0 ][ "post_title" ], "PRIVATE" );
         }
         else {
            $this->disabled = true; /* not a valid post, or user doesn't have access to the post */
            return;
         }
      }

      parent::doAction( ); 
   }

   function getResultRow( ) {
      /* call the ancestor */
      $row = parent::getResultRow( );

      if ( $this->isError( ) || !is_array( $row ) ) {
         return;
      }

      if ( isset( $row[ "comment_date" ] ) ) {
         $commentTime = $this->session->getLocalDate( BASE_TIME_FORMAT, $row[ "comment_date" ] );
         $row[ "comment_date" ] = $commentTime[ 0 ];
         
         if ( $this->showTimeZone ) {
            $row[ "comment_time_zone_abbrev" ] = $commentTime[ 1 ];
         }
         else {
            $row[ "comment_time_zone_abbrev" ] = "";
         }  
      }

      return $row;
   }
} 


class RecentCommentsDisplay extends CommentsDisplay {
   var $package = "comment";
   var $name = "RecentCommentsDisplay";
   var $command = "=|Comment.recent.get|=";
   var $templateHeader = "=|Comment.recent.header|=";
   var $template = "=|Comment.recent|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Displays recent comments for the blog, in descending order by date." );
      $this->addExpects( "comment_table", "APPLICATION" );
      $this->addExpects( "post_table", "APPLICATION" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-28" );
   }
} 

class CommentAuthorURL extends PhooDisplay {
   var $package = "comment";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Comment.author.urlexists|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Displays the URL for a comment author, if one is set" );
      $this->addExpects( "comment_author_url", "CONTAINER" );
      $this->addExpects( "comment_author", "CONTAINER" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-28" );
      $this->addModificationHistory( "fixed the expects for author and url", "2006-03-05" );
   }

   function setup(  ) {
      parent::setup( );
      $commentAuthorURL = $this->getVar( "comment_author_url" );
      /* note: not checking from the 0 position as that would return a logical false even on a match */
      if ( trim( $commentAuthorURL ) == "" ) {
         $this->template = "=|Comment.author.urlnotexists|=";
      }
      else {
         if ( !strpos( strtolower( $commentAuthorURL ), "ttp://" ) ) {
            /* prefix it with http:// */
            $commentAuthorURL = "http://" . $commentAuthorURL;
            $this->setVar( "comment_author_url", $commentAuthorURL, "PRIVATE" ); /* PRIVATE has the highest priority */
         }
      }
   }
}

class CommentAuthorIP extends PhooDisplay {
   var $package = "comment";
   var $name = "CommentAuthorIP";
   var $securityRole = ROLE_POSTER;
   /* move this to an overlay */
   var $template = " - [_@comment_author_ip@_]";
   var $invisibleOnSecurityErrors = true;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Displays the IP for a comment author, if one is set and if the current viewer has the appropriate security to see the IP." );
      $this->addExpects( "comment_author_ip", "CONTAINER" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-28" );
   }

   function setup( ) {
      parent::setup( );
      
      $commentAuthorIP = $this->getVar( "comment_author_ip" );
      
      if ( empty( $commentAuthorIP ) ) {
         $this->disabled = true;
      }
   }
}

class CommentLinkPopup extends PhooDisplay {
   var $securityRole = ROLE_EVERYONE;
   var $commentsClosedTemplate = "=|Comment.comments_closed|=";
   var $zeroCommentsTemplate = "=|Comment.countzero|=";
   var $oneCommentTemplate = "=|Comment.countone|=";
   var $multipleCommentsTemplate = "=|Comment.countmultiple|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-10-27" );
      $this->setPurpose( "Displays post comment popup link, with number of comments" );
      $this->addExpects( "post_id", "CONTAINER" );
   }
   
   function doAction( ) {
   
      $row = $this->getRowsFromQueryCommand( "=|Comment.countbypost|=" );

      $commentCount = $row[ 0 ][ "comment_count" ];
   
      /* note: if the comment count is zero AND comments are closed for this post, display the comments_closed
          template */
      if ( $commentCount == 0 ) {

         /* check the comment closed status */
         /* first check the overall */
         $commentsAllowed = $this->session->getSetting( "comments_allowed" );
         if ( $commentsAllowed == "Y" ) { /* check the individual post */
         
            $row = $this->getRowsFromQueryCommand( "=|Comment.comments_allowed_on_post.get|=" );

            $commentsAllowed = $row[ 0 ][ "comments_allowed_flag" ];
         }

         if ( !($commentsAllowed == "Y" ) ) {
            /* comments are closed and there are no existing comments */
            $this->template = $this->commentsClosedTemplate;
         }         
      }

      if ( $this->template == "" ) { /* not set yet */
         $this->setVar( "comment_count", $commentCount, "PRIVATE" );

         switch ( $commentCount ) {
            case 0 :
               $this->template = $this->zeroCommentsTemplate;
               break;

            case 1 :
               $this->template = $this->oneCommentTemplate;
               break;

            default :
               $this->template = $this->multipleCommentsTemplate;
               break;
         }
      }      
   }
}

class EditCommentLink extends PhooDisplay {
   var $package = "comment";
   var $name = "EditCommentLink";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Comment.link.edit|=";
   var $invisibleOnSecurityErrors = true;

   function setup( ) {
      /* set internal variables */
      $this->setVar( "comment_id", $this->container->getVar( "comment_id" ), "PRIVATE" );
      $this->setVar( "post_id", $this->container->getVar( "post_id" ), "PRIVATE" );
      $this->setVar( "comment_author_ip", $this->container->getVar( "comment_author_ip" ), "PRIVATE" );
      $this->setVar( "comment_guest_edit_key", $this->container->getVar( "comment_guest_edit_key" ), "PRIVATE" );
      $this->setVar( "comment_date", $this->container->getVar( "comment_date" ), "PRIVATE" );

      /* see if we have security */
      $s =& $this->session->snapOnFactory( "CommentFormSecurityChecker", $this );
      if ( !$s->commentEditable( ) ) {
         $this->disabled = true;
      }
   }
} 

class DeleteCommentLink extends PhooDisplay {
   var $package = "comment";
   var $name = "DeleteCommentLink";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Comment.link.delete|=";
   var $invisibleOnSecurityErrors = true;
   
   function setup( ) {
      /* set internal variables */
      $this->setVar( "comment_id", $this->container->getVar( "comment_id" ), "PRIVATE" );
      $this->setVar( "post_id", $this->container->getVar( "post_id" ), "PRIVATE" );
      $this->setVar( "comment_author_ip", $this->container->getVar( "comment_author_ip" ), "PRIVATE" );
      $this->setVar( "comment_guest_edit_key", $this->container->getVar( "comment_guest_edit_key" ), "PRIVATE" );
      $this->setVar( "comment_date", $this->container->getVar( "comment_date" ), "PRIVATE" );

      /* see if we have security */
      $s =& $this->session->snapOnFactory( "CommentFormSecurityChecker", $this );
      if ( !$s->commentEditable( ) ) {
         $this->disabled = true;
      }
   }
}

class MarkSpamCommentLink extends PhooDisplay {
   var $package = "comment";
   var $name = "MarkSpamCommentLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|Comment.link.markspam|=";
   var $invisibleOnSecurityErrors = true;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-11-03" );
      $this->setPurpose( "Renders the comment mark spam link" );
      $this->addExpects( "comment_id", "CONTAINER" );
      $this->addExpects( "post_id", "CONTAINER" );
   }
}

class MarkTrollCommentLink extends PhooDisplay {
   var $package = "comment";
   var $name = "MarkTrollCommentLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|Comment.link.marktroll|=";
   var $invisibleOnSecurityErrors = true;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-11-03" );
      $this->setPurpose( "Renders the comment mark troll link" );
      $this->addExpects( "comment_id", "CONTAINER" );
      $this->addExpects( "post_id", "CONTAINER" );
   }
}

class CommentDeleter extends PhooProcess {
   var $package = "comment";
   var $name = "CommentDeleter";
   var $securityRole = ROLE_EVERYONE;
   var $securityErrorMessage = "Cannot delete a comment. you do not have security privileges";
   var $command = "=|Comment.delete|=";
   
   function setup( ) {
      /* set internal variables */
      $this->setVar( "comment_id", $this->container->getVar( "comment_id" ), "PRIVATE" );
      $this->setVar( "post_id", $this->container->getVar( "post_id" ), "PRIVATE" );
      $this->setVar( "comment_author_ip", $this->container->getVar( "comment_author_ip" ), "PRIVATE" );
      $this->setVar( "comment_guest_edit_key", $this->container->getVar( "comment_guest_edit_key" ), "PRIVATE" );
      $this->setVar( "comment_date", $this->container->getVar( "comment_date" ), "PRIVATE" );

      /* see if we have security */
      $s =& $this->session->snapOnFactory( "CommentFormSecurityChecker", $this );
      if ( !$s->commentEditable( ) ) {
         $this->disabled = true;
      }
   }
}

class CommentMarker extends PhooProcess {
   var $package = "comment";
   var $name = "CommentMarker";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot mark a comment. you do not have security privileges";
   var $command = "=|Comment.mark|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-19" );
      $this->setPurpose( "Processes the marking of a comment" );
      $this->addExpects( "comment_mark_action", "CONTAINER" );
   }

   function doAction( ) {
      $commentMarkAction = $this->container->getVar( "comment_mark_action", "PRIVATE" );

      switch ( $commentMarkAction ) {
         case "markspam" :
            $this->setVar( "comment_mark", "spam", "PRIVATE" );
            break;
            
         case "marktroll" :
            $this->setVar( "comment_mark", "troll", "PRIVATE" );
            break;
      }

      parent::doAction( );
   }
}

class CommentApprover extends PhooProcess {
   var $package = "comment";
   var $name = "CommentApprover";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot approve a comment. you do not have security privileges";
   var $command = "=|Comment.approve_list|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-03" );
      $this->setPurpose( "Processes the approval of a list of comments" );
   }
}

/* This class will determine if a comment can be edited */
class CommentFormSecurityChecker extends PhooClass {
   var $name = "CommentFormSecurityChecker";
   var $securityRole = ROLE_EVERYONE;

   /* returns a true if the comment is editable */
   function commentEditable( ) {
      $editable = false;
      
      /* check to see if the current user has POSTER privileges */
      $userRole = $this->session->getUserRoles( );

      if ( in_array( ROLE_POSTER, $userRole ) ) {
         $editable = true;
      }
      else {
         /* check guest comment editing setting */
         $guestCommentEditAllowed = $this->session->getSetting( "guest_comment_edit_allowed" );
         if ( $guestCommentEditAllowed == "Y" ) {
            /* both ip and the comment edit key must match */
            $commentIp = $this->container->getVar( "comment_author_ip" ); 
            if ( $commentIp == "" ) { /* need to retrieve from the db */
               /* note: this query will generally only need to happen when editing a comment */
               $this->setVar( "comment_id", $this->container->getVar( "comment_id" ), "PRIVATE" );
               $row = $this->getRowsFromQueryCommand( "=|Comment.get|=" );
               if ( is_array( $row ) ) {
                  $commentIp = $row[ 0 ][ "comment_author_ip" ];
               }
            }
            $currentIp = $this->getVar( "REMOTE_ADDR", "SERVER" );
            if ( $commentIp == $currentIp ) {
               /* check the guest edit key */
               $commentGuestEditKey = $this->container->getVar( "comment_guest_edit_key" ); 
               if ( $commentGuestEditKey == "" ) {
                  /* get from row array pulled in previous section */
                  $commentGuestEditKey = $row[ 0 ][ "comment_guest_edit_key" ];
               }
               $currentGuestEditKey = $this->getVar( "comment_guest_edit_key", "COOKIE" );
               if ( $commentGuestEditKey == $currentGuestEditKey ) {
                  /* check the elapsed time */
                  $allowedSeconds = $this->session->getSetting( "guest_comment_edit_time_limit" ) * 60;
                  $commentDate = $this->container->getVar( "comment_date" );
                  if ( $commentDate == "" ) {
                     /* get from row array pulled in previous section */
                     $commentDate = $row[ 0 ][ "comment_date" ];
                  }
                  $commentTime = strtotime( $commentDate );
                  $localTimeAr = $this->session->getLocalDate( BASE_TIME_FORMAT ); 
                  $localTime = strtotime( $localTimeAr[ 0 ] );
                  if ( ( $localTime - $commentTime ) <= $allowedSeconds ) {
                     $editable = true;
                  }
               }
            }
         }
      }
      
      return $editable;
   }
}

class CommentForm extends PhooForm {
   var $package = "comment";
   var $name = "CommentForm";
   var $securityRole = ROLE_EVERYONE;
   var $securityErrorMessage = "Cannot display comment form. You do not have comment editing privileges";
   var $command = ""; /* will be set based on the action */
   var $trigger = array( "btnPostComment" => array( "process" => "insert" )
                        ,"btnEdit" => array( "process" => "update" ) );
                        
   var $field = array( "comment_author" => array( "required" => "Y"
                                                 ,"formalName" => "Name" )
                      ,"comment_content" => array( "required" => "Y"
                                               ,"formalName" => "Comment" )
                     );
   var $loadCommand = "=|Comment.get|=";
   var $commentsClosedTemplate = "=|Comment.form.comments_closed|=";
   var $updateCommand = "=|Comment.update|=";
   var $updateTemplate = "=|Comment.form.update|=";
   var $insertCommand = "=|Comment.insert|=";
   var $insertTemplate = "=|Comment.form.insert|=";
   
   /* set templates and expects based on the type of action to be taken (insert vs update) */  
   function setup( ) {
      /* Grab container variables */
      $this->setVar( "post_id", $this->container->getVar( "post_id" ), "PRIVATE" );
      $this->setVar( "comment_id", $this->container->getVar( "comment_id" ), "PRIVATE" );

      /* check to see if comments are closed for this post, or closed overall */
      $commentsAllowed = $this->session->getSetting( "comments_allowed" );
      if ( $commentsAllowed == "Y" ) { /* check the individual post */
         $row = $this->getRowsFromQueryCommand( "=|Comment.comments_allowed_on_post.get|=" );
         $commentsAllowed = $row[ 0 ][ "comments_allowed_flag" ];
      }

      if ( $commentsAllowed == "" ) { /* null - could not read the post - either it's not a valid
                                         post_id or it's a private post and user is not logged in */
         $this->disabled = true;
         return;
      }                         
      elseif ( !( $commentsAllowed == "Y" ) ) {
         /* close comments */
         $this->template = $this->commentsClosedTemplate; 
         return;
      }

      parent::setup( );

      switch ( $this->action ) {
         case "update" :
            /* check update security */
            $s =& $this->session->snapOnFactory( "CommentFormSecurityChecker", $this );
            if ( $s->commentEditable( ) ) {
               $this->command = "=|Comment.update|=";
               $this->template = "=|Comment.form.update|=";
            }
            else {
               $this->disabled = true;
            }
            break;
         default:
            $this->setVar( "action", "insert", "PRIVATE" );
         case "insert" :
            $this->command = "=|Comment.insert|=";
            $this->template = "=|Comment.form.insert|=";
            /* grab the ip of the commenter */
            $this->setVar( "comment_author_ip",
                  $this->getVar( "REMOTE_ADDR", "SERVER" ), "PRIVATE" );
 
            /* generate a comment guest edit key */
            $this->setVar( "comment_guest_edit_key", rand( 0, 32768 ), "POST" );

            /* assume the comment is approved (default) */
            $this->setVar( "approved_flag", "Y", "POST" );

            break;
      }
   }

   function doAction( ) {
      
      parent::doAction( );

      /* if there is an error message, set it as a session message here */
      if ( $this->isError( ) ) {
         $this->session->setSessionMessage( $this->getError( ), MSG_FATAL );
      }   
   }
   
   function redirect( ) {

      if ( $this->successPage == "" ) {
         return;
      }

      /* redirect back to the list of comments for a post */
      $ar = explode( "?", $this->successPage );
      $pageNoArgs = $ar[0];
      $this->setSuccessPage( $pageNoArgs. "?post_id=" . $this->getVar( "post_id" ) . "#comment_form" );
      
      $approvedFlag = $this->getVar( "approved_flag");

      if ( $approvedFlag == "Y" ) {
         /* set a success message */
         $this->session->setSessionMessage( "Thank you for your comment", MSG_INFO );
      }

      parent::redirect( );
   }
} 

/*************************************************************/
/* CommentFilter - abstract ancestor event-handler class     */
/* abstract class - sets the approved_flag and potentially   */
/* the filter_agent and filter_reason (in the descendents,   */
/* of course                                                 */
/*************************************************************/
class CommentFilter extends PreSaveEventHandler {
   var $package = "comment";
   var $name = "CommentFilter";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "abstract class. sets the approved_flag, and possibly filter_agent and " .
                         "filter_reason (in the descendent)" );
   }

   function unapproveComment( $aFilterAgent, $aFilterReason ) {
      /* set the post variables with the filterAgent, filterReason, and unapprovedFlag */
      $this->setVar( "filter_agent", $aFilterAgent, "POST" );
      $this->setVar( "filter_reason", $aFilterReason, "POST" );
      $this->setVar( "approved_flag", "N", "POST" );
      $this->session->setSessionMessage( "The comment you just left looks like it might be spam. It has been placed in moderation. It will be displayed here once approved by the moderator.", MSG_WARNING );
   }

   /* abstract function - should be overridden. Should call unapproveComment if
      a comment is unapproved */
   function checkComment( ) {
   }
 
   function doAction( ) {
      
      /* if there is already a comment filter warning message setup, just return (no need to keep checking) */
      $commentFilterWarningMessage = $this->getVar( "comment_filter_warning_message" );
      if ( $commentFilterWarningMessage > "" ) {
         return;
      }
      
      /* note: only do this on inserts */
      $action = $this->container->action;

      switch ( $action ) {
         case "update" :
            return;
            break;
         
         default:
         case "insert" :
            $this->checkComment( );
      }      
   }
}

/*****************************************************/
/* classes for moderating comments                   */
/*****************************************************/

/* abstract class for both unapproved and approved comments display */
class ModeratedCommentsDisplay extends CommentsDisplay {
   var $name = "ModeratedCommentsDisplay";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "You do not have comment moderation privileges";
   var $resultPageSize = 40; /* default result page size */

   function doAction( ) {
      $actionType = $this->getVar( "btnModerateType" );

      if ( $actionType > "" ) {

         /* gather all ids */
         $listAr = $this->getVar( "moderationCBox" );

         if ( is_array( $listAr ) ) {
            $list = implode( ",", $listAr );
         }

         if ( $list > "" ) { /* if blank, no ids were selected */
            switch ( $actionType ) {
               case "SPAM" :
               case "TROLL" :
                  $this->setVar( "comment_mark_action", $actionType, "PRIVATE" );
                  $s =& $this->session->snapOnFactory( "CommentMarker", $this );
                  $s->command = "=|Comment.mark_list|=";
                  $s->setVar( "comment_id_list", $list, "PRIVATE" );
                  $s->process( );
                  break;
         
               case "DELETE" :
                  $s =& $this->session->snapOnFactory( "CommentDeleter", $this );
                  $s->command = "=|Comment.delete_list|=";
                  $s->setVar( "comment_id_list", $list, "PRIVATE" );
                  $s->process( );
                  break;
               
               case "APPROVE" :
                  $s =& $this->session->snapOnFactory( "CommentApprover", $this );
                  $s->command = "=|Comment.approve_list|=";
                  $s->setVar( "comment_id_list", $list, "PRIVATE" );
                  $s->process( );
                  break;
            }      
         }
      }

      parent::doAction( );
   }

   function getResultRow( ) {
      /* call the ancestor */
      $row = parent::getResultRow( );

      if ( $this->isError( ) || !is_array( $row ) ) {
         return;
      }

      if ( isset( $row[ "comment_content" ] ) ) {
         if ( strlen( $row[ "comment_content" ] ) > 250 ) {
            $pos = strpos( $row[ "comment_content" ], " ", 250 );
            if ( $pos > 0 ) {
               $row[ "comment_content" ] = substr( $row[ "comment_content" ], 0, $pos ) . " . . .";
            }
         }
      }

      return $row;
   }
}

class ApprovedCommentsDisplay extends ModeratedCommentsDisplay {
   var $name = "ApprovedCommentsDisplay";
   var $command = "=|Comment.approved.get|=";
   var $templateHeader = "=|Comment.approved.header|=";
   var $template = "=|Comment.approved|=";
   var $templateFooter = "=|Comment.approved.footer|=";
}

class UnApprovedCommentsDisplay extends ModeratedCommentsDisplay {
   var $name = "UnApprovedCommentsDisplay";
   var $command = "=|Comment.unapproved.get|=";
   var $templateHeader = "=|Comment.unapproved.header|=";
   var $template = "=|Comment.unapproved|=";
   var $templateFooter = "=|Comment.unapproved.footer|=";
}

class ApprovedCommentsResultPageNavigation extends PhooResultPageNavigation {
   var $name = "ApprovedCommentsResultPageNavigation";
   var $template = "=|Comment.moderation.navigation|=";
   var $nextPageTemplate = "=|Comment.moderation.link.next_page|=";
   var $previousPageTemplate = "=|Comment.moderation.link.previous_page|=";
   var $divider = " | ";
   var $countCommand = "=|Comment.approved.count|=";
   var $securityRole = ROLE_ADMINISTRATOR;

}

class UnapprovedCommentsResultPageNavigation extends PhooResultPageNavigation {
   var $name = "UnapprovedCommentsResultPageNavigation";
   var $template = "=|Comment.moderation.navigation|="; 
   var $nextPageTemplate = "=|Comment.moderation.link.next_page|=";
   var $previousPageTemplate = "=|Comment.moderation.link.previous_page|=";
   var $divider = " | ";
   var $countCommand = "=|Comment.unapproved.count|=";
   var $securityRole = ROLE_ADMINISTRATOR;

}

?>
