<?php
/* 
   Name: extension.classes.php
   Author: Bill Roberts
   Created: 2/25/2006
   Purpose: Handles the loading and managing of extensions
   $Revision: 1.2 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

define( "DEFAULT_EXTENSION_CATEGORY", "General" );

class SnapOnListMenu extends BlooTabbedMenu {
   var $package = "extension";
   var $name = "SnapOnListMenu";
   var $securityRole = ROLE_ADMINISTRATOR;

   function addSnapOnCategory( $aCategoryName ) {
      $tm =& $this->session->snapOnFactory( "BlooTabbedMenuItem", $this );
      $tm->menuItemName = $aCategoryName;
      $tm->menuItemLink = "_@site_url@_/_@operations_page@_?page=extensionAdmin&" . $tm->selectedVar . "=" .
                          urlencode( $aCategoryName );
      $tm->securityRole = $this->securityRole;
      $tm->selectedValue = $aCategoryName;
      $this->addMenuItem( $tm );
   }
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-06-16" );
      $this->setPurpose( "tabbed menu for SnapOns" );
   }

   function setup( ) {
      /* create the tabs */
      $this->extensionDescriptor = $this->session->getSnapOnDescriptorsForType( "extension" );
      $category = array( );
      $defaultExists = false;
      
      foreach ( $this->extensionDescriptor as $descriptor ) {
         if ( $descriptor[ "category" ] == "" ) {
            /* blank categories default to the default */
            $descriptor[ "category" ] = DEFAULT_EXTENSION_CATEGORY;
         }
         
         if ( !in_array( $descriptor[ "category" ], $category ) &&
              $descriptor[ "category" ] != DEFAULT_EXTENSION_CATEGORY ) {
            array_push( $category, $descriptor[ "category" ] );
         }
         else { 
            if ( $descriptor[ "category" ] == DEFAULT_EXTENSION_CATEGORY ) {
               $defaultExists = true;
            }
         }
      }
   
      if ( count( $category ) > 0 ) {
         sort( $category );
      }

      /* add the default category in if it exists in the list */
      if ( $defaultExists ) {
         array_unshift( $category, DEFAULT_EXTENSION_CATEGORY );
      }
      
      foreach( $category as $eCat ) {
         $this->addSnapOnCategory( $eCat );
      }

      parent::setup( );
   }
}
   
class ListSnapOnList extends PhooQuery {
   var $package = "extension";
   var $name = "ListSnapOnList";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|SnapOn.list.header|=";
   var $template = "=|SnapOn.list|=";
   var $templateFooter = "=|SnapOn.list.footer|=";
   var $extensionDescriptor;
   var $subPage;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-25" );
      $this->setPurpose( "lists all active and inactive extensions" );
   }

   /* for sorting the extensionDescriptor array */
   function compareFunction( $ar1, $ar2 ) {
      return strcmp( $ar1[ "display_name" ], $ar2[ "display_name" ] );
   }

   function setup( ) {
      parent::setup( );

      $category = array( );
      
      /* if no GET variable setup for the selected var, set it to the default Category */
      $this->subPage = $this->getVar( "subpage" );

      if ( $this->subPage == "" ) {
         $this->subPage = DEFAULT_EXTENSION_CATEGORY;
         $this->setVar( "subpage", $this->subPage, "GET" );
      }

      /* grab all the extension descriptors */
      $this->extensionDescriptor = $this->session->getSnapOnDescriptorsForType( "extension" );
      
      /* inventory the categories in the extension descriptors */
      foreach ( $this->extensionDescriptor as $snapOn => $descriptor ) {
         $cat = $descriptor[ "category" ];
         if ( $cat == "" ) {
            $cat = DEFAULT_EXTENSION_CATEGORY;
            $this->extensionDescriptor[ $snapOn ][ "category" ] = $cat;
         }

         /* stick it in the array, just to keep track */
         if ( !in_array( $cat, $category ) ) {
            array_push( $category, $cat );
         }
      }

      /* If the subpage isn't a valid one, set it to the first category */
      if ( !in_array( $this->subPage, $category ) ) {
         if ( count( $category ) > 0 ) {
            sort( $category );
            $this->subPage = $category[ 0 ];
            $this->setVar( "subpage", $this->subPage, "GET" );
         }
      }
      
      /* Remove any descriptors that aren't in the current subPage */
      foreach ( $this->extensionDescriptor as $snapOn => $descriptor ) {
         if ( $descriptor[ "category" ] != $this->subPage ) {
            /* remove the extension descriptor */
            unset( $this->extensionDescriptor[ $snapOn ] );
         }
      }
      /* sort the extensionDescriptor array */
      uasort( $this->extensionDescriptor, array( "ListSnapOnList", "compareFunction" ) );
   }

   /* create an array of the current extension descriptors */   
   function doAction( ) {
      /* set the key of the array as an element called "name" */
      if ( is_array( $this->extensionDescriptor ) ) {
         foreach ( $this->extensionDescriptor as $name => $descriptor ) {
            $this->extensionDescriptor[ $name ][ "extension_name" ] = $name;
            if ( !isset( $this->extensionDescriptor[ $name ][ "display_name" ] ) ) {
               $this->extensionDescriptor[ $name ][ "display_name" ] = $name;
            }
            /* set the activation class for display purposes */
            if ( $this->extensionDescriptor[ $name ][ "loadable" ] ) {
               $this->extensionDescriptor[ $name ][ "activation_class" ] = "activated";
            }
            else {
               $this->extensionDescriptor[ $name ][ "activation_class" ] = "deactivated";
            }
         }
         reset( $this->extensionDescriptor );
      }
   }
   
   /* this function overrides the ancestor and just returns 
      the next row out of the extensionDescriptor array */
   function getResultRow( ) {
      if ( count( $this->extensionDescriptor ) == 0 ) {
         return null;
      }

      $row = current( $this->extensionDescriptor );
      
      if ( !$row ) {
         return null;
      }
      else {
         next( $this->extensionDescriptor );
         return $row;
      }
   }
}

/*********************************************************/
/* SnapOnActivationLink                               */
/* Controls the appearance and contents of the extension */
/* activation link                                       */
/*********************************************************/
class SnapOnActivationLink extends PhooDisplay {
   var $package = "extension";
   var $name = "SnapOnActivationLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = ""; /* will be chosen based on the status of the current extension */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-26" );
      $this->setPurpose( "sets the syntax of the extension activation link" );
      $this->addExpects( "loadable", "CONTAINER" );
      $this->addExpects( "extension_name", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      $activationFlag = $this->getVar( "loadable" );
      switch( $activationFlag ) {
         case true :
            $this->template = "=|SnapOn.link.deactivate|=";
            break;
         case false :
            $this->template = "=|SnapOn.link.activate|=";
            break;
      }
   }
}

/*********************************************************/
/* SnapOnHelpLink                                     */
/* If a help overlay exists, this link will be shown     */
/*********************************************************/
class SnapOnHelpLink extends PhooDisplay {
   var $package = "extension";
   var $name = "SnapOnHelpLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|SnapOn.link.help|="; 

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-26" );
      $this->setPurpose( "displays the help link if help is available" );
      $this->addExpects( "help_overlay", "CONTAINER" );
      $this->addExpects( "extension_name", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      $helpOverlay = $this->getVar( "help_overlay" );
      if ( !isset( $helpOverlay ) || $helpOverlay == "" ) {
         $this->disabled = true;
      }
   }
}

/*********************************************************/
/* SnapOnAdministrationLink                           */
/* If administration snapon is specified, this link will */
/* be shown (provided the snapon is activated)           */
/*********************************************************/
class SnapOnAdministrationLink extends PhooDisplay {
   var $package = "extension";
   var $name = "SnapOnAdministrationLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|SnapOn.link.administer|="; 

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-26" );
      $this->setPurpose( "displays the administration link if an administration snapon is specified (and the extension is activated)" );
      $this->addExpects( "administration_class", "CONTAINER" );
      $this->addExpects( "loadable", "CONTAINER" );
      $this->addExpects( "extension_name", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      $administrationSnapOn = $this->getVar( "administration_class" );
      $activated = $this->getVar( "loadable" );
      if ( $administrationSnapOn > "" && $activated ) {
      }
      else {
         $this->disabled = true;
      }
   }
}


/*********************************************************/
/* SnapOnActivator                                    */
/* Activates or deActivates an extension based on the    */
/* action passed                                         */
/*********************************************************/
class SnapOnActivator extends PhooProcess {
   var $package = "extension";
   var $name = "SnapOnActivator";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = ""; /* will be chosen based on the action passed */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-02-26" );
      $this->setPurpose( "activates or deactives an extension based on the action passed" );
      $this->addExpects( FORM_ACTION, "GET" );
      $this->addExpects( "extension_name", "GET" );
   }

   function setup( ) {
      parent::setup( );
      $action = $this->getVar( FORM_ACTION );
      /* load the SnapOn's info */
      $descriptor = $this->session->getSnapOnDescriptor( $this->getVar( "extension_name" ) );
      
      switch( $action ) {
         case "deactivate" :
            $this->command = "=|SnapOn.deactivate|=";
            break;
            
         case "activate" :
            /* see if there's an activation snapon */
            /* if it exists, run it */
            if ( $descriptor[ "activation_class" ] > "" ) {
               /* set the extension to loadable and re-load it */
               $snapOnName = $this->getVar( "extension_name" );
               $this->session->extensionInventory[ $snapOnName ][ "loadable" ] = true;
               
               $filename = $this->session->snapOn[ $snapOnName ][ "filename" ];
               $directory = $this->session->snapOn[ $snapOnName ][ "directory" ];
               $source = $this->session->snapOn[ $snapOnName ][ "source" ];

               /* clear out the original descriptor prior to reload */
               unset( $this->session->snapOn[ $snapOnName ] );
               
               $this->session->loadDescriptor( $directory, $filename, $source );
                           
               $s =& $this->session->snapOnFactory( $descriptor[ "activation_class" ], $this );
               $s->process( );
               if ( $s->isError( ) ) {
                  return;
               }
            }
            /* if snapOn does not exist in the extension table, add it before activating */
            $rows = $this->getRowsFromQueryCommand( "=|SnapOn.exists|=" );
            if ( $this->isError( ) ) {
               return;
            }
            $exists = false;
            if ( is_array( $rows ) ) {
               if ( $rows[ 0 ][ "snap_on_exists" ] ) {
                  $exists = true;
               }
            }

            if ( !$exists ) {
               $snapOnName = $this->getVar( "extension_name" );
               
               $this->setVar( "extension_directory", $this->session->snapOn[ $snapOnName ][ "directory" ], "PRIVATE" );
               $this->setVar( "extension_source", $this->session->snapOn[ $snapOnName ][ "source" ], "PRIVATE" );
               $this->runModificationCommand( "=|SnapOn.insert|=" );
               if ( $this->isError( ) ) {
                  return;
               }
            }
            
            $this->command = "=|SnapOn.activate|=";
            break;

         default :
            $this->setError( "Illegal extension activation action: $action", MSG_FATAL );
            break;
      }

      /* set the success page, assuming this will be successful */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                            "?page=extensionAdmin&subpage=" . urlencode( $descriptor[ "category" ] ) );

   }
} 
?>
