<?php
/* 
   Name: look_and_feel.classes.php
   Author: Bill Roberts
   Created: 3/9/2006
   Purpose: SnapOns related to listing and editing overlays and themes
   $Revision: 1.2 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

define( "CUSTOM_OVERLAY", "Custom" );
define( "OVERLAY_FOR_ALL_THEMES", "Page.sidebar" ); /* defaults this overlay, when edited, to all themes. */

class StyleSelector extends PhooDisplay {
   var $package = "look_and_feel";
   var $name = "StyleSelector";
   var $securityRole = ROLE_EVERYONE;
   var $templateHeader = "=|CSS.link.header|=";
   var $template = "=|CSS.link|=";
   var $templateFooter = "=|CSS.link.footer|=";

   function setDescriptor(  ) {
      /* populate descriptor */
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Sets the style sheet for the site" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }

   function renderSnapOn( ) {
      $syntax = "";
      $ar = array( );
     
      $styleSheet = explode( ";", $this->session->getSetting( "style_sheet" ) );

      foreach ( $styleSheet as $sheet ) {
         /* put the stylesheet in a private variable */
         if ( $sheet > "" ) {
            $this->setVar( "style_sheet", $sheet, "PRIVATE" );
            $syntax .= parent::renderSnapOn( );
         }
      }

      $t = $this->template;
      $this->template = $this->templateHeader;
      $syntax = parent::renderSnapOn( ) . $syntax;

      $t = $this->template;
      $this->template = $this->templateFooter;
      $syntax .= parent::renderSnapOn( );

      return $syntax;
   }
}
   
class InlineCSS extends PhooDisplay {
   var $package = "look_and_feel";
   var $name = "InlineCSS";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|InlineCSS|=";
}


class ThemeDropDown extends PhooDropDown {
   var $package = "look_and_feel";
   var $name = "ThemeDropDown";
   var $securityRole = ROLE_EVERYONE;
   var $templateHeader = "=|Theme.dropdown.header|=";
   var $template = "=|Theme.dropdown.item|=";
   var $templateFooter = "=|Theme.dropdown.footer|=";
   var $dropDownKeyName = "theme";
   var $rowsReturned = false;
   var $themeDescriptor;

   /* for sorting the theme array */
   function compareFunction( $ar1, $ar2 ) {
      return strcmp( $ar1[ "display_name" ], $ar2[ "display_name" ] );
   }

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-19" );
      $this->setPurpose( "Displays a drop-down containing themes" );
      $this->addExpects( "theme", "CONTAINER" ); /* to override the theme stored in the application var - optional */
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }

   function setup( ) {
      $this->setVar( "drop_down_key_name", $this->dropDownKeyName, "PRIVATE" );

      /* place the current_theme in the drop down key name, so that it will default to
         the current theme */
      /* note: only do this if it's not already set */
      $dropDownKeyValue = $this->getVar( $this->dropDownKeyName );
      if ( !isset( $dropDownKeyValue  ) ) {
         $this->setVar( $this->dropDownKeyName, $this->getVar( "current_theme", "APPLICATION" ), "PRIVATE" );
      }

      /* go ahead and expand the header template here, to set that name, since
         in the render the private vars will be replaced with each row of data retrieved.
        
         Note: I realize this could be more efficient/less clumsy. Mea culpa */
      $this->templateHeader = $this->populateParameters( $this->expandOverlays( $this->templateHeader ) );
      
      parent::setup( );
   }

   function doAction( ) {
      $this->themeDescriptor = $this->session->getSnapOnDescriptorsForType( "theme" );
      
      /* set the theme name as one of the keys */
      foreach ( $this->themeDescriptor as $themeName => $descriptor ) {
         $this->themeDescriptor[ $themeName ][ "theme" ] = $themeName;
      }

      /* sort the themeDescriptor array */
      uasort( $this->themeDescriptor, array( "ThemeDropDown", "compareFunction" ) );

      $this->rowsReturned = false;
   }

   function getResultRow( ) {
      if ( !$this->rowsReturned ) { /* at the first row */
         reset( $this->themeDescriptor );
         $row = current( $this->themeDescriptor );
         $this->rowsReturned = true;
      }   
      else {
         $row = next( $this->themeDescriptor );
      }

      if ( is_array( $row ) ) { 
         return $row;
      }

      return null; /* done */
   }
}

class TemplateCategoryDropDown extends PhooDropDown {
   var $package = "look_and_feel";
   var $name = "TemplateCategoryDropDown";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Template.category.dropdown.header|=";
   var $template = "=|Template.category.dropdown.item|=";
   var $templateFooter = "=|Template.category.dropdown.footer|=";
   var $dropDownKeyName = "template_category";
   var $templateCategory = array( );

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-09" );
      $this->setPurpose( "Displays a drop-down containing overlay categories" );
   }
   
   /* for sorting the dropdown */
   function compareFunction( $ar1, $ar2 ) {
      return strcmp( $ar1[ "display_name" ], $ar2[ "display_name" ] );
   }

   /* overrides the ancestor - creates a list of categories from snapOn names */
   function doAction( ) {
      $this->templateCategory = array( ); /* reset the array */
 
      foreach ( $this->session->snapOn as $snapOnName => $descriptor ) {
         /* only load loadable snapOns */
         if ( $descriptor[ "loadable" ] ) {

            $found = false;
            /* see if there are any template overlays for this SnapOn */
            foreach ( $this->session->packageOverlay as $overlay => $ar ) {
               if ( $ar[ "snap_on" ] == $snapOnName && $ar[ "type" ] == "template" ) {
                  $found = true;
                  break;
               }
            }
 
            if ( $found ) {
               $displayName = $descriptor[ "display_name" ];
               if ( $displayName == "" ) {
                  $displayName = $snapOnName;
               }
            
               array_push( $this->templateCategory, array( "category" => $snapOnName,
                                                           "display_name" => $displayName ) );
            }
         }
      }       

      usort( $this->templateCategory, array( "TemplateCategoryDropDown", "compareFunction" ) );

      /* create an entry for favorite templates */
      array_unshift( $this->templateCategory, array( "category" => CUSTOM_OVERLAY
                                                    ,"display_name" => CUSTOM_OVERLAY ) );
                                                    
                                                    
      reset( $this->templateCategory );
   }
   
   /* overrides the ancestor - returns a row from the internal array */
   function getResultRow( ) {
      $ar = current( $this->templateCategory );

      if ( is_array( $ar ) ) {
         
         $templateCategory = $ar[ "category" ];
         $templateCategoryDisplay = $ar[ "display_name" ];
         
         $row = array( "template_category" => $templateCategory
                      ,"template_category_display" => $templateCategoryDisplay );
         next( $this->templateCategory );
      }
      else {
         $row = null;
      }
      return $row;
   }
}

class TemplateCategorySelector extends PhooForm {
   var $package = "look_and_feel";
   var $name = "TemplateCategorySelector";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|Template.category.selector|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-16" );
      $this->addExpects( "template_category", "POST" );
      $this->setPurpose( "Displays the template categories available" );
   }
}

class TemplateList extends PhooQuery {
   var $package = "look_and_feel";
   var $name = "TemplateList";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Template.category.list.header|=";
   var $template = "=|Template.category.list|=";
   var $templateFooter = "=|Template.category.list.footer|=";
   var $templateList = array( );
   var $templateCount = 0;
   var $field = array( "overlay_description" => array( "onblank" => "&nbsp;" )
                     );

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-10" );
      $this->addExpects( "template_category", "POST" );
      $this->setPurpose( "Displays a list of template overlays for a template category" );
   }

   /* for sorting overlays */
   function compareFunction( $ovl1, $ovl2 ) {
      /* same status? Sort by name */
      $cmp = strcmp( $ovl1[ "overlay_name" ], $ovl2[ "overlay_name" ] );
      if ( $cmp == 0 ) {
         /* sort by activated status at the top */
         if ( $ovl1[ "overlay_sort_category" ] > $ovl2[ "overlay_sort_category" ] ) {
            $cmp = 1;
         }
         elseif ( $ovl1[ "overlay_sort_category" ] < $ovl2[ "overlay_sort_category" ] ) {
            $cmp = -1;
         }
         else { /* sort by theme */
            $cmp = strcmp( $ovl1[ "overlay_theme" ], $ovl2[ "overlay_theme" ] );
         }   
      }

      return $cmp;
   }

   /* overrides the ancestor - creates a list of overlays for a given category */
   function doAction( ) {
      
      $templateCategory = $this->getVar( "template_category" );
      
      if ( $templateCategory == "" ) {
         /* default to the favorite category, which should be the first in the drop-down */
         $templateCategory = CUSTOM_OVERLAY;
         $this->setVar( "template_category", $templateCategory, "POST" );
         $this->setVar( "template_category_display_name", $templateCategory, "POST" );
      }
      else {
         /* set the display name */
         $templateCategoryDisplay = $this->session->snapOn[ $templateCategory ][ "display_name" ];
         $this->setVar( "template_category_display_name", $templateCategoryDisplay, "POST" );
      }
      
      $overlayList = array( );

      /* load all deactivated overlays for all themes and for the current theme (in that order)
         into an array for later use */
      $deactivated = array( );
      
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole;
      $s->command = "=|Template.deactivated.list.get|=";
      $s->process( );
      while ( $row = $s->getResultRow( ) ) {
         $row[ "overlay_source" ] = TABLE_BASED;
         $deactivated[ $row[ "overlay_name" ] ] = $row;
      }

      /* if the template category is for CUSTOM overlays, read in the list of
         favorites (since they should show up in the CUSTOM list */
      if ( $templateCategory == CUSTOM_OVERLAY ) {
         /* read the favorites list */
         $s =& $this->session->snapOnFactory( "PhooQuery", $this );
         $s->securityRole = $this->securityRole;
         $s->command = "=|Overlay.favorites.get|=";
         $s->process( );
         if ( !$s->isError( ) ) {
            while ( $row = $s->getResultRow( ) ) {
               array_push( $overlayList, $row[ "favorite" ] );
            }
         }

         /* include deactivated table-based overlays in the list */
         foreach ( $deactivated as $name => $row ) {
            if ( !in_array( $name, $overlayList ) ) {
               array_push( $overlayList, $name );
            }
         }
         
         /* include any active table-based overlay in the favorites */
         $s =& $this->session->snapOnFactory( "PhooQuery", $this );
         $s->securityRole = $this->securityRole;
         $s->command = "=|Template.activated.name.get|=";
         $s->process( );
         while ( $row = $s->getResultRow( ) ) {
            if ( !in_array( $row[ "overlay_name" ], $overlayList ) ) {
               array_push( $overlayList, $row[ "overlay_name" ] );
            }
         }
      }
      else {
         /* find overlay templates that match the current snapOn */
         foreach ( $this->session->packageOverlay as $overlayName => $descriptor ) {
            if ( $descriptor[ "snap_on" ] == $templateCategory && $descriptor[ "type" ] == "template" ) {
               array_push( $overlayList, $overlayName );
            }
         }
      }

      
      /* Get list of overlays for this SnapOn (or favorites) category */
      $overlay = array( );
      foreach ( $overlayList as $overlayName ) {
         $descriptor = $this->packageOverlay[ $overlayName ];
         $packageName = $this->session->getPackageName( $descriptor[ "package_key" ] );
         if ( !$this->session->package[ $packageName ][ "loaded" ] ) {
            /* load the overlay package */
            $this->session->loadOverlayPackage( $overlayName );
         }

         /* get overlays without appendages */
         $ovl = $this->session->getOverlay( $overlayName, false );
            
         /* prepare the syntax for display */
         $syntax = $ovl[ "syntax" ];
         $syntax = htmlspecialchars( $syntax );
         $syntax = str_replace( "\n", "<br />", $syntax );

         if ( $ovl[ "source" ] == TABLE_BASED ) {
            $activationClass = "activated";
         }
         else {
            $activationClass = "system";
         }
            
         /* if this is a "pendage" overlay, create the pendage information */
         if ( $ovl[ "pend_to" ] > "" ) {
            $pendMessage = $ovl[ "pend_type" ] . "s to " . $ovl[ "pend_to" ];
         }
         else {
            $pendMessage = "None";
         }
            
         array_push( $this->templateList, array( "overlay_name" => $overlayName
                                                ,"overlay_description" => $ovl[ "description" ]
                                                ,"overlay_syntax" => $syntax
                                                ,"overlay_source" => $ovl[ "source" ]
                                                ,"overlay_theme" => $ovl[ "theme" ]
                                                ,"activation_class" => $activationClass
                                                ,"overlay_id" =>  $ovl[ "id" ]
                                                ,"overlay_activated_flag" => "Y"
                                                ,"overlay_pend_message" => $pendMessage
                                               ) );

         /* see if a deactivated overlay exists - if one does, add it */
         if ( isset( $deactivated[ $overlayName ] ) ) {
            $activationClass = "deactivated";
            $ovl = $deactivated[ $overlayName ];

            $syntax = htmlspecialchars( $ovl[ "overlay_syntax" ] );
            $syntax = str_replace( "\n", "<br />", $syntax );
               
            /* if this is a "pendage" overlay, create the pendage information */
            if ( $ovl[ "overlay_pend_to" ] > "" ) {
               $pendMessage = $ovl[ "overlay_pend_type" ] . "s to " . $ovl[ "overlay_pend_to" ];
            }
            else {
               $pendMessage = "None";
            }
               
            array_push( $this->templateList, array( "overlay_name" => $overlayName
                                                   ,"overlay_description" => $ovl[ "overlay_description" ]
                                                   ,"overlay_syntax" => $syntax
                                                   ,"overlay_source" => $ovl[ "overlay_source" ]
                                                   ,"overlay_theme" => $ovl[ "overlay_theme" ]
                                                   ,"activation_class" => $activationClass
                                                   ,"overlay_id" =>  $ovl[ "overlay_id" ]
                                                   ,"overlay_activated_flag" => "N"
                                                   ,"overlay_pend_message" => $pendMessage
                                                  ) );
         }     
      }

      $this->templateCount = 0;
      usort( $this->templateList, array( "TemplateList", "compareFunction" ) );
   }
   

   /* overrides the ancestor - returns a row from the internal array */
   function getResultRow( ) {
      if ( $this->templateCount >= count( $this->templateList ) ) {
         return null;
      }
      else {
         $row = $this->templateList[ $this->templateCount ];
         $this->templateCount++;
      }
      
      $row = $this->cleanRow( $row );
      
      return $row;
   }
} 

/*********************************************************/
/* TemplateDeleteLink                                    */
/* Will only be visible if this is a table-based overlay */
/*********************************************************/
class TemplateDeleteLink extends PhooDisplay {
   var $package = "look_and_feel";
   var $name = "TemplateDeleteLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|Template.link.delete|="; 

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-11" );
      $this->setPurpose( "displays the delete link for an overlay template if it is table-based" );
      $this->addExpects( "overlay_source", "CONTAINER" );
      $this->addExpects( "overlay_id", "CONTAINER" );
   }

   /* determine if this link should be used */
   function setup( ) {
      parent::setup( );
      $source = $this->getVar( "overlay_source" );
      if ( $source != "table" ) {
         $this->disabled = true;
      }
   }
}

/*********************************************************/
/* TemplateEditLink                                      */
/* Will have a different set of parameters for editing   */
/* table-based versus xml based overlays                 */
/*********************************************************/
class TemplateEditLink extends PhooDisplay {
   var $package = "look_and_feel";
   var $name = "TemplateEditLink";
   var $securityRole = ROLE_ADMINISTRATOR;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-11" );
      $this->setPurpose( "displays the edit link for an overlay template. Used different links for table-based vs xml-based" );
      $this->addExpects( "overlay_source", "CONTAINER" );
      $this->addExpects( "overlay_id", "CONTAINER" );
      $this->addExpects( "overlay_name", "CONTAINER" );
      $this->addExpects( "overlay_additive_flag", "CONTAINER" );
      $this->addExpects( "template_category", "CONTAINER" );
   }

   /* determine if this link should be used */
   function setup( ) {
      parent::setup( );
      if ( $this->getVar( "overlay_additive_flag", "PRIVATE" ) == "Yes" ) {
         $this->template = "=|Template.nolink.edit.message|=";
      }
      else {
         $source = $this->getVar( "overlay_source" );
         if ( $source == "table" ) {
            $this->template = "=|Template.link.tablebased.edit|=";
         }
         else {
            $this->template = "=|Template.link.edit|=";
         }
      }
   }
}

/*********************************************************/
/* TemplateActivationLink                               */
/* Controls the appearance and contents of the extension */
/* activation link                                       */
/*********************************************************/
class TemplateActivationLink extends PhooDisplay {
   var $package = "look_and_feel";
   var $name = "TemplateActivationLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = ""; /* will be chosen based on the status of the current template */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-11" );
      $this->setPurpose( "sets the syntax of the template activation link" );
      $this->addExpects( "overlay_source", "CONTAINER" );
      $this->addExpects( "overlay_id", "CONTAINER" );
      $this->addExpects( "template_category", "CONTAINER" );
      $this->addExpects( "overlay_activated_flag", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      $source = $this->getVar( "overlay_source" );
      if ( $source != "table" ) { /* this only counts for table-based overlay templates */
         $this->disabled = true;
      }
      else {
         $activationFlag = $this->getVar( "overlay_activated_flag" );
         switch( $activationFlag ) {
            case "Y" :
               $this->template = "=|Template.link.deactivate|=";
               break;
            case "N" :
               $this->template = "=|Template.link.activate|=";
               break;
         }
      }
   }
}

class TemplateThemeDropDown extends ThemeDropDown {
   var $package = "look_and_feel";
   var $name = "TemplateThemeDropDown";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $dropDownKeyName = "overlay_theme";
   var $firstRow;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-03-13" );
      $this->setPurpose( "Displays a drop-down containing themes, with 'All' as one option" );
   }

   function setup( ) {
      $this->firstRow = true;      

      /* if this overlay is the specified "ALL" overlay (usually Page.sidebar), set the theme drop-down to All */
      $overlayName = $this->getVar( "overlay_name" );      
      if ( $overlayName == OVERLAY_FOR_ALL_THEMES ) {
         /* only default to All if the action is an insert action */
         if ( $this->container->getVar( "action" ) == "insert" ) {
            $this->setVar( $this->dropDownKeyName, "all", "PRIVATE" );
         }
      }

      parent::setup( );
   }

   function getResultRow( ) {
      if ( $this->firstRow ) {
         $this->firstRow = false; 
         /* set "all" as the first row */
         $row = array( );
         $row[ "theme" ] = "all";
         $row[ "display_name" ] = "All Themes";
      }
      else {
         $row = parent::getResultRow( );
      }

      return $row;
   }
}

class TemplateForm extends PhooForm {
   var $package = "look_and_feel";
   var $name = "TemplateForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $updateForm = "=|Template.form.update|=";
   var $insertForm = "=|Template.form.insert|=";
   var $customizeForm = "=|Template.form.customize|=";
   var $loadCommand = ""; /* will be set in the action method */
   var $updateCommand = "=|Template.update|=";
   var $insertCommand = "=|Template.insert|=";
   var $trigger = array( "btnInsert" => array( "process" => "insert" )
                        ,"btnUpdate" => array( "process" => "update" )
                        ,"btnUpdateStay" => array( "process" => "update" ));
   var $insertConstraint = "=|Template.duplicate.insert.constraint|=";
   var $updateConstraint = "=|Template.duplicate.update.constraint|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-13" );
      $this->addExpects( FORM_ACTION, "CONTAINER" );
      $this->setPurpose( "Displays and processes the template create/update form" );
   }

   function setup( ) {
      parent::setup( );

      $templateName = $this->getVar( "overlay_name" );

      if ( $this->getVar( "action", "GET" ) == "insert" ) { /* use the customize form */
         $this->template = $this->customizeForm;
         /* will load the template from the current overlay bin, if necessary */
         $btnInsert = $this->getVar( "btnInsert" );
         if ( $templateName > "" && !isset( $btnInsert ) ) {
            $ovl = $this->session->getOverlay( $templateName, false );
            if ( $ovl ) {
               /* load the syntax */
               /* NOTE: the clean is needed here to keep embedded placeholders and overlays from expanding */
               $syntax = $this->clean( $ovl[ "syntax" ] );
               $this->setVar( "overlay_syntax", $syntax, "PRIVATE" );

               /* set the pend type and pend to (if they exist ) */
               $this->setVar( "overlay_pend_to", $this->clean( $ovl[ "pend_to" ] ), "PRIVATE" );
               $this->setVar( "overlay_pend_type", $this->clean( $ovl[ "pend_type" ] ), "PRIVATE" );
               $this->setVar( "overlay_description", $this->clean( $ovl[ "description" ] ), "PRIVATE" );
            }
         }
      }

      /* set the load command if an overlay_id is set */
      $overlayId = $this->getVar( "overlay_id" ); 
      if ( isset( $overlayId ) ) {
         $this->loadCommand = "=|Template.get|="; /* set the load command */
      }
      
      if ( $templateName > "" ) {
         /* set the success page in case this works */
         $btnUpdateStay = $this->getVar( "btnUpdateStay", "POST" );
         if ( isset( $btnUpdateStay ) ) {
            $overlayId = $this->getVar( "overlay_id" );
            $overlaySource = $this->getVar( "overlay_source" );
            /* set the success page to be the update form */ 
            $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" . 
                                   $this->getVar( "operations_page", "APPLICATION" ) . "?page=lookAndFeelAdmin&" .
                                   "subpage=overlays&action=update&overlay_name=$templateName&" .
                                   "overlay_source=$overlaySource&overlay_id=$overlayId" );
         }
         else {
            $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" . 
                                   $this->getVar( "operations_page", "APPLICATION" ) . "?page=lookAndFeelAdmin&" .
                                   "subpage=overlays" );
         }
      }
   }

   function doAction( ) {
      parent::doAction( );

      $syntax = $this->getVar( "overlay_syntax", "PRIVATE" );
      /* special conversion: need to change </textarea> tag out to something else */
      $syntax = str_replace( "</textarea", "&lt;/textarea", $syntax );
      $this->setVar( "overlay_syntax", $syntax, "PRIVATE" );
   }    

   function triggerProcess( $aProcess ) {
      $pendTo = $this->getVar( "overlay_pend_to", "POST" );

      if ( !isset( $pendTo ) ) {
         /* clear the pend type */
         $this->clearVar( "overlay_pend_type", "POST" );
      }

      parent::triggerProcess( $aProcess );

      if ( !$this->isError( ) ) {
         $this->session->setSessionMessage( "Overlay Saved", MSG_INFO );
      }
   }
}

/*********************************************************/
/* TemplateActivator                                     */
/* Activates or deActivates a user-defined templatebased */
/* on the action passed                                  */
/*********************************************************/
class TemplateActivator extends PhooProcess {
   var $package = "look_and_feel";
   var $name = "TemplateActivator";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = ""; /* will be chosen based on the action passed */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-14" );
      $this->setPurpose( "activates or deactives a template based on the action passed" );
      $this->addExpects( "overlay_id", "CONTAINER" );
   }

   function setup( ) {
      parent::setup( );
      $action = $this->getVar( FORM_ACTION );
      
      switch( $action ) {
         case "deactivate" :
            $this->command = "=|Template.deactivate|=";
            break;
            
         case "activate" :
            $this->command = "=|Template.activate|=";
            break;

         default :
            $this->setError( "Illegal template activation action: $action", MSG_FATAL );
            break;
      }

      /* set the success page in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" . 
                             $this->getVar( "operations_page", "APPLICATION" ) . "?page=lookAndFeelAdmin&" .
                             "template_category=" . $this->getVar( "template_category" ) . 
                             "&subpage=overlays" );
   }

   function doAction( ) {
      
      parent::doAction( );

      if ( !$this->isError( ) ) {
         switch( $this->getVar( FORM_ACTION ) ) {
            case "activate" :
               $this->session->setSessionMessage( "Overlay Activated", MSG_INFO );
               break;

            case "deactivate" :
               $this->session->setSessionMessage( "Overlay Deactivated", MSG_INFO );
               break;
         }
      }
   }
}   


/*********************************************************************/
/* TemplateDeleter                                                   */
/* Deletes a template specification out of the overlay table         */
/*********************************************************************/
class TemplateDeleter extends PhooProcess {
   var $name = "TemplateDeleter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete a template. you do not have security privileges";
   var $command = "=|Template.delete|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-14" );
      $this->setPurpose( "Processes the deletion of overlay templates" );
      $this->addExpects( "overlay_id", "CONTAINER" );
      $this->addExpects( "template_category", "CONTAINER" );
   }

   function setup( ) {
      parent::setup( );

      /* set the success page in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" . 
                             $this->getVar( "operations_page", "APPLICATION" ) . "?page=lookAndFeelAdmin&" .
                             "template_category=" . $this->getVar( "template_category" ) .
                             "&subpage=overlays" );
   }      
}

class ThemeList extends PhooQuery {
   var $package = "look_and_feel";
   var $name = "ThemeList";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Theme.list.header|=";
   var $template = "=|Theme.list|=";
   var $templateFooter = "=|Theme.list.footer|=";
   var $rowsReturned = false;
   var $themeDescriptor;

   /* for sorting the themeDescriptor array */
   function compareFunction( $ar1, $ar2 ) {
      return strcmp( $ar1[ "display_name" ], $ar2[ "display_name" ] );
   }
   
   /* overrides ancestor - loads themes */
   function doAction( ) {
      $this->themeDescriptor = $this->session->getSnapOnDescriptorsForType( "theme" );

      /* set the theme name as one of the keys */
      foreach ( $this->themeDescriptor as $themeName => $descriptor ) {
         $this->themeDescriptor[ $themeName ][ "theme" ] = $themeName;
      }
      $this->rowsReturned = false; /* initialize */

      /* sort the themeDescriptor array */
      uasort( $this->themeDescriptor, array( "ThemeList", "compareFunction" ) );
   }
   
   /* overrides ancestor, returns data out of the themeDescriptor array */
   function getResultRow( ) {
      if ( !$this->rowsReturned ) { /* at the first row */
         reset( $this->themeDescriptor );
         $row = current( $this->themeDescriptor );
         $this->rowsReturned = true;
      }   
      else {
         $row = next( $this->themeDescriptor );
      }

      if ( is_array( $row ) ) { 
         if ( $row[ "display_name" ] > "" ) {
            $row[ "theme_name" ] = $row[ "display_name" ];
         }
         else {
            $row[ "theme_name" ] = $row[ "theme" ];
         }
         /* post the credits */         
         if ( isset( $row[ "credits" ] ) ) {
            $row[ "theme_description" ] .= $row[ "credits" ];
         }

         if ( trim( $row[ "theme_description" ] ) == "" ) { /* if nothing there, post the description and the author */
            $row[ "theme_description" ] = $row[ "description" ];

            if ( trim( $row[ "theme_description" ] ) > "" ) {
               $row[ "theme_description" ] .= "<br />";
            }
            
            $row[ "theme_description" ] .= "Author: ";
            if ( $row[ "author_url" ] > "" ) {
               $row[ "theme_description" ] .= "<a href=\"" . $row[ "author_url" ] . "\" target=_blank>";
            }
            $row[ "theme_description" ] .= $row[ "author" ];

            if ( $row[ "author_url" ] > "" ) {
               $row [ "author_url" ] . "</a>";
            }
         }

         /* if the source is TABLE BASED, will display "Custom" */
         if ( $row[ "source" ] == TABLE_BASED ) {
            $row[ "source_display" ] = "Custom";
         }
         else {
            $row[ "source_display" ] = $row[ "source" ];
         }

         /* set the cascades_from_display_name */
         if ( isset( $row[ "cascades_from_snap_on" ] ) ) {
            $cascadesFromDisplayName = $this->session->snapOn[ $row[ "cascades_from_snap_on" ] ][ "display_name" ];
            if ( isset( $cascadesFromDisplayName ) ) { 
              $row[ "cascades_from_display_name" ] = $cascadesFromDisplayName;
            }
            else {
              $row[ "cascades_from_display_name" ] = $row[ "cascades_from_snap_on" ];
            }
         }               

         /* see if this is the current default theme */
         if ( $row[ "theme" ] == $this->getVar( "default_theme", "APPLICATION" ) ) {
            $row[ "activation_class" ] = "activated";
            $row[ "is_default_theme" ] = true;
         }
         elseif ( $row[ "theme" ] == $this->getVar( "preview_theme", "COOKIE" ) ) {
            $row[ "activation_class" ] = "previewed";
            $row[ "is_default_theme" ] = false;
         }
         elseif ( $row[ "theme" ] == $this->getVar( "current_theme", "APPLICATION" ) ) {
            $row[ "activation_class" ] = "selected";
            $row[ "is_default_theme" ] = false;
         }
         else {
            $row[ "activation_class" ] = "deactivated";
            $row[ "is_default_theme" ] = false;
         }

         return $row;
      }
      
      return null; /* done */
   }
}

/*********************************************************/
/* ThemeSetDefaultLink                                   */
/* Controls the appearance and contents of the theme     */
/* set default link                                      */
/*********************************************************/
class ThemeSetDefaultLink extends PhooDisplay {
   var $package = "extension";
   var $name = "ThemeSetDefaultLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = ""; /* will be chosen based on the status of the current extension */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-18" );
      $this->setPurpose( "sets the syntax of the theme set default link" );
      $this->addExpects( "is_default_theme", "CONTAINER" );
      $this->addExpects( "theme", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      $isDefaultTheme = $this->getVar( "is_default_theme" );
      if ( $isDefaultTheme ) {
         $this->template = "=|Theme.isdefault|=";
      }
      else {
         $this->template = "=|Theme.link.setdefault|=";
      }
   }
}

/*********************************************************/
/* ThemeEditLink                                         */
/* Displays an edit link for table-based themes          */
/*********************************************************/
class ThemeEditLink extends PhooDisplay {
   var $package = "extension";
   var $name = "ThemeEditLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|Theme.link.edit|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-21" );
      $this->setPurpose( "renders the theme edit link" );
      $this->addExpects( "source", "CONTAINER" );
      $this->addExpects( "theme", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      
      $themeType = $this->getVar( "source" );
      if ( $themeType != TABLE_BASED ) {
         $this->disabled = true;
      }
   }
}

/*********************************************************/
/* ThemeDeleteLink                                       */
/* Displays a delete link for table-based themes         */
/*********************************************************/
class ThemeDeleteLink extends PhooDisplay {
   var $package = "extension";
   var $name = "ThemeDeleteLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|Theme.link.delete|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-28" );
      $this->setPurpose( "renders the theme delete link" );
      $this->addExpects( "source", "CONTAINER" );
      $this->addExpects( "theme", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      
      $themeSource = $this->getVar( "source" );
      if ( $themeSource != TABLE_BASED ) {
         $this->disabled = true;
      }

      /* check to see if this is the default theme - if so, don't allow deletion */
      else {
         if ( $this->getVar( "theme" ) == $this->getVar( "default_theme" ) ) {
            $this->disabled = true;
         }
      }
   }
}

/*********************************************************/
/* ThemePreviewLink                                      */
/* Displays a preview link for themes                    */
/*********************************************************/
class ThemePreviewLink extends PhooDisplay {
   var $package = "extension";
   var $name = "ThemePreviewLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|Theme.link.preview|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-18" );
      $this->setPurpose( "renders the theme preview link" );
      $this->addExpects( "theme", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      
      $previewTheme = $this->getVar( "preview_theme", "COOKIE" );
      
      if ( isset( $previewTheme ) && $previewTheme == $this->getVar( "theme", "PRIVATE" ) ) {
         /* alreary previewing. Show the "Stop Preview" link */
         $this->template = "=|Theme.link.preview.stop|=";
      }
      else {
         /* not previewing, show the "Preview" link */
         $this->template = "=|Theme.link.preview|=";
      }         
   }
}

/*********************************************************/
/* ThemeScreenShotLink                                   */
/* Displays a screenShot link for themes                 */
/*********************************************************/
class ThemeScreenShotLink extends PhooDisplay {
   var $name = "ThemeScreenShotLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|Theme.link.screen_shot|=";

   /* determine which link to use */
   function setup( ) {
      parent::setup( );

      $theme = $this->container->getVar( "theme" );
      
      $screenShot = $this->session->snapOn[ $theme ][ "screen_shot" ];

      if ( trim( $screenShot ) > "" ) {
         $this->setVar( "screen_shot_url", $screenShot, "PRIVATE" );
      }
      else {
         $this->disabled = true; /* if no screenShot, disable the link */
      }
   }
}

class ThemeDefaultSetter extends PhooProcess {
   var $package = "look_and_feel";
   var $name = "ThemeDefaultSetter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|Theme.default.update|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-18" );
      $this->setPurpose( "Sets the default theme for the blog" );
   }

   function setup( ) {
      /* set the current GET variable as a private variable, to be picked up in the SQL */
      $this->setVar( "theme", $this->getVar( "theme", "GET" ), "PRIVATE" );

      /* set the success page in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" . 
                             $this->getVar( "operations_page", "APPLICATION" ) . "?page=lookAndFeelAdmin&" .
                             "&subpage=themes" );
   }
}

class ThemeForm extends PhooForm {
   var $package = "look_and_feel";
   var $name = "ThemeForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $updateForm = "=|Theme.form.update|=";
   var $insertForm = "=|Theme.form.insert|=";
//   var $loadCommand = "=|Theme.get|="; 
   var $updateCommand = "=|Theme.update|=";
   var $insertCommand = "=|Theme.insert|=";
   var $trigger = array( "btnInsert" => array( "process" => "insert" )
                        ,"btnUpdate" => array( "process" => "update" ));
   var $field = array( "theme_tag" => array( "required" => "Y"
                                             ,"formalName" => "Theme Tag" )
                      ,"theme_name" => array( "required" => "Y"
                                               ,"formalName" => "Theme Name" )
                      ,"theme_cascades_from" => array( "required" => "Y"
                                               ,"formalName" => "Cascades From" )
                     );
                        
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-18" );
      $this->setPurpose( "Displays and processes the theme create/update form" );
   }

   function setup( ) {
      parent::setup( );

      $this->action = $this->getVar( "action", "GET" );
      
      $process = $this->getTriggerProcess( );

      if ( $this->action == "update" &&  !isset( $process ) ) {
         /* no update requested yet - load the variables */
         $themeTag = $this->getVar( "theme_tag" );
         $this->setVar( "theme_name", $this->session->themeInventory[ $themeTag ][ "theme_name" ], "POST" );
         $this->setVar( "theme_cascades_from", 
               $this->session->themeInventory[ $themeTag ][ "cascades_from_snap_on" ], "POST" );
      }

      /* set the success page in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" . 
                             $this->getVar( "operations_page", "APPLICATION" ) . "?page=lookAndFeelAdmin" .
                             "&subpage=themes" );
   }

   function validateForm( ) {
      parent::validateForm( );

      if ( $this->isError( ) ) {
         return;
      }

      /* check the theme tag against current themes */
      $themeTag = $this->getVar( "theme_tag", "POST" );
      if ( $this->action == "insert" && isset( $this->session->themeInventory[ $themeTag ] ) ) {
         $this->setError( "Theme $themeTag already exists", MSG_WARNING );
         return;
      }
   }

   function triggerProcess( $aProcess ) {
      /* create descriptor */
      $descriptor = "<?xml version=\"1.0\"?>" .
                    "  <snapon>" .
                    "    <name>" . $this->getVar( "theme_tag", "POST" ) . "</name>" .
                    "    <display_name>" . $this->getVar( "theme_name", "POST" ) . "</display_name>" .
                    "    <version>1.0</version>" .
                    "    <description></description>" .
                    "    <type>theme</type>" .
                    "    <author>" . $this->session->getUserKnownBy( ) . "</author>" .
                    "    <create_date>" . $this->session->getLocalDate( "m/d/Y" ) . "</create_date>" .
                    "    <cascades_from_snap_on>" . 
                           $this->getVar( "theme_cascades_from", "POST" ) . "</cascades_from_snap_on>" .
                    "    <style_sheet></style_sheet>" .
                    "  </snapon>";

      $this->setVar( "theme_descriptor", addslashes( $descriptor ), "POST" );

      parent::triggerProcess( $aProcess );

      if ( !$this->isError( ) ) {
         $this->session->setSessionMessage( "Theme header information saved", MSG_INFO );
      }
   }
}

class ThemeCascadesFromDropDown extends ThemeDropDown {
   var $package = "look_and_feel";
   var $name = "ThemeCascadesFromDropDown";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $dropDownKeyName = "theme_cascades_from";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-19" );
      $this->setPurpose( "Displays a drop-down containing themes that the current theme can cascade from" );
      $this->addExpects( "theme_tag", "CONTAINER" ); /* will not allow this to be in the list */
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }

   function setup( ) {
      /* set the default cascades from theme to the base theme */
      $themeCascadesFrom = $this->getVar( "theme_cascades_from" );
      if ( $themeCascadesFrom == "" ) {
         $this->setVar( "theme_cascades_from", "bloo_base", "PRIVATE" );
      }

      parent::setup( );
   }

   /* filter out the current theme_tag (so that if a theme is being edited it can't be set
      to cascade from itself */
   function getResultRow( ) {
      while ( 1 == 1 ) {
         $row = parent::getResultRow( );
         if ( $row == null ) { 
            return null;
         }  
         if ( $row[ "theme" ] != $this->getVar( "theme_tag", "PRIVATE" ) ) {
            return $row;
         }
      }
   }
}

/*********************************************************************/
/* ThemeeDeleter                                                     */
/* Deletes a theme from the theme table                              */
/*********************************************************************/
class ThemeDeleter extends PhooProcess {
   var $name = "ThemeDeleter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete a theme. you do not have security privileges";
   var $command = "=|Theme.delete|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-29" );
      $this->setPurpose( "Processes the deletion of a created theme" );
      $this->addExpects( "theme_tag", "CONTAINER" );
   }

   function setup( ) {
      parent::setup( );

      /* make sure this theme is not the current theme */
      if ( $this->getVar( "theme_tag" ) == $this->getVar( "default_theme" ) ) {
         $this->setError( "You cannot delete the current default theme", MSG_FATAL );
         return;
      }

      /* see if another theme cascades from this theme */
      $themeTag = $this->getVar( "theme_tag" );
      foreach ( $this->session->theme as $t ) {
         if ( $t[ "theme_cascades_from" ] == $themeTag ) {
            $this->setError( "You cannot delete the theme $themeTag as other themes are cascaded from it", MSG_FATAL );
            return;
         }
      }
      
      /* set the success page in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" . 
                             $this->getVar( "operations_page", "APPLICATION" ) . "?page=lookAndFeelAdmin&" .
                             "&subpage=themes" );
   }      
}

class AppendPrependDropDown extends HardCodedDropDown {
   var $securityRole = ROLE_EVERYONE;
   var $keyValue = array( array( "key_value" => "prepend", "key_name" => "Prepend To" )
                         ,array( "key_value" => "append", "key_name" => "Append To" )                         
                        );
                        
   var $templateHeader = "<select name=overlay_pend_type>";
   var $template = "<option value=\"_@key_value@_\">_@key_name@_</option>";
   var $templateFooter = "</select>";
   var $dropDownKeyName = "overlay_pend_type";
} 

class ThemeCredits extends PhooDisplay {
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Theme.credits|=";

   function setup( ) {
      $currentTheme = $this->session->getSetting( "current_theme" );

      $this->setVar( "theme_credits", $this->session->snapOn[ $currentTheme ][ "credits" ], "PRIVATE" );
   }
}

?>
