<?php
/*   
   Name: menu.classes.php
   Author: Bill Roberts
   Created: 11/18/2005
   Purpose: SnapOn classes related to menu items
   $Revision: 1.2 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

class BlooMenuItem extends PhooMenuItem {
   var $name = "BlooMenuItem";
   var $package = "menu";
   var $selectedTemplate = "=|MenuItem.selected|=";
   var $unselectedTemplate = "=|MenuItem.unselected|=";
   var $selectedVar = "page";
   var $selectedValue;

   function isSelected( ) {
      $value = $this->getVar( $this->selectedVar );
      return ( $value == $this->selectedValue );
   }
}

class BlooTabbedMenuItem extends BlooMenuItem {
   var $name = "BlooMenuItem";
   var $package = "menu";
   var $selectedTemplate = "=|AdminPage.tab.selected|=";
   var $unselectedTemplate = "=|AdminPage.tab.unselected|=";
   var $selectedVar = "subpage";
}

class BlooTabbedMenu extends PhooMenu {
   var $name = "BlooTabbedMenu";
   var $package = "menu";
   var $templateHeader = "=|AdminPage.tabs.header|=";
   var $templateFooter = "=|AdminPage.tabs.footer|=";
}
   

/* define menu item snapons */
class MenuItemPosts extends BlooMenuItem {
   var $name = "MenuItemPosts";
   var $menuItemName = "Posts";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=postAdmin";
   var $securityRole = ROLE_POSTER;
   var $selectedValue = "postAdmin";

   function isSelected( ) { /* if the selectedVar isn't set, default to this one */
      $value = $this->getVar( $this->selectedVar );
      if ( $value == "" ) { 
         return true; 
      }
      else {
         return ( $value == $this->selectedValue );
      }
   }
}

class MenuItemPages extends BlooMenuItem {
   var $name = "MenuItemPages";
   var $menuItemName = "Pages";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=pageAdmin";
   var $securityRole = ROLE_POSTER;
   var $selectedValue = "pageAdmin";
}

class MenuItemSettings extends BlooMenuItem {
   var $name = "MenuItemSettings";
   var $menuItemName = "Lists and Settings";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=settingsAdmin";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "settingsAdmin";
}

class MenuItemRecentLogs extends BlooMenuItem {
   var $name = "MenuItemRecentLogs";
   var $menuItemName = "Recent Logs";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=log";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "log";
}

class MenuItemUsers extends BlooMenuItem {
   var $name = "MenuItemUsers";
   var $menuItemName = "Users";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=userAdmin";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "userAdmin";
}

class MenuItemProfile extends BlooMenuItem {
   var $name = "MenuItemProfile";
   var $menuItemName = "My Profile";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=profileAdmin";
   var $securityRole = ROLE_EVERYONE;
   var $selectedValue = "profileAdmin";
}

class MenuItemLogout extends BlooMenuItem {
   var $name = "MenuItemLogout";
   var $menuItemName = "Logout";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?action=logout";
   var $securityRole = ROLE_EVERYONE;

   function isSelected( ) {
      return false; /* never show as selected */
   }
}

class MenuItemBackToBlog extends BlooMenuItem {
   var $name = "MenuItemBackToBlog";
   var $menuItemName = "Back to the Blog";
   var $menuItemLink = "_@site_url@_";
   var $securityRole = ROLE_EVERYONE;

   function isSelected( ) {
      return false; /* never show as selected */
   }
}

class MenuItemExtensions extends BlooMenuItem {
   var $name = "MenuItemExtensions";
   var $menuItemName = "SnapOns";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=extensionAdmin";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "extensionAdmin";
}

class MenuItemLookAndFeel extends BlooMenuItem {
   var $name = "MenuItemLookAndFeel";
   var $menuItemName = "Look and Feel";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=lookAndFeelAdmin&subpage=themes";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "lookAndFeelAdmin";
}

class MenuItemModeration extends BlooMenuItem {
   var $name = "MenuItemModeration";
   var $menuItemName = "Moderation";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=moderationAdmin&subpage=approved_comments";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "moderationAdmin";
}

class MenuItemApprovedComments extends BlooTabbedMenuItem {
   var $name = "MenuItemApprovedComments";
   var $menuItemName = "Approved Comments";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=moderationAdmin&subpage=approved_comments";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "approved_comments";
}

class MenuItemUnapprovedComments extends BlooTabbedMenuItem {
   var $name = "MenuItemUnapprovedComments";
   var $menuItemName = "Suspected Spam Comments";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=moderationAdmin&subpage=unapproved_comments";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "unapproved_comments";
}

class MenuItemApprovedTrackbacks extends BlooTabbedMenuItem {
   var $name = "MenuItemApprovedTrackbacks";
   var $menuItemName = "Approved Trackbacks";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=moderationAdmin&subpage=approved_trackbacks";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "approved_trackbacks";
}

class MenuItemUnapprovedTrackbacks extends BlooTabbedMenuItem {
   var $name = "MenuItemUnapprovedTrackbacks";
   var $menuItemName = "Suspected Spam Trackbacks";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=moderationAdmin&subpage=unapproved_trackbacks";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "unapproved_trackbacks";
}

class ModerationMenu extends BlooTabbedMenu {
   var $securityRole = ROLE_ADMINISTRATOR;

   function setup( ) {
      parent::setup( );

      /* construct the menu */
      $m =& $this->session->snapOnFactory( "MenuItemApprovedComments", $this );
      $this->addMenuItem( $m );

      $m =& $this->session->snapOnFactory( "MenuItemUnapprovedComments", $this );
      $this->addMenuItem( $m );

      $m =& $this->session->snapOnFactory( "MenuItemApprovedTrackbacks", $this );
      $this->addMenuItem( $m );
      
      $m =& $this->session->snapOnFactory( "MenuItemUnapprovedTrackbacks", $this );
      $this->addMenuItem( $m );
   }
}

class BlooMenu extends PhooMenu {
   var $package = "menu";
   var $name = "BlooMenu";
   var $securityRole = ROLE_EVERYONE;

   function setup( ) {
      parent::setup( );

      /* construct the menu */
      $m =& $this->session->snapOnFactory( "MenuItemPosts", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemPages", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemSettings", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemUsers", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemProfile", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemExtensions", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemLookAndFeel", $this );
      $this->addMenuItem( $m );
/*** Note: remove this for now      
      $m =& $this->session->snapOnFactory( "MenuItemRecentLogs", $this );
      $this->addMenuItem( $m );
***/
      $m =& $this->session->snapOnFactory( "MenuItemModeration", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemLogout", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemBackToBlog", $this );
      $this->addMenuItem( $m );
   }
}

/* menu items for the look and feel page tabs */
/* define menu item snapons */
class MenuItemLookAndFeelOverlays extends BlooTabbedMenuItem {
   var $name = "MenuItemLookAndFeelOverlays";
   var $menuItemName = "Overlays";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=lookAndFeelAdmin&subpage=overlays";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "overlays";
}

class MenuItemLookAndFeelThemes extends BlooTabbedMenuItem {
   var $name = "MenuItemLookAndFeelThemes";
   var $menuItemName = "Themes";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=lookAndFeelAdmin&subpage=themes";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "themes";
}

class MenuItemLookAndFeelDisplay extends BlooTabbedMenuItem {
   var $name = "MenuItemLookAndFeelDisplay";
   var $menuItemName = "Post and Comment Display Settings";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=lookAndFeelAdmin&subpage=display";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "display";
}

class LookAndFeelMenu extends BlooTabbedMenu {
   var $package = "menu";
   var $name = "LookAndFeelMenu";
   var $securityRole = ROLE_ADMINISTRATOR;

   function setup( ) {
      parent::setup( );

      /* construct the menu */
      $m =& $this->session->snapOnFactory( "MenuItemLookAndFeelThemes", $this );
      $this->addMenuItem( $m );

      $m =& $this->session->snapOnFactory( "MenuItemLookAndFeelOverlays", $this );
      $this->addMenuItem( $m );

      $m =& $this->session->snapOnFactory( "MenuItemLookAndFeelDisplay", $this );
      $this->addMenuItem( $m );
   }
}

/* menu items for the settings page tabs */
class MenuItemSettingsBloogroll extends BlooTabbedMenuItem {
   var $name = "MenuItemSettingsBloogroll";
   var $menuItemName = "Bloogroll";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=settingsAdmin&subpage=bloogroll";
   var $securityRole = ROLE_POSTER;
   var $selectedValue = "bloogroll";
}

class MenuItemSettingsPostCategories extends BlooTabbedMenuItem {
   var $name = "MenuItemSettingsPostCategories";
   var $menuItemName = "Post Categories";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=settingsAdmin&subpage=post.category";
   var $securityRole = ROLE_POSTER;
   var $selectedValue = "post.category";
}

/* menu items for the settings page tabs */
class MenuItemSettingsBloogrollCategories extends BlooTabbedMenuItem {
   var $name = "MenuItemSettingsBloogrollCategories";
   var $menuItemName = "Bloogroll Categories";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=settingsAdmin&subpage=bloogroll.category";
   var $securityRole = ROLE_POSTER;
   var $selectedValue = "bloogroll.category";
}

class MenuItemSettingsBlogProfile extends BlooTabbedMenuItem {
   var $name = "MenuItemSettingsBlogProfile";
   var $menuItemName = "Blog Profile";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=settingsAdmin&subpage=blog.profile";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "blog.profile";
}

class MenuItemSettingsDiscussion extends BlooTabbedMenuItem {
   var $name = "MenuItemSettingsDiscussion";
   var $menuItemName = "Discussion Settings";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=settingsAdmin&subpage=discussion";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "discussion";
}

class MenuItemSettingsTimeZone extends BlooTabbedMenuItem {
   var $name = "MenuItemSettingsTimeZone";
   var $menuItemName = "Time Zone";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=settingsAdmin&subpage=time_zone";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "time_zone";
}

class MenuItemSettingsPingers extends BlooTabbedMenuItem {
   var $name = "MenuItemSettingsPingers";
   var $menuItemName = "Pingers";
   var $menuItemLink = "_@site_url@_/_@operations_page@_?page=settingsAdmin&subpage=pingers";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $selectedValue = "pingers";
}

class SettingsMenu extends BlooTabbedMenu {
   var $package = "menu";
   var $name = "SettingsMenu";
   var $securityRole = array( ROLE_POSTER, ROLE_ADMINISTRATOR );

   function setup( ) {
      parent::setup( );

      /* construct the menu */
      $m =& $this->session->snapOnFactory( "MenuItemSettingsBloogroll", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemSettingsPostCategories", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemSettingsBloogrollCategories", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemSettingsBlogProfile", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemSettingsDiscussion", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemSettingsTimeZone", $this );
      $this->addMenuItem( $m );
      $m =& $this->session->snapOnFactory( "MenuItemSettingsPingers", $this );
      $this->addMenuItem( $m );
   }
}
?>
