<?php
/* 
   Name: pinger.classes.php
   Author: Bill Roberts
   Created: 4/11/2005
   Purpose: SnapOn classes related to pinging blog aggregator services
   $Revision: 1.2 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/** WeblogPinger
    class for blog aggregator pingers (to ping
    services like weblogs, blogrolling, blo.gs, etc 
 **/
class WeblogPinger extends PhooDisplay {
   var $package = "pinger";
   var $name = "WeblogPinger";
   var $securityRole = ROLE_POSTER;
   var $port = 80;
   var $share;
   var $host;
   var $requestHeader = "=|Pinger.request.header|=";
   var $request = "=|Pinger.request.xml|=";
   var $successIndicator = "200 OK"; /* for now, just look for an OK */
   var $template = "";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-21" );
      $this->setPurpose( "Weblog pinger class." );
      $this->addExpects( "blog_name", "APPLICATION" );
      $this->addExpects( "site_url", "APPLICATION" );
      $this->addExpects( "pinger_host", "POST" );
      $this->addExpects( "pinger_share", "POST" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }

   function doAction( $aArray = null ) {
      if ( !$this->isError( ) ) {
	      /* determine the pinger user agent, and assure it's trimmed, as linefeeds may be significant */
	      $pingerUserAgent = $this->expandOverlays( "=|Pinger.user.agent|=" );
         /* remove any linefeeds */
         $pingerUserAgent = str_replace( "\n", "", $pingerUserAgent );
	      $this->setVar( "pinger_user_agent", $pingerUserAgent, "PRIVATE" );

         /* load the request and the request header */
         $this->request = $this->populateParameters( $this->expandOverlays( $this->request ) );
         $this->requestHeader = $this->populateParameters( $this->expandOverlays( $this->requestHeader ) );
         $this->requestHeader = str_replace( "__CONTENT_LENGTH__", strlen( $this->request ), $this->requestHeader );
      }

      $this->share = $this->getVar( "pinger_share", "POST" );
      $this->host = $this->getVar( "pinger_host", "POST" );

      $fp = @fsockopen( $this->host, $this->port, $errno, $errstr );

      if ( !$fp ) {
         $error = "Could not open socket for pinger at host " . $this->host . " port: " . $this->port .
                          ". Your post was saved, but the ping did not occur.";
                          
         /* write the error out to a log */
         $s =& $this->session->snapOnFactory( "SystemLogWriter", $this );
         $s->setMessage( $this->package, "Error: " . $error );
         $s->process( );
/* Removed - writing the error to the log is enough - don't want to scare people
         $this->setError( $error, MSG_WARNING );
*/         
         return;
      }
     
      $fullRequest = $this->requestHeader . $this->request; 

      if ( !@fputs( $fp, $fullRequest, strlen( $fullRequest ) ) ) {
         $error = "Write error on socket for pinger";
         
         /* write the error out to a log */
         $s =& $_session->snapOnFactory( "SystemLogWriter", $this );
         $s->setMessage( $this->package, "Error: " . $this->response );
         $s->process( );

/* Removed - writing the error to the log is enough - don't want to scare people
         $this->setError( $error, MSG_WARNING );
*/         
         return;
      }

      $this->response = "";
      while ( !@feof( $fp ) ) {
         $this->response .= @fgets( $fp );
      }

      @fclose($fp);

      /* write the response out to the log */
      $s =& $this->session->snapOnFactory( "SystemLogWriter", $this );
      $s->setMessage( $this->package, "Host: " . $this->host . ", Response: " . $this->response );
      $s->process( );

      if ( $this->isError( ) ) {
         return;
      }
         
      /* check the response */
      $pos = strPos( $this->response, $this->successIndicator );
      if ( !$pos ) {
         /* Note: not setting this error for now */
         // $this->setError( "Weblogs ping failed", MSG_WARNING );
      }
   }
}

class PingerLoop extends PhooQuery {
   var $package = "pinger";
   var $name = "PingerLoop";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Pinger.list.getactive|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.11" );
      $this->setCreateDate( "2005-04-21" );
      $this->setPurpose( "Loops and pings through the pinger services (if any) that are setup for pinging" );
      $this->addModificationHistory( "Refined this by replacing the HOST/URL with HOST/SHARE - which may be the cause of the" .
                              "problems with the pingers", "2005-05-11" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }

   function doAction( $aArray = null ) {
      /* check the site url - if it starts with http://localhost then this is probably a test/local system - don't ping */
      $siteUrl = $this->getVar( "site_url", "APPLICATION" );
      if ( substr( strtolower( $siteUrl ), 0, 16 ) == "http://localhost" ) {
         return;
      }
      
      parent::doAction( $aArray );

      if ( $this->isError( ) ) {
         return;
      }

      /* loop through the pinger services, if any */
      $row = $this->getResultRow( );

      $i = 0;

      while( is_array( $row ) ) {
         $pingerSpec[ $i ] = $row;
         $i++;
         $row = $this->getResultRow( );
      }

      for ( $j=0; $j<$i; $j++ ) {
         $this->setVar( "pinger_host", $pingerSpec[$j]["pinger_host" ], "POST" );
         $this->setVar( "pinger_share", $pingerSpec[$j][ "pinger_share" ], "POST" );
         $pinger =& $this->session->snapOnFactory( "WeblogPinger", $this );
         if ( $this->isError( ) ) {
            return;
         }

         $pinger->process( );
      }
   }
}

/*****************************************/
/*** SnapOns for Pinger administration ***/
/*****************************************/
class ListPingerDisplay extends PhooQuery {
   var $package = "pinger";
   var $name = "ListPingerDisplay";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|Pinger.list.get|=";
   var $templateHeader = "=|Pinger.list.header|=";
   var $template = "=|Pinger.list|=";
   var $templateFooter = "=|Pinger.list.footer|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-01-15" );
      $this->setPurpose( "Displays the pinger administration list" );
   }

   /* traps for activation/deactivation actions, if sent */
   function doAction( ) {
      $action = $this->getVar( FORM_ACTION, "GET" );
      
      /* process based on the action */
      switch ( $action ) {
         case "activate" :
         case "deactivate" :
            /* activate or deactivate an extension */
            $s =& $this->session->snapOnFactory( "PingerActivator", $this );
            $s->process( ); /* process the activation/deactivation */
            if ( $s->isError( ) ) {
               return;
            }
            break;
      }

      /* and now back to our regularly scheduled action */      
      parent::doAction( );
   } 

   /* sets the css style for each row based on its activated flag 
   */
   function getResultRow( ) {
      $row = parent::getResultRow( );
      if ( $row ) {
         /* set the css style for the row */
         if ( $row[ "activated_flag" ] == "Y" ) {
            $row[ "activation_class" ] = "activated";
         }
         else {
            $row[ "activation_class" ] = "deactivated";
         } 
      }
      return $row;
   }
   
   
}

class ListPingerForm extends PhooForm {
   var $package = "pinger";
   var $name = "ListPingerForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $loadCommand = "=|Pinger.get|=";
   var $updateForm = "=|Pinger.form.update|=";
   var $updateCommand = "=|Pinger.update|=";
   var $insertForm = "=|Pinger.form.insert|=";
   var $insertCommand = "=|Pinger.insert|=";
   var $insertConstraint = "=|Pinger.duplicate.constraint|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   var $field = array( "pinger_name" => array( "required" => "Y"
                                              ,"formalName" => "Pinger Service" )
                      ,"pinger_host" => array( "required" => "Y"
                                               ,"formalName" => "Host" )
                      ,"pinger_share" => array( "required" => "Y"
                                               ,"formalName" => "Share" )
                     );
 
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-01-15" );
      $this->setPurpose( "Displays and processes the pinger administration form" );
   }

   function setup( ) {
      parent::setup( );

      /* set the list type here */
      $this->setVar( "list_type", "pingers", "POST" ); /* set in post vars to make it accessible to 
                                                            the new_id command */

      /* set the success page, in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=settingsAdmin&subpage=pingers" );
   }
   
}

class PingerActivationLink extends PhooDisplay {
   var $package = "pinger";
   var $name = "PingerActivationLink";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = ""; /* will be chosen based on the status of the current event */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-02" );
      $this->setPurpose( "sets the syntax of the pinger activation link" );
      $this->addExpects( "activated_flag", "CONTAINER" );
      $this->addExpects( "pinger_id", "CONTAINER" );
   }

   /* determine which link to use */
   function setup( ) {
      parent::setup( );
      $activationFlag = $this->getVar( "activated_flag" );
      switch( $activationFlag ) {
         case "Y" :
            $this->template = "=|Pinger.link.deactivate|=";
            break;
         case "N" :
            $this->template = "=|Pinger.link.activate|=";
            break;
      }
   }
}

/*********************************************************/
/* PingerActivator                                       */
/* Activates or deActivates a pinger based on the        */
/* action passed                                         */
/*********************************************************/
class PingerActivator extends PhooProcess {
   var $package = "pinger";
   var $name = "PingerActivator";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = ""; /* will be chosen based on the action passed */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-02" );
      $this->setPurpose( "activates or deactives a pinger based on the action passed" );
      $this->addExpects( FORM_ACTION, "GET" );
      $this->addExpects( "list_item_id", "GET" );
   }

   function setup( ) {
      parent::setup( );
      $action = $this->getVar( FORM_ACTION );
      
      switch( $action ) {
         case "deactivate" :
            $this->command = "=|Pinger.deactivate|=";
            break;
            
         case "activate" :
            $this->command = "=|Pinger.activate|=";
            break;

         default :
            $this->setError( "Illegal pinger activation action: $action", MSG_FATAL );
            break;
      }

      /* set the success page, assuming this will be successful */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                            "?page=settingsAdmin&subpage=pingers" );

   }
}   

class PingerDeleter extends PhooProcess {
   var $package = "pinger";
   var $name = "PingerDeleter";
   var $command = "=|Pinger.delete|=";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete a pinger. you do not have security privileges";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-16" );
      $this->addExpects( "list_type", "CONTAINER" );
      $this->addExpects( "pinger_id", "CONTAINER" );
      $this->setPurpose( "Controls deletion of pinger items" );
   }
}


?>
