<?php
/* 
   Name: post.classes.php
   Author: Bill Roberts
   Created: 4/19/2005
   Purpose: SnapOn classes related to creating, editing, and updating posts 
   $Revision: 1.6 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
   
define( "POST_PUBLIC", 1 );
define( "POST_DRAFT", 2 );
define( "POST_PRIVATE", 3 );

/****************************************************/
/* PostDisplay - displays a page full of posts      */
/****************************************************/
class PostDisplay extends PhooQuery { 
   var $name = "PostDisplay";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Post.list.get|=";
   var $template = "=|Post|=";
   var $templateFooter = "_{PostResultPageNavigation}_";
   var $search = false; /* will set to true if we're doing a search */                                            
   var $searchText;
   var $singlePost = false;
   var $showTimeZone = false;
   var $categoryTemplate = "=|Post.category_list.item|=";
   var $noCategoryTemplate = "=|Post.category_list.none|=";
   var $countCommand = "=|Post.count|=";
   var $resultPageSize;

   function setup( ) {
      /* set the result page size */
      $this->resultPageSize = $this->session->getSetting( "posts_to_display" );
      
      parent::setup( );

      /* set date format */
      $timeFormat = $this->session->getSetting( "post_time_format" );

      /* note: since PHP 4 doesn't recognize the time zone format indicator, and since we're
         using a Phoo based, rather than PHP native, time zone, modify the format here */
      if ( strpos( $timeFormat, "e" ) ) { /* check for time zone indicator */
         $timeFormat = str_replace( "e", "", $timeFormat );
         $this->showTimeZone = true;
      }
      
      $this->field = array( "post_date" => array( "format" => $timeFormat
                           ,"type" => "date" ) );

      /* see if a post was specified in a GET parameter */
      $postId = $this->getVar( "post_id", "GET" );
      if ( isset( $postId ) ) {
         /* if there is a comment id and an update action, don't show the post */
         $action = $this->getVar( "action" );
         $commentId = $this->getVar( "comment_id" );
         
         if ( isset( $commentId ) && ( $action == "update" ) ) {
            $this->template = "";
         }
         else {
            /* load a single post */
            $this->command = "=|Post.get|=";      
            $this->singlePost = true;
            $this->templateFooter = ""; /* clear the result page navigation */
         }
      }
      else {
         /* see if a post year and month was specified in a GET parameter */
         $postYearMonth = $this->getVar( "post_year_month", "GET" );
         /* split the post year and month out and stick in the post variables for later use */
         if ( isset( $postYearMonth ) && !empty( $postYearMonth ) ) {
            /* load all posts for a given month */
            $this->setVar( "post_year", substr( $postYearMonth, 0, 4 ), "POST" );
            $this->setVar( "post_month", substr( $postYearMonth, 4 ), "POST" );
            $this->setVar( "additional_parameters", "&post_year_month=". $postYearMonth, "POST" );
            $this->command = "=|Post.list.month.get|=";
            $this->countCommand = "=|Post.month.count|=";
         }
         else {
         /* see if a post category was specified in a GET parameter */
            $postCategoryId = $this->getVar( "post_category_id", "GET" );
            if ( isset( $postCategoryId ) ) {
               /* load all posts for a given category */
               $this->command = "=|Post.list.category.get|=";
               $this->countCommand = "=|Post.category.count|=";
               $this->setVar( "additional_parameters", "&post_category_id=". $postCategoryId, "POST" );
            }
            else {
              /* see if search text was specified in a GET parameter */
               $searchText = $this->getVar( "search_text" );
               if ( isset( $searchText ) ) {
                  /* load posts that match a category */
                  $this->command = "=|Post.getbysearch|=";
                  $this->countCommand = "=|Post.search.count|=";
                  $this->setVar( "additional_parameters", "&search_text=". $searchText, "POST" );
                  $this->search = true;
                  $this->searchText = $searchText;
               }
            }
         }
      }
   }   

   /* adjusts the time */
   function getResultRow( ) {
      /* call the ancestor */
      $row = parent::getResultRow( );

      if ( $this->isError( ) || !is_array( $row ) ) {
         return;
      }

      if ( isset( $row[ "post_date" ] ) ) {
         $postTime = $this->session->getLocalDate( BASE_TIME_FORMAT, $row[ "post_date" ] );
         $row[ "post_date" ] = $postTime[ 0 ];
         if ( $this->showTimeZone ) {
            $row[ "post_time_zone_abbrev" ] = $postTime[ 1 ];
         }
         else {
            $row[ "post_time_zone_abbrev" ] = "";
         }
      }

      /* retrieve categories for the current post and create a category string */
      /* note: need to do this in a new PhooQuery object */
      $s = $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole;
      $s->command = "=|Post.meta.category.list|=";
      $s->setVar( "post_id", $row[ "post_id" ], "PRIVATE" );
      $s->process( );
      if ( $s->isError( ) ) {
         return;
      }

      $startOfList = true;
      $row[ "post_category_list" ] = "";
      while ( $catRow = $s->getResultRow( ) ) {
         if ( !$startOfList ) {
            $row[ "post_category_list" ] .= ", ";
         }
         $startOfList = false;
            
         $this->setVar( "category_id", $catRow[ "category_id" ], "PRIVATE" );
         $this->setVar( "category_description", $catRow[ "category_description" ], "PRIVATE" );
            
         $row[ "post_category_list" ] .= trim( $this->populateParameters( $this->expandOverlays( $this->categoryTemplate ) ) );
      }
      
      if ( $row[ "post_category_list" ] == "" ) {
         $row[ "post_category_list" ] = $this->populateParameters( $this->expandOverlays( $this->noCategoryTemplate ) );;
      }

      /* if searching, highlight the search text */
      if ( $this->search && $this->searchText > "" ) {
         $row[ "post_content" ] = $this->highlightSearchText( $row[ "post_content" ], $this->searchText );
         $row[ "post_title" ] = $this->highlightSearchText( $row[ "post_title" ], $this->searchText );
      }

      return $row;
   }

   function highlightSearchText( $aContent, $aSearchText ) {
      $looping = true;
      $matchArray = array( );
      $matchKey = 0;
         
      while ( $looping ) {
         eregi( $aSearchText, $aContent, $match );
         if ( isset( $match[ 0 ] ) ) {
            $key = $match[ 0 ];
            unset( $match[ 0 ] );
            /* Ok, I realize this looks funky . . . */
            $matchArray[ $key ] = "_M_a_t_c_h_" . $matchKey . "_h_c_t_a_M_"; /* just doing this to get a unique list */
            $matchKey++;
            $aContent = ereg_replace( $key, $matchArray[ $key ], $aContent );
         }
         else {
            $looping = false;
         }
      }

      /* perform the hilight */
      foreach ( $matchArray as $key => $value ) {     
        $aContent = ereg_replace( $value, "<font class=hilight>" . $key . "</font>", $aContent );
      }
      
      return $aContent;
   }
   
   /* highlights search text if search text was specified */
   function renderSnapOnX( ) {
      $syntax = parent::renderSnapOn( );

      /* highlight the search text */
      if ( $this->search && $this->searchText > "" ) {
         /* all that follows, believe it or not, was to preserve the case of the highlighted text. 
            of course, there's probably an easier way */
         $looping = true;
         $matchArray = array( );
         $matchKey = 0;
         
         while ( $looping ) {
            eregi( $this->searchText, $syntax, $match );
            if ( isset( $match[ 0 ] ) ) {
               $key = $match[ 0 ];
               unset( $match[ 0 ] );
               /* Ok, I realize this looks funky . . . */
               $matchArray[ $key ] = "_M_a_t_c_h_" . $matchKey . "_h_c_t_a_M_"; /* just doing this to get a unique list */
               $matchKey++;
               $syntax = ereg_replace( $key, $matchArray[ $key ], $syntax );
            }
            else {
               $looping = false;
            }
         }
          
         /* perform the hilight */
         foreach ( $matchArray as $key => $value ) {     
            $syntax = ereg_replace( $value, "<font class=hilight>" . $key . "</font>", $syntax );
         }
      }
      
      return $syntax;
   }
} 

/**********************************************************/
/* PostResultPageNavigation - creates navigation links    */
/* (next page, previous page, main page) for posts        */
/**********************************************************/
class PostResultPageNavigation extends PhooResultPageNavigation {
   var $name = "PostResultPageNavigation";
   var $nextPageTemplate = "=|Post.navigation.link.next_page|=";
   var $previousPageTemplate = "=|Post.navigation.link.previous_page|=";
   var $mainPageTemplate = "=|Post.navigation.link.main_page|=";
   var $countVariableName = "post_count";
   var $divider = " | ";
   var $preNavigationTemplate = "=|Post.navigation.pre|=";
   var $postNavigationTemplate = "=|Post.navigation.post|=";
   var $securityRole = ROLE_EVERYONE;

   function setup( ) {
      $this->resultPageSize = $this->container->resultPageSize;
      $this->countCommand = $this->container->countCommand;
      parent::setup( );
   }
   
   /* since posts display in reverse order, need to reverse the order of the links. This also
      puts a link to the main page in */
   function renderSnapOn( ) {
      $prevResultPage = $this->getVar( "previous_result_page", "PRIVATE" );
      $nextResultPage = $this->getVar( "next_result_page", "PRIVATE" );
      
      if ( $prevResultPage > "" && $nextResultPage > "" ) {
         /* switch their order, put in a main page link */
         $this->template = $this->nextPageTemplate . $this->divider . $this->mainPageTemplate .
                           $this->divider . $this->previousPageTemplate;
      }
      elseif ( $prevResultPage > "" && $nextResultPage == "" ) {
         /* just add the main page template */
         $this->template = $this->mainPageTemplate .
                           $this->divider . $this->previousPageTemplate;
      }

      $this->template = $this->preNavigationTemplate . $this->template . $this->postNavigationTemplate;

      return parent::renderSnapOn( );
   }
}

/**********************************************************/
/* PostDateDisplay - displays the post date, without      */
/* repeats                                                */
/**********************************************************/
class PostDateDisplay extends PhooDisplay {
   var $name = "PostDateDisplay";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Post.date|=";
   var $noRepeat = true; /* don't want to repeat the date for every post. just when it changes */ 
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Displays the date for a post" );
      $this->addExpects( "post_date", "CONTAINER" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-28" );
   }

   function setup( ) {
      parent::setup( );
      /* set date format */
      $dateFormat = $this->session->getSetting( "post_date_format" );

      /* get the local time for the post date */
      /* Note: I know this is kludgey, having to convert back and forth here. Will be fixed in a later version when
         I clean up this date display code */
         
      $gmtTime = $this->session->timeZone->getGMTTime( BASE_TIME_FORMAT, $this->getVar( "post_date", "PRIVATE" ) );
         
      $postDate = $this->session->getLocalDate( BASE_TIME_FORMAT, $gmtTime );
      $this->setVar( "post_date", $postDate[ 0 ], "PRIVATE" );
      
      /* note: since PHP 4 doesn't recognize the time zone format indicator, and since we're
         using a Phoo based, rather than PHP native, time zone, modify the format here */
      if ( strpos( $dateFormat, "e" ) ) { /* check for time zone indicator */
         $dateFormat = str_replace( "e", "", $dateFormat );
         $this->setVar( "post_time_zone_abbrev", $postDate[ 1 ], "PRIVATE" );
      }
      else {
         $this->setVar( "post_time_zone_abbrev", "", "PRIVATE" );
      }         
      
      $this->field = array( "post_date" => array( "format" => $dateFormat
                           ,"type" => "date" ) );
   }      
}                                           

class PostLastPublishedDate extends PhooQuery {
   var $name = "PostLastPublishedDate";
   var $securityRole = ROLE_EVERYONE;
   var $template = "_@post_date@_";
   var $command = "=|Post.lastpublisheddate.get|=";
   var $field = array( "post_date" => array( "format" => "D, d M Y H:i:s" 
                                            ,"type" => "date" ) );

   function getResultRow( ) {
      /* need to determine which is newer - modify date or create date */
      $row = parent::getResultRow( );

      if ( $row != null ) {
      
         if ( $row[ "max_modify_date" ] > $row[ "max_create_date" ] ) {
            $row[ "post_date" ] = $row[ "max_modify_date" ];
         }
         else {
            $row[ "post_date" ] = $row[ "max_create_date" ];
         }
      }      
      return $row;
   }
}                                           

class PostAuthorEmail extends PhooQuery {
   var $name = "PostAuthorEmail";
   var $securityRole = ROLE_EVERYONE;
   var $template = "_@post_author_email@_";
   var $command = "=|Post.authoremail.get|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-08-12" );
      $this->setPurpose( "returns the email of the post author" );
      $this->addExpects( "post_id", "POST" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }
}                                           

class PostTitle extends PhooQuery {
   var $name = "PostTitle";
   var $securityRole = ROLE_EVERYONE;
   var $template = "_@post_title@_";
   var $command = "=|Post.title.get|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-08-18" );
      $this->setPurpose( "returns the title of the post" );
      $this->addExpects( "post_id", "POST" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }
}                                           

class PostMonthArchiveList extends PhooQuery {
   var $name = "PostMonthArchiveList";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Post.yearmonths.get|=";
   var $template = "=|Post.montharchive|=";
   var $templateHeader = "=|Post.montharchive.header|=";
   var $templateFooter = "=|Post.montharchive.footer|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Displays links to the monthly archives" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-28" );
   }
} 

class EditPostLink extends PhooDisplay {
   var $name = "EditPostLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|Post.link.edit|=";
   var $invisibleOnSecurityErrors = true;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Displays a linke to edit a post" );
      $this->addExpects( "post_id", "CONTAINER" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-28" );
   }
} 

class DeletePostLink extends PhooDisplay {
   var $name = "DeletePostLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|Post.link.delete|=";
   var $invisibleOnSecurityErrors = true;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-11" );
      $this->setPurpose( "Displays a linke to delete a post" );
      $this->addExpects( "post_id", "CONTAINER" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-28" );
   }
} 

class TrackbackSendLink extends PhooDisplay {
   var $name = "TrackbackSendLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|Trackback.link.send|=";
   var $invisibleOnSecurityErrors = true;

   function setup( ) {
      $this->setVar( "post_id", $this->container->getVar( "post_id" ), "PRIVATE" );
      parent::setup( );
   }
} 

class PostDeleter extends PhooProcess {
   var $name = "PostDeleter";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot delete a post. you do not have security privileges";
   var $command = "=|Post.delete|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-19" );
      $this->setPurpose( "Processes the deletion of posts" );
      $this->addExpects( "post_id", "POST" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }
}

/*************************************************************/
/*** PostForm - controls the creating and editing of posts ***/
/*************************************************************/
class PostForm extends PhooForm {
   var $name = "PostForm";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot display post form. You do not have posting privileges";
   var $command = ""; /* will be set based on the action */
   var $trigger = array( "btnPublish" => array( "process" => "insert" )
                        ,"btnEdit" => array( "process" => "update" ) );
                        
   var $field = array( "post_title" => array( "required" => "Y"
                                             ,"formalName" => "Post Title" )
                      ,"post_content" => array( "required" => "Y"
                                               ,"formalName" => "Content" )
                     );
   var $loadCommand = "=|Post.admin.get|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.2" );
      $this->setCreateDate( "2005-04-19" );
      $this->setPurpose( "Form for the creation and editing of posts" );
      $this->addExpects( "post_id", "CONTAINER" ); /* note: only really needed in the case of an update */
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
      $this->addModificationHistory( "modified for use of attachments (removed specific calls to conversion and balancing functions", "2006-01-22" );
   }

   /* set templates and expects based on the type of action to be taken (insert vs update) */  
   function setup( ) {
      parent::setup( );
      
      switch ( $this->action ) {
         case "update" :
            $this->command = "=|Post.update|=";
            $this->template = "=|Post.form.update|=";
            break;

         default:
         case "insert" :
            $this->command = "=|Post.insert|=";
            $this->template = "=|Post.form.insert|=";
            
            /* if this is the initial display of the post form, run the script to initialize the 
               categories */
            $btnPublish = $this->getVar( "btnPublish" );
            if ( !$btnPublish ) {
               $loadScript .= "buildDestinations( 'post_category_text_list', 'post_category_id_list' );";
               $this->setVar( "post_category_load_script", $loadScript, "POST" );
            }

            break;
            
      }

      /* look for checkbox values */
      $commentsAllowedFlag = $this->getVar( "comments_allowed_flag" );
      $trackbacksAllowedFlag = $this->getVar( "trackbacks_allowed_flag" );
      

      if ( $commentsAllowedFlag == "on" ) {
         $this->setVar( "comments_allowed_flag", "Y", "PRIVATE" );
      }
      else {
         $this->setVar( "comments_allowed_flag", "N", "PRIVATE" );
      }
      
      if ( $trackbacksAllowedFlag == "on" ) {
         $this->setVar( "trackbacks_allowed_flag", "Y", "PRIVATE" );
      }
      else {
         $this->setVar( "trackbacks_allowed_flag", "N", "PRIVATE" );
      }
   }  

   /* load the form, including loading all post categories */
   function loadForm( ) {
      parent::loadForm( );

      if ( $this->isError ) {
         return;
      }

      $rows = $this->getRowsFromQueryCommand( "=|Post.meta.category.list|=" );
      if ( $this->isError( ) ) {
         return;
      }
      
      /* this builds a javascript load script for the categories */
      $loadScript = "";
      if ( is_array( $rows ) ) {
         foreach ( $rows as $row ) {
            /* note: apostrophes will prevent this from barfing. So convert to an html entity */
            $loadScript .= "addToList( '" . str_replace( "'", "&rsquo;", $row[ "category_description" ] ) . "', " .
                            "'" . $row[ "category_id" ] . "', 'post_category_text_list', 'post_category_id_list' );";
         }
      }
      if ( $loadScript == "" ) { /* empty */
         /* just create a "buildDestinations" call, which will create the default empty list */
         $loadScript .= "buildDestinations( 'post_category_text_list', 'post_category_id_list' );";
      }

      $this->setVar( "post_category_load_script", $loadScript, "POST" );
   }
   
   function executeProcessCommand( ) {
      parent::executeProcessCommand( );
      if ( !$this->isError( ) ) { /* no errors, save the categories too */
         /* check for an inserted id first, if this is an insert */
         if ( $this->insertedId > 0 ) {
            $this->setVar( "post_id", $this->insertedId, "PRIVATE" );
         }
         /* get list of categories */
         $postCategoryIdList = $this->getVar( "post_category_id_list", "POST" );
         
         if ( $postCategoryIdList > "" ) { 
            $postCategoryIdList = split( ",", $postCategoryIdList );
         }
         else {
            $postCategoryIdList = array( ); /* empty */
         }
            
         /* get list of categories for this post ( if any ) from the meta table */
         $rows = $this->getRowsFromQueryCommand( "=|Post.meta.category.list|=" );
         if ( $this->isError( ) ) {
            return;
         }
         
         $existingCat = array( );
         if ( is_array( $rows ) ) {
            foreach ( $rows as $row ) {
               array_push( $existingCat, $row[ "category_id" ] );   
            }
         }
            
         /* check for any of the new ones that don't currently exist in meta */
         $insert = array( );
         foreach( $postCategoryIdList as $newCat ) {
            if ( !in_array( $newCat, $existingCat ) ) {
               array_push( $insert, $newCat );
            }
         }
         
         /* check for any of the old ones that aren't in the new list */
         $delete = array( );
         foreach( $existingCat as $oldCat ) {
            if ( !in_array( $oldCat, $postCategoryIdList ) ) {
               array_push( $delete, $oldCat );
            }
         }

         /* delete any removed cats from meta */
         if ( count( $delete ) > 0 ) {
            $delCatList = implode( ",", $delete );
            $this->setVar( "delete_category_id_list", $delCatList, "PRIVATE" );
            $this->runModificationCommand( "=|Post.meta.category.list.delete|=" );
            if ( $this->isError( ) ) {
               return;
            }         
         }
         
         /* insert any new cats to meta */
         foreach( $insert as $insCat ) {
            $this->setVar( "insert_category_id", $insCat, "PRIVATE" );
            $this->runModificationCommand( "=|Post.meta.category.insert|=" );
            if ( $this->isError( ) ) {
               return;
            }
         }         
      }
   }

   function redirect( ) {

      /* set the success page */
      $this->setSuccessPage( $this->getVar( "PHP_SELF", "SERVER" ) . "?page=postAdmin" );
      
      if ( $this->action == "insert" ) {
         $this->successPage .= "&post_id=" . $this->insertedId;
      }
      else { /* editing */
         $this->successPage .= "&post_id=" . $this->getVar( "post_id", "POST" );
      }
         
      /* only ping on public posts */
      if ( $this->getVar( "post_type_id", "POST" ) == POST_PUBLIC ) {

         $pinger =& $this->session->snapOnFactory( "PingerLoop", $this );
         if ( !$this->isError( ) ) {
            $pinger->process( );
         }
      }

      /* set a success message */
      if ( !$this->isError( ) ) {
         $this->session->setSessionMessage( "Post has been saved", MSG_INFO );
      }
      
      parent::redirect( );
   }
} 

/* this class is used in the Post form */
class PostCategoryDropDown extends PhooDropDown {
   var $package = "post";
   var $name = "PostCategoryDropDown";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Post.category.list.get|=";
   var $templateHeader = "=|Post.category.dropdown.header|=";
   var $template = "=|Post.category.dropdown.item|=";
   var $templateFooter = "=|Post.category.dropdown.footer|=";
   var $dropDownKeyName = "post_category_id";
}

/* This class is used on the front page for selecting posts for a given category */
class PostCategorySelectDropDown extends PhooDropDown {
   var $package = "post";
   var $name = "PostCategorySelectDropDown";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Post.category.list.get|=";
   var $templateHeader = "=|Post.category.select.dropdown.header|=";
   var $template = "=|Post.category.dropdown.item|=";
   var $templateFooter = "=|Post.category.select.dropdown.footer|=";
   var $dropDownKeyName = "post_category_id";
}

class PostTypeDropDown extends PhooDropDown {
   var $package = "post";
   var $name = "PostTypeDropDown";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Post.type.list.get|=";
   var $templateHeader = "=|Post.type.dropdown.header|=";
   var $template = "=|Post.type.dropdown.item|=";
   var $templateFooter = "=|Post.type.dropdown.footer|=";
   var $dropDownKeyName = "post_type_id";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-19" );
      $this->setPurpose( "Displays a drop-down containing post types" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
      $this->addModificationHistory( "made Public the default", "2006-03-16" );
   }


   function setup( ) {
      $postTypeId = $this->getVar( "post_type_id" );
      if ( $postTypeId == "" ) { /* default to public */
         $this->setVar( "post_type_id", POST_PUBLIC, "PRIVATE" );
      }

      parent::setup( );
   }
}

class PostCheckBoxCommentsAllowed extends PhooDisplay {
   var $securityRole = ROLE_POSTER;
   var $template = "=|Post.checkbox.comments_allowed|=";

   function setup( ) {
      $this->addExpects( "comments_allowed_flag", "CONTAINER" );
      parent::setup( );
   }

   function doAction( ) {
      $commentsAllowedFlag = $this->getVar( "comments_allowed_flag" );
      if ( $commentsAllowedFlag  == "Y" ) {
         $this->setVar( "comments_allowed_checked", "CHECKED", "PRIVATE" );
      }
      else {
         if ( $commentsAllowedFlag != "N" ) {
            /* use the default setting */
            $commentsAllowedDefault = $this->session->getSetting( "comments_allowed_default" );
            if ( $commentsAllowedDefault == "Y" ) {
               $this->setVar( "comments_allowed_checked", "CHECKED", "PRIVATE" );
            }
         }
      }
   }
}
  
class PostCheckBoxTrackbacksAllowed extends PhooDisplay {
   var $securityRole = ROLE_POSTER;
   var $template = "=|Post.checkbox.trackbacks_allowed|=";

   function setup( ) {
      $this->addExpects( "trackbacks_allowed_flag", "CONTAINER" );
      parent::setup( );
   }

   function doAction( ) {
      $trackbacksAllowedFlag = $this->getVar( "trackbacks_allowed_flag" );
      if ( $trackbacksAllowedFlag == "Y" ) {
         $this->setVar( "trackbacks_allowed_checked", "CHECKED", "PRIVATE" );
      }
      else {
         if ( $trackbacksAllowedFlag != "N" ) {
            /* use the default setting */
            $trackbacksAllowedDefault = $this->session->getSetting( "trackbacks_allowed_default" );
            if ( $trackbacksAllowedDefault == "Y" ) {
               $this->setVar( "trackbacks_allowed_checked", "CHECKED", "PRIVATE" );
            }
         }
      }
   }
} 

class PostRecentList extends PhooDropDown {
   var $package = "post";
   var $name = "PostRecentList";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Post.recent.list.get|=";
   var $templateHeader = "=|Post.recent.list.header|=";
   var $template = "=|Post.recent.list|=";
   var $disableIfNoRows = true;
}

class PostDraftsList extends PhooQuery {
   var $package = "post";
   var $name = "PostDraftsList";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Post.draft.list.get|=";
   var $templateHeader = "=|Post.draft.list.header|=";
   var $template = "=|Post.draft.list|=";
   var $disableIfNoRows = true;
}

/********************************************/
/* SnapOns for Post Category administration */
/********************************************/
class ListPostCategoryDisplay extends PhooQuery {
   var $package = "post";
   var $name = "ListPostCategoryDisplay";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Post.category.list.get|=";
   var $templateHeader = "=|Post.category.list.header|=";
   var $template = "=|Post.category.list|=";
   var $templateFooter = "=|Post.category.list.footer|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-12-21" );
      $this->setPurpose( "Displays the post category administration list" );
   }
}

class ListPostCategoryForm extends PhooForm {
   var $package = "post";
   var $name = "ListPostCategoryForm";
   var $securityRole = ROLE_POSTER;
   var $loadCommand = "=|Post.category.get|=";
   var $updateForm = "=|Post.category.form.update|=";
   var $updateCommand = "=|Post.category.update|=";
   var $insertForm = "=|Post.category.form.insert|=";
   var $insertCommand = "=|Post.category.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   var $insertConstraint = "=|Post.category.duplicate.constraint|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-12-21" );
      $this->setPurpose( "Displays and processes the post category administration form" );
   }

   function setup( ) {
      parent::setup( );

      /* set the list type here */
      $this->setVar( "list_type", "post.category", "POST" ); /* set in post vars to make it accessible to 
                                                            the new_id command */

      /* set the success page, in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=settingsAdmin&subpage=post.category" );
   }
}

/**********************************************************/
/* Controls the deletion of post categories. Checks       */
/* inuse constraints                                      */
/**********************************************************/
class PostCategoryDeleter extends PhooProcess {
   var $package = "post";
   var $name = "PostCategoryDeleter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|Post.category.delete|=";
   var $securityErrorMessage = "Cannot delete a post category. you do not have security privileges";
   var $constraint = array( "=|Post.category.inuse.constraint|=" );

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-02" );
      $this->addExpects( "list_type", "CONTAINER" );
      $this->addExpects( "post_category_id", "CONTAINER" );
      $this->setPurpose( "Controls deletion of post categories" );
   }
}


/**************************************************************/
/* Displays indicator of private post if post type is private */
/**************************************************************/
class PostTypeIndicator extends PhooDisplay {
   var $name = "PostTypeIndicator";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Post.private.indicator|=";

   function setup( ) {
      $postTypeId = $this->container->getVar( "post_type_id" );
      if ( $postTypeId != POST_PRIVATE ) { 
         $this->disabled = true;
      }
   }
}
?>
