<?php
/* 
   Name: rss.classes.php
   Author: Bill Roberts
   Created: 4/11/2005
   Purpose: SnapOn classes related to RSS feeds
   $Revision: 1.4 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
  
class RSS20Feed extends PhooQuery {
   var $name = "RSS20Feed";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Post.list.get|=";
   var $templateHeader = "=|RSS20.header|=";
   var $template = "=|RSS20.item|=";
   var $templateFooter = "=|RSS20.footer|=";
   var $field = array( "post_date" => array( "format" => "D, d M Y H:i:s"
                                            ,"type" => "date" )
                      ,"comment_date" => array( "format" => "D, d M Y H:i:s" 
                                               ,"type" => "date" ) );
   var $isComment = false;                                               

   function setup( ) {
      parent:: setup( );

      $commentsFeed = $this->getVar( "comments", "GET" );
      $postId = $this->getVar( "post_id", "GET" );
      if ( $commentsFeed ) {
         if ( $postId > "" ) {
            $this->command = "=|Comment.get.bypost|=";
         }
         else {
            $this->command = "=|Comment.recent.get|=";
         }
         $this->template = "=|RSS20.comment.item|=";
         $this->isComment = true;
      }
      else { /* default is posts */
         /* set the result page size */
         $this->resultPageSize = $this->session->getSetting( "posts_to_display" );
      }
      /* establish RSS vars for the header, stripping any html tags found */
      $this->setVar( "rss_blog_name", $this->xmlClean( $this->session->getSetting( "blog_name" ) ) );
      $this->setVar( "rss_tag_line", $this->xmlClean( $this->session->getSetting( "tag_line" ) ) );

   }

   /* utility function, cleans up XML. If $cData is set to true, level of cleaning is less, as this will
      be data within CDATA tags */
   function xmlClean( $aString, $cData = false ) {
      $newString = "";

      /* convert any characters above 128 to xml entities */
   	for ( $i = 0; $i < strlen( $aString ); $i++ ) {
   		$c = substr( $aString, $i, 1 );
	   	$ord = ord($c);
   		if ( $ord >= 128 ) { /* note: 7/27/2007, BR. This was "12800", which doesn't seem right. Set to 128 */
	   		$c = "&#" . $ord . ";";
         }
         
         $newString .= $c;
      }
      
      /* replace Word-style quotes with regular quotes */
      $newString = str_replace( "&#226;&#128;&#156;", "\"", $newString );
      $newString = str_replace( "&#226;&#128;&#157;", "\"", $newString );

      /* other fixes (single quotes, dashes) */
      $newString = str_replace( "&#226;&#128;&#153;", "'", $newString );
      $newString = str_replace( "&#226;&#128;&#147;", "-", $newString );
      $newString = str_replace( "&#226;&#128;&#166;", "...", $newString );

      if ( !$cData ) { /* further cleaning */
         /* decode "all" entities */
         $newString = html_entity_decode( $newString );

         /* further decoding, for ones missed by html_entity_decode */
         $newString = str_replace( "&lsquo;", "'", $newString );
         $newString = str_replace( "&rsquo;", "'", $newString );
         $newString = str_replace( "&ldquo;", '"', $newString );
         $newString = str_replace( "&rdquo;", '"', $newString );
         $newString = str_replace( "&trade;", '(tm)', $newString );

         /* replace any remaining ones with spaces */
         $newString = ereg_replace( "&[a-zA-Z]+;", ' ', $newString );
         
         /* replace any remaining ampersands with an entity */
         $newString = ereg_replace( "&", '&amp;', $newString );

         /* convert tags to entities */
         $newString = str_replace( "<", "&lt;", $newString );
         $newString = str_replace( ">", "&gt;", $newString );
      }

      /* wordwrap it, just to make it more readable */
      return wordwrap( $newString, 75 );
   }
   
   function getResultRow( ) {
      /* call the ancestor */
      $row = parent::getResultRow( );

      if ( $this->isError( ) || !is_array( $row ) ) {
         return;
      }

      if ( $this->isComment ) { 
         /* process for comment */
         /* for description, limit to the first space after the first 100 characters (or end of string) */
         if ( strlen( $row[ "comment_content" ] ) < 101 ) {
            $commentDescription = $row[ "comment_content" ];
         }
         else {
            $pos = strpos( $row[ "comment_content" ], " ", 100 );
            if ( $pos > 0 ) {
               $commentDescription = substr( $row[ "comment_content" ], 0, $pos ) . " [...]";
            }
            else {
               $commentDescription = $row[ "comment_content" ];
            }
         }
         $row[ "post_title" ] = $this->xmlClean( $row[ "post_title" ] );
         $row[ "comment_description" ] = $this->xmlClean( $commentDescription );
         $row[ "comment_content" ] = $this->xmlClean( $row[ "comment_content" ], true );
         $row[ "comment_author" ] = $this->xmlClean( $row[ "comment_author" ]  );
      }
      else {
         /* process for post */
         /* for description, limit to the first space after the first 100 characters (or end of string) */
         if ( strlen( $row[ "post_content" ] ) < 101 ) {
            $postDescription = $row[ "post_content" ];
         }
         else {
            $pos = strpos( $row[ "post_content" ], " ", 100 );
            if ( $pos > 0 ) {
               $postDescription = substr( $row[ "post_content" ], 0, $pos ) . " [...]";
            }
            else {
               $postDescription = $row[ "post_content" ];
            }
         }

         $row[ "post_title" ] = $this->xmlClean( $row[ "post_title" ] );
         $row[ "post_description" ] = $this->xmlClean( $postDescription );
         $row[ "post_content" ] = $this->xmlClean( $row[ "post_content" ], true );
         $row[ "post_author" ] = $this->xmlClean( $row[ "post_author" ]  );
         $row[ "post_category" ] = $this->xmlClean( $row[ "post_category" ] );
      }
      
      return $row;
   }

   function renderSnapOn( ) {
      /* send the appropriate header */
      header("Content-type: text/xml");
      return parent::renderSnapOn( );
   }
} 

class RSS20FeedPostCategories extends PhooQuery {
   var $name = "RSS20FeedPostCategories";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Post.meta.category.list|=";
   var $template = "=|RSS20.category.item|=";

   function setup( ) {
      parent::setup( );
      
      $postId = $this->container->getVar( "post_id" );
      $this->setVar( "post_id", $postId, "PRIVATE" );
   }
}
?>
