<?php
/* 
   Name: security.classes.php
   Author: Bill Roberts
   Created: 4/11/2005
   Purpose: SnapOn classes related to user login and security
   $Revision: 1.4 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/* note: this form doesn't actually do the login. It just
   populates post variables that will be picked up in the
   init program */
class LoginForm extends PhooForm {
   var $package = "security";
   var $name = "LoginForm";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Login.form|=";
}

/* validates the current user - reads application variables set from the cookies.
   Clears them out as soon as user is validated */
class UserValidator extends PhooQuery {
   var $package = "security";
   var $name = "UserValidator";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|User.getcurrent|="; 

   function doAction( ) {
      parent::doAction( );
      
      $this->clearVar( "user_login_id", "APPLICATION" );
      $this->clearVar( "user_password", "APPLICATION" );
   }
}

class RoleInformation extends PhooQuery {
   var $package = "security";
   var $name = "RoleInformation";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Role.get|="; 

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-10-05" );
      $this->setPurpose( "returns information for a role id" );
      $this->addExpects( "role_id", "POST" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }
}

class LoggedInAs extends PhooDisplay {
   var $package = "security";
   var $name = "LoggedInAs";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Information.currentlyloggedon|=";

   function setup( ) {
      $this->setVar( "user_known_by", $this->session->getUserKnownBy( ), "POST" );
      parent::setup( );
   }
}

class UserList extends PhooQuery {
   var $name = "UserList";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot display users. You do not have security clearance to view users";
   var $command = "=|User.list.get|=";
   var $templateHeader = "=|User.list.header.readonly|=";
   var $template = "=|User.list.readonly|=";
   var $templateFooter = "=|User.list.footer.readonly|=";
   
   /* see if current user is an adminmistrator. If so, switch templates to the maintenance templates */
   function setup( ) {
      $roles = $this->session->getUserRoles( );

      if ( in_array( ROLE_ADMINISTRATOR, $roles ) ) {
         $this->templateHeader = "=|User.list.header|=";
         $this->template = "=|User.list|=";
      }

      parent::setup( );
   }
}

class AddUserButton extends PhooForm {
   var $name = "AddUserButton";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot display user form. You do not have security clearance to add users";
   var $trigger = array( "btnAdd" => array( "process" => "insert" ) );
   var $template = "=|User.form.adduser|=";
}

class UserForm extends PhooForm {
   var $name = "UserForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot display user form. You do not have security clearance to edit users";
   var $command = ""; /* will be set based on the action */
   var $trigger = array( "btnAdd" => array( "process" => "insert" )
                        ,"btnEdit" => array( "process" => "update" ) );
   var $constraint = array( "=|User.duplicate.constraint|=" );
                                            
   var $field = array( 
                       "maint_user_first_name" => array( "required" => "Y"
                                                  ,"formalName" => "First Name" ),
                       "maint_user_last_name" => array( "required" => "Y"
                                                  ,"formalName" => "Last Name" ),
                       "maint_user_nick_name" => array( "required" => "Y"
                                                  ,"formalName" => "Name on Blog" ),
                       "maint_user_email" => array( "required" => "Y"
                                                  ,"formalName" => "Email" ),
                       "role_id" => array( "required" => "Y"
                                                  ,"formalName" => "Security Role" ),
                                                  
                                            );
                                            
   
   function setup( ) {
      parent::setup( );

      $action = $this->getVar( FORM_ACTION );
      
      switch ( $action ) {

         case "update" :
            $this->command = "=|User.update|=";
            $this->template = "=|User.form.update|=";
            $this->loadCommand = "=|User.get|=";
            break;

         default:
         case "insert" :
            $this->command = "=|User.insert|=";
            $this->template = "=|User.form.insert|=";
            /* add more required fields */
            $this->field[ "maint_user_login_id" ]= array( "required" => "Y"
                                                         ,"formalName" => "Login ID" );
            $this->field[ "maint_user_password" ] = array( "required" => "Y"
                                                          ,"formalName" => "Password" );
            break;
            
      }
   }

   function validateForm( ) {
      parent::validateForm( $aArray );

      if ( !$this->isError( ) ) {
         $pwd1 = $this->getVar( "maint_user_password", "POST" );
         $pwd2 = $this->getVar( "maint_user_password_2", "POST" );

         if ( $pwd1 != $pwd2 ) {
            $this->setError( "Passwords must match", MSG_WARNING );
            return;
         }
         /* convert the password to md5 */
         $this->setVar( "maint_user_password", md5( $pwd1 ), "POST" );
      }
   }   

   /* Developer's note - see about moving this to the setup function */
   function redirect( ) {
      $this->setSuccessPage( $this->getVar( "PHP_SELF", "SERVER" ) . "?page=userAdmin" );
     
      $action = $this->getVar( "FORM_ACTION" ); 
      if ( $action == "insert" ) {
         $this->successPage .= "&maint_user_id=" . $this->insertedId;
      }
      else { /* editing */
         $this->successPage .= "&maint_user_id=" . $this->getVar( "maint_user_id", "POST" );
      }

      $this->session->setSessionMessage( "User Information Saved", MSG_INFO );     
      parent::redirect( );
   }
} 

class SecurityRoleDropDown extends PhooDropDown {
   var $package = "security";
   var $name = "SecurityRoleDropDown";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|Role.list.get|=";
   var $templateHeader = "=|Role.dropdown.header|=";
   var $template = "=|Role.dropdown.item|=";
   var $templateFooter = "=|Role.dropdown.footer|=";
   var $dropDownKeyName = "role_id";

}

class ResetPasswordForm extends PhooForm {
   var $name = "ResetPasswordForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot display user form. You do not have security clearance to reset passwords";
   var $command = "=|User.password.reset|=";
   var $template = "=|User.password.form.resetothers|=";
   var $loadCommand = "=|User.get|=";
   var $trigger = array( "btnReset" => array( "process" => "reset_password" ) );
   var $field = array( 
                       "maint_user_password" => array( "required" => "Y"
                                                      ,"formalName" => "Password 1" ),
                       "maint_user_password_2" => array( "required" => "Y"
                                                        ,"formalName" => "Password 2" )
                     );
   function setup( ) {
      parent::setup( );
      /* set the success page for if this form works */
      $this->setSuccessPage( $this->getVar( "PHP_SELF", "SERVER" ) . "?page=userAdmin" );
   }

   function validateForm( ) {
      parent::validateForm( $aArray );

      if ( !$this->isError( ) ) {
         $pwd1 = $this->getVar( "maint_user_password", "POST" );
         $pwd2 = $this->getVar( "maint_user_password_2", "POST" );

         if ( $pwd1 != $pwd2 ) {
            $this->setError( "Passwords must match", MSG_WARNING );
         }
         /* convert the password to md5 */
         $this->setVar( "maint_user_password", md5( $pwd1 ), "POST" );
      }
   }   
   
   /* Developer's Note: I don't much like this */
   function doAction( ) {
      /* since the load command is used differently in this form than most other forms (where it is
         only processed in the case of an update form that hasn't been processed yet, whereas here
         we want it processed every time) I am inserting a call to the load command here */
      $this->session->connection->executeSelectSql( $this->setSqlParameters( $this->expandOverlays( $this->loadCommand ) ) );

      if ( $this->isError( ) ) {
         return;  
      }
      $row = $this->getResultRow( );
      /* load the row in the post variables for the later call to render */
      foreach ( $row as $key => $value ) {
         $this->setVar( $key, $value, "POST" );
      }

      parent::doAction( $aArray );
      
   }

   function redirect( ) {
      $this->session->setSessionMessage( "Password Reset", MSG_INFO );
      parent::redirect( );
   }
} 

class UserDeleter extends PhooProcess {
   var $name = "UserDeleter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete a user. you do not have security privileges";
   var $command = "=|User.delete|=";

   function redirect( ) {
      $this->session->setSessionMessage( "User Deleted", MSG_INFO );
      parent::redirect( );
   }
}

class UserProfileForm extends PhooForm {
   var $name = "UserProfileForm";
   var $securityRole = ROLE_EVERYONE;
   var $securityErrorMessage = "Cannot display profile form. You do not have security clearance to edit your profile";
   var $command = "=|User.profile.update|=";
   var $loadCommand = "=|User.profile.get|=";
   var $template = "=|User.profile.form.update|=";
   var $trigger = array( "btnEdit" => array( "process" => "update" )  
                        ,"btnChangePassword" => array( "process" => "change_password" ) );
   var $action = "update"; /* this is always an update form */                        
                                            
   var $field = array( 
                       "maint_user_first_name" => array( "required" => "Y"
                                                  ,"formalName" => "First Name" ),
                       "maint_user_last_name" => array( "required" => "Y"
                                                  ,"formalName" => "Last Name" ),
                       "maint_user_nick_name" => array( "required" => "Y"
                                                  ,"formalName" => "Name on Blog" ),
                       "maint_user_email" => array( "required" => "Y"
                                                  ,"formalName" => "Email" ) 
                     );
                                            
   function setup( ) {
      parent::setup( );
      /* set the success page for if this form works */
      $this->setSuccessPage( $this->getVar( "PHP_SELF", "SERVER" ) . "?page=profileAdmin" );
   } 

   function triggerProcess( $aProcess ) {
      /* trap for the change_password process */
      switch ( $aProcess ) {
         case "change_password" :
            $this->successPage = $this->getVar( "PHP_SELF", "SERVER" ) . "?page=profilePasswordAdmin";
            parent::redirect( );
            break;
            
         default:
            parent::triggerProcess( $aProcess, $aArray );

            if ( !$this->isError( ) ) {
               $this->session->setSessionMessage( "User Information Saved", MSG_INFO );     
            }
            break;
      }
   }   
} 

class UserProfilePasswordForm extends PhooForm {
   var $name = "UserProfilePasswordForm";
   var $securityRole = ROLE_EVERYONE;
   var $securityErrorMessage = "Cannot display user form. You do not have security clearance to change your password";
   var $command = "=|User.profile.password.update|=";
   var $template = "=|User.password.form.resetprofile|=";
   var $loadCommand = "=|User.profile.password.get|=";
   var $trigger = array( "btnChangePassword" => array( "process" => "change_password" ) );
   var $field = array( 
                       "maint_user_password" => array( "required" => "Y"
                                                      ,"formalName" => "Password 1" ),
                       "maint_user_password_2" => array( "required" => "Y"
                                                        ,"formalName" => "Password 2" )
                     );
   
                        
   function setup( ) {
      parent::setup( );
      /* set the success page for if this form works */
      $this->successPage = $this->getVar( "PHP_SELF", "SERVER" ) . "?page=profileAdmin";
   } 

   
   function validateForm( ) {
      parent::validateForm( $aArray );

      if ( !$this->isError( ) ) {
         $pwdcdb = $this->getVar( "maint_user_current_password_db", "POST" );
         $pwdc = $this->getVar( "maint_user_current_password", "POST" );
         $pwd1 = $this->getVar( "maint_user_password", "POST" );
         $pwd2 = $this->getVar( "maint_user_password_2", "POST" );

         if ( $pwdcdb != md5($pwdc) ) {
            $this->setError( "Current password is incorrect", MSG_WARNING );
            return;
         }
         
         if ( $pwd1 != $pwd2 ) {
            $this->setError( "Passwords must match", MSG_WARNING );
            return;
         }
         /* convert the password to md5 */
         $this->setVar( "maint_user_password", md5( $pwd1 ), "POST" );
         /* and this one too, since the sql checks it */
         $this->setVar( "maint_user_current_password", md5( $pwdc ), "POST" );
      }
   }   

   /* Developer's Note: I don't like this */
   function doAction( ) {
      /* since the load command is used differently in this form than most other forms (where it is
         only processed in the case of an update form that hasn't been processed yet, whereas here
         we want it processed every time) I am inserting a call to the load command here */
      $this->session->connection->executeSelectSql( $this->setSqlParameters( $this->expandOverlays( $this->loadCommand ) ) );
      
      if ( $this->isError( ) ) {
         return;  
      }

      $row = $this->getResultRow( );
      /* load the row in the post variables for the later call to render */
      foreach ( $row as $key => $value ) {
         $this->setVar( $key, $value, "POST" );
      }

      parent::doAction( $aArray );
      
   }

   function redirect( ) {
      /* now that the password has been changed successfully (we can tell because a redirect
         has been called), log the user in with the new password */
      $loginId = $this->getVar( "maint_user_login_id", "POST" );
      $password = $this->getVar( "maint_user_password", "POST" );
      
      $this->session->login( $loginId, $password ); 

      $this->session->setSessionMessage( "Password Changed", MSG_INFO );     
     
      parent::redirect( );
   }
} 

/**************************************************************/
/* Event Handler                                              */
/* Checks for a lost_password GET parameter. If found,        */
/* replaces the content section of the Admin page with a form */
/* for resetting a lost password                              */
/**************************************************************/
class SecurityLostPasswordCheck extends PreProcessEventHandler {
   var $name = "SecurityLostPasswordCheck";
   var $securityRole = ROLE_EVERYONE;

   function doAction( ) {
      parent::doAction( );

      /* check for a Get variable called BloogrollPosts */
      $lostPasswordFlag = $this->getVar( "lost_password", "GET" );
      if ( $lostPasswordFlag == 1 ) {
         /* replace the template for the container */
         $this->container->template = "_{SecurityLostPasswordRequestForm}_";
      }
   }
}

class SecurityLostPasswordRequestForm extends PhooForm {
   var $name = "SecurityLostPasswordRequestForm";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Security.lost_password.email_userid.form|=";
   var $trigger = array( "btnSubmit" => array( "process" => "submit" ) );
   var $email;
   var $userId;
   var $successPage = ""; /* clearing the success page will stop the default redirect upon success */
   
   function validateForm( ) {
      /* grab information, if there */
      $loginId = $this->getVar( "lost_password_login_id", "POST" );
      $email = $this->getVar( "lost_password_email", "POST" );
      /* If loginid is filled in, compare it to the user table */
      if ( $loginId > "" ) {
         $rows = $this->getRowsFromQueryCommand( "=|LostPassword.getUserByLoginId|=" );
         if ( count( $rows ) == 1 ) { /* one row returned */
            /* see if the email matches */
            if ( $email > "" ) {
               if ( $rows[ 0 ][ "user_email" ] != $email ) {
                  $this->setError( "The email entered doesn't match the email on file. Please correct", MSG_WARNING );
               }
            }
         }
         else {
            $this->setError( "Login id not found. Please correct or try entering your email address instead", MSG_WARNING );
         }
      }
      else { /* no login id - see if email address was entered */
         if ( $email > "" ) {
            $rows = $this->getRowsFromQueryCommand( "=|LostPassword.getUserByEmail|=" );
            if ( count( $rows ) > 1 ) { /* more than one row returned */
               $this->setError( "The email entered matches more than one login id. Please enter your login id also", 
                                MSG_WARNING );
            }
            elseif ( count( $rows ) == 0 ) {
               $this->setError( "Could not find a match for the email entered. Please correct", 
                                MSG_WARNING );
            }
         }
      }

      /* if no error, set the email address and user id */
      if ( !$this->isError( ) ) {
         $this->email = $rows[ 0 ][ "user_email" ];
         $this->userId = $rows[ 0 ][ "user_id" ];
      }
   }

   function doAction( ) {
      parent::doAction( );
      
      $process = $this->getTriggerProcess( );
      
      if ( isset( $process ) && $this->email > "" ) { /* process triggered and email set */
         /* create a blooQueue key and email it to the user */
         $bq = $this->session->snapOnFactory( "BlooQueueKey", $this );
         $key = $bq->createKey( "LostPasswordRestore", 4*60*60, $this->userId ); 

         /* create the email object and send */
         $s =& $this->session->snapOnFactory( "MailSender", $this );
         /* create a from email of the DONOTREPLY variety */
         $from = "DONOTREPLY@" . str_replace( "http://", "", $this->session->getSetting( "site_url" ) );
         $blogName = $this->session->getSetting( "blog_name" ); 
         $subject =  $blogName . ": Lost password restore";
         $message = "To restore your password on $blogName, please click on the following link:\n\n";
         $uri = $this->session->getSetting( "site_url" ) . "/" . 
                     $this->session->getSetting( "operations_page" ) . "?queue_key=$key";
         $message .= "$uri\n\n";
         $message .= "If the link above is not clickable, please copy and paste it into your browser.";
         $s->setMail( $this->email, $subject, $message, $from );
         $s->process( );

         /* set the template to display a success message */
         $this->template = "=|Security.lost_password.request.message|=";
      }
   }
}

/**************************************************************/
/* SecurityLostPasswordProcessor                              */
/* Checks for Bloo Queue LostPasswordRestore command, and     */
/* displays the Lost Password reset form if found             */
/**************************************************************/
class SecurityLostPasswordProcessor extends PreProcessEventHandler {
   var $name = "SecurityLostPasswordProcessor";
   var $securityRole = ROLE_EVERYONE;

   function doAction( ) {
      parent::doAction( );
      $this->command = $this->container->command;

      if ( $this->command == "LostPasswordRestore" ) {
         $this->metaData = $this->container->metaData;
         /* since it is expected that this event is hooked to the event in the BlooQueue package that
            is itself hooked to the admin page, have to go back to levels of containers */
         $this->container->container->template = "_{SecurityLostPasswordResetForm}_";
         $this->container->container->setVar( "maint_user_id", $this->metaData, "PRIVATE" );
      }
   }
}

class SecurityLostPasswordResetForm extends ResetPasswordForm {
   var $name = "SecurityLostPasswordResetForm";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|User.password.reset|=";
   var $loadCommand = "";
   var $template = "=|Security.lost_password.reset.form|=";
   var $trigger = array( "btnReset" => array( "process" => "reset_password" ) );

   function setup( ) {
      /* re-validate the queue key passed. This is just an extra security check */
      $q = $this->session->SnapOnFactory( "BlooQueueKey", $this );
      if ( !$q->keyValid( $this->getVar( "queue_key" ) ) ) {
         $this->disabled = true;
         return;
      } 
      parent::setup( );
      $this->setVar( "maint_user_id", $this->container->getVar( "maint_user_id" ), "PRIVATE" );

      /* and, in case this works */
      $this->successPage = $this->session->getSetting( "site_url" ) . "/" . $this->session->getSetting( "operations_page" );
   }

   /* overrides the extended functions in the ancestors doAction */
   function doAction( ) {
      PhooForm::doAction( );
   }

   function triggerProcess( $aProcess ) {
      parent::triggerProcess( $aProcess );
      if ( !$this->isError( ) ) {
         /* deactivate the queue id */
         $q = $this->session->SnapOnFactory( "BlooQueueKey", $this );
         $q->deactivateKey( $this->getVar( "queue_key" ) );
         
         $this->session->setSessionMessage( "Your password has been reset", MSG_INFO );
      }
   }

}

?>
