<?php
/* 
   Name: settings.classes.php
   Author: Bill Roberts
   Created: 1/18/2006
   Purpose: SnapOn classes related to administering site settings
   $Revision: 1.2 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
   
/*************************************************************/
/* AdminSettingsPageSection                                  */
/* This controls the section of the administration page      */
/* for administering blog settings                           */
/* It is assumed that these settings are stored in the list  */
/*************************************************************/
class AdminSettingsPageSection extends PhooDisplay {
   var $package = "bloo_pages";
   var $securityRole = ROLE_EVERYONE; /* Security for individual settings are set at the lower levels */
   var $name = "AdminSettingsPageSection";
   var $template = "_{SettingsMenu}_"; /* this element is on all list pages */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-04-13" );
      $this->setPurpose( "Page for settings administration" );
   }
   
   function setup( ) {
      parent::setup( );

      /* determine the subpage type, if any */
      $subPage = $this->getVar( "subpage" );

      if ( $subPage == "" ) {
         $subPage = "bloogroll"; /* default */
         $this->setVar( "subpage", $subPage, "GET" );
      }

      /* determine the action to take */
      $btnInsert = $this->getVar( "btnInsert", "POST" );
      
      if ( $btnInsert > "" ) {
         $action = "insert";
      }
      else {
         $btnUpdate = $this->getVar( "btnUpdate", "POST" );
         if ( $btnUpdate > "" ) {
            $action = "update";
         }
         else {
            $action = $this->getVar( FORM_ACTION, "GET" );
         }
      }

      /* if no action, assume an action of list (to list the setting) */
      if ( $action == "" ) {
         $action = "list";
      }

      /* set the action */
      $this->setVar( FORM_ACTION, $action, "GET" );
      
      /* assign the detail snapon based on the action. 
         append it to the template */
      switch ( $action ) {
         case "insert" :
         case "update" :
            $this->template .= "_{" . $this->getSnapOnName( $subPage, "modify" ) . "}_";               
            break;
               
         case "list" :
            $this->template .= "_{" . $this->getSnapOnName( $subPage, "display" ) . "}_";               
            break;

         case "activate" :
         case "deactivate" : /* will pass the action to the display form for processing (if it handles de/activates) */
            $this->template .= "_{" . $this->getSnapOnName( $subPage, "display" ) . "}_";               
            break;
      }
   }  

   function doAction( ) {
      /* check for a delete command. If one was passed,
         perform the delete action instead of the default
         action */
      $action = $this->getVar( FORM_ACTION, "GET" );
      $subPage = $this->getVar( "subpage", "GET" );
      
      if ( $action == "delete" ) {
         /* get the delete SnapOn */
         $d =& $this->session->snapOnFactory( $this->getSnapOnName( $subPage, "delete" ), $this );
         /* set the success page to take you back to the list administration page */
         $d->setSuccessPage( $this->getVar( "site_url" ) . "/" . $this->getVar( "operations_page" ) . 
                             "?page=settingsAdmin&subpage=" . urlencode( $this->getVar( "subpage" ) ) );
         $d->process( );
         if ( $d->isError( ) ) {
            /* load the template up with the error syntax, etc */
            $this->template .= $d->render( );
            return;
         }
      }
      else { /* otherwise, continue on */
         parent::doAction( );
      }
   }

   /* returns the SnapOn to use for a given subpage and action */
   function getSnapOnName( $aSubPage, $aAction ) {
      $snapOnName = "PhooEmpty";

      switch( $aSubPage ) {
         case "bloogroll" :
            switch( $aAction ) {
               case "display" :
                  $snapOnName = "ListBloogrollDisplay";            
                  break;
               case "modify" :
                  $snapOnName = "ListBloogrollForm";            
                  break;
               case "delete" :
                  $snapOnName = "BloogrollDeleter";            
                  break;
            }

            break;
            
         case "post.category" :
            switch( $aAction ) {
               case "display" :
                  $snapOnName = "ListPostCategoryDisplay";            
                  break;
               case "modify" :
                  $snapOnName = "ListPostCategoryForm";            
                  break;
               case "delete" :
                  $snapOnName = "PostCategoryDeleter";            
                  break;
            }
            break;
            
         case "bloogroll.category" :
            switch( $aAction ) {
               case "display" :
                  $snapOnName = "ListBloogrollCategoryDisplay";            
                  break;
               case "modify" :
                  $snapOnName = "ListBloogrollCategoryForm";            
                  break;
               case "delete" :
                  $snapOnName = "BloogrollCategoryDeleter";            
                  break;
            }

            break;
            
         case "blog.profile" :
            switch( $aAction ) {
               case "display" :
               case "modify" :
                  $snapOnName = "BlogProfileAdminForm";            
                  break;
            }
            break;
            
         case "display" :
            switch( $aAction ) {
               case "display" :
               case "modify" :
                  $snapOnName = "DisplaySettingsAdminForm";            
                  break;
            }

            break;
            
         case "discussion" :
            switch( $aAction ) {
               case "display" :
               case "modify" :
                  $snapOnName = "DiscussionSettingsAdminForm";            
                  break;
            }

            break;
            
         case "time_zone" :
            switch( $aAction ) {
               case "display" :
               case "modify" :
                  $snapOnName = "TimeZoneSettingsAdminForm";            
                  break;
            }

            break;
            
         case "pingers" :
            switch( $aAction ) {
               case "display" :
                  $snapOnName = "ListPingerDisplay";            
                  break;
               case "modify" :
                  $snapOnName = "ListPingerForm";            
                  break;
               case "delete" :
                  $snapOnName = "PingerDeleter";            
                  break;
            }
            break;

      }

      return $snapOnName;      

   }
}  

/* ancestor for the multiple-item settings pages */
/* Note: there is no need to setup a load command, since settings are already
   stored in application variables */
class MultiItemSettingsAdminForm extends PhooForm {
   var $loadCommandSpecial; /* since the underlying table will return multiple rows, need
                               to load this manually, below */
   var $trigger = array( "btnUpdate" => array( "process" => "update" ) );
   var $subPage;

   function setup( ) {
      parent::setup( );

      /* set the success page, in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=settingsAdmin&subpage=" . $this->subPage );
   }
 
   /* overridden to handle the multi-line process 
   
      Developer's note: a little too much is picked up here from the ancestor. Try to find
      a better way of doing this */
   function triggerProcess( $aProcess ) {
      /* by default, just run the command. Descendents can have multiple triggers */
      $this->validateForm( );
      if ( $this->isError( ) ) {
         return;
      }

      /* fire any pre-save events */
      $this->fireEvents( EVENT_PRESAVE );
      if ( $this->isError( ) ) {
         return;
      }
      
      $s = $this->session->SnapOnFactory( "PhooMultiProcess", $this );
      $s->securityRole = $this->securityRole;
      $s->command = $this->command;
      /* since the values being potentially changed here are already set in APPLICATION variables, which in the 
         scope precedence come before POST variables but after PRIVATE variables, move all post vars into the private
         variables */
      $p = $this->getVars ( "POST" );
      foreach ( $p as $key => $value ) {
         $s->setVar( $key, $value, "PRIVATE" );
      }      
      
      $s->process( );
     
      if ( $this->isError( ) ) {
         return;
      }

      /* fire any post-save events */
      $this->fireEvents( EVENT_POSTSAVE );
      if ( $this->isError( ) ) {
         return;
      }

      /* set a system message */
      $this->session->setSessionMessage( "Settings Saved", MSG_INFO );
      $this->redirect( );
   }
}

class BlogProfileAdminForm extends MultiItemSettingsAdminForm {
   var $name = "BlogProfileAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|BlogProfile.update|="; 
   var $template = "=|BlogProfile.form.update|=";
   var $subPage = "blog.profile";

}

/* Note: this is part of the settings package, but it appears under the
   look and feel section of the menus, because that just felt like a more
   intuitive place for it.
 */
class DisplaySettingsAdminForm extends MultiItemSettingsAdminForm {
   var $name = "DisplaySettingsAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|DisplaySettings.update|="; 
   var $template = "_{LookAndFeelMenu}_=|DisplaySettings.form.update|=";
   var $subPage = "display";

   /* override this to point to the look and feel admin */
   function setup( ) {
      parent::setup( );

      /* set the success page, in case this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=lookAndFeelAdmin&subpage=" . $this->subPage );

      /* set examples */
      $postDateFormat = $this->session->getSetting( "post_date_format" );
      if ( strpos( $postDateFormat, "e" ) ) { /* check for time zone indicator */
         $postDateFormat = str_replace( "e", "", $postDateFormat );
         $showTimeZone = true;
      }

      $postDateExample = $this->session->getLocalDate( $postDateFormat );

      $this->setVar( "post_date_example", $postDateExample[ 0 ], "PRIVATE" );      

      if ( $this->showTimeZone ) {
         $this->setVar( "post_date_time_zone_abbrev", $postTimeExample[ 1 ], "PRIVATE" );
         $showTimeZone = false;
      }
      
      $postTimeFormat = $this->session->getSetting( "post_time_format" );
      
      if ( strpos( $postTimeFormat, "e" ) ) { /* check for time zone indicator */
         $postTimeFormat = str_replace( "e", "", $postTimeFormat );
         $showTimeZone = true;
      }
      
      $postTimeExample = $this->session->getLocalDate( $postTimeFormat );

      $this->setVar( "post_time_example", $postTimeExample[ 0 ], "PRIVATE" );      

      if ( $showTimeZone ) {
         $this->setVar( "post_time_time_zone_abbrev", $postTimeExample[ 1 ], "PRIVATE" );
         $showTimeZone = false;
      }

      $commentDateFormat = $this->session->getSetting( "comment_date_format" );
      
      if ( strpos( $commentDateFormat, "e" ) ) { /* check for time zone indicator */
         $commentDateFormat = str_replace( "e", "", $commentDateFormat );
         $showTimeZone = true;
      }
      
      $commentDateExample = $this->session->getLocalDate( $commentDateFormat );

      $this->setVar( "comment_date_example", $commentDateExample[ 0 ], "PRIVATE" );      

      if ( $showTimeZone ) {
         $this->setVar( "comment_date_time_zone_abbrev", $commentDateExample[ 1 ], "PRIVATE" );
         $showTimeZone = false;
      }
   }
}

class DiscussionSettingsAdminForm extends MultiItemSettingsAdminForm {
   var $name = "DiscussionSettingsAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|DiscussionSettings.update|="; 
   var $template = "=|DiscussionSettings.form.update|=";
   var $subPage = "discussion";
}

class TimeZoneSettingsAdminForm extends MultiItemSettingsAdminForm {
   var $name = "TimeZoneSettingsAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|TimeZoneSettings.update|="; 
   var $template = "=|TimeZoneSettings.form.update|=";
   var $subPage = "time_zone";
}

class SettingsAllowCommentsYesNo extends YesNoDropDown {
   var $dropDownKeyName = "comments_allowed";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Setting.yesno.dropdown.header|=";
   var $template = "=|Setting.yesno.dropdown.item|=";
   var $templateFooter = "=|Setting.yesno.dropdown.footer|=";

}

class SettingsAllowTrackbacksYesNo extends YesNoDropDown {
   var $dropDownKeyName = "trackbacks_allowed";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Setting.yesno.dropdown.header|=";
   var $template = "=|Setting.yesno.dropdown.item|=";
   var $templateFooter = "=|Setting.yesno.dropdown.footer|=";

}

class SettingsAllowCommentsDefaultYesNo extends YesNoDropDown {
   var $dropDownKeyName = "comments_allowed_default";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Setting.yesno.dropdown.header|=";
   var $template = "=|Setting.yesno.dropdown.item|=";
   var $templateFooter = "=|Setting.yesno.dropdown.footer|=";

}

class SettingsAllowTrackbacksDefaultYesNo extends YesNoDropDown {
   var $dropDownKeyName = "trackbacks_allowed_default";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Setting.yesno.dropdown.header|=";
   var $template = "=|Setting.yesno.dropdown.item|=";
   var $templateFooter = "=|Setting.yesno.dropdown.footer|=";

}

class SettingsAllowGuestCommentEditYesNo extends YesNoDropDown {
   var $dropDownKeyName = "guest_comment_edit_allowed";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Setting.yesno.dropdown.header|=";
   var $template = "=|Setting.yesno.dropdown.item|=";
   var $templateFooter = "=|Setting.yesno.dropdown.footer|=";
}

class BlogImage extends PhooDisplay {
   var $securityRole = ROLE_EVERYONE; /* Security for individual settings are set at the lower levels */
   var $name = "BlogImage";
   var $template = "=|BlogImage.display|="; 

   function setup( ) {
      /* see if the blog image url setting is set */
      $blogImageUrl = $this->session->getSetting( "blog_image_url" );
      if ( !isset( $blogImageUrl ) || $blogImageUrl == "" ) { /* if no image, disable */
         $this->disabled = true;
      }
   }
}
?>
