<?php
/*
   Name: static_page.classes.php
   Author: Bill Roberts
   Created: 11/7/2007
   Purpose: SnapOn classes related to creating, editing, and updating pages
   $Revision: 1.3 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
   
define( "PAGE_PUBLIC", 1 );
define( "PAGE_DRAFT", 2 );

/****************************************************/
/* StaticPageDisplay - displays a page full of pages      */
/****************************************************/
class StaticPageDisplay extends PhooQuery { 
   var $name = "StaticPageDisplay";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|StaticPage.get|=";
   var $template = "=|StaticPage|=";

   function setup( ) {
      /* if there isn't a static_page_id value passed, disable */
      $staticPageId = $this->getVar( "static_page_id" );
      if ( $staticPageId == "" ) {
         $this->disabled = true;
      }
   }
}

class StaticPageTitle extends PhooQuery {
   var $name = "StaticPageTitle";
   var $securityRole = ROLE_EVERYONE;
   var $template = "_@page_title@_";
   var $command = "=|StaticPage.title.get|=";
}                                           

class EditStaticPageLink extends PhooDisplay {
   var $name = "EditStaticPageLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|StaticPage.link.edit|=";
   var $invisibleOnSecurityErrors = true;
} 

class DeleteStaticPageLink extends PhooDisplay {
   var $name = "DeleteStaticPageLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|StaticPage.link.delete|=";
   var $invisibleOnSecurityErrors = true;
} 

class StaticPageDeleter extends PhooProcess {
   var $name = "StaticPageDeleter";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot delete a page. you do not have security privileges";
   var $command = "=|StaticPage.delete|=";
}

/*************************************************************/
/*** StaticPageForm - controls the creating and editing of pages ***/
/*************************************************************/
class StaticPageForm extends PhooForm {
   var $name = "StaticPageForm";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot display page form. You do not have pageing privileges";
   var $trigger = array( "btnPublish" => array( "process" => "insert" )
                        ,"btnEdit" => array( "process" => "update" ) );
   var $field = array( "static_page_title" => array( "required" => "Y"
                                             ,"formalName" => "Page Title" )
                      ,"static_page_content" => array( "required" => "Y"
                                               ,"formalName" => "Content" )
                     );
   var $loadCommand = "=|StaticPage.admin.get|=";

   /* set templates and expects based on the type of action to be taken (insert vs update) */  
   function setup( ) {
      parent::setup( );
      
      switch ( $this->action ) {
         case "help" :
            $this->template = "=|StaticPage.help|=";
            $descriptor = $this->session->getSnapOnDescriptor( "static_page" );
            $this->setVar( "static_page_help", $descriptor[ "help_overlay" ], "PRIVATE" );
            break;
         
         case "update" :
            $this->command = "=|StaticPage.update|=";
            $this->template = "=|StaticPage.form.update|=";
            break;

         default:
         case "insert" :
            $this->command = "=|StaticPage.insert|=";
            $this->template = "=|StaticPage.form.insert|=";
            break;
      }
   }  

   function redirect( ) {

      /* set the success page */
      $this->setSuccessPage( $this->getVar( "PHP_SELF", "SERVER" ) . "?page=pageAdmin" );
      
      if ( $this->action == "insert" ) {
         $this->successPage .= "&static_page_id=" . $this->insertedId;
      }
      else { /* editing */
         $this->successPage .= "&static_page_id=" . $this->getVar( "static_page_id", "POST" );
      }
         
      /* set a success message */
      if ( !$this->isError( ) ) {
         $this->session->setSessionMessage( "Page has been saved", MSG_INFO );
      }
      
      parent::redirect( );
   }
} 

class StaticPageTypeDropDown extends PhooDropDown {
   var $name = "StaticPageTypeDropDown";
   var $securityRole = ROLE_POSTER;
   var $command = "=|StaticPage.type.list.get|=";
   var $templateHeader = "=|StaticPage.type.dropdown.header|=";
   var $template = "=|StaticPage.type.dropdown.item|=";
   var $templateFooter = "=|StaticPage.type.dropdown.footer|=";
   var $dropDownKeyName = "static_page_type_id";

   function setup( ) {
      $pageTypeId = $this->getVar( "static_page_type_id" );
      if ( $pageTypeId == "" ) { /* default to public */
         $this->setVar( "static_page_type_id", PAGE_PUBLIC, "PRIVATE" );
      }

      parent::setup( );
   }
}

class StaticPageList extends PhooDropDown {
   var $name = "StaticPageRecentList";
   var $securityRole = ROLE_POSTER;
   var $command = "=|StaticPage.list.get|=";
   var $templateHeader = "=|StaticPage.list.header|=";
   var $template = "=|StaticPage.list.item|=";
   var $disableIfNoRows = true;
}

class StaticPageDraftsList extends PhooQuery {
   var $name = "StaticPageDraftsList";
   var $securityRole = ROLE_POSTER;
   var $command = "=|StaticPage.draft.list.get|=";
   var $templateHeader = "=|StaticPage.draft.list.header|=";
   var $template = "=|StaticPage.draft.list.item|=";
   var $disableIfNoRows = true;
}

/* events for converting BRs to line feeds and visa-versa */
class StaticPageBRToLineFeedConverter extends BRToLineFeedConverter {
   var $name = "StaticPageBRToLineFeedConverter";
   var $securityRole = ROLE_POSTER;
   var $fieldName = "static_page_content";
}

class StaticPageLineFeedToBRConverter extends LineFeedToBRConverter {
   var $name = "StaticPageLineFeedToBRConverter";
   var $securityRole = ROLE_POSTER;
   var $fieldName = "static_page_content";
}

class StaticPageTagBalancer extends TagBalancer {
   var $name = "StaticPageTagBalancer";
   var $securityRole = ROLE_POSTER;
   var $fieldName = array( "static_page_content", "static_page_title" );
}

?>
