<?php
/* 
   Name: trackback_classes.php
   Author: Bill Roberts
   Created: 5/13/2005
   Purpose: SnapOn classes related to receiving and sending trackback pings
   $Revision: 1.5 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
  
class TrackbackServer extends PhooProcess {
   var $package = "trackback";
   var $name = "TrackbackServer";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Trackback.insert|=";
   var $template = "=|Trackback.response.success|=";
   var $error;
   var $requestURI;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-05-13" );
      $this->setPurpose( "Recieves trackback pings, returns OK or error message and stores the trackback" );
      $this->addExpects( "url", "CONTAINER" );
      $this->addExpects( "trackback_table", "APPLICATION" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }

   function setup( ) {
      parent::setup( );
      
      /* check to see if trackbacks are closed for this post, or closed overall */
      $trackbacksAllowed = $this->session->getSetting( "trackbacks_allowed" );

      if ( !$trackbacksAllowed == "Y" ) { 
         $this->error = "Trackbacks are currently not allowed";
         return;
      }
         
      /* grab the request URI */
      $this->requestURI = $this->getVar( "REQUEST_URI", "SERVER" );
   }
 
   function doAction( ) {
      /* parse out the post id */
      $postId = explode('/', $this->requestURI );
	   $postId = $postId[ count( $postId ) - 1 ];
      $this->setVar( "post_id", $postId, "PRIVATE" );

      /* check the trackback allowed status on this individual post */
      $row = $this->getRowsFromQueryCommand( "=|Trackback.trackbacks_allowed_on_post.get|=" );

      $trackbacksAllowed = $row[ 0 ][ "trackbacks_allowed_flag" ];

      if ( !( $trackbacksAllowed == "Y" ) ) {
         /* close trackbacks by disabling this object */
         $this->error = "Trackbacks are currently not allowed on this post";
         return;
      }
      
      /* must have been sent at least a URL (based on the spec at http://www.sixapart.com/pronet/docs/trackback_spec) */
      $url = $this->getVar( "url", "POST" );
      
      if ( !isset( $url ) ) {
         $this->error = "No URL sent with the trackback ping";
         return;
      }

      /* set the private variables to the desired names */
      $this->setVar( "trackback_blog_url", $url, "PRIVATE" );      
      $this->setVar( "trackback_post_id", $this->getVar( "post_id", "PRIVATE" ), "PRIVATE" );      
      $this->setVar( "trackback_blog_name", $this->getVar( "blog_name", "POST" ), "PRIVATE" );      
      $this->setVar( "trackback_excerpt", $this->getVar( "excerpt", "POST" ), "PRIVATE" );      
      $this->setVar( "trackback_title", $this->getVar( "title", "POST" ), "PRIVATE" );      
      $this->setVar( "trackback_ip", $this->getVar( "REMOTE_ADDR", "SERVER" ), "PRIVATE" );
      
      /* assume the trackback is approved (default) */
      $this->setVar( "approved_flag", "Y", "POST" );
         
      parent::doAction( );
      /* just put any errors we got in the response */
      while ( $this->isError( ) ) {
         $this->error = $this->getError( );
      }

      if ( isset( $this->error ) ) { /* switch the template */
         $this->template = "=|Trackback.response.error|=";
      }
   }

   function renderSnapOn( ) {
      /* put the response var in the post variables so it will render */
      if ( isset( $this->error ) ) {
         $this->setVar( "error", $this->error, "POST" );
      }

      return parent::renderSnapOn( );
   }
}

class TrackbackDisplay extends PhooQuery {
   var $package = "trackback";
   var $name = "TrackbackDisplay";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Trackback.get.bypost|=";
   var $templateHeader = "=|Trackback.section.header|=";
   var $template = "=|Trackback|=";
   var $field = array( "trackback_date" => array( "format" => "m/d/Y g:i a"
                                                 ,"type" => "date" ) );

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-05-27" );
      $this->setPurpose( "Displays trackbacks for a post, in ascending order by date" );
      $this->addExpects( "trackback_table", "APPLICATION" );
      $this->addExpects( "post_table", "APPLICATION" );
      $this->addExpects( "post_id", "CONTAINER" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }

   function setup( ) {
      /* trackbacks are only allowed on public posts - if this is not a public post, disable 
         the display the object */
      /* note: only run this if there is a post_id available */
      $postId = $this->getVar( "post_id" );
      if ( $postId > 0 ) {
         $rows = $this->getRowsFromQueryCommand( "=|Post.type.get|=" );
         if ( is_array( $rows ) ) {
            if ( $rows[ 0 ][ "post_type_id" ] == 1 ) { /* public post */
               parent::setup( );
            }
            else {
               $this->disabled = true;
            }
         }
         else {
            $this->disabled = true;
         }
      }

      parent::setup( );
   }
         
   function doAction( ) {
      /* check for a delete command. If one was passed,
         perform the delete action instead of the default
         action */
      $action = $this->getVar( FORM_ACTION, "GET" );
      switch ( $action ) {
         case "delete" :
            /* process a trackback deleter */
            $s =& $this->session->snapOnFactory( "TrackbackDeleter", $this );
            /* set the success page to take you back to the calling page, with no parameters
            */
            $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
            $s->process( );
            if ( $s->isError( ) ) {
               /* load the template up with the error syntax, etc */
               $this->template .= $s->render( );
               return;
            }

            break;

         case "markspam" :
            /* process a trackback marker */
            $this->setVar( "trackback_mark_action", $action, "PRIVATE" );
            $s =& $this->session->snapOnFactory( "TrackbackMarker", $this );
            /* set the success page to take you back to the calling page, with no parameters
            */
            $s->setSuccessPage( $this->getVar( "HTTP_REFERER", "SERVER" ) );
            $s->process( );
            if ( $s->isError( ) ) {
               /* load the template up with the error syntax, etc */
               $this->template .= $s->render( );
               return;
            }

            break;
            
         default : /* otherwise, continue on */
            parent::doAction( );
      }
   }

   function getResultRow( ) {
      /* call the ancestor */
      $row = parent::getResultRow( );

      if ( $this->isError( ) || !is_array( $row ) ) {
         return;
      }

      if ( isset( $row[ "trackback_date" ] ) ) {
         $trackbackTime = $this->session->getLocalDate( BASE_TIME_FORMAT, $row[ "trackback_date" ] );
         $row[ "trackback_date" ] = $trackbackTime[ 0 ];
         $row[ "trackback_time_zone_abbrev" ] = $trackbackTime[ 1 ];
      }

      return $row;
   }
} 

class TrackbackLink extends PhooDisplay {
   var $securityRole = ROLE_EVERYONE;
   var $trackbacksClosedTemplate = "=|Trackback.trackbacks_closed|=";
   var $zeroTrackbacksTemplate = "=|Trackback.count.zero|=";
   var $oneTrackbackTemplate = "=|Trackback.count.one|=";
   var $multipleTrackbacksTemplate = "=|Trackback.count.multiple|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-10-27" );
      $this->setPurpose( "Displays post trackbacks link, with number of trackbacks" );
      $this->addExpects( "post_id", "CONTAINER" );
   }
   
   function doAction( ) {
      $row = $this->getRowsFromQueryCommand( "=|Trackback.count.bypost|=" );

      $trackbackCount = $row[ 0 ][ "trackback_count" ];
   
      /* note: if the trackback count is zero AND trackbacks are closed for this post, display the trackbacks_closed
          template */
      if ( $trackbackCount == 0 ) {

         /* check the trackback closed status */
         /* first check the overall */
         $trackbacksAllowed = $this->session->getSetting( "trackbacks_allowed" );
         if ( $trackbacksAllowed == "Y" ) { /* check the individual post */
            $row = $this->getRowsFromQueryCommand( "=|Trackback.trackbacks_allowed_on_post.get|=" );

            $trackbacksAllowed = $row[ 0 ][ "trackbacks_allowed_flag" ];
         }

         if ( !($trackbacksAllowed == "Y" ) ) {
            /* trackbacks are closed and there are no existing trackbacks */
            $this->template = $this->trackbacksClosedTemplate;
         }         
      }

      if ( $this->template == "" ) { /* not set yet */
      
         $this->setVar( "trackback_count", $trackbackCount, "PRIVATE" );

         switch ( $trackbackCount ) {
            case 0 :
               $this->template = $this->zeroTrackbacksTemplate;
               break;

            case 1 :
               $this->template = $this->oneTrackbackTemplate;
               break;

            default :
               $this->template = $this->multipleTrackbacksTemplate;
               break;
         }
      }      
   }
}

class DeleteTrackbackLink extends PhooDisplay {
   var $package = "trackback";
   var $name = "DeleteTrackbackLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|Trackback.link.delete|=";
   var $invisibleOnSecurityErrors = true;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-06-03" );
      $this->setPurpose( "Renders the trackback deletion link" );
      $this->addExpects( "trackback_id", "CONTAINER" );
      $this->addModificationHistory( "modified for new role-based security", "2005-10-30" );
   }
}

class MarkSpamTrackbackLink extends PhooDisplay {
   var $package = "trackback";
   var $name = "MarkSpamTrackbackLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|Trackback.link.markspam|=";
   var $invisibleOnSecurityErrors = true;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-11-03" );
      $this->setPurpose( "Renders the trackback mark spam link" );
      $this->addExpects( "trackback_id", "CONTAINER" );
      $this->addExpects( "post_id", "CONTAINER" );
   }
}

/*****************************************************/
/* classes for moderating trackbacks                 */
/*****************************************************/

/* abstract class for both unapproved and approved trackbacks display */
class ModeratedTrackbacksDisplay extends TrackbackDisplay {
   var $name = "ModeratedTrackbacksDisplay";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "You do not have trackback moderation privileges";
   var $resultPageSize = 40; /* default result page size */

   function doAction( ) {
      $actionType = $this->getVar( "btnModerateType" );

      if ( $actionType > "" ) {

         /* gather all ids */
         $listAr = $this->getVar( "moderationCBox" );

         $list = implode( ",", $listAr );

         if ( $list > "" ) { /* if blank, no ids were selected */
            switch ( $actionType ) {
               case "SPAM" :
                  $this->setVar( "trackback_mark_action", "markspam", "PRIVATE" );
                  $s =& $this->session->snapOnFactory( "TrackbackMarker", $this );
                  $s->command = "=|Trackback.mark_list|=";
                  $s->setVar( "trackback_id_list", $list, "PRIVATE" );
                  $s->process( );
                  break;
         
               case "DELETE" :
                  $s =& $this->session->snapOnFactory( "TrackbackDeleter", $this );
                  $s->command = "=|Trackback.delete_list|=";
                  $s->setVar( "trackback_id_list", $list, "PRIVATE" );
                  $s->process( );
                  break;
               
               case "APPROVE" :
                  $s =& $this->session->snapOnFactory( "TrackbackApprover", $this );
                  $s->command = "=|Trackback.approve_list|=";
                  $s->setVar( "trackback_id_list", $list, "PRIVATE" );
                  $s->process( );
                  break;
            }      
         }
      }

      parent::doAction( );
   }


   function getResultRow( ) {
      /* call the ancestor */
      $row = parent::getResultRow( );

      if ( $this->isError( ) || !is_array( $row ) ) {
         return;
      }

      if ( isset( $row[ "trackback_excerpt" ] ) ) {
         if ( strlen( $row[ "trackback_excerpt" ] ) > 250 ) {
            $pos = strpos( $row[ "trackback_excerpt" ], " ", 250 );
            if ( $pos > 0 ) {
               $row[ "trackback_excerpt" ] = substr( $row[ "trackback_excerpt" ], 0, $pos ) . " . . .";
            }
         }
      }

      return $row;
   }
   
}

class ApprovedTrackbacksDisplay extends ModeratedTrackbacksDisplay {
   var $command = "=|Trackback.approved.get|=";
   var $templateHeader = "=|Trackback.approved.header|=";
   var $template = "=|Trackback.approved|=";
   var $templateFooter = "=|Trackback.approved.footer|=";

}

class UnapprovedTrackbacksDisplay extends ModeratedTrackbacksDisplay {
   var $command = "=|Trackback.unapproved.get|=";
   var $templateHeader = "=|Trackback.unapproved.header|=";
   var $template = "=|Trackback.unapproved|=";
   var $templateFooter = "=|Trackback.unapproved.footer|=";

}

class ApprovedTrackbacksResultPageNavigation extends PhooResultPageNavigation {
   var $name = "ApprovedTrackbacksResultPageNavigation";
   var $template = "=|Trackback.moderation.navigation|=";
   var $nextPageTemplate = "=|Trackback.moderation.link.next_page|=";
   var $previousPageTemplate = "=|Trackback.moderation.link.previous_page|=";
   var $divider = " | ";
   var $countCommand = "=|Trackback.approved.count|=";
   var $securityRole = ROLE_ADMINISTRATOR;

}

class UnapprovedTrackbacksResultPageNavigation extends PhooResultPageNavigation {
   var $name = "UnapprovedTrackbacksResultPageNavigation";
   var $template = "=|Trackback.moderation.navigation|=";
   var $nextPageTemplate = "=|Trackback.moderation.link.next_page|=";
   var $previousPageTemplate = "=|Trackback.moderation.link.previous_page|=";
   var $divider = " | ";
   var $countCommand = "=|Trackback.unapproved.count|=";
   var $securityRole = ROLE_ADMINISTRATOR;

}


class TrackbackDeleter extends PhooProcess {
   var $package = "trackback";
   var $name = "TrackbackDeleter";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot delete a trackback. you do not have security privileges";
   var $command = "=|Trackback.delete|=";
}

class TrackbackMarker extends PhooProcess {
   var $package = "trackback";
   var $name = "TrackbackMarker";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot mark a trackback. you do not have security privileges";
   var $command = "=|Trackback.mark|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-04-19" );
      $this->setPurpose( "Processes the marking of a trackback" );
      $this->addExpects( "trackback_mark_action", "CONTAINER" );
   }

   function doAction( ) {
      $trackbackMarkAction = $this->getVar( "trackback_mark_action", "PRIVATE" );

      switch ( $trackbackMarkAction ) {
         case "markspam" : /* currently the only possible action */
            $this->setVar( "trackback_mark", "spam", "PRIVATE" );
            break;
      }

      parent::doAction( );
   }
}

class TrackbackApprover extends PhooProcess {
   var $package = "trackback";
   var $name = "TrackbackApprover";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot approve a trackback. you do not have security privileges";
   var $command = "=|Trackback.approve_list|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-03" );
      $this->setPurpose( "Processes the approval of a list of trackbacks" );
   }
}

/*************************************************************/
/* TrackbackFilter - abstract ancestor event-handler class   */
/* abstract class - sets the approved_flag and potentially   */
/* the filter_agent and filter_reason (in the descendents,   */
/* of course                                                 */
/*************************************************************/
class TrackbackFilter extends PreSaveEventHandler {
   var $package = "trackback";
   var $name = "TrackbackFilter";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-11-03" );
      $this->setPurpose( "abstract class. sets the approved_flag, and possibly filter_agent and " .
                         "filter_reason (in the descendent)" );
   }

   function unapproveTrackback( $aFilterAgent, $aFilterReason ) {
      /* set the post variables with the filterAgent, filterReason, and unapprovedFlag */
      $this->setVar( "filter_agent", $aFilterAgent, "POST" );
      $this->setVar( "filter_reason", $aFilterReason, "POST" );
      $this->setVar( "approved_flag", "N", "POST" );
      $this->setError( "This trackback has been marked as suspected spam", MSG_FATAL );
   }

   /* abstract function - should be overridden. Should call unapproveTrackback if
      a trackback is unapproved */
   function checkTrackback( ) {
   }
 
   function doAction( ) {
      $this->checkTrackback( );
   }
}

/******************************************************************************/
/* TrackbackSendPopupPage - Renders the Trackback send page in a popup window */
/******************************************************************************/
class TrackbackSendPopupPage extends BlooPage {
   var $name = "TrackbackSendPopupPage";
   var $securityRole = ROLE_POSTER;
   var $templateHeader = "=|Page.trackback_send.header|=";
   var $template = "_{TrackbackSendForm}_";
   var $templateFooter = "=|Page.trackback_send.footer|=";
}

class TrackbackSendForm extends PhooForm {
   var $name = "TrackbackSendForm";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot display trackback send form. You do not have comment editing privileges";
   var $trigger = array( "btnSendTrackback" => array( "process" => "send" ) );
   var $loadCommand = "=|Post.get|=";                        
   var $field = array( "trackback_url" => array( "required" => "Y"
                                                ,"formalName" => "Trackback URL" )
                     );
   var $template = "=|Trackback.form.send|=";
   var $action = "update"; /* this is just set here so the loadForm function will be called */
   
   function setup( ) {
      parent::setup( );
      
      /* see if a trackback has already been sent (meaning we were redirected here from the end of the
         trackback send process */
      $trackbackSent = $this->getVar( "trackback_sent", "GET" );
      if ( $trackbackSent == 1 ) {
         $this->template = "=|Trackback.sent.message|=";
         $this->loadCommand = ""; /* clear this so it won't wast time running the load command */
      }
      else {
         /* clear the success page to prevent redirects */
         $this->setSuccessPage( "" );
      }
   }
   
   function loadForm( ) {
      parent::loadForm( );

      if ( !$this->isError( ) ) {
         /* get a post excerpt, approximately 250 characters in length or less. 
         Strip the tags too, except for <br> tags and paragraph tags */
         $postContent = $this->getVar( "post_content", "POST" );
         $postContent = strip_tags( $postContent, "<br>,<br />,<p>,</p>" );
         if ( strlen( $postContent ) > 250 ) {
            $pos = strpos( $postContent, " ", 250 );
            if ( $pos > 0 ) {
               $postExcerpt = substr( $postContent, 0, $pos ) . " . . .";
            }
         }
         else {
            $postExcerpt = $postContent;
         }

         $this->setVar( "post_excerpt", $postExcerpt, "PRIVATE" );

      }
   }

   function triggerProcess( $aProcess ) {
      parent::triggerProcess( $aProcess );

      if ( !$this->isError( ) ) {
         /* verify that the trackback url at the least starts with http:// */
         $trackbackURL = trim( $this->getVar( "trackback_url", "POST" ) );
         
         if ( strpos( "X" . strtolower( $trackbackURL ), "http://" ) != 1 ) {
            $this->setError( "The Trackback URL must begin with 'http://'", MSG_WARNING );
         }
      }

      if ( !$this->isError( ) ) {
         /* parse the share from the trackback url */
         $pos = strpos( $trackbackURL, "/", 7 );

         if ( $pos > 0 ) { /* get the share */
            $share = substr( $trackbackURL, $pos );
            $host = substr( $trackbackURL, 0, $pos );
         }
         else {
            $share = "/";
            $host = $trackbackURL;
         }
         
         /* remove the "http://" from the host */
         $host = str_replace( "http://", "", $host );
         
         /* grab variables */
         $blogName = urlencode( stripslashes( $this->session->getSetting( "blog_name" ) ) );
         $title = urlencode( stripslashes( $this->getVar( "post_title", "POST" ) ) );
         $url = $this->session->getSetting( "site_url" ) . "?post_id=" . $this->getVar( "post_id", "POST" );
         $excerpt = urlencode( stripslashes( $this->getVar( "post_excerpt", "POST" ) ) );


         /* send the trackback */
         $fp = @fsockopen( $host, 80, $errno, $errstr, 30);
         
         if ( !$fp ) {
            $this->setError( "Could not open a socket on the remote trackback server", MSG_WARNING );
         }
         else {
            
            /* build the header and data */
            $header = "POST $share HTTP/1.1\r\n";
            $header .= "Host: $host\r\n";
            $header .= "Content-Type: application/x-www-form-urlencoded; charset=utf-8\r\n";
            
            $data = "blog_name=$blogName&excerpt=$excerpt&title=$title" . "&url=$url\r\n";
            $header .= "Content-length: " . strlen( $data ) . "\r\n\r\n";
            $content = $header . $data;   
   
            @fwrite( $fp, $content );
   
            $result = "";
            while ( !feof( $fp ) ) {
               $result .= fgets( $fp, 128 );
            }

            @fclose($fp);

            /* check the result - need to find xml response tags. If not found, assume it didn't work */
            $noValidResponse = false;
            $responseError = false;
            $posStart = strpos( strtolower( $result ), "<?xml" );
            if ( $posStart > 0 ) {
               $posEnd = strpos( strtolower( $result ), "</response>", $posStart );
               if ( $posEnd > 0 ) {
                  /* we got result xml. Parse it */
                  $xmlResponse = substr( $result, $posStart, $posEnd+11 - $posStart );
                  $x = new XML2ArrayParser( $xmlResponse, true );

                  if ( !$x->isError( ) ) {
                     if ( $x->tagData[ "RESPONSE" ][ 0 ][ "ERROR" ] == "0" ) {
                        $this->session->setSessionMessage( "Trackback successfully sent", MSG_INFO );
                     }
                     else {
                        $responseError = true;
                        $errorMessage = $x->tagData[ "RESPONSE" ][ 0 ][ "MESSAGE" ];
                     }
                  }
                  else {                     
                     $noValidResponse = true;
                     $xmlError = "";
                     while ( $x->isError( ) ) {
                        $xmlError .= $x->getError( );
                     }
                     $this->session->setSessionMessage( "Bad response: $xmlError", MSG_FATAL );
                  }
               }
               else {
                  $noValidResponse = true;
               }
            }
            else {
               $noValidResponse = true;
            }

            if ( $noValidResponse ) {
               $this->session->setSessionMessage( 
                  "Warning: Did not receive a valid response from the remote trackback host", MSG_WARNING );
            }

            if ( $responseError ) {
               $this->session->setSessionMessage( "Warning: Error returned from remote trackback server: $errorMessage", 
                                                  MSG_WARNING );
            }
            /* regardless of error status, redirect to the trackback sent page */
            $this->setSuccessPage( $this->session->getSetting( "site_url" ) . 
                                   "/bloo.trackback_send.popup.php?trackback_sent=1" );
            $this->redirect( );
         }
      }
   }
} 
?>
