<?php
/* 
   Name: bloo_quote.classes.php
   Author: Bill Roberts
   Created: 11/1/2005

   Purpose: Extension related to managing and displaying quotes. 

   $Revision: 1.7 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/   
  
class BlooQuoteDisplay extends PhooQuery { 
   /* Abstract - need to inherit to and populate
      the chooseQuote method */
   /* class displays a quote */
   var $package = "bloo_quote";
   var $name = "BlooQuoteDisplay";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|BlooQuote.getbyid|=";
   var $templateHeader = "=|BlooQuote.header|=";
   var $template = "=|BlooQuote.item|=";
   var $templateFooter = "=|BlooQuote.footer|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-01" );
      $this->setPurpose( "Abstract class - displays a quote" );
   }

   function quoteCount( ) {
      /* returns the number of quotes */
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole; /* set the security to same as containter */
      $s->command = "=|BlooQuote.count|=";
      $s->process( );
      if ( $s->isError( ) ) {
         return;
      }
      $row = $s->getResultRow( );

      if ( $row[ "num_quotes" ] == 0 ) {
         $this->disabled = true; /* disable - no quotes to display */
      }
      
      return $row[ "num_quotes" ];
   }      
   
   function chooseQuote( ) {
      /* chooses the quote to display based upon whatever algorithm the inheriting class decides
         to implement */
      return 0; /* abstract - return 0 */
   } 
   
   /* choose a quote and process */
   function doAction( $aArray = null ) {
      $selectedQuoteId = $this->chooseQuote( );

      $this->setVar( "bloo_quote_id", $selectedQuoteId, "PRIVATE" );
      
      parent::doAction( $aArray );
   }
} 

class BlooQuoteCitation extends PhooDisplay {
   var $package = "bloo_quote";
   var $name = "BlooQuoteCitation";
   var $securityRole = ROLE_EVERYONE;
   var $template = "";
   var $authorTemplate = "=|BlooQuote.author|=";
   var $workTemplate = "=|BlooQuote.work|=";
   var $bothTemplate = "=|BlooQuote.author.and.work|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-04" );
      $this->addExpects( "bloo_quote_author", "CONTAINER" );
      $this->addExpects( "bloo_quote_work", "CONTAINER" );
      $this->setPurpose( "Displays a quote's author and work, if they exist" );
   }

   function renderSnapOn( ) {
      /* if the Author exists (list_meta_01) set that in the template. Same for the work (list_item_name) */
      $author = $this->getVar( "bloo_quote_author" );
      $work = $this->getVar( "bloo_quote_work" );
      
      if ( $author > "" && $work == "" ) {
         $this->template = $this->authorTemplate;
      }
      elseif ( $author == "" && $work > "" ) {
         $this->template = $this->workTemplate;
      }
      elseif ( $author > "" && $work > "" ) {
         $this->template = $this->bothTemplate;
      }
         
      return parent::renderSnapOn( ); 
   }
}

class BlooQOTD extends BlooQuoteDisplay {
   var $name = "BlooQOTD";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-01" );
      $this->setPurpose( "Displays a quote of the day" );
   }

   function chooseQuote( ) { 

      /* initialize - get current local day */
      $localTime = $this->session->getLocalDate( "Y-m-d" );
      $currentLocalDay = $localTime[ 0 ];

      /* get last date that a quote was advanced */
      $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotelastdate.get|=" );
      if ( $this->isError( ) ) {
         return;
      }
      
      if ( is_array( $rows ) && count( $rows ) > 0 ) {
         $lastDate = $rows[ 0 ][ "bloo_quote_last_date" ];
      }
      else { /* could not determine last date. That record must not exist. Insert today's date */
         $lastDate = $currentLocalDay;
         $this->setVar( "bloo_quote_last_date", $lastDate, "PRIVATE" );      
         $this->runModificationCommand( "=|BlooQuote.quotelastdate.insert|=" );
         if ( $this->isError( ) ) {
            return;
         }
      }

      /* get current id and order */
      $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotecurrentid.get|=" );
      if ( $this->isError( ) ) {
         return;
      }
      
      
      if ( is_array( $rows ) && count( $rows ) > 0 ) {
         $currentId = $rows[ 0 ][ "bloo_quote_current_id" ];
      }
      else { /* could not determine current id. Record must not exist. Insert the earliest quote, if it exists */
         $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotefirstid.get|=" );
         if ( $this->isError( ) ) {
            return;
         }
         
         if ( is_array( $rows ) && count( $rows ) > 0 ) {
            $firstId = $rows[ 0 ][ "bloo_quote_first_id" ];
         }
         else { /* could not determine first id. No quotes exist, evidently */
            return -1; /* return the "there aren't any quote ids" id */
         }

         $this->setVar( "bloo_quote_current_id", $firstId, "PRIVATE" );      
         $currentId = $firstId;

         /* check to see if the current id setting is already in the table */
         $exists = 'N';
         $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.currentidsetting.exists|=" );
         if ( $this->isError( ) ) {
            return;
         }

         if ( is_array( $rows ) && $rows[ 0 ][ "setting_exists" ] > 0 ) {
            $exists = 'Y';
         }

         if ( $exists == 'Y' ) {
            /* already exists */
            $this->setVar( "bloo_quote_current_date", $currentLocalDay, "PRIVATE" );
            $this->runModificationCommand( "=|BlooQuote.quotecurrentid.update|=" );
         }
         else {
            $this->runModificationCommand( "=|BlooQuote.quotecurrentid.insert|=" );
         }
         if ( $this->isError( ) ) {
            return;
         }
      }
      
      /* check last date - if different than the current local day, advance the quote */
      if ( $currentLocalDay != $lastDate ) {
         $this->setVar( "bloo_quote_current_id", $currentId, "PRIVATE" );      
         $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotenextid.get|=" );
         if ( $this->isError( ) ) {
            return;
         }
         
         if ( is_array( $rows ) && count( $rows ) > 0 ) {
            $currentId = $rows[ 0 ][ "bloo_quote_next_id" ];
         }
         else {
            /* must be at end, go back to the first id */
            $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotefirstid.get|=" );
            if ( $this->isError( ) ) {
               return;
            }
         
            if ( is_array( $rows ) && count( $rows ) > 0 ) {
               $currentId = $rows[ 0 ][ "bloo_quote_first_id" ];
            }
            else { /* could not determine first id. No quotes exist, evidently */
               return -1; /* return the "there aren't any quote ids" id */
            }
         }

         /* set the next current id and date */
         $this->setVar( "bloo_quote_current_id", $currentId, "PRIVATE" );      
         $this->setVar( "bloo_quote_current_date", $currentLocalDay, "PRIVATE" );
         $this->runModificationCommand( "=|BlooQuote.quotecurrentid.update|=" );
         if ( $this->isError( ) ) {
            return;
         }
         $this->runModificationCommand( "=|BlooQuote.quotelastdate.update|=" );
         if ( $this->isError( ) ) {
            return;
         }
      }

      /* return the id selected */
      return $currentId;
   }
}
   
class RandomBlooQuote extends BlooQuoteDisplay {
   var $name = "RandomBlooQuote";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-03" );
      $this->setPurpose( "Randomly chooses a quote to display" );
   }

   function chooseQuote( ) { 
      /* algorithm: generate a random number to use to select a quote
       */
      $quoteCount = $this->quoteCount( ); 
      if ( $quoteCount == 0 ) {
         return -1;
      }
      
      $quoteNum = rand( 0, $quoteCount - 1 );

      /* this is kind of brute force. . . Could be made more efficient */
      $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.ids.get|=" );
      if ( $this->isError( ) ) {
         return;
      }
      
      if ( is_array( $rows ) && count( $rows ) > 0 ) {
         return $rows[ $quoteNum ][ "bloo_quote_id" ];
      }
      else {
         return -1;
      }
   }
} 

/* displays the number of active quotes */
class BlooQuoteCount extends PhooQuery { 
   var $name = "BlooQuoteCount";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|BlooQuote.count|=";
   var $template = "=|BlooQuote.count.display|=";
} 

/*********************************************/
/*** SnapOns for Quote administration      ***/
/*********************************************/
class BlooQuoteAdminPageSection extends PhooDisplay {
   var $package = "bloo_quote";
   var $name = "QuoteAdminPageSection";
   var $securityRole = ROLE_POSTER;
   var $template = "_{BlooQuoteAdminDisplay}_";
   
   /* check the parameters passed to determine what to do */
   function setup( ) {
      parent::setup( );

      $action = $this->getVar( FORM_ACTION );

      switch( $action ) {
         case "insert" :
         case "update" :
            $this->template = "_{BlooQuoteAdminForm}_";
            
            break;
            
         case "delete" : 
            /* instantiate and run a deleter */
            $s =& $this->session->snapOnFactory( "BlooQuoteDeleter", $this );
            $s->process( );
            if ( $s->isError( ) ) {
               return;
            }
            break;

         case "set" : 
            /* instantiate and run a setter */
            $s =& $this->session->snapOnFactory( "BlooQuoteSetter", $this );
            $s->process( );
            if ( $s->isError( ) ) {
               return;
            }
            break;

         default :
            break;
      }
   }
}

class BlooQuoteAdminDisplay extends PhooQuery {
   var $package = "bloo_quote";
   var $name = "BlooQuoteAdminDisplay";
   var $securityRole = ROLE_POSTER;
   var $command = "=|BlooQuote.list.get|=";
   var $templateHeader = "=|BlooQuote.list.header|=";
   var $template = "=|BlooQuote.list|=";
   var $templateFooter = "=|BlooQuote.list.footer|=";
   var $currentId;
   var $currentOrder;
   
   function setup( ) {
      parent::setup( );

      /* determine the current quote (for daily quote rotation only) */
      $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotecurrentid.get|=" );
      if ( $this->isError( ) ) {
         return;
      }
      
      if ( is_array( $rows ) && count( $rows ) > 0 ) {
         $this->currentId = $rows[ 0 ][ "bloo_quote_current_id" ];
      }
   }

   function getResultRow( ) {
      $row = parent::getResultRow( );

      if ( is_array( $row ) ) {

         /* see if row matches current id. If it does, set the suppress flag for the "set current" link */
         if ( $row[ "bloo_quote_id" ] == $this->currentId ) {
            $row[ "suppress_set_link" ] = "Y";
            $row[ "activation_class" ] = "activated";
         }
         else {
            $row[ "suppress_set_link" ] = "N";
            $row[ "activation_class" ] = "na";
         }
      }

      return $row;
   }
}

/* QuoteAdminForm 
*/
class BlooQuoteAdminForm extends PhooForm {
   var $package = "bloo_quote";
   var $name = "BlooQuoteAdminForm";
   var $securityRole = ROLE_POSTER;
   var $updateForm = "=|BlooQuote.form.update|=";
   var $insertForm = "=|BlooQuote.form.insert|=";
   var $loadCommand = "=|BlooQuote.getbyid|=";
   var $updateCommand = "=|BlooQuote.update|=";
   var $insertCommand = "=|BlooQuote.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   var $field = array( "bloo_quote_text" => array( "required" => "Y"
                                                  ,"formalName" => "Quote" )
                                                 );
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-01-23" );
      $this->setPurpose( "Displays and processes the quote administration form" );
   }

   function setup( ) {
      parent::setup( );
      
      /* set the list type here */
      $this->setVar( "list_type", "bloo_quote", "POST" ); /* set in post vars to make it accessible to 
                                                            the new_id command */

      /* for if this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&action=administer&extension_name=" . $this->getVar( "extension_name" ) );
   }

   function triggerProcess( $aProcess ) {

      /* if this is an insert, initialize the submitter and order */
      if ( $aProcess == "insert" ) {
         /* initialize to equal the current user as quote submitter */
         $this->setVar( "bloo_quote_submitter", $this->getVar( "user_id", "APPLICATION" ), "POST" );

         $s =& $this->session->snapOnFactory( "PhooQuery", $this );
         $s->securityRole = $this->securityRole; /* set the security to same as containter */
         $s->command = "=|BlooQuote.get.nextorder|=";
         $s->process( );
         if ( $s->isError( ) ) {
            return;
         }
         $row = $s->getResultRow( );
         
         if ( $row[ "next_order" ] == "" ) {
            /* this is the first row */
            $row[ "next_order" ] = 10;

            /* check to see if the current id setting is already in the table */
            /* insert the current id setting, if it doesn't already exist */
            $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.currentidsetting.exists|=" );
            if ( $this->isError( ) ) {
               return;
            }
   
            if ( !is_array( $rows ) || $rows[ 0 ][ "setting_exists" ] == 0 ) {
               /* if this is the first row, the id will be 1 */
               $this->setVar( "bloo_quote_current_id", 1, "PRIVATE" );
               $this->runModificationCommand( "=|BlooQuote.quotecurrentid.insert|=" );
               if ( $this->isError( ) ) {
                  return;
               }
            }

            /* see if the last date setting exists. If not, set it to today */
            $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotelastdate.get|=" );
            if ( $this->isError( ) ) {
               return;
            }
      
            if ( !is_array( $rows ) || count( $rows ) == 0 ) {
               $localTime = $this->session->getLocalDate( "Y-m-d" );
               $currentLocalDay = $localTime[ 0 ];
               $this->setVar( "bloo_quote_last_date", $currentLocalDay, "PRIVATE" );      
               $this->runModificationCommand( "=|BlooQuote.quotelastdate.insert|=" );
               if ( $this->isError( ) ) {
                  return;
               }
            }
         }
         
         $this->setVar( "list_order", $row[ "next_order" ], "POST" );
      }
      parent::triggerProcess( $aProcess );
   }
}

class BlooQuoteDeleter extends PhooProcess {
   var $package = "bloo_quote";
   var $name = "BlooQuoteDeleter";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot delete a quote. you do not have security privileges";
   var $command = "=|BlooQuote.delete|=";
   var $listType = "bloo_quote";

   /* check to see if the quote to be deleted is the current quote. If it is, move to the next quote prior
      to deleting */
   function setup( ) {
      $deleteId = $this->getVar( "bloo_quote_id", "GET" );

      /* get the current id */
      $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotecurrentid.get|=" );
      if ( $this->isError( ) ) {
         return;
      }
      
      if ( is_array( $rows ) && count( $rows ) > 0 ) {
         $currentId = $rows[ 0 ][ "bloo_quote_current_id" ];

         if ( $currentId == $deleteId ) {
            /* current id is being deleted - advance the quote */
            $this->setVar( "bloo_quote_current_id", $currentId, "PRIVATE" );      
            $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotenextid.get|=" );
            if ( $this->isError( ) ) {
               return;
            }

            if ( is_array( $rows ) && count( $rows ) > 0 ) {
               $currentId = $rows[ 0 ][ "bloo_quote_next_id" ];
            }
            else {
               /* must be at end, go back to the first id */
               $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.quotefirstid.get|=" );
               if ( $this->isError( ) ) {
                  return;
               }
         
               if ( is_array( $rows ) && count( $rows ) > 0 ) {
                  $currentId = $rows[ 0 ][ "bloo_quote_first_id" ];
               }
            }      
               
            $this->setVar( "bloo_quote_current_id", $currentId, "PRIVATE" );      

            $localTime = $this->session->getLocalDate( "Y-m-d" );
            $currentLocalDay = $localTime[ 0 ];
            
            $this->setVar( "bloo_quote_current_date", $currentLocalDay, "PRIVATE" );
            $this->runModificationCommand( "=|BlooQuote.quotecurrentid.update|=" );
            if ( $this->isError( ) ) {
               return;
            }
         }
      }
   }
}

class BlooQuoteSetQuoteLink extends PhooDisplay {
   var $name = "BlooQuoteSetQuoteLink";
   var $securityRole = ROLE_POSTER;
   var $template = "=|BlooQuote.link.setquote|=";
   var $templateSuppress = "=|BlooQuote.link.suppresssetquote|=";

   /* don't display if the suppress flag is set */
   function setup( ) {
      $suppressSetLink = $this->container->getVar( "suppress_set_link" );
      
      if ( $suppressSetLink == "Y" ) {
         $this->template = $this->templateSuppress;
      }
      else {
         $blooQuoteId = $this->container->getVar( "bloo_quote_id", "PRIVATE" );
         $this->setVar( "bloo_quote_id", $blooQuoteId, "PRIVATE" );
      }
   }
}

class BlooQuoteSetter extends PhooProcess {
   var $package = "bloo_quote";
   var $name = "BlooQuoteSetter";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot set the current quote. you do not have security privileges";
   var $command = "=|BlooQuote.quotecurrentid.update|=";
   /* note: there is a danger that if they haven't actually looked at a quote yet for the day, and if they
      set the current quote, because it doesn't set the last viewed date, it would advance to the next quote. They would
      have to go back in and reset it back to the current quote. This would be a pretty rare occurrence.

      Another rare occurrence - if they've never viewed the default daily quote SnapOn, setting current will have no effect
      because the daily quote records won't have been created in the list table. This is also assumed to be very rare
    */

   function setup( ) {
      parent::setup( );

      /* set the variables needed by the command */
      $rows = $this->getRowsFromQueryCommand( "=|BlooQuote.getbyid|=" );
      if ( $this->isError( ) ) {
         return;
      }
      
      if ( is_array( $rows ) && count( $rows ) > 0 ) {
         $currentOrder = $rows[ 0 ][ "list_order" ];
         $this->setVar( "bloo_quote_current_order", $currentOrder, "PRIVATE" );
         $currentId = $rows[ 0 ][ "bloo_quote_id" ];
         $this->setVar( "bloo_quote_current_id", $currentId, "PRIVATE" );
      }
      else {
         $this->disable; /* don't do the command */
      }
   }
}

?>
