<?php
/* 
   Name: bloogroll_posts.classes.php
   Author: Bill Roberts
   Created: 6/27/2006 

   Purpose: Extension collects and displays recent posts from the RSS feeds 
            in the bloogroll
   $Revision: 1.5 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
            
*/   
  
/*********************************************/
/*** SnapOn to display Bloogroll Posts     ***/
/*********************************************/
class BloogrollPosts extends PhooQuery {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPosts";
   var $securityRole = ROLE_EVERYONE;
   var $templateHeader = "=|BloogrollPosts.header|=";
   var $template = "=|BloogrollPosts|=";
   var $templateFooter = "=|BloogrollPosts.footer|=";
   var $post = array( );
   var $numPosts;
   var $currentPost;
   var $maxPosts;
   var $field = array( "post_date" => array( "format" => "m/d/Y g:i a"
                                            ,"type" => "date" ) );
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-06" );
      $this->setPurpose( "Displays bloogroll posts" );
   }

   /* stores the post passed in the post array. Note: if we're past the limit of the
      number of posts to show, return false
    */
   function storePost( &$aPost ) {
      /* note: storing in GMT. This assumes the post date follows the standard of including hours different from 
         GMT in its format */
      $aPost[ "post_date" ] = gmdate( "Y-m-d H:i:s", strtotime( $aPost[ "post_date" ] ) );
      /* search through the post array backwards - adding the new post in the proper order, without
         going over the max posts */
      for ( $i = count( $this->post ); $i > 0; $i-- ) {
         if ( $aPost[ "post_date" ] < $this->post[ $i - 1 ][ "post_date" ] ) {
            break;
         }
      }
      
      if ( $i < $this->maxPosts ) {
         if ( isset( $this->post[ $i ] ) ) {
            /* roll the other posts already stored down */
            for ( $j = count( $this->post ) - 1; $j > $i; $j-- ) {
               $this->post[ $j ] = $this->post[ $j - 1 ];
            }
         }
         
         $this->post[ $i ] = $aPost;
         return true;
      }
      else {
         return false;
      }
   }
   
   /* will read all the data from the cache, and then will sort the posts
      in date order */
   function doAction( ) {
      /* get the settings for collecting and displaying bloogroll posts */
      $settings = $this->getRowsFromQueryCommand( "=|BloogrollPosts.settings.get|=" );

      if ( isset( $settings ) ) {
         $cronTabCollection = $settings[ 0 ][ "bloogroll_posts_crontab_collection" ];
         $this->maxPosts = $settings[ 0 ][ "bloogroll_posts_num_posts" ];
      }
      else { /* set to defaults */
         $cronTabCollection = 'Y';
         $this->maxPosts = 20;
      }      

      if ( $cronTabCollection != 'Y' ) { /* need to collect */
         $c =& $this->session->snapOnFactory( "BloogrollPostsRSSCollector", $this );
         $c->securityRole = $this->securityRole;
         $c->process( );   /* collect */
      }

      $cachedData = $this->getRowsFromQueryCommand( "=|BloogrollPosts.cachedposts.get|=" );

      if ( isset( $cachedData ) ) {

         foreach( $cachedData as $c ) {
            $serialized = $c[ "serialized_rss" ] . $c[ "serialized_rss_overflow_1" ] . $c[ "serialized_rss_overflow_2" ];
            $rssData = unserialize( $serialized );
            $items = count( $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ] );
            /* determine an hour adjustment, if one is needed */
            $hourAdjustment = $rssData[ "HOUR_ADJUSTMENT" ];
            if ( !is_numeric( $hourAdjustment ) ) {
               $hourAdjustment = 0;
            }
            
            $traversing = true;
            for ( $i=0; $i < $items && $traversing; $i++ ) {
               $post = array( );
               /* gather pertinent data. Notice that we have to try various fields
                  as not all RSS is created equal */
               
               $post[ "post_blog_title" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "TITLE" ];
               $post[ "post_title" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "TITLE" ];
               $post[ "post_blog_url" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "LINK" ];
               if ( !isset( $post[ "post_blog_url" ] ) ) {
                  $post[ "post_blog_url" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "GUID" ];
               }
               $post[ "post_date" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "PUBDATE" ];
               if ( !isset( $post[ "post_date" ] ) ) {
                  $post[ "post_date" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "DC:DATE" ];
               }

               $post[ "post_comments_url" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "COMMENTS" ];


               /* do some modifications on post date it is nn ISO 8601 date (i.e. yyyy-mm-ddThh:mm:ss-hh:ss).
                  PHP 4.x doesn't understand that format */
               if ( ereg( "([0-9]+-[0-9]+-[0-9]+)T([0-9]+:[0-9]+:[0-9]+[-\+][0-9]+):([0-9]+)", $post[ "post_date" ]
                    , $match ) ) {
                  /* recreate the date, using the parts of the format that work in PHP 4.x */
                  $post[ "post_date" ] = $match[1] . " " . $match[2] . $match[3];
               }   
               
               /* if there is an hour adjustment needed, make it here */
               if ( $hourAdjustment != 0 ) {
                  $post[ "post_date" ] = gmdate( "Y-m-d H:i:s", strtotime( $post[ "post_date" ] ) + 
                                                                         ( $hourAdjustment * 3600 ) ) . "+0000";
               }
               
               $post[ "post_author" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "DC:CREATOR" ];
               $post[ "post_content" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "CONTENT:ENCODED" ];
               if ( !isset( $post[ "post_content" ] ) ) {
                  $post[ "post_content" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "ITEM" ][ $i ][ "DESCRIPTION" ];
               }

               /* show a post image (if there is one) */
               $post[ "post_blog_image_url" ] = $rssData[ "RSS" ][0][ "CHANNEL" ][0][ "IMAGE" ][ 0 ][ "URL" ];
               
               if ( !$this->storePost( $post ) ) {
                  $traversing = true;
               }
            }
         }
      }

      /* reset */
      $this->numPosts = count( $this->post );
      $this->currentPost = 0;

      parent::doAction( );
   }
   
   /* return each row of the post array until none are left */
   function getResultRow( ) {
      if ( $this->currentPost < $this->numPosts ) {
         $postTime = $this->session->getLocalDate( BASE_TIME_FORMAT, $this->post[ $this->currentPost ][ "post_date" ] );
         
         /* convert post date to local date */
         $this->post[ $this->currentPost ][ "post_date" ] = $postTime[ 0 ];
         $this->post[ $this->currentPost ][ "post_time_zone_abbrev" ] = $postTime[ 1 ];

         return $this->post[ $this->currentPost++ ];
      }
      else {
         return null;
      }
   }
}

class BloogrollPostsLink extends PhooQuery {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsLink";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|BloogrollPosts.link|=";
   var $command = "=|BloogrollPosts.settings.get|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-13" );
      $this->setPurpose( "Displays the bloogroll posts link" );
   }
}

class BloogrollPostsTitle extends PhooQuery {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsTitle";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|BloogrollPosts.title|=";
   var $command = "=|BloogrollPosts.settings.get|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-13" );
      $this->setPurpose( "Displays the bloogroll posts title" );
   }
}

class BloogrollPostsByline extends PhooDisplay {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsByline";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|BloogrollPosts.post.byline|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-06" );
      $this->addExpects( "post_author", "CONTAINER" );
      $this->setPurpose( "Displays a bloogroll byline, if the post author exists" );
   }

   function setup( ) {
      parent::setup( );

      $postAuthor = $this->getVar( "post_author", "PRIVATE" );

      /* if no post author, disable the display */
      if ( !isset( $postAuthor ) ) {
         $this->disabled = true;
      }
   }
}

class BloogrollPostsBlogImage extends PhooDisplay {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsBlogImage";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|BloogrollPosts.blog.image|=";
   var $maxImageWidth = 150;
   var $maxImageHeight = 150;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-15" );
      $this->addExpects( "post_blog_image_url", "CONTAINER" );
      $this->addExpects( "post_blog_image_height", "CONTAINER" );
      $this->addExpects( "post_blog_image_width", "CONTAINER" );
      $this->setPurpose( "Displays a bloogroll blog image, if one exists exists" );
   }

   function setup( ) {
      parent::setup( );

      $postBlogImageUrl = $this->getVar( "post_blog_image_url", "PRIVATE" );

      /* if no image, disable the display */
      if ( !isset( $postBlogImageUrl ) ) {
         $this->disabled = true;
      }
      else {
         $postBlogImageWidth = $this->getVar( "post_blog_image_width", "PRIVATE" );
         $postBlogImageHeight = $this->getVar( "post_blog_image_height", "PRIVATE" );

         /* set the height and width if set too large */

         /* first, see if we have dimensions */
         if ( !isset( $postBlogImageWidth ) ) {
            /* get the dimensions */
            $dimensions = getimagesize( $postBlogImageUrl );

            $postBlogImageWidth = $dimensions[ 0 ];
            $postBlogImageHeight = $dimensions[ 1 ];
         }

         /* check the width and height */
         if ( $postBlogImageWidth > $this->maxImageWidth ) {
            /* keep the proportions the same */
            $postBlogImageHeight = round( $postBlogImageHeight * 
                          ( $this->maxImageWidth / $postBlogImageWidth ) );
            $this->setVar( "post_blog_image_width", $this->maxImageWidth, "PRIVATE" );
            $this->setVar( "post_blog_image_height", $postBlogImageHeight, "PRIVATE" );
         }
         
         if ( $postBlogImageHeight > $this->maxImageHeight ) {
            /* keep the proportions the same */
            $postBlogImageWidth = round( $postBlogImageWidth * 
                          ( $this->maxImageHeight / $postBlogImageHeight ) );
            $this->setVar( "post_blog_image_width", $postBlogImageWidth, "PRIVATE" );
            $this->setVar( "post_blog_image_height", $this->maxImageHeight, "PRIVATE" );
         }
      }      
   }
}

class BloogrollPostsCommentsLink extends PhooDisplay {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsCommentsLink";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|BloogrollPosts.comments.link|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-06" );
      $this->addExpects( "post_comments_url", "CONTAINER" );
      $this->setPurpose( "Displays a bloogroll comments link, if the link exists" );
   }

   function setup( ) {
      parent::setup( );

      $postCommentsUrl = $this->getVar( "post_comments_url", "PRIVATE" );

      /* if no url, disable the display */
      if ( !isset( $postCommentsUrl ) ) {
         $this->disabled = true;
      }
   }
}

/*********************************************/
/*** SnapOns for administration            ***/
/*********************************************/
class BloogrollPostsAdminPageSection extends PhooDisplay {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{BloogrollPostsAdminForm}_";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-06-26" );
      $this->setPurpose( "Page for administrating the bloogroll posts snapon" );
   }
}

/* Administration Form 
*/
class BloogrollPostsAdminForm extends PhooForm {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|BloogrollPosts.settings.form|=";
   var $command = "=|BloogrollPosts.settings.update|=";
   var $loadCommand = "=|BloogrollPosts.settings.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                       );
   var $action = "update";                       

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-06-26" );
      $this->setPurpose( "Displays and processes the bloogroll posts settings administration form" );
   }

   /* set the action to Update */
   function setup( ) {
      parent::setup( );

      $this->setVar( "action", "update", "GET" );

      /* for if this works */
      $extensionName = $this->getVar( "extension_name", "GET" );
      $cat = $this->session->snapOn[ $extensionName ][ "category" ];
      
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&subpage=" . urlencode( $cat ) );
   }
}

/**************************************************************/
/* Activator SnapOn                                           */
/* creates the cache table if none exists                     */
/**************************************************************/
class BloogrollPostsActivator extends PhooProcess {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsActivator";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|BloogrollPosts.table.create|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-06-26" );
      $this->setPurpose( "Controls activation of the bloogroll posts snapon" );
   }

   /* checks to see if the bloogroll posts rss cache table exists. If it doesn't, creates it.
      also adds it into the settings table if not there */
   function doAction( ) { 
      /* sees if the setting "bloogroll_post_rss_table" exists. */
      $bprTable = $this->getVar( "bloogroll_posts_rss_table" );
      if ( !isset( $bprTable ) ) {
         /* create the setting */
         /* first, grab the name of the posts table - figure out the
            table prefix from that */
         $postTable = $this->getVar( "post_table", "APPLICATION" );
         $tablePrefix = substr( $postTable, 0, strpos( $postTable, "_" ) + 1 );
        
         /* create name of table */
         $this->setVar( "bloogroll_posts_rss_table", $tablePrefix . "bloogroll_posts_rss", "APPLICATION" ); 

         /* get the next id in the settings table for settings */
         /* set the list type here */
         $s =& $this->session->snapOnFactory( "PhooQuery", $this );
         $s->securityRole = $this->securityRole;
         $s->command = "=|List.newid.get|=";
         $s->setVar( "list_type", "settings", "PRIVATE" ); 
         $s->process( );
         if ( $s->isError( ) ) {
            return;
         }
         $row = $s->getResultRow( );
         
         $s =& $this->session->snapOnFactory( "PhooProcess", $this );
         $s->securityRole = $this->securityRole;
         $s->command = "=|BloogrollPosts.setting.tablename.create|=";
         $s->setVar( "new_id", $row[ "new_id" ], "PRIVATE" );
         $s->process( );
         if ( $s->isError( ) ) {
            return;
         }
         /* load the variable for later use */
         $bprTable = $this->getVar( "bloogroll_posts_rss_table", "APPLICATION" );
      }

      /* create setting records if necessary */
      $settings = $this->getRowsFromQueryCommand( "=|BloogrollPosts.settings.get|=" );
            
      if ( !isset( $settings ) ) { /* no rows exist, create the setting */
         /* there are multiple statements in the create so use the multi process */
         $p =& $this->session->snapOnFactory( "PhooMultiProcess", $this );
         $p->securityRole = $this->securityRole;
         $p->command = "=|BloogrollPosts.settings.create|=";
         $p->process( );
         if ( $this->isError( ) ) {
            return;
         }
      }   

      /* now see if the table exists */ 
      if ( $this->session->connection->tableExists( $bprTable ) ) {
         /* table exists, just return */
         return;
      }
      
      /* table doesn't exist, go ahead and process the snapon */
      parent::doAction( );
   }
}

/**************************************************************/
/* BloogrollPosts Event Handler                               */
/* Checks for a BloogrollPosts GET parameter. If found,       */
/* replaces the content section of the PostPageSection        */
/**************************************************************/
class BloogrollPostsContentSwitcher extends PreProcessEventHandler {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsContentSwitcher";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-06" );
      $this->setPurpose( "switches the content of the PostPageSection to Bloogroll Posts if the command has been passed" );
   }

   function doAction( ) {
      parent::doAction( );

      /* check for a Get variable called BloogrollPosts */
      $bloogrollPosts = $this->getVar( "BloogrollPosts", "GET" );
      if ( isset( $bloogrollPosts ) ) {
         /* replace the template for the container */
         $this->container->template = "_{BloogrollPosts}_";
      }
   }
}

/**************************************************************/
/* BloogrollPosts Event Handler                               */
/* Checks for a BloogrollPostsCollect GET parameter. If found */
/* runs a collection and returns simple output of results     */
/**************************************************************/
class BloogrollPostsCollectorChecker extends PreProcessEventHandler {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsCollectorChecker";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-06" );
      $this->setPurpose( "switches the operation of the Bloo Main Page to call the rss collector, rather than " .
                         "display a normal page." );
   }

   function doAction( ) {
      parent::doAction( );
      
      /* might be called from a command line. If so, add the command line
         arguments in as GET vars */
      if ( $argv ) {
         for ($i=1; $i < count( $argv ); $i++ )
         {
            $it = split( "=", $argv[ $i ] );
            $this->setVar( $it[ 0 ], $it[ 1 ], "GET" );
         }
      }
      
      /* check for a Get variable called BloogrollPosts */
      $bloogrollPostsCollect = $this->getVar( "BloogrollPostsCollect", "GET" );
      if ( isset( $bloogrollPostsCollect ) ) {
         /* run the collection process */
         $c =& $this->session->snapOnFactory( "BloogrollPostsRSSCollector", $this );
         $c->securityRole = $this->securityRole;
         $c->process( );   /* collect */
         
         /* set result logs into private variables */
         $this->container->setVar( "bloogroll_posts_log_error", $c->logError, "PRIVATE" );
         $this->container->setVar( "bloogroll_posts_log_status", $c->logStatus, "PRIVATE" );
         /* set the current gmt date */
         $this->container->setVar( "bloogroll_posts_gmt_date", gmdate( "Y-m-d H:i:s" ), "PRIVATE" );

         /* replace the template for the container */
         $this->container->template = "=|BloogrollPosts.collector.status.xml|=";
         $this->container->templateHeader = "";
         $this->container->templateFooter = "";
         $this->container->templateMenu = "";
      }
   }
}

/**************************************************************/
/* Collector SnapOn                                           */
/* Collects RSS streams and puts them in the cache            */
/**************************************************************/
class BloogrollPostsRSSCollector extends PhooProcess {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsRSSCollector";
   var $securityRole = ROLE_EVERYONE;
   var $logError = "";
   var $logStatus = "";
   var $maxTextLength = 64000;
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-06-29" );
      $this->setPurpose( "Collects Bloogroll RSS feeds and caches them" );
   }
   
   function doAction( ) {
      /* check the timestamp of last collection. If it's within 4 minutes and 59 seconds of now (GMT)
         don't run the collection - some safety against flooding */
      /* read each rss feed from the bloogroll */
      $timestamp = $this->getRowsFromQueryCommand( "=|BloogrollPosts.settings.timestamp.get|=" );
      
      if ( isset( $timestamp ) ) {
         $lastCollectionTimestamp = $timestamp[0][ "bloogroll_posts_timestamp" ];
         if ( $lastCollectionTimestamp > time( ) - ( 4*60 + 59 ) ) {
            $this->logError .= "An rss collection was run less than five minutes ago. No need to run again\n";
            return;
         }   
      }
      
      $updateOrInsert = false;
      
      /* read each rss feed from the bloogroll */
      $rssFeed = $this->getRowsFromQueryCommand( "=|BloogrollPosts.rssfeeds.get|=" );

      if ( !isset( $rssFeed ) ) {
         if ( $this->isError( ) ) {
            /* log the error */
            While ( $error = $x->getError( ) ) {
               $this->logError .= $error . "\n";               
            }
         }
         else {
            $this->logError .= "There are no Bloogroll RSS feeds to be read\n";
         }
         return;
      }            
         
      foreach ( $rssFeed as $rss ) {
         /* parse the rss */
         $x = new XML2ArrayParser( $rss[ "rss_feed" ] );

         if ( $x->isError( ) ) {
            /* just log the error - don't want to stop the process because of a bad rss somewhere */
            While ( $error = $x->getError( ) ) {
               $this->logError .= $error . "\n";               
            }
         }
         else {
            /* check the lastBuildDate against what's in the cache */
            /* will need to see what's available - lastBuildDate or RSS.CHANNEL.PUBDATE. Use BuildDate by 
               preferably */
            $lastBuildDate = $x->tagData[ "RSS" ][0][ "CHANNEL" ][0][ "LASTBUILDDATE" ];

            if ( !isset( $lastBuildDate ) ) {
               $lastBuildDate = $x->tagData[ "RSS" ][0][ "CHANNEL" ][0][ "PUBDATE" ];
            }

            /* Note: also checking DC:DATE if needed */
            if ( !isset( $lastBuildDate ) ) {
               $lastBuildDate = $x->tagData[ "RSS" ][0][ "CHANNEL" ][0][ "DC:DATE" ];
            }

            /* if there is no build date, just set to the nearest hour. This will update the RSS
               in the cache once an hour */
            if ( !isset( $lastBuildDate ) ) { 
               $lastBuildDate = $this->session->getLocalDate( "Y-m-d H" ) . ":00:00";
            }
            
            /* do some modifications on the last build date it is nn ISO 8601 date (i.e. yyyy-mm-ddThh:mm:ss-hh:ss).
               PHP 4.x doesn't understand that format */
            if ( ereg( "([0-9]+-[0-9]+-[0-9]+)T([0-9]+:[0-9]+:[0-9]+[-\+][0-9]+):([0-9]+)", $lastBuildDate
                 , $match ) ) {
               /* recreate the date, using the parts of the format that work in PHP 4.x */
               $lastBuildDate = $match[1] . " " . $match[2] . $match[3];
            }   

            /* initialize flag */
            $found = false;

            /* convert last build date to GMT - assumes that the GMT offset is formatted in the date */
            $gmtLastBuildDate = gmdate( "Y-m-d H:i:s", strtotime( $lastBuildDate ) );

            /* retrieve any cached data from the cache */
            $this->setVar( "rss_feed", $rss[ "rss_feed" ], "PRIVATE" );
            $cachedData = $this->getRowsFromQueryCommand( "=|BloogrollPosts.cacheurldate.get|=" );

            $command = "";
            $commandType = "";
            
            if ( isset( $cachedData ) ) {
               /* compare to the date in the cache. For now, if they are different,
                  reload the cache */
               if ( $cachedData[ 0 ][ "last_build_date" ] != $gmtLastBuildDate ) {
                  /* reloading the cache */
                  $command = "=|BloogrollPosts.cache.update|=";
                  $commandType = "update";
               }

               $found = true;                     
            }   

            if ( !$found ) {
               
               /* This feed has not been read before. will 
                  insert it into the cache */
               $command = "=|BloogrollPosts.cache.insert|=";
               $commandType = "update";

            }

            if ( $command > "" ) { /* need to insert or update */
               /* check the build date on the RSS - if it is more than 30 minutes in the future, 
                  assume a problem with the date on the server being queried, and insert a special
                  ADJUSTED_BUILD_DATE in the tagData, which represents the adjusted date (adjusted to 
                  the nearest date in the past for the minute passed */
               $currentGMDate =  gmdate( "Y-m-d H:i:s" );

               if ( $gmtLastBuildDate > $currentGMDate ) {
                  /* this rss was built in the future, which is impossible. Something wrong with the way
                     the post last build date is being stored. Adjust it back so it's earlier than the current GM 
                     date */
                  $secDiff = strtotime( $gmtLastBuildDate ) - strtotime( $currentGMDate );
                  $hourDiff = -ceil( $secDiff / 3600 );
                  $x->tagData[ "HOUR_ADJUSTMENT" ] = $hourDiff;
               }

               /* prepare the data. If serialized data is too long, break it up */
               /* Note: a design decision here: if an RSS has to be broken up into more than
                  three pieces due to length, don't store. This is a bit limited, but better
                  than having to have an overflow table or something */
               $serialized = addslashes( serialize( $x->tagData ) );
               $serializedOver1 = "";
               $serializedOver2 = "";
               $lengthProblem = false;
               if ( strlen( $serialized ) > $this->maxTextLength ) { /* field too long */
                  $serializedOver1 = substr( $serialized, $this->maxTextLength );
                  $serialized = substr( $serialized, 0, $this->maxTextLength );
                  if ( strlen( $serializedOver1 ) > $this->maxTextLength ) { /* field too long */
                     $serializedOver2 = substr( $serializedOver1, $this->maxTextLength );
                     $serializedOver1 = substr( $serializedOver1, 0, $this->maxTextLength );
                     if ( strlen( $serializedOver2 ) > $this->maxTextLength ) { /* Too long! bail . . . */
                        $lengthProblem = true;
                     }

                  }
               }
               if ( $lengthProblem ) {
                  $this->logError .= "rss for " . $rss[ "rss_feed" ] . " is too long\n";
               }
               else {
                  
                  $this->setVar( "serialized_rss", $serialized, "PRIVATE" );
                  $this->setVar( "serialized_rss_overflow_1", $serializedOver1, "PRIVATE" );
                  $this->setVar( "serialized_rss_overflow_2", $serializedOver2, "PRIVATE" );
                  $this->setVar( "last_build_date", $gmtLastBuildDate, "PRIVATE" );
                  $this->setVar( "rss_feed", $rss[ "rss_feed" ], "PRIVATE" );
                  $this->runModificationCommand( $command );
                  if ( $this->isError( ) ) {
                     /* just log the error - don't want to stop the process because of a bad rss somewhere */
                     While ( $error = $x->getError( ) ) {
                        $this->logError .= $error . "\n";               
                     }
                  }
                  else {
                     /* log the successful update */
                     $updateOrInsert = true;
                     if ( $commandType == "update" ) {
                        $this->logStatus .= "updated rss for " . $rss[ "rss_feed" ] . " in the cache\n";
                     }
                     else {
                        $this->logStatus .= "inserted rss for " . $rss[ "rss_feed" ] . " in the cache\n";
                     }                  
                  }
               }
            }
         }
      }

      if ( !$updateOrInsert ) {
         $this->logStatus .= "No rss feeds were updated or inserted.\n";
      }
      /* update the timestamp (flood prevention) */
      $this->setVar( "bloogroll_posts_collection_timestamp", time( ), "PRIVATE" );
      $this->runModificationCommand( "=|BloogrollPosts.settings.timestamp.update|=" );
      if ( $this->isError( ) ) {
         /* log the error */
         While ( $error = $x->getError( ) ) {
            $this->logError .= $error . "\n";               
         }
      }
   }
}

class BloogrollPostsCronDropDown extends YesNoDropDown {
   var $package = "bloogroll_posts";
   var $name = "BloogrollPostsCronDropDown";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|BloogrollPosts.cron.dropdown.header|=";
   var $template = "=|BloogrollPosts.cron.dropdown.item|=";
   var $templateFooter = "=|BloogrollPosts.cron.dropdown.footer|=";
   var $dropDownKeyName = "bloogroll_posts_crontab_collection";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-10" );
      $this->setPurpose( "Displays a drop-down containing Yes or No for the Crontab question" );
   }
}

?>
