<?php
/* 
   Name: contact.classes.php
   Author: Bill Roberts
   Created: 10/4/2007 

   Purpose: Provides for a contact me/us type form for users to interact with 
            blog admins and post authors

   $Revision: 1.3 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
            
*/           
  
class ContactForm extends PhooForm { 
   var $name = "ContactForm";
   var $securityRole = ROLE_EVERYONE;
   var $trigger = array( "btnSubmit" => array( "process" => "submit" ) );
   var $field = array( "contact_email" => array( "required" => "Y"
                                                ,"formalName" => "Email" )
                      ,"contact_message" => array( "required" => "Y"
                                                   ,"formalName" => "Message" )
                     );
   var $from;
   var $email;
   var $subject;
   var $subjectPrefix;
   var $message;
   var $meta = array( );
   var $dontProcessCommand = true; /* prevents errors since there is no sql associated with this trigger */

   function doAction( ) {
      parent::doAction( );
      
      if ( $this->isError( ) ) {
         return;
      }

      /* populate email information - from post vars */
      $p = $this->getVars( "POST" );
      foreach ( $p as $key => $value ) {
         switch( $key ) {
            case "contact_email" :
               $this->from = $this->getVar( "contact_email", "POST" );
               break;

            case "contact_subject" :
               $this->subject = $this->getVar( "contact_subject", "POST" );
               break;

            case "contact_message" :
               $this->message = $this->getVar( "contact_message", "POST" );
               break;

            case "btnSubmit" :
            case "SnapOnFormName" :
               /* don't capture these */
               break;

            default : /* for extra fields besides these */
               /* anything else goes in the meta */
               $this->meta[ $key ] = $value;
               break;
         }
      }
   }

   function sendMail( ) {
      /* create a mail message */
      $email = $this->email;
      $subject = $this->subjectPrefix . $this->subject;
      $message = $this->message;
      $from = $this->from;   
     
      foreach ( $this->meta as $key => $value ) {
         if ( $value > "" && $key != "post_id" ) {
            $message .= "\n $key = $value";
         }
      }

      $s =& $this->session->snapOnFactory( "MailSender", $this );

      $s->setMail( $email, $subject, $message, $from );

      $s->process( );
   }      

   function redirect( ) {
      /* overrides. Just set to the success template */
      $this->template = "=|Contact.success|=";
   }
} 

class ContactFormAdministrator extends ContactForm { 
   var $name = "ContactFormAdministrator";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Contact.form.administrator|=";

   function doAction( ) {
      /* see if the form was submitted */
      $btnSubmit = $this->getVar( "btnSubmit", "POST" );
      if ( $btnSubmit == "" ) {
         return;
      }
      
      /* ancestor creates the email information */
      parent::doAction( );
      
      if ( $this->isError( ) ) { /* validation error */
         return;
      }
      
      /* determine who the administrator is, based on the administration form for this extension */
      $rows = $this->getRowsFromQueryCommand( "=|Contact.getParameters|=" );

      /* if no administrator chosen, choose All */
      if ( !is_array( $rows ) ) {
         $adminUserId = 0; /* 0 will represent all */
      }
      else {
         $adminUserId = $rows[ 0 ][ "contact_admin_user_id" ];
      }

      $emailList = "";
      
      /* if All, populate array with all admins, otherwise just populate array with the admin chosen */
      if ( $adminUserId == 0 ) {
         $rows = $this->getRowsFromQueryCommand( "=|Contact.getAdminEmails|=" );

      }
      else { 
         /* get the email for the userid selected */
         $this->setVar( "admin_user_id", $adminUserId, "PRIVATE" );
         $rows = $this->getRowsFromQueryCommand( "=|Contact.getUserEmail|=" );
      }
      
      if ( !is_array( $rows ) ) {
         $this->setError( "Could not determine admin email(s) for Contact", MSG_WARNING );
         return;
      }
      else {
         $first = true;
         foreach ( $rows as $row ) {
            if ( $first ) {
               $first = false;
            }
            else {
               $emailList .= ","; /* add a comma separator */
            }

            $emailList .= $row[ "email" ];
         }
      }

      $this->email = $emailList;
      
      /* create the subject prefix */
      $blogName = $this->session->getSetting( "blog_name" );
      $this->subjectPrefix = $blogName . " - Message to Admin: ";

      $this->sendMail( );

      /* set to the success template after the mail is sent */
      $this->redirect( );
   }
} 

class ContactFormAuthor extends ContactForm { 
   var $name = "ContactFormAuthor";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|Contact.form.author|=";
   var $postTitle;
   var $postAuthor;
   
   function setup( ) {
      parent::setup( );

      /* get the post information */
      $rows = $this->getRowsFromQueryCommand( "=|Post.get|=" );

      $this->postTitle = $rows[ 0 ][ "post_title" ];
      $this->postAuthor = $rows[ 0 ][ "post_author" ];

      $this->setVar( "post_author", $this->postAuthor, "PRIVATE" );

   }

   function doAction( ) {
      /* see if the form was submitted */
      $btnSubmit = $this->getVar( "btnSubmit", "POST" );
      if ( $btnSubmit == "" ) {
         return;
      }
      
      /* ancestor creates the email information */
      parent::doAction( );
      
      if ( $this->isError( ) ) { /* validation error */
         return;
      }
      
      /* get the post author email */
      $rows = $this->getRowsFromQueryCommand( "=|Post.authoremail.get|=" );

      /* if no administrator chosen, choose All */
      if ( !is_array( $rows ) ) {
         $this->setError( "Could not determine post author email", MSG_WARNING ); /* there's some problem somewhere */
         return;
      }

      $this->email = $rows[ 0 ][ "post_author_email" ];
      
      /* create the subject prefix */
      $this->subjectPrefix = "Message to " . $this->postAuthor . " regarding post \"" . $this->postTitle . "\": ";

      $this->sendMail( );

      /* set to the success template after the mail is sent */
      $this->redirect( );
   }
} 

/* replace the post display with the appropriate contact form 
   if a contact form request has been made */
class ContactProcessor extends PreProcessEventHandler {
   var $name = "ContactProcessor";
   var $securityRole = ROLE_EVERYONE;

   function setup( ) {
      $contactAdministrator = $this->getVar( "contact_administrator" );
      if ( $contactAdministrator == "1" ) {
         $this->container->template = "_{ContactFormAdministrator}_";
      }
      else {
         $contactAuthor = $this->getVar( "contact_author" );
         if ( $contactAuthor == "1" ) {
            $this->container->template = "_{ContactFormAuthor}_";
         }
      }
      parent::setup( );
   }
}

class ContactAdminPageSection extends PhooDisplay {
   var $name = "ContactAdminPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{ContactAdminForm}_";
}

/* note: this form is based off of PhooQuery, not PhooForm
   It manages a list of checkboxes */
class ContactAdminForm extends PhooQuery {
   var $name = "ContactAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|Contact.adminEmailList.header|=";
   var $template = "=|Contact.adminEmailList|=";
   var $templateFooter = "=|Contact.adminEmailList.footer|=";
   var $email = array( );
   var $emailIndex = 0;
   
   function doAction( ) {
      $btnUpdate = $this->getVar( "btnUpdate", "POST" ); /* see if an update has been requested */

      if ( !$btnUpdate ) { /* initialize */
         /* get list of currently selected administrators to contact from the settings table */
         $selectedRecipient = array( );
         $rows = $this->getRowsFromQueryCommand( "=|Contact.getParameters|=" );
         if ( is_array( $rows ) ) {
            /* put ids into the selectedRecipient array */
            $selectedRecipient = explode( ",", $rows[ 0 ][ "contact_admin_user_id" ] );
         }
         else { /* create the setting - it must not exist */
            $this->runModificationCommand( "=|Contact.createContactParametersSetting|=" );
            /* now try again */
            $rows = $this->getRowsFromQueryCommand( "=|Contact.getParameters|=" );
            if ( $this->isError( ) ) {
               return;
            }
         }
         $selectedRecipient = explode( ",", $rows[ 0 ][ "contact_admin_user_id" ] );
         $this->setVar( "contact_admin_link_title", $rows[ 0 ][ "contact_admin_link_title" ], "POST" );
         $this->setVar( "contact_author_link_title", $rows[ 0 ][ "contact_author_link_title" ], "POST" );
      }
      
      /* get list of emails that are possible to select from the user table */
      $rows = $this->getRowsFromQueryCommand( "=|Contact.adminEmailList.get|=" );

      /* store in member array */
      if ( is_array( $rows ) ) {
         foreach ( $rows as $row ) {
            array_push( $this->email, $row );
         }
      }

      /* now, check for an update. If one was requested, adjust for any changes made to the status
         of emails */
      if ( $btnUpdate > "" ) {
         /* store the list of userids */
         $userIdList = "";
         $selectedRecipient = array( ); 
         foreach ( $this->email as $email ) {
            if ( $this->getVar( $email[ "contact_user_id" ], "POST" ) == "on" ) { /* see if the email index was checked */
               if ( $userIdList > "" ) {
                  $userIdList .= ",";
               }
               $userIdList .= $email[ "contact_user_id" ];
               array_push( $selectedRecipient, $email[ "contact_user_id" ] ); /* store in the selected recipient array */
            }
         }

         /* store the user id list */
         $this->setVar( "user_id_list", $userIdList, "PRIVATE" );
         $this->runModificationCommand( "=|Contact.updateContactParametersSetting|=" );
         
         if ( !$this->isError( ) ) {
            $this->session->setSessionMessage( "Contact Settings Have Been Saved", MSG_INFO );
         }
      }

      /* now, set the checkbox status */
      for ( $i = 0; $i < count( $this->email ); $i++ ) {
         $this->email[ $i ][ "contact_checked" ] = in_array( $this->email[ $i ][ "contact_user_id" ], $selectedRecipient );
      }

      parent::doAction( );
   }
   
   function getResultRow( ) {
      if ( $this->emailIndex >= count( $this->email ) ) {
         return null;
      }
      else {
         $row = $this->email[ $this->emailIndex ];

         $this->emailIndex++;
         return $row;
      }
   }
}

class ContactCheckBox extends PhooDisplay {
   var $name = "ContactCheckBox";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|Contact.checkbox|=";

   function doAction( ) {

      $contactName = $this->container->getVar( "contact_name", "PRIVATE" );
      $contactUserId = $this->container->getVar( "contact_user_id", "PRIVATE" );
      $contactChecked = $this->container->getVar( "contact_checked", "PRIVATE" );

      if ( $contactChecked ) {
         $this->setVar( "contact_checked", "CHECKED", "PRIVATE" );
      }

      $this->setVar( "contact_name", $contactName, "PRIVATE" );
      $this->setVar( "contact_user_id", $contactUserId, "PRIVATE" );
   }
}

class ContactLink extends PhooDisplay {
   var $name = "ContactLink";
   var $securityRole = ROLE_EVERYONE;
   var $parameters;
   
   function setup( ) {
      /* get the parameters for contacts */      
      $rows = $this->getRowsFromQueryCommand( "=|Contact.getParameters|=" );
      if ( !is_array( $rows ) ) {
         /* create the parameters */
         $this->runModificationCommand( "=|Contact.createContactParametersSetting|=" );
         /* now try again */
         $rows = $this->getRowsFromQueryCommand( "=|Contact.getParameters|=" );
         if ( $this->isError( ) ) {
            return;
         }

      }
      $this->parameters = $rows[ 0 ];

      parent::setup( );
   }
}

class ContactAdminLink extends ContactLink {
   var $name = "ContactAdminLink";
   var $template = "=|Contact.admin.link|=";

   function setup( ) {
      parent::setup( );

      $this->setVar( "contact_admin_link_title", $this->parameters[ "contact_admin_link_title" ], "PRIVATE" );
   }      
}

class ContactAuthorLink extends ContactLink {
   var $name = "ContactAuthorLink";
   var $template = "=|Contact.author.link|=";

   function setup( ) {
      parent::setup( );

      $this->setVar( "contact_author_link_title", $this->parameters[ "contact_author_link_title" ], "PRIVATE" );
      $this->setVar( "post_id", $this->container->getVar( "post_id" ), "PRIVATE" );
   }      
}
?>
