<?php
/* 
   Name: ct_code.classes.php
   Author: Bill Roberts
   Created: 1/25/2007 

   Purpose: Extension which places obfuscated codes in the comment form and 
            trackback link to help prevent comment spam
           
            This is a port of the CTCode MT plugin, found on 
            alogblog (http://alogblog.com/movabletype/plugins/ccode_and_tcode_for_blocking_commenttrackback_spam_for_mt_32/). 
           
            This includes the obfuscation code found at 
            http://www.jottings.com/obfuscator/ 
            Email obfuscator script 2.1 by Tim Williams, University of Arizona
            Random encryption key feature by Andrew Moulden, Site Engineering Ltd
            This code is freeware provided these four comment lines remain intact
            A wizard to generate this code is at http://www.jottings.com/obfuscator

   $Revision: 1.4 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/            
  
/* utility class - not extended from Phoo */
class Munge {
   var $mungeCoded;
   var $mungeCipher;
   var $mungeMode;
   var $mungePath;
   var $mungeHidden;
   var $rnd_seed_c;
   
   function Munge( $address, $mode, $path='', $hidden='') {
	   $coded = '';
   	$unmixedkey = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
      $inprogresskey = $unmixedkey;
   	$mixedkey = '';
   	$unshuffled = 62;

      $this->rnd_seed_c = time( );

      for( $i=0; $i<=61; $i++) {
         /* select a random character from the unmixed key */
	   	$ranpos = $this->_rand_c($unshuffled) - 1; 
   		$nextchar = substr($inprogresskey, $ranpos, 1);
         /* place it in the mixed key */
   		$mixedkey .= $nextchar;
         /* remove the character selected from the unmixed key */
   		$before = substr($inprogresskey, 0, $ranpos);
   		$after = substr($inprogresskey, $ranpos+1, $unshuffled);
   		$inprogresskey = $before.$after;
   		$unshuffled -= 1;
   	}
   
   	$cipher = $mixedkey;

   	$shifts = strlen( $address ) % strlen( $cipher );

   	for( $j=0; $j<strlen( $address ); $j++ ) {
         /* loop through every character of the item to be encrypted,
            locating it in the cipher */

   		if(strpos( $cipher, substr( $address, $j, 1) ) === false) {
            /* not found - just use the character */
		   	$chr = substr($address, $j, 1);
			   $coded .= substr($address, $j, 1);
   		}
	   	else {
            /* character is the position in the cipher of the character found, shifted by
               the length of the item being encrypted, and wrapped around */
   			$chr = ( strpos( $cipher, substr( $address, $j, 1) ) + $shifts ) % strlen( $cipher );
	   		$coded .= substr( $cipher, $chr, 1 );
		   }
   	}

      /* set variables */
      $this->mungeCoded = $coded;
      $this->mungeCipher = $cipher;
      $this->mungeMode = $mode;
      $this->mungePath = $path;
      $this->mungeHidden = $hidden;
   }

   function rnd_c() {
   	$this->rnd_seed_c = fmod( $this->rnd_seed_c * 9301 + 49297, 233280.0 );
   	return( $this->rnd_seed_c / 233280.0 );
   }

   function _rand_c($number) {
      return ceil( $this->rnd_c() * $number);
   }
}

class CTCodeCommentCode extends PhooDisplay { 
   var $name = "CTCodeCommentCode";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|CTCodeCommentCode.obfuscator|=";

   function doAction( ) {
      
      /* get post data */
      $rows = $this->getRowsFromQueryCommand( "=|Post.get|=" );
      if ( $this->isError( ) ) {
         return;
      }

      if ( is_array( $rows ) ) {
         $postDate = $rows[ 0 ][ "post_date" ];
         $postDate = strftime( "%Y%m%d%H%M%s", strtotime( $postDate ) );
         
         $code = $rows[0][ "post_title" ] . $postDate;
         $revisedCode = "";
         
         /* remove all non letter/number characters */
         for ( $i = 0; $i < strlen( $code ); $i++ ) {
            $chr = substr( $code, $i, 1 );
            if ( strpos( " ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", $chr ) ) {
               $revisedCode .= $chr;
            }
         }
      }

      /* Munge this */
      $m = new Munge( $revisedCode, "hidden_input" );
      
      $this->setVar( "munge_coded", $m->mungeCoded, "PRIVATE" );
      $this->setVar( "munge_cipher", $m->mungeCipher, "PRIVATE" );
      $this->setVar( "munge_mode", $m->mungeMode, "PRIVATE" );
      $this->setVar( "munge_path", $m->mungePath, "PRIVATE" );
      $this->setVar( "munge_hidden", $m->mungeHidden, "PRIVATE" );

      parent::doAction( );
   }
}

class CTCodeCommentCodeCheck extends CommentFilter {
   var $name = "CTCodeCommentCodeCheck";
   var $securityRole = ROLE_EVERYONE;

   function checkComment( ) {
      $ctCode = $this->getVar( "CCode", "POST" );

      $rows = $this->getRowsFromQueryCommand( "=|Post.get|=" );
      if ( $this->isError( ) ) {
         return;
      }

      if ( is_array( $rows ) ) {
         $postDate = $rows[ 0 ][ "post_date" ];
         $postDate = strftime( "%Y%m%d%H%M%s", strtotime( $postDate ) );
         
         $code = $rows[0][ "post_title" ] . $postDate;
         $revisedCode = "";
         
         /* remove all non letter/number characters */
         for ( $i = 0; $i < strlen( $code ); $i++ ) {
            $chr = substr( $code, $i, 1 );
            if ( strpos( " ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", $chr ) ) {
               $revisedCode .= $chr;
            }
         }
      }
      
      if ( $ctCode != $revisedCode ) {
         $this->container->setError( "Your comment looks like it might be spam." .
                                     " Please try again.", MSG_WARNING );
      }
   }
}

class CTCodeTrackbackCode extends PhooDisplay { 
   var $name = "CTCodeTrackbackCode";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|CTCodeTrackbackCode.obfuscator|=";


   function doAction( ) {
      /* get post data */
      $rows = $this->getRowsFromQueryCommand( "=|Post.get|=" );
      if ( $this->isError( ) ) {
         return;
      }

      if ( is_array( $rows ) ) {
         $postTitle = $rows[ 0 ][ "post_title" ];
         $postDate = $rows[ 0 ][ "post_date" ];
         /* will grab an arbitrary set of digits here - the second, 1 from the month, and the third letter of the title */
         /* note: since "." is not valid in the code match (since it will precede it), ensure that the third letter of
            the title is not a dot */
         $third = substr( $postTitle, 2, 1 );
         if ( $third == "." ) {
            $third = "d0t";
         }
         $code = substr( $postDate, 17, 2 ) . substr( $postDate, 6, 1 ) . $third;
         /* remove all non letter/number characters */
         $revisedCode = "";
         
         for ( $i = 0; $i < strlen( $code ); $i++ ) {
            $chr = substr( $code, $i, 1 );
            if ( strpos( " ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", $chr ) ) {
               $revisedCode .= $chr;
            }
         }
      }

      /* Munge this */
      $m = new Munge( $revisedCode, "__MTTBLINK__" );
      
      $this->setVar( "munge_coded", $m->mungeCoded, "PRIVATE" );
      $this->setVar( "munge_cipher", $m->mungeCipher, "PRIVATE" );
      $this->setVar( "munge_mode", $m->mungeMode, "PRIVATE" );
      $this->setVar( "munge_path", $m->mungePath, "PRIVATE" );
      $this->setVar( "munge_hidden", $m->mungeHidden, "PRIVATE" );
      
      parent::doAction( );
   }
}

class CTCodeTrackbackURIParse extends PreProcessEventHandler {
   var $securityRole = ROLE_EVERYONE;

   /* parse out the post_id and the ct_code, and save the ct_code in the trackback server */
   function doAction( ) {
      $pos = strrpos( $this->container->requestURI, "." );
      if ( $pos ) {
         $this->container->ctCode = substr( $this->container->requestURI, $pos+1 );
         $this->container->requestURI = substr( $this->container->requestURI, 0, $pos );
         /* this will be needed for a later query */
         $pos = strrpos( $this->container->requestURI, "/" );
         $this->container->ctCodePostId = substr( $this->container->requestURI, $pos+1 );
      }
   }
}

class CTCodeTrackbackCheck extends TrackbackFilter {
   var $securityRole = ROLE_EVERYONE;

   /* verify their answer against the list of possible answers */
   function checkTrackback( ) {
      /* this will check the ctCode passed in against what's expected for this trackback */
      /* get post data */
      $this->setVar( "post_id", $this->container->ctCodePostId, "PRIVATE" );
      $rows = $this->getRowsFromQueryCommand( "=|Post.get|=" );
      if ( $this->isError( ) ) {
         return;
      }

      if ( is_array( $rows ) ) {
         $postTitle = $rows[ 0 ][ "post_title" ];
         $postDate = $rows[ 0 ][ "post_date" ];
         /* will grab an arbitrary set of digits here - the second, 1 from the month, and the third letter of the title */
         /* note: since "." is not valid in the code match (since it will precede it), ensure that the third letter of
            the title is not a dot */
         $third = substr( $postTitle, 2, 1 );
         if ( $third == "." ) {
            $third = "d0t";
         }
         $code = substr( $postDate, 17, 2 ) . substr( $postDate, 6, 1 ) . $third;
         /* remove all non letter/number characters */
         $revisedCode = "";
         
         for ( $i = 0; $i < strlen( $code ); $i++ ) {
            $chr = substr( $code, $i, 1 );
            if ( strpos( " ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789", $chr ) ) {
               $revisedCode .= $chr;
            }
         }
      }
      
      if ( $revisedCode != $this->container->ctCode ) {
         $this->unapproveTrackback( "LagTimeSpamChecker", "Trackback failed spam check" );
      }
   }
}

?>
