<?php
/* 
   Name: favorite_posts.classes.php
   Author: Bill Roberts
   Created: 1/13/2008 
   Purpose: Displays and manages a list of favorite posts
   $Revision: 1.2 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/   
  
class FavoritePosts extends PhooQuery { 
   /* displays the favorite posts list */
   var $package = "favorite_posts";
   var $name = "FavoritePosts";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|FavoritePosts.list.get|=";
   var $templateHeader = "=|FavoritePosts.header|=";
   var $templateFooter = "=|FavoritePosts.footer|=";
   var $template = "=|FavoritePosts|=";
} 

/***************************************************/
/*** Classes for FavoritePosts administration    ***/
/***************************************************/
class FavoritePostsAdminPageSection extends PhooDisplay {
   var $package = "favorite_posts";
   var $name = "FavoritePostsAdminPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{FavoritePostsAdminDisplay}_";
   
   /* check the parameters passed to determine what to do */
   function setup( ) {
      parent::setup( );

      $action = $this->getVar( FORM_ACTION );
      switch( $action ) {
         case "insert" :
         case "update" :
            $this->template = "_{FavoritePostsAdminForm}_";
            
            break;
         case "delete" : 
            /* instantiate and run a deleter */
            $s =& $this->session->snapOnFactory( "FavoritePostsDeleter", $this );
            $s->process( );
            if ( $s->isError( ) ) {
               return;
            }
            break;
         default :
            break;
      }
   }
}

class FavoritePostsAdminDisplay extends PhooQuery {
   var $package = "favorite_posts";
   var $name = "FavoritePostsAdminDisplay";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|FavoritePosts.list.get|=";
   var $templateHeader = "=|FavoritePosts.list.header|=";
   var $template = "=|FavoritePosts.list|=";
   var $templateFooter = "=|FavoritePosts.list.footer|=";
}

class FavoritePostsAdminForm extends PhooForm {
   var $package = "favorite_posts";
   var $name = "FavoritePostsAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $updateForm = "=|FavoritePosts.form.update|=";
   var $insertForm = "=|FavoritePosts.form.insert|=";
   var $loadCommand = "=|FavoritePosts.getbyid|=";
   var $updateCommand = "=|FavoritePosts.update|=";
   var $insertCommand = "=|FavoritePosts.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   var $field = array( "favorite_posts_post_id" => array( "required" => "Y"
                                                         ,"formalName" => "Post ID" )
                      ,"list_order" => array( "required" => "Y"
                                             ,"formalName" => "Display Order" )                                                     
                     );

   function setup( ) {
      parent::setup( );
      
      /* set the list type here */
      $this->setVar( "list_type", "favorite_posts", "POST" );

      /* for if this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&action=administer&extension_name=" . $this->getVar( "extension_name" ) );
   }

   function validateForm( ) {
      parent::validateForm( );

      if ( !$this->isError( ) ) {
         /* verify that the post id set is valid */
         $rows = $this->getRowsFromQueryCommand( "=|FavoritePosts.post_exists.check|=" );
         if ( !is_array( $rows ) ) {
            $this->setError( "The post id entered is not a valid post id. There is no post available with that id.", MSG_WARNING );
         }      
      }
   }

   function triggerProcess( $aProcess ) {

      /* make sure the list_order is a number. If not, set to 0 */
      $listOrder = $this->getVar( "list_order", "POST" );
      if ( !is_numeric( $listOrder ) ) {
         $this->setVar( "list_order", 0, "POST" );
      }
      parent::triggerProcess( $aProcess );
   }
   
}

class FavoritePostsDeleter extends PhooProcess {
   var $package = "favorite_posts";
   var $name = "FavoritePostsDeleter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete a Favorite Posts list item. You do not have security privileges";
   var $command = "=|FavoritePosts.delete|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-06" );
      $this->setPurpose( "Controls deletion of a Favorite Posts list item" );
   }
}
?>
