<?php
/* Name: ip_blacklist.classes.php
   Author: Bill Roberts
   Created: 5/20/2006 

   Purpose: Extension related to managing an ip blacklist - for banning troublemakers and spammers from commenting
            and tracking back

   $Revision: 1.2 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/            
  
/*********************************************/
/*** SnapOns for Blacklist administration  ***/
/*********************************************/
class IPBlackListAdminPageSection extends PhooDisplay {
   var $package = "ip_blacklist";
   var $name = "IPBlackListAdminPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{IPBlackListAdminDisplay}_";
   
   /* check the parameters passed to determine what to do */
   function setup( ) {
      parent::setup( );

      $action = $this->getVar( FORM_ACTION );
      switch( $action ) {
         case "insert" :
         case "update" :
            $this->template = "_{IPBlackListAdminForm}_";
            
            break;
         case "delete" : 
            /* instantiate and run a deleter */
            $s =& $this->session->snapOnFactory( "IPBlackListDeleter", $this );
            $s->process( );
            if ( $s->isError( ) ) {
               return;
            }
            break;
         default :
            break;
      }
   }
}

class IPBlackListAdminDisplay extends PhooQuery {
   var $package = "ip_blacklist";
   var $name = "IPBlackListAdminDisplay";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|IPBlackList.list.get|=";
   var $templateHeader = "=|IPBlackList.list.header|=";
   var $template = "=|IPBlackList.list|=";
   var $templateFooter = "=|IPBlackList.list.footer|=";
}

/* Administration Form 
*/
class IPBlackListAdminForm extends PhooForm {
   var $package = "ip_blacklist";
   var $name = "IPBlackListAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $updateForm = "=|IPBlackList.form.update|=";
   var $insertForm = "=|IPBlackList.form.insert|=";
   var $loadCommand = "=|IPBlackList.getbyid|=";
   var $updateCommand = "=|IPBlackList.update|=";
   var $insertCommand = "=|IPBlackList.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   var $field = array( "ip_blacklist_ip" => array( "required" => "Y"
                                                  ,"formalName" => "IP Address" )
                                                 );
   function setup( ) {
      parent::setup( );
      
      /* set the list type here */
      $this->setVar( "list_type", "ip_blacklist", "POST" ); /* set in post vars to make it accessible to 
                                                            the new_id command */

      /* for if this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&action=administer&extension_name=" . $this->getVar( "extension_name" ) );
   }
}

class IPBlackListDeleter extends PhooProcess {
   var $package = "ip_blacklist";
   var $name = "IPBlackListDeleter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete this blacklisted ip. You do not have security privileges";
   var $command = "=|IPBlackList.delete|=";
}

/**************************************************************/
/* IPBlackListComment Event Handler                           */
/* Inherits from the CommentSpamChecker class - verifies that */
/* the IP address of the current commenter is not blacklisted */
/**************************************************************/
class IPBlackListComment extends CommentFilter {
   var $package = "ip_blacklist";
   var $name = "IPBlackListComment";
   var $securityRole = ROLE_EVERYONE;

   /* verify that the IP address is not blacklisted */
   function checkComment( ) {
      $commentAuthorIp = $this->container->getVar( "comment_author_ip" );
      
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole;
      $s->command = "=|IPBlackList.isBlackListed|=";
      $s->setVar( "ip_address", $commentAuthorIp, "PRIVATE" );
      $s->process( );
      if ( $s->isError( ) ) {
         return;
      }

      $blackListed = false;
      
      while ( $row = $s->getResultRow( ) ) {
         if ( $row[ "answer" ] == "yes" ) {
            $blackListed = true;
         }
      }

      if ( $blackListed ) {
         $this->unapproveComment( "IPBlackList", "IP Address is Blacklisted" );
      }
   }
}

/**************************************************************/
/* IPBlackListTrackback Event Handler                         */
/* Inherits from the CommentSpamChecker class - verifies that */
/* the IP address of the current commenter is not blacklisted */
/**************************************************************/
class IPBlackListTrackback extends TrackbackFilter {
   var $package = "ip_blacklist";
   var $name = "IPBlackListTrackback";
   var $securityRole = ROLE_EVERYONE;

   /* verify that the IP address is not blacklisted */
   function checkTrackback( ) {
      $trackbackIp = $this->container->getVar( "trackback_ip" );
      
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole;
      $s->command = "=|IPBlackList.isBlackListed|=";
      $s->setVar( "ip_address", $trackbackIp, "PRIVATE" );
      $s->process( );
      if ( $s->isError( ) ) {
         return;
      }

      $blackListed = false;
      
      while ( $row = $s->getResultRow( ) ) {
         if ( $row[ "answer" ] == "yes" ) {
            $blackListed = true;
         }
      }

      if ( $blackListed ) {
         $this->unapproveTrackback( "IPBlackList", "IP Address is Blacklisted" );
      }
   }
}
?>
