<?php
/* 
   Name: lag_time_spam_checker.classes.php 
   Author: Bill Roberts
   Created: 11/05/2006 

   Purpose: Extension related to checking for suspicious comments and trackbacks, based on the assumption 
   that comments or trackbacks left on older posts might be spam 
   
   $Revision: 1.4 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/            

/*********************************************/
/*** SnapOns for Question administration   ***/
/*********************************************/
class LagTimeSpamCheckerAdminPageSection extends PhooDisplay {
   var $package = "lag_time_spam_checker";
   var $name = "LagTimeSpamCheckerAdminPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{LagTimeSpamCheckerAdminForm}_";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-11-06" );
      $this->setPurpose( "Page for administrating the parameters for the lag time spam checker" );
   }
}

/* Administration Form 
*/
class LagTimeSpamCheckerAdminForm extends PhooForm {
   var $package = "lag_time_spam_checker";
   var $name = "LagTimeSpamCheckerAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|LagTimeSpamChecker.form|=";
   var $loadCommand = "=|LagTimeSpamChecker.parameters.get|=";
   var $command = "=|LagTimeSpamChecker.update|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" ) );
   var $action = "update"; /* this form is update only */
   var $field = array( "lag_time_spam_checker_comment_lag" => array( "required" => "Y"
                                                  ,"formalName" => "Comment Lag Time" )
                      ,"lag_time_spam_checker_trackback_lag" => array( "required" => "Y"
                                                  ,"formalName" => "Trackback Lag Time" )
                                                 );
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "Displays and processes the simple human check administration form" );
   }

   function setup( ) {
      parent::setup( );

      /* for if this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&subpage=Spamicide" );
   }
}

/**************************************************************/
/* Activator SnapOn                                           */
/* Inserts the default parameters into the list table if no   */
/* parameters exist                                           */
/**************************************************************/
class LagTimeSpamCheckerActivator extends PhooProcess {
   var $package = "lag_time_spam_checker";
   var $name = "LagTimeSpamCheckerActivator";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|LagTimeSpamChecker.activate|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-11-06" );
      $this->setPurpose( "Controls activation of the lag time spam checker" );
   }

   /* adds default parameters to the list table, if none exist */
   function process( ) { 
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole;
      $s->command = "=|LagTimeSpamChecker.parameters.exist|=";
      $s->process( );

      if ( $s->isError( ) ) {
         return;
      }

      $row = $s->getResultRow( );
      if ( is_array( $row ) ) {
         /* the parameters already exist */
         return;
      }

      /* no parameters exist, go ahead and process the snapon */
      parent::process( );
   }
}

/**************************************************************/
/* Initializer SnapOn                                         */
/**************************************************************/
class LagTimeSpamCheckerInitializer extends PhooProcess {
   var $package = "lag_time_spam_checker";
   var $name = "LagTimeSpamCheckerInitializer";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "Controls initialization of the lag time spam checker SnapOn" );
   }

   /* adds the comment check event to the list of event handlers */
   function process( ) { 
      /* note that the cost of a lag time spam checker is considered in the middle of the 0-100 cost
         range, as it does db calls */
      $this->session->addEvent( "CommentForm", EVENT_PRESAVE, "LagTimeSpamCheckerComment", 50 );
      $this->session->addEvent( "TrackbackServer", EVENT_PRESAVE, "LagTimeSpamCheckerTrackback", 50 );
   }
}

/**************************************************************/
/* LagTimeSpamChecker Event Handler                           */
/* Inherits from the CommentFilter class - checks the lag time*/
/* between a comment and the post that is being commented on  */
/**************************************************************/
class LagTimeSpamCheckerComment extends CommentFilter {
   var $package = "lag_time_spam_checker";
   var $name = "LagTimeSpamCheckerComment";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-11-06" );
      $this->setPurpose( "checks the lag time on comments to see if they are hitting old posts." );
   }

   function checkComment( ) {
      /* this will check the latest comment for the post in question, and will compare the dates on that
         comment to now. If the difference is greater than the lag, this comment will be considered suspected spam
       */

      /* get the day differential between now and the post in question */
      $rows = $this->getRowsFromQueryCommand( "=|LagTimeSpamChecker.comment_date.get|=" );
      if ( is_array( $rows ) && $rows[ 0 ][ "comment_date" ] > "" ) {
         $differential = strtotime( "now" ) - strtotime( $rows[ 0 ][ "comment_date" ] );
         $dayDiff = round( $differential / ( 60*60*24 ) );
      }
      else { /* no comments, check the post age */
         $rows = $this->getRowsFromQueryCommand( "=|LagTimeSpamChecker.post_date.get|=" );
         if ( is_array( $rows ) ) {
            $differential = strtotime( "now" ) - strtotime( $rows[ 0 ][ "post_date" ] );
            $dayDiff = round( $differential / ( 60*60*24 ) );
         }
      }

      /* retrieve the parameters for this filter */
      $rows = $this->getRowsFromQueryCommand( "=|LagTimeSpamChecker.parameters.get|=" );
      if ( is_array( $rows ) ) {
         if ( $dayDiff > $rows[ 0 ][ "lag_time_spam_checker_comment_lag" ] ) {
            /* comment failed */
            $this->unapproveComment( "LagTimeSpamChecker", "Comment made on older post" );
            return;
         }
      }
   }
}

/******************************************************************/
/* LagTimeSpamCheckerTrackback Event Handler                      */
/* Inherits from the TrackbackFilter class - checks the lag time  */
/* between a trackback and the post that is being tracked back    */
/******************************************************************/
class LagTimeSpamCheckerTrackback extends TrackbackFilter {
   var $package = "lag_time_spam_checker";
   var $name = "LagTimeSpamCheckerTrackback";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-11-06" );
      $this->setPurpose( "checks the lag time on trackbacks to see if they are hitting old posts." );
   }

   function checkTrackback( ) {
      /* this will check the create date for the post in question, and will compare the dates on that
         to now. If the difference is greater than the lag, this trackback will be considered suspected spam
       */
      $this->setVar( "post_id", $this->container->getVar( "post_id" ), "PRIVATE" );

      /* get the day differential between now and the post in question */
      $rows = $this->getRowsFromQueryCommand( "=|LagTimeSpamChecker.post_date.get|=" );
      if ( is_array( $rows ) ) {
         $differential = strtotime( "now" ) - strtotime( $rows[ 0 ][ "post_date" ] );
         $dayDiff = round( $differential / ( 60*60*24 ) );
      }

      /* retrieve the parameters for this filter */
      $rows = $this->getRowsFromQueryCommand( "=|LagTimeSpamChecker.parameters.get|=" );
      if ( is_array( $rows ) ) {
         if ( $dayDiff > $rows[ 0 ][ "lag_time_spam_checker_trackback_lag" ] ) {
            /* trackback failed */
            $this->unapproveTrackback( "LagTimeSpamChecker", "Trackback made on older post" );
            return;
         }
      }
   }
}

?>
