<?php
/* Name: popular_posts.classes.php
   Author: Bill Roberts
   Created: 3/23/2006

   Purpose: Display the most commented posts

   $Revision: 1.4 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   
*/   

class PopularPosts extends PhooQuery {
   var $package = "popular_posts";
   var $name = "PopularPosts";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|PopularPosts.get|=";
   var $templateHeader = "=|PopularPosts.header|=";
   var $template = "=|PopularPosts|=";
   var $templateFooter = "=|PopularPosts.footer|=";
   var $postsToDisplay;
   var $timeHorizon;
   
   function setup( ) {
      /* get parameters for this extension */
      $parms = $this->getRowsFromQueryCommand( "=|PopularPosts.get.parameters|=" );
      
      if ( is_array( $parms )  ) {
         $this->postsToDisplay = $parms[ 0 ][ "popular_posts_posts_to_display" ];
         $this->timeHorizon = $parms[ 0 ][ "popular_posts_time_horizon" ];
      }
      else { /* create the settings */
         $this->runModificationCommand( "=|PopularPosts.create.parameters|=" );
      }

      if ( $this->isError( ) ) { /* turn any error into a warning, so the snapOn and page will still display */
         $msg = $this->getError( );
         $this->setError( $msg, MSG_WARNING );
      }

      if ( $this->postsToDisplay == "" ) { /* set to default */
         $this->postsToDisplay = 10;
      }

      if ( $this->timeHorizon == "" ) { /* set to default */
         $this->timeHorizon = 0;
      }

      if ( $this->timeHorizon > 0 ) {
         $localTime = $this->session->getLocalDate( "Y-m-d" );
         $this->setVar( "where_clause_adder", " and c.create_date >= DATE_SUB( '" . $localTime[ 0 ] . "', INTERVAL " . 
                                                                               $this->timeHorizon . " DAY )", "PRIVATE" );
      }                                                                              

      $this->setVar( "popular_posts_time_horizon", $this->timeHorizon, "PRIVATE" );
      $this->setVar( "results_limit", $this->postsToDisplay, "PRIVATE" ); 
     
      parent::setup( );
   }
}

class PopularPostsAdminPageSection extends PhooDisplay {
   var $name = "PopularPostsAdminPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{PopularPostsAdminForm}_";
}

class PopularPostsAdminForm extends PhooForm {
   var $name = "PopularPostsAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|PopularPosts.form.update|=";
   var $command = "=|PopularPosts.set.parameters|=";
   var $loadCommand = "=|PopularPosts.get.parameters|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" ) );
   var $action = "update";

   function setup( ) {
      parent::setup( );

      /* if parameters aren't setup for this extension, create them */
      $parms = $this->getRowsFromQueryCommand( "=|PopularPosts.get.parameters|=" );
      
      if ( !is_array( $parms )  ) {
         $this->runModificationCommand( "=|PopularPosts.create.parameters|=" );
      }
      
      $this->session->setSessionMessage( "Popular Post Parameters Updated", MSG_INFO );
      /* for if this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&subpage=General" );
   }
}

class PopularPostsTimeHorizon extends PhooDisplay {
   var $name = "PopularPostsTimeHorizon";
   var $securityRole = ROLE_EVERYONE;
   var $template = "=|PopularPosts.time_horizon|=";

   function setup( ) {
      $timeHorizon = $this->container->getVar( "popular_posts_time_horizon", "PRIVATE" );
      
      if ( $timeHorizon <= 0 ) {
         $this->disabled = true;
      }
      else {
         $this->setVar( "popular_posts_time_horizon", $timeHorizon, "PRIVATE" );
      }
   }
}

?>
