<?php
/* Name: reading_list.classes.php
   Author: Bill Roberts
   Created: 3/6/2005
   Purpose: Displays and manages a reading list
   $Revision: 1.4 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   
*/   
  
class ReadingList extends PhooQuery { 
   /* displays the reading list */
   var $package = "reading_list";
   var $name = "ReadingList";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|ReadingList.list.get|=";
   var $templateHeader = "=|ReadingList.header|=";
   var $template = "=|ReadingList|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-06" );
      $this->setPurpose( "Displays the reading list" );
   }
} 

/***************************************************/
/*** SnapOns for ReadingList administration      ***/
/***************************************************/
class ReadingListAdminPageSection extends PhooDisplay {
   var $package = "reading_list";
   var $name = "ReadingListAdminPageSection";
   var $securityRole = ROLE_POSTER;
   var $template = "_{ReadingListAdminDisplay}_";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-06" );
      $this->setPurpose( "Page for reading list administration" );
   }

   /* check the parameters passed to determine what to do */
   function setup( ) {
      parent::setup( );

      $action = $this->getVar( FORM_ACTION );
      switch( $action ) {
         case "insert" :
         case "update" :
            $this->template = "_{ReadingListAdminForm}_";
            
            break;
         case "delete" : 
            /* instantiate and run a deleter */
            $s =& $this->session->snapOnFactory( "ReadingListDeleter", $this );
            $s->process( );
            if ( $s->isError( ) ) {
               return;
            }
            break;
         default :
            break;
      }
   }
}

class ReadingListAdminDisplay extends PhooQuery {
   var $package = "reading_list";
   var $name = "ReadingListAdminDisplay";
   var $securityRole = ROLE_POSTER;
   var $command = "=|ReadingList.list.get|=";
   var $templateHeader = "=|ReadingList.list.header|=";
   var $template = "=|ReadingList.list|=";
   var $templateFooter = "=|ReadingList.list.footer|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-01-23" );
      $this->setPurpose( "Displays the reading_list administration list" );
   }
}

/* ReadingListAdminForm
*/
class ReadingListAdminForm extends PhooForm {
   var $package = "reading_list";
   var $name = "ReadingListAdminForm";
   var $securityRole = ROLE_POSTER;
   var $updateForm = "=|ReadingList.form.update|=";
   var $insertForm = "=|ReadingList.form.insert|=";
   var $loadCommand = "=|ReadingList.getbyid|=";
   var $updateCommand = "=|ReadingList.update|=";
   var $insertCommand = "=|ReadingList.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   var $field = array( "reading_list_book" => array( "required" => "Y"
                                                    ,"formalName" => "Book" )
                                                 );
   
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-06" );
      $this->setPurpose( "Displays and processes the reading_list administration form" );
   }

   function setup( ) {
      parent::setup( );
      
      /* set the list type here */
      $this->setVar( "list_type", "reading_list", "POST" ); /* set in post vars to make it accessible to 

      /* for if this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&action=administer&extension_name=" . $this->getVar( "extension_name" ) );
   }

   function triggerProcess( $aProcess ) {

      /* make sure the list_order is a number. If not, set to 0 */
      $listOrder = $this->getVar( "list_order", "POST" );
      if ( !is_numeric( $listOrder ) ) {
         $this->setVar( "list_order", 0, "POST" );
      }
      parent::triggerProcess( $aProcess );
   }
   
}

class ReadingListDeleter extends PhooProcess {
   var $package = "reading_list";
   var $name = "ReadingListDeleter";
   var $securityRole = ROLE_POSTER;
   var $securityErrorMessage = "Cannot delete a reading list item. You do not have security privileges";
   var $command = "=|ReadingList.delete|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-03-06" );
      $this->setPurpose( "Controls deletion of a reading list item" );
   }
}
?>
