<?php
/* 
   Name: simple_human_check.classes.php
   Author: Bill Roberts
   Created: 5/20/2006 

   Purpose: Extension related to managing and displaying the "simple human check" comment questions 

   $Revision: 1.4 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
   
*/   
  
class SimpleHumanCheckDisplay extends PhooQuery { 
   var $package = "simple_human_check";
   var $name = "SimpleHumanCheckDisplay";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|SimpleHumanCheck.get|=";
   var $templateHeader = "";
   var $template = "=|SimpleHumanCheck.display|=";
   var $templateFooter = "";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "displays a simple human check question and answer entry form" );
   }

   function questionCount( ) {
      /* returns the number of simple human check questions */
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole; /* set the security to same as containter */
      $s->command = "=|SimpleHumanCheck.count|=";
      $s->process( );
      if ( $s->isError( ) ) {
         return;
      }
      $row = $s->getResultRow( );

      if ( $row[ "num_questions" ] == 0 ) {
         $this->disabled = true; /* disable - no questions to display */
      }
      
      return $row[ "num_questions" ];
   }      

   function chooseQuestion( ) { 
      /* algorithm: generate a random number to use to select a question
       */
      return rand( 0, $this->questionCount( ) - 1 );
   }
   
   
   /* choose a question and process */
   function doAction( $aArray = null ) {
      $selectedQuestion = $this->chooseQuestion( );
      $this->setVar( "selected_question", $selectedQuestion, "POST" );
      
      parent::doAction( $aArray );
   }
} 
  
/*********************************************/
/*** SnapOns for Question administration   ***/
/*********************************************/
class SimpleHumanCheckAdminPageSection extends PhooDisplay {
   var $package = "simple_human_check";
   var $name = "SimpleHumanCheckAdminPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{SimpleHumanCheckAdminDisplay}_";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "Page for administrating the simple human check" );
   }

   /* check the parameters passed to determine what to do */
   function setup( ) {
      parent::setup( );

      $action = $this->getVar( FORM_ACTION );
      switch( $action ) {
         case "insert" :
         case "update" :
            $this->template = "_{SimpleHumanCheckAdminForm}_";
            
            break;
         case "delete" : 
            /* instantiate and run a deleter */
            $s =& $this->session->snapOnFactory( "SimpleHumanCheckDeleter", $this );
            $s->process( );
            if ( $s->isError( ) ) {
               return;
            }
            break;
         default :
            break;
      }
   }
}

class SimpleHumanCheckAdminDisplay extends PhooQuery {
   var $package = "simple_human_check";
   var $name = "SimpleHumanCheckAdminDisplay";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|SimpleHumanCheck.list.get|=";
   var $templateHeader = "=|SimpleHumanCheck.list.header|=";
   var $template = "=|SimpleHumanCheck.list|=";
   var $templateFooter = "=|SimpleHumanCheck.list.footer|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-01-23" );
      $this->setPurpose( "Displays the simple human check administration list" );
   }
}

/* Administration Form 
*/
class SimpleHumanCheckAdminForm extends PhooForm {
   var $package = "simple_human_check";
   var $name = "SimpleHumanCheckAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $updateForm = "=|SimpleHumanCheck.form.update|=";
   var $insertForm = "=|SimpleHumanCheck.form.insert|=";
   var $loadCommand = "=|SimpleHumanCheck.getbyid|=";
   var $updateCommand = "=|SimpleHumanCheck.update|=";
   var $insertCommand = "=|SimpleHumanCheck.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   var $field = array( "simple_human_check_question" => array( "required" => "Y"
                                                  ,"formalName" => "Question" )
                      ,"simple_human_check_answer" => array( "required" => "Y"
                                                  ,"formalName" => "Answer" )
                                                 );
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "Displays and processes the simple human check administration form" );
   }

   function setup( ) {
      parent::setup( );
      
      /* set the list type here */
      $this->setVar( "list_type", "simple_human_check", "POST" ); /* set in post vars to make it accessible to 
                                                            the new_id command */

      /* for if this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&action=administer&extension_name=" . $this->getVar( "extension_name" ) );
   }
}

class SimpleHumanCheckDeleter extends PhooProcess {
   var $package = "simple_human_check";
   var $name = "SimpleHumanCheckDeleter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete a simple human check question. You do not have security privileges";
   var $command = "=|SimpleHumanCheck.delete|=";
}

/**************************************************************/
/* Activator SnapOn                                           */
/* Inserts the default question into the list table if no     */
/* question exists                                            */
/**************************************************************/
class SimpleHumanCheckActivator extends PhooProcess {
   var $package = "simple_human_check";
   var $name = "SimpleHumanCheckActivator";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|SimpleHumanCheck.activate|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "Controls activation of the simple human check questions" );
   }

   /* adds a simple human check question to the list table, if none exists */
   function process( ) { 
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole;
      $s->command = "=|SimpleHumanCheck.question.exists|=";
      $s->process( );

      if ( $s->isError( ) ) {
         return;
      }

      $row = $s->getResultRow( );
      if ( is_array( $row ) ) {
         /* a question already exists */
         return;
      }

      /* no question exists, go ahead and process the snapon */
      parent::process( );
   }
}

/**************************************************************/
/* Initializer SnapOn                                         */
/**************************************************************/
class SimpleHumanCheckInitializer extends PhooProcess {
   var $package = "simple_human_check";
   var $name = "SimpleHumanCheckInitializer";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "Controls initialization of the simple human check SnapOn" );
   }

   /* adds the comment check event to the list of event handlers */
   function process( ) { 
      $this->session->addEvent( "CommentForm", EVENT_PRESAVE, "SimpleHumanCheck" );
   }
}

/**************************************************************/
/* SimpleHumanCheck Event Handler                             */
/* Inherits from the CommentSpamChecker class - verifies that */
/* the answer given to the simple human check question is     */
/* correct.                                                   */
/**************************************************************/
class SimpleHumanCheck extends CommentFilter {
   var $package = "simple_human_check";
   var $name = "SimpleHumanCheck";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-05-23" );
      $this->setPurpose( "checks the answer to the simple human check question to see if it matches." );
   }


   /* verify their answer against the list of possible answers */
   function checkComment( ) {
      $answer = trim( strtolower( $this->getVar( "simple_human_check_answer", "POST" ) ) );
      if ( $answer == "" ) {
         $this->container->setError( "Please answer the Simple Human Check question.", MSG_WARNING );
      }
      
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole;
      $s->command = "=|SimpleHumanCheck.getanswer|=";
      $s->process( );
      if ( $s->isError( ) ) {
         return;
      }
      $row = $s->getResultRow( );
      $realAnswerList = split( ",", strtolower( $row[ "simple_human_check_answer" ] ) );
      for ( $i = 0; $i < count( $realAnswerList ); $i++ ) {
         /* make sure there are no leading or trailing spaces */
         $realAnswerList[ $i ] = trim( $realAnswerList[ $i ] );
      }
      
      if ( !in_array( $answer, $realAnswerList ) ) {
         $this->container->setError( "Based on your answer to the question asked, I'm not sure that you're human. You might be a spambot or some other blight upon the blogosphere. Please try again.", MSG_WARNING );
      }
   }
}
?>
