<?php
/* Name: smilies.classes.php
   Author: Bill Roberts
   Created: 7/18/2006
   Purpose: manages a list of images that will replace smilie codes in comments and posts 
   $Revision: 1.4 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/   
  
class Smilies extends PhooQuery {
   /* displays the list of smilies */
   var $package = "smilies";
   var $name = "Smilies";
   var $securityRole = ROLE_EVERYONE;
   var $command = "=|Smilies.list.get|=";
   var $templateHeader = "=|Smilies.header|=";
   var $template = "=|Smilies|=";
   var $templateFooter = "=|Smilies.footer|=";

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-18" );
      $this->setPurpose( "Displays the Smilies list" );
   }

   function setup( ) {
      parent::setup( );

      /* stick these in the POST variables because the PRIVATE vars get replaced with each row retrieved */
      switch ( $this->container->name ) {
         case "PostForm" :
            $this->setVar( "smilies_content_name", "post_content", "POST" );
            $this->setVar( "smilies_content_description", "post", "POST" );
            break;
            
         case "CommentForm" :
            $this->setVar( "smilies_content_name", "comment_content", "POST" );
            $this->setVar( "smilies_content_description", "comment", "POST" );
            break;
      }
   } 
}

/***************************************************/
/*** SnapOns for Smilies administration          ***/
/***************************************************/
class SmiliesAdminPageSection extends PhooDisplay {
   var $package = "smilies";
   var $name = "SmiliesAdminPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{SmiliesAdminDisplay}_";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-18" );
      $this->setPurpose( "Page for smilies administration" );
   }

   /* check the parameters passed to determine what to do */
   function setup( ) {
      parent::setup( );

      $action = $this->getVar( FORM_ACTION );
      switch( $action ) {
         case "insert" :
         case "update" :
            $this->template = "_{SmiliesAdminForm}_";
            
            break;
         case "delete" : 
            /* instantiate and run a deleter */
            $s =& $this->session->snapOnFactory( "SmiliesDeleter", $this );
            $s->process( );
            if ( $s->isError( ) ) {
               return;
            }
            break;
         default :
            break;
      }
   }
}

class SmiliesAdminDisplay extends PhooQuery {
   var $package = "smilies";
   var $name = "SmiliesAdminDisplay";
   var $securityRole = ROLE_POSTER;
   var $command = "=|Smilies.list.get|=";
   var $templateHeader = "=|Smilies.list.header|=";
   var $template = "=|Smilies.list|=";
   var $templateFooter = "=|Smilies.list.footer|=";
   
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-18" );
      $this->setPurpose( "Displays the smilies administration list" );
   }
}

/* SmiliesAdminForm
*/
class SmiliesAdminForm extends PhooForm {
   var $package = "smilies";
   var $name = "SmiliesAdminForm";
   var $securityRole = ROLE_POSTER;
   var $updateForm = "=|Smilies.form.update|=";
   var $insertForm = "=|Smilies.form.insert|=";
   var $loadCommand = "=|Smilies.getbyid|=";
   var $updateCommand = "=|Smilies.update|=";
   var $insertCommand = "=|Smilies.insert|=";
   var $newIdCommand = "=|List.newid.get|=";
   var $trigger = array( "btnUpdate" => array( "process" => "update" )
                        ,"btnInsert" => array( "process" => "insert" ) );
   var $field = array( "smilies_code" => array( "required" => "Y"
                                               ,"formalName" => "Smilie Code" )
                      ,"smilies_image_url" => array( "required" => "Y"
                                                ,"formalName" => "Smilie Image" )                                                                 );
      
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-18" );
      $this->setPurpose( "Displays and processes the smilies administration form" );
   }

   function setup( ) {
      parent::setup( );
      
      /* set the list type here */
      $this->setVar( "list_type", "smilies", "POST" ); /* set in post vars to make it accessible to 

      /* for if this works */
      $this->setSuccessPage( $this->getVar( "site_url", "APPLICATION" ) . "/" .
                             $this->getVar( "operations_page", "APPLICATION" ) .
                             "?page=extensionAdmin&action=administer&extension_name=" . $this->getVar( "extension_name" ) );
   }
   
}

class SmiliesDeleter extends PhooProcess {
   var $package = "smilies";
   var $name = "SmiliesDeleter";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $securityErrorMessage = "Cannot delete a smilie. You do not have security privileges";
   var $command = "=|Smilies.delete|=";
}

/**************************************************************/
/* Smilies Event Handler                                      */
/* Checks for a Smilie codes in a comment or post, and        */
/* replaces them with the smilie image urls                   */
/**************************************************************/
class SmiliesCodeReplacer extends PreSaveEventHandler {
   var $package = "smilies";
   var $name = "SmiliesCodeReplacer";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-18" );
      $this->setPurpose( "converts any smilie codes into image urls" );
   }

   function doAction( ) {
      parent::doAction( );

      /* load up the smilies list */
      $smilies = $this->getRowsFromQueryCommand( "=|Smilies.list.get|=" );

      switch ( $this->container->name ) {
         case "PostForm" :
            $field = "post_content";
            break;
            
         case "CommentForm" :
            $field = "comment_content";
            break;
      }

      $content = $this->container->getVar( $field, "PRIVATE" );
      
      foreach ( $smilies as $smilie ) {
         $this->setVar( "smilies_code", $smilie[ "smilies_code" ], "PRIVATE" );
         $this->setVar( "smilies_image_url", $smilie[ "smilies_image_url" ], "PRIVATE" );
         $img = trim( $this->populateParameters( $this->expandOverlays( "=|Smilies.image|=" ) ) );
         $content = str_replace( $smilie[ "smilies_code" ], $img, $content );
      }

      $this->container->setVar( $field, $content, "PRIVATE" );      
   }
}

/**************************************************************/
/* Smilies Event Handler                                      */
/* Checks for a Smilie urls in a comment or post and reverts  */
/* them to the smilie code                                    */
/**************************************************************/
class SmiliesImageReplacer extends PreRenderEventHandler {
   var $package = "smilies";
   var $name = "SmiliesImageReplacer";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-18" );
      $this->setPurpose( "converts any smilie images back to codes" );
   }

   function doAction( ) {
      parent::doAction( );

      /* load up the smilies list */
      $smilies = $this->getRowsFromQueryCommand( "=|Smilies.list.get|=" );

      switch ( $this->container->name ) {
         case "PostForm" :
            $field = "post_content";
            break;
            
         case "CommentForm" :
            $field = "comment_content";
            break;
      }

      $content = $this->container->getVar( $field, "PRIVATE" );

      foreach ( $smilies as $smilie ) {
         $this->setVar( "smilies_code", $smilie[ "smilies_code" ], "PRIVATE" );
         $this->setVar( "smilies_image_url", $smilie[ "smilies_image_url" ], "PRIVATE" );
         $img = trim( $this->populateParameters( $this->expandOverlays( "=|Smilies.image|=" ) ) );

         $content = str_replace( $img, $smilie[ "smilies_code" ], $content );
      }

      $this->container->setVar( $field, $content, "PRIVATE" );      
   }
}


/**************************************************************/
/* Activator SnapOn                                           */
/* Creates the initial smilies list if none exist             */
/**************************************************************/
class SmiliesActivator extends PhooProcess {
   var $package = "smilies";
   var $name = "SmiliesActivator";
   var $securityRole = ROLE_ADMINISTRATOR;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2006-07-18" );
      $this->setPurpose( "Controls activation of the smilies snapon" );
   }

   /* checks to see if any smilies exist. If they don't, create the basic list */
   function doAction( ) { 
      $count = $this->getRowsFromQueryCommand( "=|Smilies.count|=" );
   
      if ( $count[ 0 ][ "smilies_count" ] == 0 ) { /* none exist, create them */
         $p =& $this->session->snapOnFactory( "PhooMultiProcess", $this );
         $p->securityRole = $this->securityRole;
         $p->command = "=|Smilies.create|=";
         $p->process( );
         if ( $this->isError( ) ) {
            return;
         }
      }
   }
}

?>
