<?php
/* Name: theme_selector.classes.php
   Author: Bill Roberts
   Created: 3/23/2006

   Purpose: Display the a dropdown that the user can use to select a theme

   $Revision: 1.6 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/   

class ThemeSelectorDropDown extends ThemeDropDown { /* note: extends from the main theme drop down in the
                                                       look and feel package */
   var $name = "ThemeSelectorDropDown";
   var $templateHeader = "=|ThemeSelector.dropdown.header|=";
   var $template = "=|ThemeSelector.dropdown.item|=";
   var $templateFooter = "=|ThemeSelector.dropdown.footer|=";
   
   function doAction( ) {
      parent::doAction( ); /* this loads the themeDescriptor array */
      
      $rows = $this->getRowsFromQueryCommand( "=|ThemeSelector.active.get|=" );
      if ( is_array( $rows ) ) {
         foreach ( $rows as $row ) {
            if ( $row[ "theme_selector_available_flag" ] == "Y" ) {
               $this->themeDescriptor[ $row[ "theme_selector_theme_tag" ] ][ "active" ] = true;
            }
            else { 
               $this->themeDescriptor[ $row[ "theme_selector_theme_tag" ] ][ "active" ] = false;
            }
         }
      }

      /* make sure the default theme is always included */
      $defaultTheme = $this->session->getSetting( "default_theme" );
      if ( !$this->themeDescriptor[ $defaultTheme ][ "active" ] ) {
         $this->themeDescriptor[ $defaultTheme ][ "active" ] = true;
      }
   }

   function getResultRow( ) {
      $searchActive = true;
      while ( $searchActive ) {
         if ( !$this->rowsReturned ) { /* at the first row */
            reset( $this->themeDescriptor );
            $row = current( $this->themeDescriptor );
            $this->rowsReturned = true;
         }   
         else {
            $row = next( $this->themeDescriptor );
         }

         if ( !is_array( $row ) ) {
            $searchActive = false;
         }
         elseif ( $row[ "active" ] ) {
            $searchActive = false;
         }
      }
      
      if ( is_array( $row ) ) { 
         return $row;
      }

      return null; /* done */
   }
}


/* this snapon really just wraps the Theme.form.selector overlay, for ease of use */
class ThemeSelector extends PhooForm {
   var $package = "theme_selector";
   var $name = "ThemeSelector";
   var $securityRole = ROLE_EVERYONE;
   var $template = "_{ThemeSelectorDropDown}_"; 
   
}

/****************************************************************/
/*** ThemeSelectorCheck - checks for a theme selection POST   ***/
/*** variable - if set, it sets cookies variables             ***/
/*** to tell Bloo to select the theme                         ***/
/****************************************************************/
class ThemeSelectorCheck extends SessionInitEventHandler {
   var $name = "ThemeSelectorCheck";
   var $securityRole = ROLE_EVERYONE;
   var $fieldName;
   
   function doAction( ) {
      /* see if a theme was posted */
      $themeSelectorTheme = $this->getVar( "theme", "POST" );

      if ( !isset( $themeSelectorTheme ) || $themeSelectorTheme == "" ) {
         /* check the cookie */
        $themeSelectorTheme = $this->getVar( "theme_selector_theme", "COOKIE" );         
      }
        
      if ( isset( $themeSelectorTheme ) && ( $themeSelectorTheme > "" ) ) {
         /* check to see if the theme is one of the "approved" themes or the default theme */
         $themeIsGood = false; /* initialize */
         $defaultTheme = $this->session->getSetting( "default_theme" );
         if ( $themeSelectorTheme == $defaultTheme ) {
            $themeIsGood = true;
         }
         else {
            $this->setVar( "theme", $themeSelectorTheme, "PRIVATE" );
            $rows = $this->getRowsFromQueryCommand( "=|ThemeSelector.is.theme.active|=" );
            if ( is_array( $rows ) ) {
               if ( $rows[0][ "theme_is_active" ] > "" ) {
                  $themeIsGood = true;
               }
            }
         }
         if ( $themeIsGood ) {
            /* note: default cookies to expire in 2 weeks */
            $this->setVar( "theme_selector_theme", $themeSelectorTheme, "COOKIE", time( ) + (60*60*24*14) );

            /* set an application variable - this will be picked up in the session */
            $this->session->setSetting( "theme_selector_theme", $themeSelectorTheme );
         }
      }
   }
}

class ThemeSelectorAdminPageSection extends PhooDisplay {
   var $name = "ThemeSelectorAdminPageSection";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "_{ThemeSelectorAdminForm}_";
}

/* note: this form is based off of PhooQuery, not PhooForm
   It manages a list of checkboxes */
class ThemeSelectorAdminForm extends PhooQuery {
   var $name = "ThemeSelectorAdminForm";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|ThemeSelector.list.header|=";
   var $template = "=|ThemeSelector.list|=";
   var $templateFooter = "=|ThemeSelector.list.footer|=";
   var $theme = array( );
   var $themeIndex = 0;
   
   /* for sorting the theme array */
   function compareFunction( $ar1, $ar2 ) {
      return strcmp( $ar1[ 0 ], $ar2[ 0 ] );
   }
   
   function doAction( ) {
      $btnUpdate = $this->getVar( "btnUpdate", "POST" ); /* see if an update has been requested */

      $themeInSelector = array( );
      /* get list of themes that are possible to select from the list table */
      $rows = $this->getRowsFromQueryCommand( "=|ThemeSelector.list.get|=" );
      if ( is_array( $rows ) ) {
         foreach ( $rows as $row ) {
            if ( $row[ "theme_selector_available_flag" ] == "Y" ) {
               $themeInSelector[ $row[ "theme_selector_theme_tag" ] ] = true;
            }
            else { 
               $themeInSelector[ $row[ "theme_selector_theme_tag" ] ] = false;
            }
         }
      }

      /* retrieve theme inventory */
      foreach ( $this->session->themeInventory as $themeTag => $themeAttributes ) {
         if ( $themeInSelector[ $themeTag ] ) {
            $inSelector = true;
         }
         else {
            $inSelector = false;
         }
         array_push( $this->theme, array( $themeAttributes[ "display_name" ], $themeTag, $inSelector ) );
      }

      /* now, check for an update. If one was requested, adjust for any changes made to the status
         of themes */
      if ( $btnUpdate > "" ) {
         $themeIndex = 0;
         foreach ( $this->theme as $theme ) {
            $this->setVar( "theme_selector_theme_tag", $theme[ 1 ], "PRIVATE" );
            $checked = $this->getVar( $theme[ 1 ], "POST" );
            if ( $checked == "on" ) {
               $checked = true;
               $this->setVar( "theme_selector_available_flag", "Y", "PRIVATE" );
            }
            else {
               $checked = false;
               $this->setVar( "theme_selector_available_flag", "N", "PRIVATE" );
            }
            
            if ( $theme[ 2 ] != $checked ) {
               /* the status of this item has changed. update it or insert it based on whether it
                  already existed in the list table */
               if ( isset( $themeInSelector[ $theme[ 1 ] ] ) ) {
                  /* exists in the list table - update it */
                  $this->runModificationCommand( "=|ThemeSelector.update|=" );
               }
               else {
                  /* doesn't exist in the list table - insert it. Get new id first */
                  $rows = $this->getRowsFromQueryCommand( "=|ThemeSelector.nextId|=" );
                  if ( isset( $rows[0][ "next_id" ] ) && ( $rows[0][ "next_id" ] > 0 ) ) {
                     $nextId = $rows[0][ "next_id" ];
                  }
                  else {
                     $nextId = 1;
                  }

                  $this->setVar( "next_id", $nextId, "PRIVATE" );
                  
                  $this->runModificationCommand( "=|ThemeSelector.insert|=" );
               }

               /* update the main array for display */
               $this->theme[ $themeIndex ][ 2 ] = $checked;
            }
            $themeIndex++;
         }
         if ( !$this->isError( ) ) {
            $this->session->setSessionMessage( "Theme Selector Changes Have Been Saved", MSG_INFO );
         }
      }

      /* sort the theme inventory */
      usort( $this->theme, array( "ThemeSelectorAdminForm", "compareFunction" ) );
      
      parent::doAction( );
   }
   
   function getResultRow( ) {
      if ( $this->themeIndex >= count( $this->theme ) ) {
         return null;
      }
      else {
         $row[ "theme_name" ] = $this->theme[ $this->themeIndex ][ 0 ];
         $row[ "theme_tag" ] = $this->theme[ $this->themeIndex ][ 1 ];
         $row[ "theme_checked" ] = $this->theme[ $this->themeIndex ][ 2 ];

         $this->themeIndex++;
         return $row;
      }
   }
}

class ThemeSelectorCheckBox extends PhooDisplay {
   var $securityRole = ROLE_ADMINISTRATOR;
   var $template = "=|ThemeSelector.checkbox|=";
   var $theme = array( );

   function doAction( ) {

      $themeName = $this->container->getVar( "theme_name", "PRIVATE" );
      $themeTag = $this->container->getVar( "theme_tag", "PRIVATE" );
      $themeChecked = $this->container->getVar( "theme_checked", "PRIVATE" );

      if ( $themeChecked ) {
         $this->setVar( "theme_checked", "CHECKED", "PRIVATE" );
      }

      $this->setVar( "theme_name", $themeName, "PRIVATE" );
      $this->setVar( "theme_tag", $themeTag, "PRIVATE" );
      
   }
}

/**************************************************************/
/* Activator SnapOn                                           */
/* Inserts records in the list table for each theme           */
/**************************************************************/
class ThemeSelectorActivator extends PhooProcess {
   var $name = "ThemeSelectorActivator";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $command = "=|ThemeSelector.insert|=";

   /* adds themes to the list table, if none exist */
   function process( ) { 
      $s =& $this->session->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = $this->securityRole;
      $s->command = "=|ThemeSelector.list.get|=";
      $s->process( );

      if ( $s->isError( ) ) {
         return;
      }

      $row = $s->getResultRow( );
      if ( is_array( $row ) ) {
         /* themes already exist */
         return;
      }

      /* no themes are in the themes table, insert them one by one */
      $nextId = 0;

      /* retrieve theme inventory */
      foreach ( $this->session->themeInventory as $themeTag => $themeAttributes ) {
         $nextId++;
         $this->setVar( "next_id", $nextId, "PRIVATE" );
         $this->setVar( "theme_selector_theme_tag", $themeTag, "PRIVATE" );

         if ( $themeTag == "bloo_base" ) {
            $availableFlag = "N"; /* don't enable bloo base. It's not a very viewable theme */
         }
         else {
            $availableFlag = "Y";
         }
         
         $this->setVar( "theme_selector_available_flag", $availableFlag, "PRIVATE" );

         parent::process( );
      }
   }
}

?>
