<?php
/*
Name: phoo.base.php
Purpose: Base classes for the phoo framework
Author: Bill Roberts
Created: 4/11/2005
$Revision: 1.44 $

Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/
   
/* define the global error list */
$_globalErrorList = array( );

define( "MSG_FATAL", 1 );
define( "MSG_WARNING", 2 );
define( "MSG_INFO", 3 );
define( "MSG_SECURITY", 4 );

class PhooMessage {
   var $level;
   
}

class PhooBase {

   function PhooBase( ) {

   }

   /**** Error Message Functions ****/

   /* returns true if there is an error - if level is specified
      only returns true if an error of that level exists */
   function isError( $aLevel = null ) {
      global $_globalErrorList;

      for ( $i = 0; $i < count( $_globalErrorList ); $i++ ) {
         $err = $_globalErrorList[ $i ];
         if ( isset( $aLevel ) ) {
            if ( $err->level == $aLevel ) {
               return true;
            }
         }
         else {
            return true;
         }
      }
      
      return false;
   }

   /* sets an error an level in the global error list */
   function setError( $aErrorMessage, $aMessageLevel ) {
      global $_globalErrorList;

      $e = new PhooMessage( );
      $e->level = $aMessageLevel;
      $e->message = $aErrorMessage;
      array_unshift( $_globalErrorList, $e );
   }
   
   /* returns the current error message */
   function getError( ) {
      global $_globalErrorList;

      if ( $this->isError( ) ) {
         $e = array_pop( $_globalErrorList );
         return $e->message;
      }
      else {
         return "";
      }
   }
}

/* parses an XML file into an object tree */
class XML2ArrayParser extends PhooBase {
   var $parser;
   var $tagData = array( ); /* this is the object tree */
   var $defaultEncoding = "UTF-8";
   var $tagStack = array( );
   var $parentStack = array( );
   var $level = array( );
   var $fileName;
   var $inTag = "";
   
   /* if $aRawXML is set, there is no file: raw XML has been passed */
   function XML2ArrayParser( $aFileName, $aRawXML = false ) {

      if ( !$aRawXML ) {
         $this->fileName = $aFileName;
         $data = @file_get_contents( $aFileName );
      }
      else {
         $data = $aFileName; /* this is raw xml */
      }

      if ( $data === false ) {
         $this->setError( "Could not open XML file: $aFileName", MSG_FATAL );
         return;
      }
      
      /* figure out encoding. If you can't figure it out, set to default */ 
      eregi( "encoding=\"([a-z,A-Z,0-9,\-]+)\"", $data, $match );
      if ( isset( $match[ 1 ] ) ) {
         $encoding = $match[ 1 ];
      }
      else {
         $encoding = $this->defaultEncoding;
      }
            
      $this->parser = xml_parser_create( $encoding );
      
      xml_set_object( $this->parser, $this );
      xml_set_element_handler( $this->parser, "startElement", "endElement" );
      xml_set_character_data_handler( $this->parser, "characterData" );
         
      if (!xml_parse($this->parser, $data, true ) ) {
         $this->setError( sprintf( "XML error on $aFileName: %s. Error at line %d",
             xml_error_string(xml_get_error_code( $this->parser ) ),
             xml_get_current_line_number( $this->parser ) ), MSG_FATAL );
         return;
      }

      if ( isset( $this->parser ) ) {
         xml_parser_free($this->parser);
      }
   }

   /* compute and return the current full tag */
   function fullTag( ) {
      $fullTag = "";
      foreach ( $this->tagStack as $tag ) {
         if ( $fullTag > "" ) {
            $fullTag .= "~";
         }

         $fullTag .= $tag;
      }

      return $fullTag;
   }

   function startElement($parser, $name, $attrs) {
      /* get current full tag - it will be the previous level */
      $fullTag = $this->fullTag( );
      $prevLevel = $this->level[ $fullTag ];
      
      /* add current name onto the fulltag */
      if ( $fullTag > "" ) {
         $fullTag .= "~";
      }
      $fullTag .= $name;

      /* increment the level */
      if ( isset( $this->level[ $fullTag ] ) ) {
         $this->level[ $fullTag ]++;
         /* reset levels lower than this one */
         $fullTagLen = strlen( $fullTag );
         
         foreach ( $this->level as $key => $level ) {
            if ( substr( $key, 0, $fullTagLen + 1 ) == $fullTag . "~" ) { /* this is a lower level */
               $this->level[ $key ] = 0;
            } 
         }      
      }
      else {
         $this->level[ $fullTag ] = 0;
      }

      /* create an object pointer pointing to the current position in the 
         object tree */
      if ( count( $this->tagStack ) > 0 ) {
         /* get the latest parent pointer */
         $parentPtr =& $this->parentStack[ count( $this->parentStack ) - 1 ]; 
         /* create a placeholder in the object tree for this item */
         if ( !isset( $parentPtr[ $prevLevel ][ $name ] ) ) {
            $parentPtr[ $prevLevel ][ $name ] = "";         
         }
         $newItemPtr =& $parentPtr[ $prevLevel ][ $name ];
      }
      else {
         /* create root pointer */
         $this->tagData[ $name ] = array( );
         $newItemPtr =& $this->tagData[ $name ];
      }

      /* push the current tag and a reference to the current item on the stacks */
      array_push( $this->tagStack, $name );
      /* note: originally, this was array_push( $this->parentStack, &$newItemPtr ), 
         but passing references to array_push has been deprecated */
      $this->parentStack[ count( $this->parentStack ) ] = &$newItemPtr;
   }

   function endElement($parser, $name) {
      /* pop items off the stacks */
      array_pop( $this->tagStack );
      array_pop( $this->parentStack );
      $this->inTag = "";
   }

   function characterData($parser, $data) {
      /* Note: if a newline is encountered, set a flag that will ONLY store it if we're concatenating data */
      if ( $data == "\n" ) {
         $storeNLOnConcatenate = true;
      }
      else {
         $storeNLOnConcatenate = false;
      }
      
      /* see if any data exists */
      if ( trim( $data ) > "" || $storeNLOnConcatenate ) {
         $currentTag = $this->tagStack[ count( $this->tagStack ) - 1 ];
         /* if in the current tag, concatenate data */
         if ( $currentTag == $this->inTag ) { /* concatenate */
            $concatenate = true;
         }
         else {
            $concatenate = false;
            /* store the current tag as the inTag */
            if ( !$storeNLOnConcatenate ) {
               $this->inTag = $currentTag;
            }
         }

         $parentPtr =& $this->parentStack[ count( $this->parentStack ) - 1 ]; /* get the current parent pointer */
         /* store this item */
         if ( $concatenate ) {
            $parentPtr .= $data; /* Note: may be a newline character */           
         }
         else {
            if ( !$storeNLOnConcatenate ) { /* if we just have a newline, don't store it here as we're at the beginning */
               $parentPtr = $data; 
            }
         }
      }     
   }
}

/* class to encapsulate various variable types */
class Vars extends PhooBase {
   /* notice - cookie variables have to be asked for by explicit scope - they are not 
      in the scope precedence */
   var $scopePrecedence = array( "PRIVATE", "SESSION", "APPLICATION", "POST", "GET", "SERVER" );
   var $appVar = array( );
   var $privateVar = array( );

   function Vars( ) {
   }

   function getVarByScope( $aName, $aScope ) {
      global $_POST, $_GET;

      switch( $aScope ) {
         case "PRIVATE" :
            return ( isset( $this->privateVar[ $aName ] ) ? $this->privateVar[ $aName ] : null ); 
            break;

         case "APPLICATION" :
            return ( isset( $this->appVar[ $aName ] ) ? $this->appVar[ $aName ] : null ); 
            break;
            
         case "SESSION" :
            return ( isset( $_SESSION[ $aName ] ) ? $_SESSION[ $aName ] : null );
            break;
            
         case "COOKIE" :
            return ( isset( $_COOKIE[ $aName ] ) ? $_COOKIE[ $aName ] : null );
            break;

         case "SERVER" :
            return ( isset( $_SERVER[ $aName ] ) ? $_SERVER[ $aName ] : null );
            break;
            
         case "POST" :
            return ( isset( $_POST[ $aName ] ) ? $_POST[ $aName ] : null );
            break;
            
         case "GET" :
            /* note the urldecode used here */
            return ( isset( $_GET[ $aName ] ) ? urldecode( $_GET[ $aName ] ) : null );
            break;

         default :
            $this->setError( "Illegal variable scope: $aScope", MSG_FATAL );
            return;
            break;
      }
   }

   function getVar( $aName, $aScope = null ) {
      
      if ( !$aScope ) {
         foreach( $this->scopePrecedence as $scope ) {
            $value = $this->getVarByScope( $aName, $scope );
            if ( isset( $value ) ) {
               return $value;
            }
         }
      }
      else {
         return $this->getVarByScope( $aName, $aScope );
      }         
   }

   function setVar( $aName, $aValue, $aScope = "PRIVATE", $aExpiration = 0, $aPath = null, $aDomain = null ) {

      global $_POST, $_GET, $_COOKIE;

      switch( $aScope ) {
         case "PRIVATE" :
            $this->privateVar[ $aName ] = $aValue;
            break;
            
         case "APPLICATION" :
            $this->appVar[ $aName ] = $aValue;
            break;

         case "SESSION" :
            $_SESSION[ $aName ] = $aValue;
            break;

         case "COOKIE" :
            // This must be done before any other output hits the browser:
            /* note: if domain is null, determine it based on the current
               website - parsing out, by default, the beginning www if it exists
               so that the cookie works for all subdomains */
            if ( !isset( $aDomain ) ) {
               $aDomain = "." . eregi_replace( "^www.", "", $_SERVER[ "HTTP_HOST" ] );  
            }

            setcookie( $aName, $aValue, $aExpiration, $aPath, $aDomain );
            /* note: set it in the cookie variables too for use on the current refresh */
            $_COOKIE[ $aName ] = $aValue;
            break;

         case "POST" :
            $_POST[ $aName ] = $aValue;
            break;

         case "GET" :
            /* note the urlencode used below */
            $_GET[ $aName ] = urlencode( $aValue );
            break;
            
      }   
   }

   function getVars( $aScope ) {
      global $_POST, $_GET, $_SERVER;

      switch( $aScope ) {
         case "PRIVATE" :
            return $this->privateVar;
            break;
         
         case "APPLICATION" :
            return $this->appVar;
            break;

         case "SESSION" :
            return $_SESSION;
            break;

         case "SERVER" :
            return $_SERVER;
            break;            

         case "COOKIE" :
            return $_COOKIE;
            break;

         case "POST" :
            return $_POST;
            break;

         case "GET" :
            return $_GET;
            break;
      }
   }

   /* stores an associative array into a specified scope */
   function setVars( $aArray, $aScope ) {
      foreach ( $aArray as $key => $value ) {
         $this->setVar( $key, $value, $aScope );
      }
   }

   function clearVar( $aName, $aScope ) { 
      global $_POST, $_GET, $_COOKIE;

     switch( $aScope ) {
          case "PRIVATE" :
            unset( $this->privateVar[ $aName ] );
            break;
            
         case "APPLICATION" :
            unset( $this->appVar[ $aName ] );
            break;

         case "SESSION" :
            unset( $_SESSION[ $aName ] );
            break;

         case "COOKIE" :
            $this->setVar( $aName, "", $aScope, -1 ); /* set cookie to expire */
            unset( $_COOKIE[ $aName ] ); /* clear it here too for the current refresh */
            break;

         case "POST" :
            unset( $_POST[ $aName ] );
            break;

         case "GET" :
            unset( $_GET[ $aName ] );
            break;

         default :
            $this->setError( "Illegal scope passed to clearVar: $aScope", MSG_FATAL );    
            break;
      }
   }

   function clearVars( $aScope ) { 
      global $_POST, $_GET;

      switch( $aScope ) {
         case "PRIVATE" :
            unset( $this->privateVar );
            break;
         
         case "APPLICATION" :
            unset( $this->appVar );
            break;

         case "SESSION" :
            unset( $_SESSION );
            break;

         case "COOKIE" :
            /* have to clear each cookie individual */
            foreach ( $_COOKIE as $key => $value ) {
               $this->setVar( $key, "", $aScope, -1 ); /* set cookie to expire */
            }
            break;

         case "POST" :
            unset( $_POST );
            break;

         case "GET" :
            unset( $_GET );
            break;

         default :
            $this->setError( "Illegal scope passed to clearVars: $aScope", MSG_FATAL );    
            break;
      }
   }
}

/* mysql abstrations */
define( "SQL_RESULT_NUM", MYSQL_NUM );
define( "SQL_RESULT_ASSOC", MYSQL_ASSOC );

/* all database-specific commands go in this class */
class PhooConnection extends PhooBase {
   var $sqlResult;
   var $session;

   /*** Initialization Functions ***/
   function PhooConnection( $aServer, $aDatabase, $aUserid, $aPassword ) {
      @mysql_connect(  $aServer, $aUserid, $aPassword );
      if ( mysql_error( ) ) {
         $this->setError( "Error on connect: " . mysql_error( ), MSG_FATAL );
         return;
      }
      
      @mysql_select_db( $aDatabase );
      if ( mysql_error( ) ) {
         $this->setError( "Error selecting database $aDatabase : " . mysql_error( ), MSG_FATAL );
         return;
      }
   }

   /* executes a select sql */
   function executeSelectSql( $sql ) {

      unset( $this->sqlResult );
      $result = @mysql_query( $sql );
      
      if ( $result ) {
         $this->sqlResult = $result;
      }
      else {
         $this->setError( "SQL Error: " . mysql_error( ) . ": " . $sql, MSG_FATAL );
         return;
      }
   }

   /*
    * returns the next available result row from the 
    * previous executeSelectSql command
    */
   function getResultRow( $returnType = SQL_RESULT_ASSOC ) {
      if ( $this->isError( MSG_FATAL ) ) {
         return;
      }

      /* default to associative array return type */
      if ( $returnType == null ) {
         $returnType = SQL_RESULT_ASSOC;
      }
      
      if ( !in_array( $returnType, array( SQL_RESULT_ASSOC, SQL_RESULT_NUM ) ) ) {
         $this->setError( "Illegal return type. Must be SQL_RESULT_ASSOC or SQL_RESULT_NUM", MSG_FATAL );
         return;
      }
      
      $row = @mysql_fetch_array( $this->sqlResult, $returnType );
      if ( mysql_error( ) ) {
         $this->setError( "SQL Error: " . mysql_error( ) . ": " . $sql, MSG_FATAL );
         return;
      }
      if ( is_array( $row ) ) {
         /* strip any slashes from the result */
         if ( $returnType == SQL_RESULT_NUM ) {
            for ( $i=0; $i<count($row); $i++ ) {
               $row[ $i ] = stripslashes( $row[ $i ] );
            }  
         }
         if ( $returnType == SQL_RESULT_ASSOC ) {
            foreach ( $row as $key => $value ) {
               $row[ $key ] = stripslashes( $value );
            }
         }
            
      }
      
      return ( $row );      
   }

   /* executes a modification sql (insert, update, delete) */
   function executeModificationSql( $sql ) {
      unset( $this->sqlResult );

      $result = @mysql_query( $sql );
      if ( $result ) {
         $this->sqlResult = $result;
      }
      else {
         $this->setError( "SQL Error: " . mysql_error( ) . ": " . $sql, MSG_FATAL );
         return;
      }
   }

   /* returns the id that was just inserted (if any) 
      from the call made to executeModificationSql 
    */
   function getInsertedId( ) {
      return( @mysql_insert_id( ) );   
   }

   /* returns true if the table specified exists in 
      the currently selected database */
   function tableExists( $aTableName ) {
      $this->executeSelectSql( "SHOW TABLES" );
      
      if ( $this->isError( ) ) {
         return;
      }

      while ( $row = $this->getResultRow( SQL_RESULT_NUM ) ) {
         if ( $row[ 0 ] == $aTableName ) {
            while ( $this->getResultRow( ) ); /* blow through the rest of the tables */
            return true;
         }
      }
      
      return false;
   }

   function columnExists( $aTableName, $aColumnName ) {
      $this->executeSelectSql( "SHOW COLUMNS FROM $aTableName" );
      
      if ( $this->isError( ) ) {
         return;
      }

      while ( $row = $this->getResultRow( SQL_RESULT_NUM ) ) {
         if ( in_array( $aColumnName, $row ) ) {
            return true;
         }
      }
      
      return false;
   }
}

/* define the default security roles */
define( "ROLE_NOONE", -1 );
define( "ROLE_EVERYONE", 1 );
define( "ROLE_ADMINISTRATOR", 2 );
define( "FILE_BASED", "filesystem" );

class PhooSession extends PhooBase {
   var $name = "PhooSession";
   var $connection;
   var $vars;
   var $loggedIn = false;
   var $user;
   var $sessionLoginName = "handle"; /* the name of the cookie variable storing the login id */
   var $sessionPasswordName = "cloak"; /* the name of the cookie variable storing the password */
   var $snapOnUserValidator = "UserValidator"; /* control to validate user */
   var $snapOnRoleInformation = "RoleInformation"; /* returns information on a role */
   var $controlUserSelect = "sql.user.select"; /* control to get user information */
   var $overlay = array( );
   var $pendage = array( ); /* will store overlay appendages and prependages */
   var $suppressObject = array();
   var $event = array( );
   var $theme = array( );
   var $snapOn = array( );
   var $package = array( );
   var $packageClass = array( );
   var $packageOverlay = array( );
   var $loadWithList = array( );
   var $redirecting = false;

   /*** Constructor ***/
   function PhooSession( ) {
      $this->vars = new Vars( );
      session_start( );
   }

   /*********************************************/
   /* SnapOn Package Load Functions             */
   /*********************************************/
   /* returns file names in a directory that match an ereg expression */
   function getEregFileNames( $aDirectory, $aEreg ) {
      $fileList = array( );
      
      if ( $handle = @opendir( $aDirectory ) ) {
         /* This is the correct way to loop over the directory. */
         while ( false !== ( $file = readdir($handle) ) ) {
            if ( ereg( $aEreg, $file ) ) {
               array_push( $fileList, $aDirectory . "/" . $file );
            }
         }
      }
      else {
         $this->setError( "Cannot load directory list from $aDirectory", MSG_FATAL );
         return;
      }

      return $fileList;
   }
   
   /* loads descriptors found in the snapOn path passed */
   /* if expected type is set, throw an error if any descriptor is found that isn't
      of the expected type */
   function loadDescriptorsFromPath( $aSnapOnPath, $aExpectedType = null) {
      $snapOnDir = explode( ";", $aSnapOnPath );
      
      foreach( $snapOnDir as $dir ) {
         $snapOnPackageDescriptorList = $this->getEregFileNames( $dir, ".+\.descriptor\.xml$" );
         if ( is_array( $snapOnPackageDescriptorList ) ) {
            foreach( $snapOnPackageDescriptorList as $descriptor ) {
               $this->loadDescriptor( $dir, $descriptor, FILE_BASED, $aExpectedType  );
            } 
         }
      }
   }

   /* default is always true. Descendent can put more conditions on this */
   function isLoadable( &$aDescriptorTagData ) {
      return true;
   }
   
   function loadDescriptor( $aDir, $aDescriptor, $aSource, $aExpectedType = null ) {
      $x = new XML2ArrayParser( $aDescriptor, false );
      if ( $x->isError( ) ) {
         return;
      }      
      /* Note: all snapon headers are loaded, no matter what. 
         Grab the proceedwithload boolean here, though, to see if the snapon
         is ultimately loadable */
      $loadable = $this->isLoadable( $x->tagData );
      
      /* store snapOn information */
      $snapOnName = $x->tagData[ "SNAPON" ][0][ "NAME" ];

      if ( isset( $this->snapOn[ $snapOnName ] ) ) { /* duplicate name */
         $this->setError( "Duplicate SnapOn: $snapOnName", MSG_WARNING );
         return;
      }

      if ( isset( $aExpectedType ) && $x->tagData[ "SNAPON" ][0][ "TYPE" ] != $aExpectedType ) {
         $this->setError( "SnapOn: $snapOnName is the wrong type. Expecting type '$aExpectedType', but it is "
                          . "of type '" . $x->tagData[ "SNAPON" ][0][ "TYPE" ] . "'", MSG_WARNING );
         return;
      }

      /* collect information */
      $this->snapOn[ $snapOnName ] = array( "display_name" => $x->tagData[ "SNAPON" ][0][ "DISPLAY_NAME" ]
                                           ,"directory" => $aDir
                                           ,"filename" => $aDescriptor
                                           ,"version" => $x->tagData[ "SNAPON" ][0][ "VERSION" ]
                                           ,"description" => $x->tagData[ "SNAPON" ][0][ "DESCRIPTION" ]
                                           ,"type" => $x->tagData[ "SNAPON" ][0][ "TYPE" ]
                                           ,"author" => $x->tagData[ "SNAPON" ][0][ "AUTHOR" ]
                                           ,"author_url" => $x->tagData[ "SNAPON" ][0][ "AUTHOR_URL" ]
                                           ,"credits" => $x->tagData[ "SNAPON" ][0][ "CREDITS" ]
                                           ,"screen_shot" => $x->tagData[ "SNAPON" ][0][ "SCREEN_SHOT" ]
                                           ,"create_date" => $x->tagData[ "SNAPON" ][0][ "CREATE_DATE" ]
                                           ,"help_overlay" => $x->tagData[ "SNAPON" ][0][ "HELP_OVERLAY" ]
                                           ,"activation_class" => $x->tagData[ "SNAPON" ][0][ "ACTIVATION_CLASS" ]
                                           ,"administration_class" => $x->tagData[ "SNAPON" ][0][ "ADMINISTRATION_CLASS" ]
                                           ,"cascades_from_snap_on" => $x->tagData[ "SNAPON" ][0][ "CASCADES_FROM_SNAP_ON" ] 
                                           /* note: it is assumed the descendent or calling classes will do something
                                              with the style sheet and inline css values. Phoo doesn't do anything 
                                              special with them other than store them */
                                           ,"style_sheet" => $x->tagData[ "SNAPON" ][0][ "STYLE_SHEET" ]
                                           ,"inline_css" =>  $x->tagData[ "SNAPON" ][0][ "INLINE_CSS" ]
                                           ,"category" => $x->tagData[ "SNAPON" ][0][ "CATEGORY" ]
                                           ,"loaded" => false
                                           ,"loadable" => $loadable
                                           ,"source" => $aSource
                                          );
                                   
      if ( $loadable ) {
         if ( isset( $x->tagData[ "SNAPON" ][0][ "CLASSES" ][0][ "CLASS_PACKAGE" ] ) ) {
            foreach( $x->tagData[ "SNAPON" ][0][ "CLASSES" ][0][ "CLASS_PACKAGE" ] as $package ) {
               /* check to see if the package is inline - if so, set the inline flag and store 
                  the inline code in the package */
               if ( isset( $package[ "INLINE" ] ) ) {
                  $inline = true;
                  $inlineCode = $package[ "INLINE" ];
               }
                  
               $packageKey = count( $this->package );
               $this->package[ $package[ "NAME" ] ] = array( "directory" => $aDir
                                                            ,"loaded" => false
                                                            ,"snap_on" => $snapOnName
                                                            ,"inline" => $inline
                                                            ,"inline_code" => $inlineCode );

               foreach( $package[ "CLASS" ] as $class ) {
                  $this->packageClass[ $class[ "NAME" ] ] = array( "parent" => $class[ "PARENT" ]
                                                                  ,"package_key" => $packageKey
                                                                  ,"snap_on" => $snapOnName );
               }
            }
         }

         if ( isset( $x->tagData[ "SNAPON" ][0][ "OVERLAYS" ][0][ "OVERLAY_PACKAGE" ] ) ) {
            
            foreach( $x->tagData[ "SNAPON" ][0][ "OVERLAYS" ][0][ "OVERLAY_PACKAGE" ] as $package ) {
               /* check to see if the package is inline - if so, set the inline flag and store 
                  the inline code in the package */
               if ( isset( $package[ "INLINE" ] ) ) {
                  $inline = true;
                  $inlineCode = $package[ "INLINE" ];
               }
               
               $packageKey = count( $this->package );
               $this->package[ $package[ "NAME" ] ] = array( "directory" => $aDir
                                                            ,"loaded" => false
                                                            ,"snap_on" => $snapOnName
                                                            ,"inline" => $inline
                                                            ,"inline_code" => $inlineCode );

               foreach( $package[ "OVERLAY" ] as $overlay ) {
                  $this->packageOverlay[ $overlay[ "NAME" ] ] = array( "package_key" => $packageKey
                                                                      ,"type" => $overlay[ "TYPE" ]
                                                                      ,"snap_on" => $snapOnName );
               }
            }
         }

         /* if there are initialization events, add those events */
         if ( is_array( $x->tagData[ "SNAPON" ][0][ "SETUP_EVENT" ] ) ) {
            foreach ( $x->tagData[ "SNAPON" ][0][ "SETUP_EVENT" ] as $event ) {
               if ( isset( $event[ "ATTACH_TO_CLASS" ] ) &&
                    isset( $event[ "TYPE" ] ) &&
                    isset( $event[ "HANDLER" ] ) ) {
                  /* setup the event */
                  /* check cost */
                  $cost = $event[ "cost" ];
                  if ( !isset( $cost ) ) {
                     $cost = 0;
                  }

                  /* add the event */                  
                  $this->addEvent( $event[ "ATTACH_TO_CLASS" ]
                                  ,$event[ "TYPE" ]
                                  ,$event[ "HANDLER" ]
                                  ,$cost );
               }
            }
         }

         /* if there is a "load_with" value, add this to that values list of snapOns to load
            when it gets loaded */
         if ( is_array( $x->tagData[ "SNAPON" ][0][ "LOAD_WITH_SNAP_ON_CLASS" ] ) ) {
            foreach ( $x->tagData[ "SNAPON" ][0][ "LOAD_WITH_SNAP_ON_CLASS" ] as $loadWith ) {
               if ( !is_array( $this->loadWithList[ $loadWith[ "CLASS" ] ] ) ) {
                  $this->loadWithList[ $loadWith[ "CLASS" ] ] = array( );
               }
               array_push( $this->loadWithList[ $loadWith[ "CLASS" ] ], $snapOnName );
            }
         }
      } 
   }

   /* tidy XML */
   function tidy( $aXml ) {
      /* add newlines in appropriate places in the xml */
      $xml = $aXml;
/*
      $xml = ereg_replace( "(<\/[a-zA-Z]*>)", "\\1\n", $aXml );
      $xml = ereg_replace( "(<br>)", "\\1\n", $xml );
      $xml = ereg_replace( "(/>)", "\\1\n", $xml );
      $xml = ereg_replace( "(><)", ">\n<", $xml );
      $xml = ereg_replace( "(<!--)", "\\1\n", $xml );
*/      
      return $xml;
   }
   
   /* load overlays from an xml file. If raw XML flag is set, there is no file, but 
      raw XML has been passed */
   function loadOverlaysInXML( $xmlFile, $aSnapOnType, $aRawXML = false ) {
     
      $x = new XML2ArrayParser( $xmlFile, $aRawXML );
      
      if ( $x->isError( ) ) {
         return;
      }

      foreach ( $x->tagData[ "OVERLAYS" ][ 0 ][ "OVERLAY" ] as $ovl ) {
         
         $name = $ovl[ "NAME" ];
         $type = $ovl[ "TYPE" ];
         $description = $ovl[ "DESCRIPTION" ];
         $syntax = $ovl[ "SYNTAX" ];
         
         /* remove any newlines in name or type (which can happen due to some strange doings
            in the parser */
         $name = str_replace( "\n", "", $name );
         $type = str_replace( "\n", "", $type );

         /* if the first character of the syntax is a newline, remove it */
         $syntax = ereg_replace( "^\n", "", $syntax );

         /* tidy the syntax */
         $syntax = $this->tidy( $syntax );

         /* check to see if this is a "pendage" overlay (i.e., prepends or appends to another) */
         $pendTo = $ovl[ "PEND_TO" ];
         if ( $pendTo > "" ) {
            $pendType = $ovl[ "PEND_TYPE" ];
         }
         else {
            $pendType = "";
         }

         $this->addOverlay( $name, $type, $syntax, $description, $xmlFile, $aSnapOnType, $pendTo, $pendType );
      }
   }

   function getPackageName( $aPackageKey ) {
      $package = array_keys( $this->package );
      $packageName = $package[ $aPackageKey ];
     
      return( $packageName );
   }

   function getClassesInPackage( $aPackageKey ) {
      $classAr = array( );

      foreach( $this->packageClass as $className => $class ) {
         if ( $class[ "package_key" ] == $aPackageKey ) {
            $classAr[ $className ] = $class;
         }
      }

      return $classAr;
   }
      
   function loadSnapOnPackage( $aSnapOnClassName ) {
      
      /* see if SnapOn is defined in the class list */
      if ( !isset( $this->packageClass[ $aSnapOnClassName ] ) ) {
         $this->setError( "SnapOn Class $aSnapOnClassName is unknown or not activated", MSG_WARNING );
         return;
      }
      
      /* see if it's already loaded */
      $packageKey = $this->packageClass[ $aSnapOnClassName ][ "package_key" ];
      $packageName = $this->getPackageName( $packageKey );
      if ( $this->package[ $packageName ][ "loaded" ] ) {
         return; /* package already loaded */
      }

      $snapOnName = $this->package[ $packageName ][ "snap_on" ];
      if ( $this->snapOn[ $snapOnName ][ "cascades_from_snap_on" ] > "" ) {
         $this->loadSnapOn( $this->snapOn[ $snapOnName ][ "cascades_from_snap_on" ] );
      }
      
      /* need to check all the parents of the classes in the package in question to make
         sure they are all loaded */
      $classesInPackage = $this->getClassesInPackage( $packageKey );
      
      foreach( $classesInPackage as $class ) {
         /* check to see if the SnapOn's parent is loaded */
         $parent = $class[ "parent" ];

         /* recursively load parent packages if necessary (only if the parent is in a different package) */
         if ( !class_exists( $parent ) && $this->packageClass[ $parent ][ "package_key" ] != $packageKey ) {
            $this->loadSnapOnPackage( $parent );
            if ( $this->isError( ) ) {
               return;
            }
         }
      }
         
      /* load this package */
      if ( $this->package[ $packageName ][ "inline" ] ) { /* the code is already stored in the package array */
         eval( $this->package[ $packageName ][ "inline_code" ] );
      }
      else {
         /* package is file-based */
         $packageNameWithDir = $this->package[ $packageName ][ "directory" ] . "/" . $packageName;
         require_once( $packageNameWithDir );
      }

      /* mark package as loaded */
      $this->package[ $packageName ][ "loaded" ] = true;
      /* set the packages snapOn header loaded as well */
      $this->setSnapOnLoaded( $this->package[ $packageName ][ "snap_on" ] );
   }    

   function loadOverlayPackage( $aOverlayName ) {
      /* load the overlay package */
      $packageKey = $this->packageOverlay[ $aOverlayName ][ "package_key" ];
      $packageName = $this->getPackageName( $packageKey );

      if ( !isset( $packageName ) || $packageName == "" ) { 
         $this->setError( "Unknown overlay: $aOverlayName", MSG_WARNING );
         return;
      }
         
      /* see if it's already loaded */
      if ( $this->package[ $packageName ][ "loaded" ] ) {

         return; /* package already loaded */
      }

     $snapOnType = $this->snapOn[ $this->package[ $packageName ][ "snap_on" ] ][ "type" ];
      
      /* load this package */
      if ( $this->package[ $packageName ][ "inline" ] ) { /* the code is already stored in the package array */
         $this->loadOverlaysInXML( $this->package[ $packageName ][ "inline_code" ], $snapOnType, true );
      }
      else {
         /* package is file-based */
         $packageNameWithDir = $this->package[ $packageName ][ "directory" ] . "/" . $packageName;
         $this->loadOverlaysInXML( $packageNameWithDir, $snapOnType, false );
         if ( $this->isError( ) ) {
            return;
         }
      }

      /* set the package as loaded */
      $this->package[ $packageName ][ "loaded" ] = true;
      /* set the packages snapOn header loaded as well */
      $this->setSnapOnLoaded( $this->package[ $packageName ][ "snap_on" ] );
   }   

   /* loads a complete SnapOn (all packages). This is generally called when a snapOn refers to another
      SnapOn that it cascades from */
   function loadSnapOn( $aSnapOnName ) {
      if ( !isset( $this->snapOn[ $aSnapOnName ] ) ) {
         $this->setError( "Unknown SnapOn: $aSnapOnName", MSG_WARNING );
      }

      /* if this snapOn cascades from another, load the other first */
      if ( $this->snapOn[ $aSnapOnName ][ "cascades_from_snap_on" ] > "" ) {
         $this->loadSnapOn( $this->snapOn[ $aSnapOnName ][ "cascades_from_snap_on" ] );
      }
      
      /* locate and load all classes belonging to this SnapOn 
         (need to load them individually in case they have dependencies) */
      foreach ( $this->packageClass as $snapOnClassName => $p ) {
         if ( $p[ "snap_on" ] == $aSnapOnName ) {
            $this->loadSnapOnPackage( $snapOnClassName );
         }
      }
      
      /* locate and load all overlays belonging to this SnapOn */
      /* note: this could possibly be more efficient */
      foreach ( $this->packageOverlay as $overlayName => $p ) {
         if ( $p[ "snap_on" ] == $aSnapOnName ) {
            $this->loadOverlayPackage( $overlayName );
         }
      }

      $this->setSnapOnLoaded( $aSnapOnName );
   }

   /* sets the snapOn loaded, if it hasn't already been set.
      returns true if set, false if already set (or doesn't exist) */
   function setSnapOnLoaded( $aSnapOnName ) {
      if ( !isset( $this->snapOn[ $aSnapOnName ] ) ) {
         return false;
      }

      if ( $this->snapOn[ $aSnapOnName ][ "loaded" ] ) {
         return false;
      }
      
      $this->snapOn[ $aSnapOnName ][ "loaded" ] = true;

      return true;
   }
      
   /* returns the specified snapOn descriptor */
   function getSnapOnDescriptor( $aName ) {
      return $this->snapOn[ $aName ];
   }

   /* returns an array of the snapOn descriptors for a given type */
   function getSnapOnDescriptorsForType( $aType ) {
      $snapOnDescriptor = array( );

      foreach ( $this->snapOn as $snapOnName => $descriptor ) {
         if ( $descriptor[ "type" ] == $aType ) {
            $snapOnDescriptor[ $snapOnName ] = $descriptor;
         }
      }

      return $snapOnDescriptor;
   }
   
   /*** base snap on factory
        This should be called when creating a snapOn. Will return an empty snapOn on error
    ***/
   function &snapOnFactory( $aSnapOnClassName, &$aContainer ) {
      if ( !class_exists( $aSnapOnClassName ) ) {
         $this->loadSnapOnPackage( $aSnapOnClassName );
         if ( $this->isError( ) ) {
            $aSnapOnClassName = "PhooEmpty";
         }   
      }

      /* produce the snapon */
      $s = new $aSnapOnClassName( $this, $aContainer );
         
      if ( is_object( $s ) ) {
         /* check to see if there were any other SnapOns that were supposed to load with this one */
         $loadWithList = $this->loadWithList[ $aSnapOnClassName ];
         if ( is_array( $loadWithList ) ) {
            foreach( $loadWithList as $snapOnName ) {
               $this->loadSnapOn( $snapOnName);
               /* if errors, just leave them to be caught and reported in the returned object */
            }
            /* now, clear out the loadWithList for this SnapOnClass, since we don't need to try reloading it if it's
               invoked later */
            unset( $this->loadWithList[ $aSnapOnClassName ] );
         }
         return $s;      
      }
      else {
         $this->setError( "Could not create object: $aSnapOnClassName", MSG_WARNING ); 
      }         
   }
   
   /*** Initialization Functions ***/
   function connect( $aServer, $aDatabase, $aUserid, $aPassword ) {
      $this->connection = new PhooConnection( $aServer, $aDatabase, $aUserid, $aPassword );
      $this->connection->session = &$this;

      if ( $this->isError( ) ) {
         return;
      }
   }

   function checkPreviousLogin( ) {
      /* see if the user has been logged in already - if so, verify the login */
      $loginId = $this->getVar( $this->sessionLoginName, "COOKIE" );
      $password = $this->getVar( $this->sessionPasswordName, "COOKIE" );
      
      if ( $loginId && $password ) {
         $this->logIn( $loginId, $password );
      }
   }

   /* renders an error */   
   function renderError( ) {
      $syntax = "";
      /* if there is an error snapon defined, use it, else
         just render the error plain */
      if ( class_exists( "PhooError" ) ) {
         /* produce the snapon */
         $s =& $this->snapOnFactory( "PhooError", $this );
         $s->process( );
         $syntax .= $s->render( );
      }
      else {
         
         $error = $this->getError( );
         while ( $error ) {
            $syntax .= $error . "<br />";
            $error = $this->getError( );
         }
      }

      return $syntax;
   }
   
   /*** wrapper functions for the Vars object ***/
   function getVar( $aName, $aScope = null ) {
      return $this->vars->getVar( $aName, $aScope );
   }

   function setVar( $aName, $aValue, $aScope = null, $expiration = null  ) {
      $this->vars->setVar( $aName, $aValue, $aScope, $expiration );
   }

   function getVars( $aScope ) {
      return $this->vars->getVars( $aScope );
   }
   
   function setVars( $aArray, $aScope ) {
      $this->vars->setVars( $aArray, $aScope );
   }

   function clearVar( $aName, $aScope ) {
      $this->vars->clearVar( $aName, $aScope );
   }

   function clearVars( $aScope ) {
      $this->vars->clearVars( $aScope );
   }
   /* end wrapper functions for the Vars object */

   function addOverlay( $aName, $aType, $aSyntax, $aDescription, $aSource, $aSnapOnType, $aPendTo = null
                       ,$aPendType = null, $aTheme = "all", $aId = 0 ) {

     $this->overlay[ $aName ] = array( "name" => $aName
                                      ,"type" => $aType
                                      ,"syntax" => $aSyntax
                                      ,"description" => $aDescription
                                      ,"source" => $aSource
                                      ,"snap_on_type" => $aSnapOnType
                                      ,"pend_to" => $aPendTo
                                      ,"pend_type" => $aPendType
                                      ,"theme" => $aTheme
                                      ,"id" => $aId );
                                             
      if ( $aPendTo > "" && $aPendType > "" ) {

         $replaced = false;

         if ( !is_array( $this->pendage[ $aPendTo ] ) ) {
            $this->pendage[ $aPendTo ] = array( );
         }
         else {
            /* see if this overlay is already in the pendage array. If it is, replace it because it is
               being replaced by another overlay */
            for ( $i = 0; $i < count( $this->pendage[ $aPendTo ] ); $i++ ) {
               if ( $this->pendage[ $aPendTo ][ $i ][ "name" ] == $aName ) {
                  $this->pendage[ $aPendTo ][ $i ] = array( "name" => $aName, "pend_type" => $aPendType );
                  $replaced = true;
               }
            }
         }
         
         if ( !$replaced ) { /* push in a new pendage array */
            array_push( $this->pendage[ $aPendTo ], array( "name" => $aName, "pend_type" => $aPendType ) );
         }
      }
   }

   function getOverlay( $aName, $aWithPendages = true ) {

      /* check to see if the overlay is loaded. If not, load its package */
      if ( !$this->isOverlay( $aName ) ) {
         if ( !isset( $this->packageOverlay[ $aName ] ) ) {
            $this->setError( "Unknown overlay: $aName", MSG_WARNING );
            return null;
         }
         else {
            $this->loadOverlayPackage( $aName );
         } 
      }

      $ovl = $this->overlay[ $aName ];
      
      /* if this flag is set, add on the additionial overlay data from any pendage overlays */
      if ( $aWithPendages ) {
         $append = "";
         $prepend = "";
         
         if ( is_array( $this->pendage[ $aName ] ) ) {

            foreach ( $this->pendage[ $aName ] as $p ) {
               $overlayName = $p[ "name" ];
               /* Sorry for the double-clutch here. Table-based overlays may need to be loaded, and this can
                  change the pend_type. So get the overlays once to make sure everything's loaded, then run this again */
               $this->getOverlay( $overlayName );
            }
            
            foreach ( $this->pendage[ $aName ] as $p ) {
               $overlayName = $p[ "name" ];
               $pendType = $p[ "pend_type" ];
               /* Now, loading . . . */
               $pendOvl = $this->getOverlay( $overlayName );
               
               switch ( $pendType ) {
                  case "append" :
                     $append .= $pendOvl[ "syntax" ];
                     break;
                     
                  case "prepend" :
                     $prepend = $pendOvl[ "syntax" ] . $prepend;
                     break;
                     
               }
            }

            $ovl[ "syntax" ] = $prepend . $ovl[ "syntax" ] . $append;
         }
      }
      
      return $ovl;
   }

   /* this function calls getOverlay to ensure that an overlay has been loaded, but
      it does not return the overlay object */
   function loadOverlay( $aName, $aWithPendages = true ) {
      $this->getOverlay( $aName, $aWithPendages );
   }
   
   function isOverlay( $aName ) { 
      /* returns true if an overlay of the name specified exists */
      return isset( $this->overlay[ $aName ] );
   }

   function isOverlayAvailable( $aName ) {
      return isset( $this->packageOverlay[ $aName ] );
   }      

   /*** Security/User Functions ***/
   function loadRoles( $aRoleId ) {
      /* determines if this role id contains other roles. Returns an array containing
         this role and its contained roles (and their contained roles, etc, etc ) */

      /* initialize the SnapOn that returns role information */
      $s =& $this->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = ROLE_EVERYONE;
      $s->command = "=|Role.information.get|=";

      $role = array( );
      $roleQueue = array( );
      array_push( $roleQueue, $aRoleId );
     
      /* loop through the role information, gathering up any contained roles as well */
      while ( count( $roleQueue ) > 0 ) {
         $roleId = array_shift( $roleQueue );
         if ( $roleId == "" ) {
            break;
         }

         /* set the role id in a variable */
         $this->setVar( "role_id", $roleId, "PRIVATE" );
         $s->process( );
         $this->clearVar( "role_id", "PRIVATE" ); /* clear it out, to prevent confusion for other queries that look
                                                     at role_id */
         $row = $s->getResultRow( );
         if ( is_array( $row ) ) {
            array_push( $role, $roleId );
         }
         else {
            break;
         }
         
         $containedRole = explode( ",", $row[ "contained_roles" ] );

         foreach( $containedRole as $c ) {
            /* very important we don't redo a role already done (unless you like stack overflows) */
            if ( !in_array( $c, $role ) ) {
               array_push( $roleQueue, $c );
            }
         }
      }

      /* return the roles found */
      return $role;
   }

   /* returns a list of roles for the user, in a string with comma separation */
   function getUserRoleList( ) {
      $roleList = "";
      
      if ( is_array( $this->user[ "user_roles" ] ) ) {
         $roleCount = 0;
         foreach ( $this->user[ "user_roles" ] as $r ) {
            if ( $roleCount > 0 ) {
               $roleList .= ",";
            }
            $roleList .= $r;
            $roleCount++;
         }
      }      

      return $roleList;
   }

   /* login the userid, password by comparing against a persistent storage entry
      and set Cookie variables and load user information if the login is good. 
      Otherwise error message and reset Session variables. Login flag is set */
   function logIn( $aLoginId, $aEncPassword, $aStoreInCookie = true ) {
      $this->loggedIn = false;
      
      $userVal = $this->snapOnFactory( $this->snapOnUserValidator, $this );
      if ( $this->isError( ) ) {
         return;
      }

      $parms = array( "user_login_id" => $aLoginId
                     ,"user_password" => $aEncPassword
                    );
                    
      $this->setVars( $parms, "APPLICATION" );
      $userVal->process( );
      if ( !$userVal->isError( ) ) {
         $row = $userVal->getResultRow( );

         if ( $row ) {
            /* success on the login check. Store the login information in cookies */
            if ( $aStoreInCookie ) {
               $this->setVar( $this->sessionLoginName, $aLoginId, "COOKIE", time()+60*60*4 );
               $this->setVar( $this->sessionPasswordName, $aEncPassword, "COOKIE", time()+60*60*4 );
            }

            $this->loggedIn = true;
            /* store the user's information */
            $this->user = $row;
            /* if there is a role id (which there should be) load an array with all the
               roles it contains */
            $this->user[ "user_roles" ] = $this->loadRoles( $this->user[ "user_role_id" ] );   
         }
         else {
            /* Note: Why set MSG_WARNING here, instead of MSG_SECURITY? Well, this is not a security
               error with the login form itself, it's just a warning that they set the wrong userid/password.
               Still want the login form to render. It won't on a MSG_SECURITY error, for good reason. */
            $this->setError( "Illegal userid/password on attempted login", MSG_INFO ); 
            $this->logOut( );
         }

      }
      return $this->loggedIn;
   }

   function logOut( ) {
      $this->clearVar( $this->sessionLoginName, "COOKIE" );
      $this->clearVar( $this->sessionPasswordName, "COOKIE" );

      unset( $this->user );
      $this->loggedIn = false;
   }
      
   /* check to see if the user is currently logged in by checking session variables */
   function isLoggedIn( ) {
      return $this->loggedIn;
   }

   function getUserInformation( ) {
      return $this->user;
   }

   /* returns the name the user is known as
      when displaying
    */
   function getUserKnownBy( ) {
      if ( is_array( $this->user ) ) {
         $knownBy = $this->user[ "user_known_by" ];
      }
      return $knownBy;
   }

   /* returns the current user id, or 0 if no user logged in
    */
   function getUserID( ) {
      if ( is_array( $this->user ) ) {
         $userID = $this->user[ "user_id" ];
      }
      else {
         $userID = 0;
      }
      
      return $userID;
   }

   /* returns an array of the current user's roles */
   function getUserRoles( ) {

      if ( is_array( $this->user ) ) {
         if ( isset( $this->user[ "user_roles" ] ) ) {
            $roles = $this->user[ "user_roles" ];
         }
      }

      if ( !isset( $roles ) ) {
         $roles = array( ROLE_EVERYONE ); /* default to the EVERYONE role */
      }

      return $roles;
   }   


   /******************************/
   /* Event Handling Functions   */
   /******************************/
   /* loads events based on an event loading command passed in */
   function loadEvents( $aCommand ) {
      /* run the command */
      $s =& $this->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = ROLE_EVERYONE;
      $s->command = $aCommand;
      $s->process( );
      while ( $row = $s->getResultRow( ) ) {
         /* add each event to the event queue */
         $this->addEvent( $row[ "target_snapon" ], $row[ "event_type" ], $row[ "event_handler" ] );
      }
   }
      
   function addEvent( $aSnapOnClassName, $aEventType, $aEventName, $aEventCost = 0 ) {
      if ( !is_array( $this->event[ $aSnapOnClassName ][ $aEventType ] ) ) {
         $this->event[ $aSnapOnClassName ][ $aEventType ] = array( );
      }

      /* note: the event cost parameter is, by convention, somewhere between 0 and 100. It is useful
         for putting costlier events at the end of the queue. This works well for spam checkers, allowing
         less costly events (or preventative events like kaptchas or human checks) to run first and thus
         potentially make running the more costly events needless */

      /* locate the first event in this array that has a cost >= the event cost passed in above */
      $loc = 0;
      foreach ( $this->event[ $aSnapOnClassName ][ $aEventType ] as $event ) {
         /* cost is in the second element */
         if ( $event[ 1 ] >= $aEventCost ) {
            break;
         }
         $loc++;
      }
       
      /* now insert at the location specified */
      if ( $loc == 0 ) {  
         /* put at the beginning */
         array_unshift( $this->event[ $aSnapOnClassName ][ $aEventType ], array( $aEventName, $aEventCost ) );
      }
      elseif ( $loc = count( $this->event[ $aSnapOnClassName ][ $aEventType ] ) ) {
         /* put at the end */         
         array_push( $this->event[ $aSnapOnClassName ][ $aEventType ], array( $aEventName, $aEventCost ) );
      }
      else {
         /* insert the new event at the location specified */
         $slice = array_slice( $this->event[ $aSnapOnClassName ][ $aEventType ], 0, $loc );
         array_push( $slice, array( $aEventName, $aEventCost ) );
         $this->event[ $aSnapOnClassName ][ $aEventType ] = array_merge( $slice, 
               array_slice( $this->event[ $aSnapOnClassName ][ $aEventType ], $loc, 
                            count( $this->event[ $aSnapOnClassName ][ $aEventType ] ) - $loc + 1 ) );
      }
   }
   
   function getNextEvent( $aSnapOnClassName, $aEventType, $aEventCounter ) {
      /* get array of events for this snapOn and type */
      $eventAr = $this->event[ $aSnapOnClassName ][ $aEventType ];
      if ( count( $eventAr ) >= $aEventCounter + 1 ) { /* there are events */
         $ea = $eventAr[ $aEventCounter ];
         return $ea[ 0 ]; /* return the name of the event */
      }
      else {
         return null;
      }     
   }

   /* will look in the event array to see if an event 
      is attached to this Session for the event type passed.
      If so, will instatiate the event handler, let it know who
      is calling it, and process it */
   function fireEvents( $aEventType ) {
      $eventCounter = 0;
      while ( $eventName = $this->getNextEvent( $this->name, $aEventType, $eventCounter ) ) {
         $e =& $this->snapOnFactory( $eventName, $this );
         if ( $e->isError( ) ) {
            return;
         }

         /* process the event */
         $e->process( );
         if ( $e->isError( ) ) {
            return;
         }
         $eventCounter++;
      }         
   }         
      
   /***** Utility Functions *****/
   
   /* "cleans" a value by altering the prefixes and suffixes for templates, snapons, parameters
       so that they won't expand */
   function clean( $aValue ) {
         $value = $aValue;
         $value = str_replace( PLACEHOLDER_PREFIX, SUBSTITUTE_PLACEHOLDER_PREFIX, $value );
         $value = str_replace( PLACEHOLDER_SUFFIX, SUBSTITUTE_PLACEHOLDER_SUFFIX, $value );
         $value = str_replace( EMBEDDEDSNAPON_PREFIX, SUBSTITUTE_EMBEDDEDSNAPON_PREFIX, $value );
         $value = str_replace( EMBEDDEDSNAPON_SUFFIX, SUBSTITUTE_EMBEDDEDSNAPON_SUFFIX, $value );
         $value = str_replace( OVERLAY_PREFIX, SUBSTITUTE_OVERLAY_PREFIX, $value );
         $value = str_replace( OVERLAY_SUFFIX, SUBSTITUTE_OVERLAY_SUFFIX, $value );
         return $value;
   }

   /* reverses the effects of clean */
   function unClean( $aValue ) {
         $value = $aValue;
         $value = str_replace( SUBSTITUTE_PLACEHOLDER_PREFIX, PLACEHOLDER_PREFIX, $value );
         $value = str_replace( SUBSTITUTE_PLACEHOLDER_SUFFIX, PLACEHOLDER_SUFFIX, $value );
         $value = str_replace( SUBSTITUTE_EMBEDDEDSNAPON_PREFIX, EMBEDDEDSNAPON_PREFIX, $value );
         $value = str_replace( SUBSTITUTE_EMBEDDEDSNAPON_SUFFIX, EMBEDDEDSNAPON_SUFFIX, $value );
         $value = str_replace( SUBSTITUTE_OVERLAY_PREFIX, OVERLAY_PREFIX, $value );
         $value = str_replace( SUBSTITUTE_OVERLAY_SUFFIX, OVERLAY_SUFFIX, $value );
         return $value;
   }
}

define( "SINGLE_QUOTE", "'" );
define( "DOUBLE_QUOTE", "\"" );

class SnapOnDescriptor {
   var $author;
   var $version;
   var $createDate;
   var $modificationHistory = array( );
   var $purpose;
   var $expects = array( );
   var $meta = array( );

   /* set functions */
   function setAuthor( $aAuthor ) {
      $this->author = $aAuthor;
   }

   function setVersion( $aVersion ) {
      $this->version = $aVersion;
   }
   
   function setCreateDate( $aCreateDate ) {
      $this->createDate = $aCreateDate;
   }

   function addModificationHistory( $aDescription, $aDate ) {
      array_push( $this->modificationHistory, array( "Description"=>$aDescription, "Date"=>$aDate ) ); 
   }

   function setPurpose( $aPurpose ) {
      $this->purpose = $aPurpose;
   }

   function addExpects( $aVariable, $aScope ) {
      array_push( $this->expects, array( "Variable"=>$aVariable, "Scope"=>$aScope ) ); 
   }

   function addMeta( $aKey, $aValue ) {
      array_push( $this->meta, array( "Key"=>$aKey, "Value"=>$aValue ) ); 
   }

   /* get functions */
   function getAuthor( ) {
      return $this->author;
   }

   function getVersion( ) {
      return $this->version;
   }
   
   function getCreateDate( ) {
      return $this->createDate;
   }

   function getModificationHistory( ) {
      return $this->modificationHistory;
   }

   function getPurpose( ) {
      return $this->purpose;
   }

   function getExpects( ) {
      return $this->expects;
   }

   function getMeta( ) {
      return $this->meta;
   }
}   

/* define event types */
define( "EVENT_PRERENDER", "eventPreRender" );
define( "EVENT_POSTRENDER", "eventPostRender" );
define( "EVENT_PREPROCESS", "eventPreProcess" );
define( "EVENT_POSTPROCESS", "eventPostProcess" );
define( "EVENT_PRESAVE", "eventPreSave" );
define( "EVENT_POSTSAVE", "eventPostSave" );
define( "EVENT_POSTROWRETRIEVE", "eventPostRowRetrieve" );

/* define snapOn and overlay prefixes and suffixes */
define( "PLACEHOLDER_PREFIX", "_@" );
define( "PLACEHOLDER_SUFFIX", "@_" );
define( "EMBEDDEDSNAPON_PREFIX", "_{" );
define( "EMBEDDEDSNAPON_SUFFIX", "}_" );
define( "OVERLAY_PREFIX", "=|" );
define( "OVERLAY_SUFFIX", "|=" );
define( "CONSTANT_PREFIX", "_=" );
define( "CONSTANT_SUFFIX", "=_" );
define( "SUBSTITUTE_PLACEHOLDER_PREFIX", "_~" );
define( "SUBSTITUTE_PLACEHOLDER_SUFFIX", "~_" );
define( "SUBSTITUTE_EMBEDDEDSNAPON_PREFIX", "_[" );
define( "SUBSTITUTE_EMBEDDEDSNAPON_SUFFIX", "]_" );
define( "SUBSTITUTE_OVERLAY_PREFIX", "=~" );
define( "SUBSTITUTE_OVERLAY_SUFFIX", "~=" );

define( "ROW_NUMBER_TEMPLATE", "_#row#_" );
define( "GMT_TIME", "_#gmt_time#_" );
define( "BASE_TIME_FORMAT", "Y-m-d H:i:s" );

/*** Abstract class SnapOn
     This is the base SnapOn object.
 ***/
class PhooClass extends PhooBase {
   var $package; /* the name of the package this snapon belongs in */
   var $descriptor; /* pointer to the snapons descriptor */
   var $session; /* current session */
   var $name;
   var $securityRole = ROLE_NOONE; /* force objects to pick a security role */
   var $securityErrorMessage = "You do not have security privileges to perform this operation";
   var $command; /* command to run */
   var $template; /* template to display */
   var $field; /* array of field descriptors */
   var $invisibleOnSecurityErrors = false; /* set to true if you just want the snapon to go invisible
                                              if the user doesn't have security privileges to it */
   var $disabled = false; /* set to true to disable the snapon (turn it invisible) */
   var $noRepeat = false; /* set to true if you don't want the snapon to repeat itself
                             in rendering - useful for report headers, etc */

   var $modifier = array( ); /* array of modifier classes */
   var $displayFilter = array( ); /* array of display filter classes */
   var $processFilter = array( ); /* array of process filter classes */
   var $privateVar = array( ); /* local variable array */
   var $container; /* points to the containing snapOn */
   
   function PhooClass( &$aSession, &$aContainer ) {
      $this->session = &$aSession;
      $this->container = &$aContainer;      
      $this->descriptor = new SnapOnDescriptor( ); 
      $this->setDescriptor( ); /* descendent should populate this */
      /* aquire any expected variables from the container */
      $this->aquireExpectedVariables( );
      
      /* perform any setups that need to be done */
      $this->setup( );
   }

   /* abstract function - descriptor should be populated here */
   function setDescriptor( ) {
   }

   function checkSecurity( ) {
      /* retrieve the user's roles and see if any of them work for this object */
      $userRole = $this->session->getUserRoles( );
      $hasSecurity = false;

      /* role for an object can be either a list of roles or a single role. See if this
         is an array. If not, make it one for just a sec */
      if ( !is_array( $this->securityRole ) ) {
         $objectRole = array( $this->securityRole );
      }
      else {
         $objectRole = $this->securityRole;
      }
      foreach( $objectRole as $or ) {
         if ( in_array( $or, $userRole ) ) {
            $hasSecurity = true;
            break;
         }
      }
      
      if ( !$hasSecurity ) {
         /* the role is not one of the user's roles */
         if ( $this->invisibleOnSecurityErrors ) { /* disable the object */
            $this->disabled = true;
         }
         else {
            $this->setError( $this->securityErrorMessage . " : " . $this->name, MSG_SECURITY );
         }
      }
   }
   
   /* check the "expects" of the new snapon. Sometimes a variable
      it's expecting is in the container's private vars, which will be unavailable to
      the new snapon. If so, create the variable it needs */
   function aquireExpectedVariables( ) {
      $varSet = array( );
         
      $expects = $this->getExpects( );
      foreach ( $expects as $e ) {
         $varName = $e["Variable"];
         $scope = $e["Scope"];
            
         /* check for the expectation of finding the variable in the container */
         if ( $scope == "CONTAINER" ) {
            $v = $this->container->getVar( $varName );

            if ( isset( $v ) ) {
               /* set the expected variable */
               $this->setVar( $varName, $v, "PRIVATE" ); 
            }            
         }
      }
   }

   /***********************************************/
   /* Wrapper functions for the session variables */
   /***********************************************/
   function getVar( $aName, $aScope = "PRIVATE" ) {
      /* if scope is PRIVATE, look at the snapon's private variables */
      if ( $aScope == "PRIVATE" ) {
         if ( isset( $this->privateVar[ $aName ] ) ) {
            return $this->privateVar[ $aName ];
         }
         else {
            $aScope = null; /* null out the scope - will check the session variables next */
         }
      }
      
      return $this->session->getVar( $aName, $aScope );
   }

   function setVar( $aName, $aValue, $aScope = "PRIVATE", $expiration = null ) {
      /* if scope is PRIVATE, set the snapon's private variables */
      if ( $aScope == "PRIVATE" ) {
         $this->privateVar[ $aName ] = $aValue;
      }
      else {
         $this->session->setVar( $aName, $aValue, $aScope, $expiration );
      }
   }

   function getVars( $aScope = "PRIVATE" ) {
      /* if scope is PRIVATE, look at the snapon's private variables */
      if ( $aScope == "PRIVATE" ) {
         return $this->privateVar;
      }
      else {
         return $this->session->getVars( $aScope );
      }
   }
   
   function setVars( $aArray, $aScope = "PRIVATE" ) {
      /* if scope is PRIVATE, set the snapon's private variables */
      if ( $aScope == "PRIVATE" ) {
         $this->privateVar = $aArray;
      }
      else {
         $this->session->setVars( $aArray, $aScope );
      }
   }

   function clearVar( $aName, $aScope = "PRIVATE" ) {
      /* if scope is PRIVATE, clear the snapon's private variable */
      if ( $aScope == "PRIVATE" ) {
         unset( $this->privateVar[ $aName ] );
      }
      else {
         $this->session->clearVar( $aName, $aScope );
      } 
   }

   function clearVars( $aScope = "PRIVATE" ) {
      /* if scope is PRIVATE, clear the snapon's private variables */
      if ( $aScope == "PRIVATE" ) {
         unset( $this->privateVar );
      }
      else {
         $this->session->clearVars( $aScope );
      }
   }

   /***************************************/
   /* snapon descriptor wrapper functions */
   /***************************************/
   function setAuthor( $aAuthor ) {
      $this->descriptor->setAuthor( $aAuthor );
   }

   function setVersion( $aVersion ) {
      $this->descriptor->setVersion( $aVersion );
   }
   
   function setCreateDate( $aCreateDate ) {
      $this->descriptor->setCreateDate( $aCreateDate );
   }

   function addModificationHistory( $aDescription, $aDate ) {
      $this->descriptor->addModificationHistory( $aDescription, $aDate );
   }

   function setPurpose( $aPurpose ) {
      $this->descriptor->setPurpose( $aPurpose );
   }

   function addExpects( $aVariable, $aScope ) {
      $this->descriptor->addExpects( $aVariable, $aScope );
   }

   function addMeta( $aKey, $aValue ) {
      $this->descriptor->addMeta( $aKey, $aValue );
   }
   
   /* get functions */
   function getAuthor( ) {
      return $this->descriptor->getAuthor( );
   }

   function getVersion( ) {
      return $this->descriptor->getVersion( );
   }
   
   function getCreateDate( ) {
      return $this->descriptor->getCreateDate( );
   }

   function getModificationHistory( ) {
      return $this->descriptor->getModificationHistory( );
   }

   function getPurpose( ) {
      return $this->descriptor->getPurpose( );
   }

   function getExpects( ) {
      return $this->descriptor->getExpects( );
   }

   function getMeta( ) {
      return $this->descriptor->getMeta( );
   }
   /* end snapon descriptor wrapper functions */

   /* locates placeholders (overlays, snapons, values, constants (these are delimited by prefixes
      and suffixes) in a string, starting from the start pos 
    */
   function getPlaceHolderName( $aPrefix, $aSuffix, $aString, &$aStartPos ) {
      /* kickin' it oldschool . . . found this easier than trying to use regexes */

      /* check to see if at end of string */
      if ( $aStartPos >= strlen( $aString ) ) {
         return null;
      }

      /* starting at startpos, locate the next occurrence of the prefix */
      $ppos = strpos( $aString, $aPrefix, $aStartPos );

      if ( $ppos === false ) { 
         return null;
      }

      /* starting from the position after the found prefix, locate the next suffix */
      $namePos = $ppos + strlen( $aPrefix );
      $spos = strpos( $aString, $aSuffix, $namePos );
      if ( !$spos ) {
         return null;
      }

      $aStartPos = $spos + strlen( $aSuffix );
      
      return substr( $aString, $namePos, $spos - $namePos );
    }
   
   /* expands to their syntax the overlays (if any) embedded in the command or template */
   function expandOverlays( $aCommandOrTemplate ) {
      $expandedList = array( );

      if ( $this->disabled ) { 
         return;
      }

      if ( $aCommandOrTemplate == "" ) {
         return "";
      }

      
      /* locate any embedded overlays */
      $pos = 0;
      $overlayName = $this->getPlaceHolderName( OVERLAY_PREFIX, OVERLAY_SUFFIX, $aCommandOrTemplate, $pos ); 
      
      /* loop here, so that embedded overlays will be loaded */
      while ( $overlayName ) {
         if ( in_array( $overlayName, $expandedList ) ) {
            /* Note: removing this from now as this check throws false errors if an overlay is referenced twice */
//            $this->setError( "Overlay: $overlayName, called from " . $this->name . ", is self-referencing, which will result in an endless loop.", MSG_WARNING );
//            return;
         }
         
         $so = $this->session->getOverlay( $overlayName );

         if ( !is_array( $so ) ) {
            /* replace it with an error message */
            $s = new PhooEmpty( $this->session, $this ); /* set to an empty snapon to avoid later errors */
            $s->setError( "Unknown overlay: $overlayName, called from " . $this->name . ".", MSG_WARNING );
            $overlaySyntax = $s->renderError( );
         }
         else {
            $overlaySyntax = $so[ "syntax" ];

            /* check for any embedded constants */
            $cPos = 0;
            $constantSyntax = $this->getPlaceHolderName( CONSTANT_PREFIX, CONSTANT_SUFFIX, $overlaySyntax, $cPos ); 
      
            while ( $constantSyntax ) {
               /* set the constant in an application variable */
               $parse = explode( "=", $constantSyntax );
               if ( isset( $parse[ 1 ] ) ) {
                  $this->setVar( trim( $parse[ 0 ] ), trim( $parse[ 1 ] ), "APPLICATION" );
               }

               /* remove the constant */
               $overlaySyntax = 
                  str_replace( CONSTANT_PREFIX . $constantSyntax . CONSTANT_SUFFIX, "", $overlaySyntax );
               $cPos = $cPos - strlen( CONSTANT_PREFIX . $constantSyntax . CONSTANT_SUFFIX );

               $constantSyntax = $this->getPlaceHolderName( CONSTANT_PREFIX, CONSTANT_SUFFIX, $overlaySyntax, $cPos ); 
            }
         }

         $aCommandOrTemplate = 
            str_replace( OVERLAY_PREFIX . $overlayName . OVERLAY_SUFFIX, $overlaySyntax, $aCommandOrTemplate );

         $pos = 0; /* always reset the pos, as there may be overlays embedded in the previous overlay */
         /* push what was just expanded onto the stack */
         array_push( $expandedList, $overlayName );
         $overlayName = $this->getPlaceHolderName( OVERLAY_PREFIX, OVERLAY_SUFFIX, $aCommandOrTemplate, $pos ); 
      }
      
      return $aCommandOrTemplate;
   }

   /* to be done before any processing. NOTE: this is also done
      before the security check, so this method should not contain
      any actions, just setup (such as adding expects, setting
      templates, etc */
   function setup( ) {

   }

   /* peform the action for this snapOn */
   function doAction( ) {
   }
   
   /* public method - should be called for every snapOn. This
      processes the snapOn.
      Don't override or extend this */
   function process( ) {
      /* check to see if snapOn can be processed */
      $this->checkSecurity( );
      
      if ( $this->disabled ) { 
         return;
      }
      
      if ( $this->isError( ) ) {
         return;
      }

      /* fire any pre-process events */
      $this->fireEvents( EVENT_PREPROCESS );
      if ( $this->isError( ) ) {
         return;
      }

      /* perform its action */
      $this->doAction( );      
      /* fire any post-process events */
      /* NEED TO CHECK FOR A SUCCESSFUL COMPLETION */
      /* $this->fireEvents( EVENT_POSTPROCESS ); */
   }

   /* will look in the event array to see if an event 
      is attached to this SnapOn for the event type passed.
      If so, will instatiate the event handler, let it know who
      is calling it, and process it */
   function fireEvents( $aEventType ) {
      $eventCounter = 0;
      while ( $eventName = $this->session->getNextEvent( $this->name, $aEventType, $eventCounter ) ) {
         $e =& $this->session->snapOnFactory( $eventName, $this );
         if ( $e->isError( ) ) {
            return;
         }
         /* process the event */
         $e->process( );
         if ( $e->isError( ) ) {
            return;
         }
         $eventCounter++;
      }         
   }         

   /* utility function to replace quotes with 
      html-safe syntax */   
   function replaceQuotes( $text, $value, $quoteType ) {

      switch ( $quoteType ) {
         
         case SINGLE_QUOTE : 
            if ( strpos( $value, SINGLE_QUOTE ) != false ) {
               while ( strpos( $text, "'$value'" ) != false ) {
                  $newValue = str_replace( "'", "&#" . ord( "'" ) . ";", $value );
                  $text = str_replace( "'$value'", "'$newValue'", $text );
               }
            }
            break;

         case DOUBLE_QUOTE :
            if ( strpos( $value, DOUBLE_QUOTE ) != false ) {
               while ( strpos( $text, "\"$value\"" ) != false ) {
                  $newValue = str_replace( "\"", "&quot;", $value );
                  $text = str_replace( "\"$value\"", "\"$newValue\"", $text );
               }
            }
            break;

         default :
            $this->setError( "Illegal Quote Type: " . $quoteType );
            break;
      }

      return $text;
   }
  
   /* converts placeholders to sql parameters. values are loaded from 
      the session variables */
   function setSqlParameters( $aSqlTemplate ) {
      $replacing = true;
      $sql = $aSqlTemplate;

      $count = 0;

      while( $replacing ) {
         unset( $match );
         eregi( PLACEHOLDER_PREFIX . "([a-z,0-9,_]*)" . PLACEHOLDER_SUFFIX, $sql, $match );
         if ( isset( $match[1] ) ) {
            $name = $match[ 1 ];
            /* find the variable in the Vars object, following scope precedence. 
               (PRIVATE scope will get first shot) */
            $value = $this->getVar( $name );
            if ( isset( $value ) ) {
               /* check to see if the value itself has placeholders in it. If so, substitute
                  them out so they don't get replaced in a later syntax replacement */
               $value = str_replace( PLACEHOLDER_PREFIX, SUBSTITUTE_PLACEHOLDER_PREFIX, $value );
               $value = str_replace( PLACEHOLDER_SUFFIX, SUBSTITUTE_PLACEHOLDER_SUFFIX, $value );
            
               if ( !get_magic_quotes_gpc( ) ) { /* need to add slashes if magic quotes not on */
                  $value = addslashes( $value );
               }
            }

            $sql = str_replace( PLACEHOLDER_PREFIX . $name . 
                                PLACEHOLDER_SUFFIX, $value, $sql );
         }
         else {
            $replacing = false;
         }
      } 
      /* replace any substitutions done earlier to prevent placeholders that come back in the actual
         result values (such as when querying the control table) from being overwritten */      
      $sql = str_replace( SUBSTITUTE_PLACEHOLDER_PREFIX, PLACEHOLDER_PREFIX, $sql );
      $sql = str_replace( SUBSTITUTE_PLACEHOLDER_SUFFIX, PLACEHOLDER_SUFFIX, $sql );

      /* replace special fields */
      $sql = str_replace( GMT_TIME, gmdate( BASE_TIME_FORMAT, time( ) ), $sql );

      return $sql;
   }   

   /* populates any parameter variables with real values in
      the template string passed in. This should be used
      for display templates, not commands */
   function populateParameters( $aTemplate ) {
      if ( $this->disabled ) { 
         return;
      }

      if ( $aTemplate == "" ) {
         return "";
      }

      $pos = 0;
      $parameterName = $this->getPlaceHolderName( PLACEHOLDER_PREFIX, PLACEHOLDER_SUFFIX, $aTemplate, $pos ); 
      
      /* loop here, so that embedded overlays will be loaded */
      while ( $parameterName ) {
         
         /* get the value for the parameter from the object's variables, if it exists */
         $value = $this->getVar( $parameterName );
         
         /* if magic quotes are set, strip slashes from the value */
         if ( get_magic_quotes_gpc( ) ) {
            /* strip slashes */
            $value = stripslashes( $value );
         }

         /* format the data, if a format is specified */
         if ( isset( $this->field[ $parameterName ] ) ) {
            if ( isset( $this->field[ $parameterName ]["format"] ) ) {
               switch ( $this->field[ $parameterName ]["type" ] ) {
                  case "date" :  /* date is the only field type handled currently */
                     if ( $value > "" ) {
                        $value = date( $this->field[ $parameterName ]["format"], strtotime( $value ) );
                     }
                     break;
                  default :
                     $this->setError( "Formats only allowed on date fields currently. Insure the field type is set", 
                                      MSG_INFO );
                     break;
               }
            }
            /* check for the value to use in case of blank */
            if ( isset( $this->field[ $parameterName ]["onblank"] ) && $value == "" ) {
               $value = $this->field[ $parameterName ]["onblank"];
            }
         }

         $aTemplate = 
            str_replace( PLACEHOLDER_PREFIX . $parameterName . PLACEHOLDER_SUFFIX, $value, $aTemplate );

         /* fix any embedded quotes (for instance, the situation where we have 
            a value with a single quote in it that is embedded with in a value=' . . ' attribute */
         $aTemplate = $this->replaceQuotes( $aTemplate, $value, SINGLE_QUOTE );                                  
         $aTemplate = $this->replaceQuotes( $aTemplate, $value, DOUBLE_QUOTE );
            
         $pos = 0; /* always reset the pos, as there may be overlays embedded in the previous overlay */
         $parameterName = $this->getPlaceHolderName( PLACEHOLDER_PREFIX, PLACEHOLDER_SUFFIX, $aTemplate, $pos ); 
      }
      return $aTemplate;
   }

   /******************************************************************************/
   /* utility functions for retrieving and changing data                         */
   /******************************************************************************/
   function getRowsFromQueryCommand( $aCommand, $aReturnType = null ) {
      /* returns an array of rows resulting from a query command. Returns null if no rows exist */
      $this->session->connection->executeSelectSql( $this->setSqlParameters( $this->expandOverlays( $aCommand ) ) );

      if ( $this->isError( ) ) {
         return;
      }

      $rows = array( );
      
      while ( $row = $this->session->connection->getResultRow( $aReturnType ) ) {
         array_push( $rows, $row );   
      }

      if ( count( $rows ) == 0 ) {
         $rows = null;
      }

      return $rows;
   }

   function runModificationCommand( $aCommand ) {
      /* an error will be set if there is a problem */
      $this->session->connection->executeModificationSql( 
             $this->setSqlParameters( $this->expandOverlays( $aCommand ) ) );
   }

   /* "cleans" a value by altering the prefixes and suffixes for templates, snapons, parameters
       so that they won't expand */
   function clean( $aValue ) {
      return ( $this->session->clean( $aValue ) );
   }

   /* reverses the effects of clean */
   function unClean( $aValue ) {
      return ( $this->session->unClean( $aValue ) );
   }

   /*
    * cleans a row by replacing certain prefixes and suffixes that would result
      in template explansion - this is primarily used for returned row data from a database call (in 
      descendent snapOns like SnapOnQuery and SnapOnProcess) */
   function cleanRow( $row ) {
      foreach ( $row as $key => $value ) {
         $row[ $key ] = $this->clean( $value );
      }
      return $row;
   }
   
   /*
    * Reverses the effects of cleanRow */
   function unCleanRow( $row ) {
      foreach ( $row as $key => $value ) {
         $row[ $key ] = $this->unClean( $value );
      }
      return $row;
   }

   /* utility function, cleans up XML */
   function xmlClean( $aString ) {
      $newString = "";

   	for ( $i = 0; $i < strlen( $aString ); $i++ ) {
   		$c = substr( $aString, $i, 1 );
	   	$ord = ord($c);
         /* convert any characters above 128 */
   		if ( $ord >= 128 ) {
	   		$c = "&#" . $ord . ";";
         }
         
         $newString .= $c;
      }

      /* replace Word-style quotes with regular quotes */
      $newString = str_replace( "&#226;&#128;&#156;", "\"", $newString );
      $newString = str_replace( "&#226;&#128;&#157;", "\"", $newString );

      /* other fixes (single quotes, dashes) */
      $newString = str_replace( "&#226;&#128;&#153;", "'", $newString );
      $newString = str_replace( "&#226;&#128;&#147;", "-", $newString );
      $newString = str_replace( "&#226;&#128;&#166;", "...", $newString );

      return wordwrap( $newString, 75 );
   }

   /* renders the snapOn */
   function renderSnapOn( ) {
      /* if object is disabled, return a blank */
      if ( $this->disabled ) { 
         return "";
      }
      /* convert overlays to syntax in the template */
      $this->template = $this->expandOverlays( $this->template );
      
      /* expand parameters */
      $syntax = $this->populateParameters( $this->template );

      /* render any embedded snapons */
      $syntax = $this->renderEmbeddedSnapOns( $syntax );

      return $syntax;
   }

   /* renders embedded snapons in a the template */ 
   function renderEmbeddedSnapOns( $aPartialRender ) {
      $renderedList = array( );
      
      /* if object is disabled, return a blank */
      if ( $this->disabled ) { 
         return "";
      }

      /* locate any embedded snapOns */
      $pos = 0;
      $snapOnClassName = $this->getPlaceHolderName( EMBEDDEDSNAPON_PREFIX, EMBEDDEDSNAPON_SUFFIX, 
                        $aPartialRender, $pos ); 
      
      /* loop here, so that embedded snapons will be loaded */
      while ( $snapOnClassName ) {
         if ( in_array( $snapOnClassName, $renderedList ) ) {
            $this->setError( "SnapOn: $snapOnClassName, called from " . 
                             $this->name . ", is self-referencing, which will result in an endless loop.", MSG_WARNING );
            return;
         }
         
         /* create the embedded snapon */
         $s =& $this->session->snapOnFactory( $snapOnClassName, $this );
         if ( $this->isError( MSG_FATAL ) ) {
            /* just return */
            return;
         }
         if ( is_object( $s ) ) {
            /* there may be errors, but this is an object, so process it */
            $s->process( );
         
            $snapOnRender = $s->render( );

            $aPartialRender = str_replace( EMBEDDEDSNAPON_PREFIX . $snapOnClassName . EMBEDDEDSNAPON_SUFFIX,
                                           $snapOnRender, $aPartialRender );
                                        
            $pos = 0; /* always reset the pos, as there may be snapons embedded in the previous embedded snapon */
            array_push( $renderedList, $snapOnClassName );

            $snapOnClassName = $this->getPlaceHolderName( EMBEDDEDSNAPON_PREFIX, EMBEDDEDSNAPON_SUFFIX, 
                          $aPartialRender, $pos ); 
         }
      }

      return $aPartialRender;
   } 

   /* wrapper function for rendering an error */
   function renderError( ) {
      return $this->session->renderError( );
   }
   
   /* render's the snapOn, expanding all parameters
      overlays and embedded snapOns. 
      Don't override or extend this 
    */
   function render( ) {
      /* if disabled, return an empty string */
      if ( $this->disabled ) { 
         return "";
      }

      /* don't render if we're just redirecting to another page */
      if ( $this->session->redirecting ) {
         return;
      }
      
      $syntax = "";
      $renderObject = true;

      /* check for fatal or security errors - don't render if there are any */
      if ( $this->isError( MSG_FATAL ) || $this->isError( MSG_SECURITY ) ) {
         /* don't render the object */
         $renderObject = false;
      }
      
      /* render any errors */
      if ( $this->isError( ) ) {
         $syntax .= $this->renderError( );
      }

      /* render the snapOn */
      if ( $renderObject ) {
         /* fire any pre-render events */
         $this->fireEvents( EVENT_PRERENDER );
         if ( $this->isError( ) ) {
            return;
         }
         
         $syntax .= $this->renderSnapOn( );
         /* check for errors in the rendering of the snapOn */
         if ( $this->isError( ) ) {
            
            $syntax .= $this->renderError( );
         }
      
         /* check for a no-repeat flag */
         if ( $this->noRepeat ) {
            /* see if the session already is initialized to keep track of this one */
            $noRepeatList = $this->getVar( "no_repeat_list", "APPLICATION" );
            if ( !is_array( $noRepeatList ) ) {
               $noRepeatList = array( );
            }

            if ( !isset( $noRepeatList[ $this->name ] ) ) {
               $noRepeatList[ $this->name ] = $syntax;
               $this->setVar( "no_repeat_list", $noRepeatList, "APPLICATION" );
            }
            else {
               /* see if the current render has already been displayed */
               if ( $noRepeatList[ $this->name ] == $syntax ) {
                  $syntax = ""; /* clear the syntax - will not render */
               }
               else {
                  /* store the syntax in the no repeat list so that it won't be repeated the next time 
                     this stapon renders if the data hasn't changed */
                  $noRepeatList[ $this->name ] = $syntax;
                  $this->setVar( "no_repeat_list", $noRepeatList, "APPLICATION" );
               }
            }
         }
      }

      /* fire any post-render events */
      $this->fireEvents( EVENT_POSTRENDER );

      return $syntax;
   }

   function display( ) {
      /* echoes the rendered SnapOn to the screen */
      $syntax = $this->render( );
      if ( $this->isError( ) ) {
         $syntax = $this->renderError( );/* . $syntax;*/
      }
            
      /* replace any escaped snapon or overlay delimiters (these are used when the snapon designer
         wants to show the syntax of a snapon or overlay) */
      $syntax = $this->unClean( $syntax );

      echo $syntax;
   }
}

/* empty snapOn */
class PhooEmpty extends PhooClass {
   var $name = "PhooEmpty";
   var $securityRole = ROLE_EVERYONE; /* everyone can use this stub snapon */
}

/* snapOn to display non-query results */
class PhooDisplay extends PhooClass {
   var $name = "PhooDisplay";
   var $securityRole = ROLE_EVERYONE; /* for display snapons, allow all to view by default */
}

define( "DEFAULT_RESULT_PAGE_SIZE", 20 );

/* snapOn to display the results of queries */
class PhooQuery extends PhooClass {

   /* items to place above and below the template which is replicated for each row */
   var $templateHeader;
   var $templateFooter;
   var $rowCount = 0;
   var $disableIfNoRows = false;
   var $resultPageSize = DEFAULT_RESULT_PAGE_SIZE;
   var $rowCache = array( );
   var $rowsRetrieved = false;
   var $rowsReturned = 0;
   
   function setup( ) {
      parent::setup( );

      /* check for result page requests passed in the GET variables */

      /* determine if a specific page has been specified */
      $resultPage = $this->getVar( "result_page", "GET" );
      if ( $resultPage > 1 ) {
         /* compute the start of the result page */
         $resultPageStart = ( $resultPage - 1 ) * $this->resultPageSize;
      }
      else {
         $resultPageStart = 0;
      }

      $this->setVar( "result_page_start", $resultPageStart, "PRIVATE" );
      $this->setVar( "page_size", $this->resultPageSize, "PRIVATE" );
   }

   function doAction( ) {
      if ( $this->command > "" && !$this->disabled ) {
         $this->session->connection->executeSelectSql( $this->setSqlParameters( $this->expandOverlays( $this->command ) ) );
      }

      /* reinitialize variables if needed */
      $this->rowsReturned = 0;
      $this->rowsRetrieved = false;
      $this->rowCache = array( );
   }

   /* wrapper function to allow callers to get the result row */
   function getResultRow( $aReturnType = null ) {
      if ( !$this->rowsRetrieved ) { /* attempt to retrieve rows */
         while ( $row = $this->session->connection->getResultRow( $aReturnType ) ) {
            $this->rowCount++;
            array_push( $this->rowCache, $this->cleanRow( $row ) ); 
         }

         $this->rowsRetrieved = true;      
         $this->rowsReturned = 0;
      }

      if ( $this->rowsReturned < $this->rowCount ) {
         $row = $this->rowCache[ $this->rowsReturned++ ];
      }
      else {
         $row = null;
      }

      return $row;
   }

   function renderSnapOn( ) {
      $syntax = "";
      $rowSet = array( );

      /* populate the template with row by row results */      
      while ( $row = $this->getResultRow( ) ) {
         array_push( $rowSet, $row );
      }

      if ( $this->rowCount == 0 && $this->disableIfNoRows ) {
         
         $this->disabled = true;
         return "";
      }

      /* expand overlays in the detail template */
      $expT = $this->expandOverlays( $this->template );

      $rowNum = 0;
      /* loop through each detail row, populating the syntax */
      foreach ( $rowSet as $r ) {
         $rowNum++;
         /* save the result in private variables */
         foreach( $r as $key => $value ) {
            $this->setVar( $key, $value, "PRIVATE" );
         }
         
         /* fire an event following the row retrieve */
         $this->fireEvents( EVENT_POSTROWRETRIEVE );
         if ( $this->isError( ) ) {
            return;
         }
         
         /* populate parameters and render any embedded snapons */
         $rowSyntax = $this->renderEmbeddedSnapOns( $this->populateParameters( $expT ) );

         /* replace special fields */
         $rowSyntax = str_replace( ROW_NUMBER_TEMPLATE, $rowNum, $rowSyntax );
         $syntax .= $rowSyntax;
      }
      
      /* load the header and footer templates */
      $h = $this->renderEmbeddedSnapOns( $this->populateParameters( $this->expandOverlays( $this->templateHeader) ) );
      $f = $this->renderEmbeddedSnapOns( $this->populateParameters( $this->expandOverlays( $this->templateFooter) ) );
     
      return $h . $syntax . $f;
   }
}

/* this is useful for creating "Next", "Previous" links on the bottom of
   result sets. Requires a command that computes the overall count of the result
   set. */
class PhooResultPageNavigation extends PhooClass {
   var $name = "PhooResultPageNavigation";
   var $nextPageTemplate;
   var $previousPageTemplate;
   var $countCommand;
   var $countVariableName;
   var $resultPageSize;
   var $divider;
   
   function doAction( ) {

      /* check to see if the count variable is set already - if so, don't run the count command */
      if ( $this->countVariableName > "" ) {
         $count = $this->getVar( $this->countVariableName );
      }

      if ( $count == 0 ) {
         /* get a count from the countCommand */
         $rows = $this->getRowsFromQueryCommand( $this->countCommand, SQL_RESULT_NUM );

         $count = $rows[ 0 ][ 0 ];

         if ( $this->countVariableName > "" ) { /* store the count in a POST variable */
            $this->setVar( $this->countVariableName, $count, "POST" );
         }
      }

      /* determine the current starting spot, if any, and the page size
         to determine which templates - next, or previous or none - get displayed */
      
      $needDivider = false; /* won't need unless there's a previous link */
      $resultPage = $this->getVar( "result_page", "GET" );
      if ( $resultPage > 1 ) {
         /* if there are records at all, enable the previous */
         if ( $count > 0 ) {
            $this->setVar( "previous_result_page", $resultPage - 1, "PRIVATE" );
            $this->template .= $this->previousPageTemplate;
            $needDivider = true;
         }
      }
      else {
         $resultPage = 1;
      }
      
      if ( $this->resultPageSize == null ) { /* not set, get it from the container */
         if ( is_object( $this->container ) ) {
            $this->resultPageSize = $this->container->resultPageSize;
         }
         if ( $this->resultPageSize == null ) {
            $this->setError( "Cannot determine result page size", MSG_INFO );
            return;
         }
      }

      $resultPageStart = ( $resultPage - 1 ) * $this->resultPageSize;
      $resultPageEnd = $resultPageStart + $this->resultPageSize;

      if ( $resultPageEnd < $count ) { /* enable the next */
         $this->setVar( "next_result_page", $resultPage + 1, "PRIVATE" );
         if ( $needDivider ) {
            $this->template .= $this->divider;
         }
         $this->template .= $this->nextPageTemplate;
      }

      if ( $resultPageEnd > $count ) {
         $resultPageEnd = $count;
      }

      if ( $resultPageStart+1 > $count ) {
         $resultPageStart = $count-1;
      }
      
      /* set other useful private variables */
      $this->setVar( "navigation_count", $count, "PRIVATE" );
      $this->setVar( "navigation_start", $resultPageStart+1, "PRIVATE" );
      $this->setVar( "navigation_end", $resultPageEnd, "PRIVATE" );
      
      parent::doAction( );
   }
}

class PhooProcess extends PhooClass {
   var $name = "PhooProcess";
   var $insertedId;
   var $constraint = array( ); /* place command overlays here -
                                  for now, a constraint is considered violated if
                                  it returns rows */
   var $dontProcessCommand = false; /* set to true to quietly not process the command */                                  

   /* wrapper function to allow callers to get the result row */
   function getResultRow( $aReturnType = null ) {
      $row = $this->session->connection->getResultRow( $aReturnType );

      if ( $row ) {
         $this->rowCount++;
         $row = $this->cleanRow( $row );
      }
      
      return $row;
   }
                                  
   /* checks any constraints (RI, key violations, etc) */
   function checkConstraints(  ) {

      foreach ( $this->constraint as $c ) {
         /* create a query with this constraint */
         $q =& $this->session->snapOnFactory( "PhooQuery", $this );
         $q->securityRole = $this->securityRole;
         $q->command = $c;
         $q->process( );
         
         $row = $q->getResultRow( );
         if ( is_array( $row ) ) {
            $this->setError( $row[ "constraint_message" ], MSG_WARNING );

            /* blow through any remaining rows */
            while ( is_array( $row ) ) {
               $row = $q->getResultRow( );
            }
         }
      }
   }

   /* executes the command for this process */
   function executeProcessCommand( ) {
      if ( $this->dontProcessCommand ) {
         /* just return. Don't process the command */
         return;
      }

      $this->session->connection->executeModificationSql( $this->setSqlParameters( $this->expandOverlays( $this->command ) ) );
      if ( !$this->isError( ) ) {
         /* store inserted id if there is one */
         $insertedId = $this->session->connection->getInsertedId( );
         if ( $insertedId > 0 ) {
            $this->insertedId = $insertedId;
         }
      }
   }      
   
   /* overrides ancestor */
   function doAction( ) {
      if ( $this->disabled ) {
         return;
      }

      /* check for any constraint violations */
      $this->checkConstraints( );
      if ( $this->isError( ) ) {
         return;
      }
      
      /* fire any pre-save events */
      $this->fireEvents( EVENT_PRESAVE );
      if ( $this->isError( ) ) {
         return;
      }
      
      if ( $this->command > "" ) {
         $this->executeProcessCommand( );
         if ( !$this->isError( ) ) {
            if ( $this->dontProcessCommand ) {
               /* the command was aborted. Just return */
               return;
            }

            /* fire any post-save events */
            $this->fireEvents( EVENT_POSTSAVE );
            if ( $this->isError( ) ) {
               return;
            }
                 
            $this->redirect( );
         }
      }
   }

   /* sets the success page */
   function setSuccessPage( $aSuccessPage ) {
      $this->successPage = $aSuccessPage;
   }

   function redirect( ) {
      if ( $this->isError( ) ) {
         return;
      }

      if ( isset( $this->successPage )  && $this->successPage > "") {
         $this->session->redirecting = true;
         header( "location: " . $this->successPage );
      }
   }      
}

/******************************************************/
/* Processes more than one sql statements, semicolon  */
/* delimited. Note: as currently written, contraints  */
/* won't make sense so not a good idea to add them    */
/******************************************************/
class PhooMultiProcess extends PhooProcess {
   var $name = "PhooMultiProcess";
   
   function doAction( ) {
      /* store the command so that we can put things back the way we found them */
      $commandStore = $this->command;
      
      /* expand all overlays first - only way this will work */
      $this->command = $this->expandOverlays( $this->command );

      /* temporarily convert any escaped semicolons (i.e., real live semicolons we need in the sql, not
         delimiters) into something else to keep them from exploding */
      $this->command = str_replace( "\;", "_SEMICOLON_", $this->command );
      /* explode the command */
      $commandAr = explode( ";", $this->command );
      foreach( $commandAr as $command ) {
         /* re-convert any converted semicolons */
         $command = str_replace( "_SEMICOLON_", ";", $command );
         if ( trim( $command ) > "" ) {
            $this->command = $command;
            parent::doAction( );
            if ( $this->isError( ) ) {
               break;
            }
         }
      }

      $this->command = $commandStore;
   }
}

define( "FORM_ACTION", "action" );

class PhooForm extends PhooProcess {
   var $trigger = array( );
   var $loadCommand;
   var $insertCommand;
   var $insertForm;
   var $updateCommand;
   var $updateForm;
   var $insertConstraint;
   var $newIdCommand; /* if a new id is needed that is not auto-generated */
   var $successPage;
   var $action;
   
   function setup( ) {
      parent::setup( );
      /* if successPage not set, set it to the referring page by default */
      if ( !isset( $this->successPage ) ) {
         $this->successPage = $this->getVar( "HTTP_REFERER", "SERVER" );
      }
   
      if ( !isset( $this->action ) ) {   
         $this->action = $this->getVar( FORM_ACTION );
      }
      /* based on the action, set the template and the commands */
      switch( $this->action ) {
         case "update" :
            if ( $this->updateForm > "" ) {
               $this->template = $this->updateForm;
            }
            
            if ( $this->updateCommand > "" ) {
               $this->command = $this->updateCommand;
            }
            /* push an update constraint if one is specified */
            if ( $this->updateConstraint > "" ) {
               array_push( $this->constraint, $this->updateConstraint );
            }

            break;

         case "insert" :
            if ( $this->insertForm > "" ) {
               $this->template = $this->insertForm;
            }
            
            if ( $this->insertCommand > "" ) {
               $this->command = $this->insertCommand;
            }
            
            /* push an insert constraint if one is specified */
            if ( $this->insertConstraint > "" ) {
               array_push( $this->constraint, $this->insertConstraint );
            }
            
            break;
      }
   }      

   function validateForm( ) {
      
      /* check for required fields */
      if ( is_array( $this->field ) ) {
         foreach ( $this->field as $name => $att ) {
            $value = $this->getVar( $name );
            if ( $att[ "required" ] == "Y" && ( !isset( $value ) || $value == "" ) ) {
               $formalName = ( isset( $att[ "formalName" ] ) ? $att[ "formalName" ] : $name );
               $this->setError( "$formalName must have a value", MSG_WARNING );
            }
         }
      }
   }

   /* returns name of trigger that has been "fired", if any */
   function getTriggerProcess( ) {

      /* see if the form was submitted - a hidden field with 
         the name of the form will be set */
      $formName = $this->getVar( "SnapOnFormName", "POST" );
      if ( $formName == $this->name ) {
      
         foreach( $this->trigger as $name => $att ) {
            $trig = $this->getVar( $name );
            if ( isset( $trig ) ) {
               if ( isset( $att[ "process" ] ) ) {
                  return $att[ "process" ];
               }
            }
         }
      }
      return null;
   }

   function loadForm( ) {
      if ( $this->loadCommand > "" ) {
         $this->session->connection->executeSelectSql( 
                $this->setSqlParameters( $this->expandOverlays( $this->loadCommand ) ) );
         if ( $this->isError( ) ) {
            return;  
         }
         $row = $this->getResultRow( );
         /* load the row in the post variables for the later call to render */
         if ( is_array( $row ) ) {
            foreach ( $row as $key => $value ) {
               $this->setVar( $key, $value, "POST" );
            }
         }
      }
   }
   
   function doAction( ) {
      if ( $this->disabled ) {
         return;
      }

      $process = $this->getTriggerProcess( );
      if ( isset( $process ) ) {
         /* check constraints */
         $this->checkConstraints( );
         if ( $this->isError( ) ) {
            return;
         }

         /* if this is an insert action, and there is a newIdCommand set, run it to determine the
            new id */
         if ( $this->action == "insert" && $this->newIdCommand > "" ) {
            $s =& $this->session->snapOnFactory( "PhooQuery", $this );
            $s->securityRole = $this->securityRole;
            $s->command = $this->newIdCommand;
            $s->process( );

            if ( $s->isError( ) ) {
               return;
            }

            while ( $row = $s->getResultRow( ) ) {
               if ( isset( $row[ "new_id" ] ) ) {
                  $newId = $row[ "new_id" ];
               }
            }

            if ( !is_numeric( $newId ) ) {  /* wasn't able to determine. Set to 1 */
               $newId = 1;
            }

            $this->setVar( "new_id", $newId, "PRIVATE" );
         }
         
         $this->triggerProcess( $process );
      }
      else { /* no process triggered. See if form needs to be loaded */
      /* if this is an update command, load the form */
         if ( $this->action == "update" ) {
            $this->loadForm( );
         }
      }
   }

   function triggerProcess( $aProcess ) {
      /* by default, just run the command. Descendents can have multiple triggers */
      $this->validateForm( );
      if ( $this->isError( ) ) {
         return;
      }

      /* fire any pre-save events */
      $this->fireEvents( EVENT_PRESAVE );
      if ( $this->isError( ) ) {
         return;
      }
      
      if ( $this->command > "" ) {   
         $this->executeProcessCommand( );      
         if ( $this->isError( ) ) {
            return;
         }

         if ( $this->dontProcessCommand ) {
            /* the command was aborted. Just return */
            return;
         }

         /* fire any post-save events */
         $this->fireEvents( EVENT_POSTSAVE );
         if ( $this->isError( ) ) {
            return;
         }
      }

      $this->redirect( );
   }

   function renderSnapOn( ) {
      /* set the form name as a hidden field in the form */
      $syntax = parent::renderSnapOn( );
      return str_replace( "</form>", 
             "<input type=hidden name=\"SnapOnFormName\" value=\"" . $this->name . "\"/></form>", $syntax );
   }
}

class PhooDropDown extends PhooQuery {
   var $name = "PhooDropDown";
   var $dropDownKeyName;
   var $dropDownKeyValue;
   
   function setup( ) {
      parent::setup( );
      /* grab the value, if any, of the drop down key, for later defaulting */
      $this->dropDownKeyValue = $this->getVar( $this->dropDownKeyName );

      /* also, set it in a private variable so the template doesn't have to spell
         out the name */
      $this->setVar( "drop_down_key_name", $this->dropDownKeyName, "PRIVATE" );
   }
   
   function renderSnapOn( ) {
      /* call the parent */
      $syntax = parent::renderSnapOn( );

      if ( isset( $this->dropDownKeyValue ) ) {
         /* search for either value=id or value="id" or value='id' */
         $syntax = str_replace( "value=" . $this->dropDownKeyValue, 
                                "value=" . $this->dropDownKeyValue . " SELECTED ", $syntax );
         $syntax = str_replace( "value='" . $this->dropDownKeyValue . "'", 
                                "value='" . $this->dropDownKeyValue . "' SELECTED ", $syntax );
         $syntax = str_replace( "value=\"" . $this->dropDownKeyValue . "\"", 
                                "value=\"" . $this->dropDownKeyValue . "\" SELECTED ", $syntax );
      }

      return $syntax;
   }   
}

class HardCodedDropDown extends PhooDropDown {
   var $keyValue = array( );
   var $index = 0;

   /* override the normal function, just returning the next key value pair each time */
   function getResultRow( ) {
      if ( $this->index >= count( $this->keyValue ) ) {
         return null; /* done */
      }

      $row = $this->keyValue[ $this->index++ ];

      $this->rowCount++;      
      return $row;
   }
}

class YesNoDropDown extends HardCodedDropDown {
   var $keyValue = array( array( "key_value" => "Y", "key_name" => "Yes" )
                         ,array( "key_value" => "N", "key_name" => "No" )
                        );
} 

class PhooError extends PhooQuery {
   var $name = "PhooError";
   var $securityRole = ROLE_EVERYONE;
   var $headerTemplateName = "Error.header";
   var $templateName = "Error";
   var $footerTemplateName = "Error.footer";
   var $errorMessage = array( );
   var $errorMessageIndex = 0;

   function PhooError( &$aSession, &$aContainer ) {
      /* must capture all the errors before constructing this, to prevent an infinite loop */
      while ( $aSession->isError( ) ) {
         array_push( $this->errorMessage, $aSession->getError( ) );         
      }

      parent::PhooQuery( $aSession, $aContainer );
   }

   function setup( ) {
      /* verify the error templates exist. If they don't substitute a very basic template */
      /* because if these don't exist, it generates an error on render, which instantiates a
         new SnapOnError, which then tries to render, which generates an error, which
         instantiates a new SnapOnError, which . . . you get the picture */
      if ( $this->session->isOverlayAvailable( $this->headerTemplateName ) ) {
         $this->templateHeader = OVERLAY_PREFIX . $this->headerTemplateName . OVERLAY_SUFFIX;
      }
      else {
         $this->templateHeader = ""; /* set it to a simple overlay */
      }

      if ( $this->session->isOverlayAvailable( $this->footerTemplateName ) ) {
         $this->templateFooter = OVERLAY_PREFIX . $this->footerTemplateName . OVERLAY_SUFFIX;
      }
      else {
         $this->templateFooter = ""; /* set it to a simple overlay */
      }
      
      if ( $this->session->isOverlayAvailable( $this->templateName ) ) {
         $this->template = OVERLAY_PREFIX . $this->templateName . OVERLAY_SUFFIX;
      }
      else {
         $this->template = "<b>_@error_message@_</b><br />"; /* set it to a simple overlay */
      }
   }
   
   
   /* override the doAction and the get rows function to simulate the retrieval of 
      error records */
   function doAction( ) {
      return; /* do nothing - no query to run */
   }

   /* overriding the ancestor - this returns rows from the error message */
   function getResultRow( ) {
      if ( $this->errorMessageIndex < count( $this->errorMessage ) ) {
         $row = array( "error_message" => $this->errorMessage[ $this->errorMessageIndex ] );
         $this->errorMessageIndex++;
      }
      else {
         $row = false;
      }

      return $row;
   }  

   function renderError( ) {
      /* null this out, just to prevent horrible infinite loops */
      return;
   }
}

class PhooRSSReader extends PhooDisplay {
   var $RSSFeed; /* set to the URL of the RSS feed */
   var $readError = "Could not access the RSS Feed";
  
   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-05-17" );
      $this->setPurpose( "Abstract RSS reader" );
   }

   /* abstract function - to be overridden by descendents. Useful for setting 
      POST variables from the RSS feed */
   function setVariables( $xmlData ) {

   }

   function doAction( ) {
      if ( $this->disabled ) {
         return;
      }
      $x = new XML2ArrayParser( $this->RSSFeed );
      
      if ( $x->isError( ) ) {
         /* replace the error, since the one given by the XML parser is a bit too long and cryptic */
         while ( $x->isError( ) ) {
            $x->getError( );
         }

         $this->setError( $this->readError, MSG_FATAL );
         return;
      }      

      $this->setVariables( $x->tagData );
      
      parent::doAction( );
   }
}

class PhooPage extends PhooDisplay {
   var $name = "PhooPage";
   var $templateHeader;
   var $templateMenu;
   var $templateFooter;
   var $setupComplete = false;

   function renderSnapOn( ) {
      if ( $this->disabled ) {
         return "";
      }
      /* load the main template */
      $syntax = parent::renderSnapOn( );

      /* load other templates */
      $h = $this->renderEmbeddedSnapOns( $this->populateParameters( $this->expandOverlays( $this->templateHeader ) ) );
      $m = $this->renderEmbeddedSnapOns( $this->populateParameters( $this->expandOverlays( $this->templateMenu ) ) );
      $f = $this->renderEmbeddedSnapOns( $this->populateParameters( $this->expandOverlays( $this->templateFooter ) ) );
      
      return $h . $m . $syntax . $f;
   }
}

class PhooMenuItem extends PhooDisplay {
   var $name = "PhooMenuItem";
   var $menuItemName;
   var $menuItemLink;
   var $selectedTemplate; /* set to the overlay to use when selected */
   var $unselectedTemplate; /* set to the overlay to use when not selected */
   var $invisibleOnSecurityErrors = true; /* will hide this item if user does not have security */

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-18" );
      $this->setPurpose( "Displays a menu item (a link), provided security exists" );
   }

   function isSelected( ) {
      /* stub-code in the ancestor */
      return false;
   }

   /* note: needs to happen after the setup of the snapOn, as close the rendering as possible */
   function doAction( ) {
      if ( $this->disabled ) {
         return;
      }

      /* check to see if this tab is selected */
      if ( $this->isSelected( ) ) {
         $this->template = $this->selectedTemplate;
      }
      else {
         $this->template = $this->unselectedTemplate;
      }
      
      /* Expand any parameters in the menu item link (such as _@site_url@_) */
      $this->menuItemLink = $this->populateParameters( $this->menuItemLink );
      /* set variables for the link */
      $this->setVar( "menu_item_name", $this->menuItemName, "PRIVATE" );
      $this->setVar( "menu_item_link", $this->menuItemLink, "PRIVATE" );
      parent::doAction( );
   }
}

class PhooMenu extends PhooDisplay {
   var $name = "Menu";
   var $menu = array( );
   var $templateHeader = "=|MenuItem.header|=";
   var $templateFooter = "=|MenuItem.footer|=";
   var $securityRole = ROLE_EVERYONE;

   function setDescriptor(  ) {
      $this->setAuthor( "Bill Roberts" );
      $this->setVersion( "0.1" );
      $this->setCreateDate( "2005-11-28" );
      $this->setPurpose( "Displays a menu" );
   }

   function addMenuItem( &$aMenuItem ) {
      array_push( $this->menu, $aMenuItem );
   }

   /* inserts the menu item passed into a specific location */
   function insertMenuItem( &$aMenuItem, $aPosition ) {
      if ( $aPosition < 0 ) {
         $aPosition = 0; /* prevents a nasty error */
      }
      
      $menuSize = count( $this->menu );
      if ( $menuSize > $aPosition ) {
         for ( $i = $menuSize; $i > $aPosition; $i-- ) {
            $this->menu[ $i ] = $this->menu[ $i-1 ];
         }
      }
      else {
         $aPosition = $menuSize; /* put it at the end */
      }
      $this->menu[ $aPosition ] = $aMenuItem;
   }

   /* note: needs to happen after the setup of the snapOn, as close to the rendering as possible */
   function doAction( ) {
      if ( $this->disabled ) {
         return;
      }

      /* build the template */
      $this->template = $this->templateHeader;
      /* add each menu item */
      foreach ( $this->menu as $m ) {
         $m->process( );
         if ( $this->isError( ) ) {
            return;
         }
         $this->template .= $m->render( );
      }
      $this->template .= $this->templateFooter;
      parent::doAction( );
   }
}

/*************************************************/
/* Event Handling classes                        */
/*************************************************/
class PhooEventHandler extends PhooClass {
}

class PreRenderEventHandler extends PhooEventHandler {
}

class PostRenderEventHandler extends PhooEventHandler {
}

class PreProcessEventHandler extends PhooEventHandler {
}

class PostProcessEventHandler extends PhooEventHandler {
}

class PreSaveEventHandler extends PhooEventHandler {
}

class PostSaveEventHandler extends PhooEventHandler {
}

class PostRowRetrieveEventHandler extends PhooEventHandler {
}

class SessionInitEventHandler extends PhooEventHandler {
}

/*************************************************/
/* Time Zone Processing                          */
/*************************************************/
define( "TZ_GMT", 27 );

/* time zone class - converts GMT times to local timeszones and visa-versa, with DST adjustments */
class PhooTimeZone extends PhooBase {
   var $id;
   var $name;
   var $shortName;
   var $gmtOffset;
   var $dstMethod;
   var $dstOffset;
   var $dstShortName;
   var $tagData;
   
   function PhooTimeZone( $aTimeZoneId ) {
      /* read in the timezones file */
      $tzx = new XML2ArrayParser( "timezones.xml" );
      if ( $tzx->isError( ) ) {
         return;
      }

      /* search the timezone array for the id passed */
      $i = 0;
   
      while ( $key == null && $i < count( $tzx->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ] ) ) {
         if ( $tzx->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ $i ][ "ID" ] == $aTimeZoneId ) {
            $key = $i;
            $tz = $tzx->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ $i ];
         }
         else {
            $i++;
         }
      }
      
      if ( !is_array( $tz ) ) {
         $this->setError( "Illegal time zone id: $aTimeZoneId", MSG_FATAL );
         return;
      }

      /* load the members */
      $this->id = $aTimeZoneId;
      $this->name = $tz[ "NAME" ];
      $this->shortName = $tz[ "SHORT-NAME" ];
      $this->gmtOffset = $tz[ "GMT-OFFSET" ];
      $this->dstMethod = $tz[ "DST-METHOD" ];
      $this->dstOffset = $tz[ "DST-OFFSET" ];
      $this->dstShortName = $tz[ "DST-SHORT-NAME" ];
      $this->tagData = $tzx->tagData;
   }

   /* Convert a GMT date-time string into a date-time in this timezone
      The string is expected to be in this format: yyyy-mm-dd hh:mi:ss, with
      no offset to GMT included. It is assumed to be in GMT time.
     
      If no date is passed in, get the current GMT time.
     
      Date returned is the equivalent date for this timezone in the date format passed.

      Returns an array containing the date and a displayable code for the timezone, if available
      */
   function getTimeInZone( $aDateFormat = BASE_TIME_FORMAT, $aGMTDateTime = null ) {
      /* add the GMT indicator (+0000) and convert to a unix timestamp */
      if ( isset( $aGMTDateTime ) ) {         
         $timeStamp = strtotime( $aGMTDateTime . "+0000" );
      }
      else {
         $timeStamp = time( );
      }

      /* adjust the date to local time */
      $localStandardTimeStamp = $this->getLocalStandardTimeStamp( $timeStamp );

      /* initialize */
      $isDST = $this->isDST( $localStandardTimeStamp );
      /* add the DST offset for this timezone if we're in DST */
      if ( $isDST ) {
         $localStandardTimeStamp += $this->dstOffset * 3600;
         $timeZoneCode = $this->dstShortName;
      }

      /* create a string containing a date local to this timestamp (don't let gmdate throw 
         you off. The date has been adjusted for local already - gmdate will keep it from being
         further adjusted to the time of your server */
      $dateString = gmdate( $aDateFormat, $localStandardTimeStamp );
      if ( !isset( $timeZoneCode ) ) {
         /* see if you can set it to the short name */
         if ( isset( $this->shortName ) ) {
            $timeZoneCode = $this->shortName;
         }
         else {
            /* use the name */
            $timeZoneCode = $this->name;
         }
      }

      return array( $dateString, $timeZoneCode, $isDST );
   }
   
   /* converts a local date string for this timezone into the equivalent GMT date string */
   function getGMTTime( $aDateFormat = BASE_TIME_FORMAT, $aLocalDateTime ) {
      /* add the gmtOffset to the datestring to get the correct timestamp, regardless
         of server time */
      if ( $this->gmtOffset >= 0 ) {
         $localDateTimeWOffset = $aLocalDateTime . "+" . $this->gmtOffset;
      }
      else {
         $localDateTimeWOffset = $aLocalDateTime . $this->gmtOffset;
      }

      /* convert to local standard timestamp */
      $gmtTimeStamp = strtotime( $localDateTimeWOffset );
      $localStandardTimeStamp = $this->getLocalStandardTimeStamp( $gmtTimeStamp );
      if ( $this->isDST( $localStandardTimeStamp ) ) {
         /* subtract the DST adder from the gmtTimeStamp */
         $gmtTimeStamp -= ( $this->dstOffset * 3600 );
      }

      /* return the GMT Date */
      return gmdate( $aDateFormat, $gmtTimeStamp );
   }
   
   /* converts a GMT timestamp to local standard time. This represents the number of seconds since
      the LOCAL epoch (local time 1/1/1970), not adjusting for DST */
   function getLocalStandardTimeStamp( $aTimeStamp ) {
      return $aTimeStamp + ( $this->gmtOffset * 3600 );
   }
   
   /* returns true if the local standard timestamp passed puts this timezone
      in DST */
   function isDST( $aLocalStandardTimeStamp ) {
      
      $isDST = false; /* initialize */
      
      if ( isset( $this->dstMethod ) ) {
         switch ( $this->dstMethod ) {

            case "unitedStates" :
               $isDST = $this->unitedStates( $aLocalStandardTimeStamp );
               break;
               
            case "easternEuropeanTime" :
               $isDST = $this->easternEuropeanTime( $aLocalStandardTimeStamp );
               break;
               
            default:

               break;
         }
      }
         
      return $isDST;
   }

   /* returns the local standard timestamp of the beginning of the first 
      occurrence of the weekday in question in this month.
      Weekdays are numbered 0=Sunday - 6=Saturday. */
   function firstOfThisDayInMonth( $aDayNum, $aYear, $aMonth ) {
      /* get GMT timestamp of first day in the month */
      $timeStamp = gmmktime( 0, 0, 0, $aMonth, 1, $aYear );

      $firstDayNum = gmdate( "w", $timeStamp );
      
      /* compute the distance from the first day of the month to the day desired */
      if ( $aDayNum < $firstDayNum ) {
         $aDayNum += 7;
      }
      
      /* return it, converting it to local standard time */
      return $timeStamp + ( ( $aDayNum - $firstDayNum ) * 3600 * 24 );
   }

   /* returns the local standard timestamp of the beginning of the last
      occurrence of the weekday in question in this month.
      Weekdays are numbered 0=Sunday - 6=Saturday. */
   function lastOfThisDayInMonth( $aDayNum, $aYear, $aMonth ) {
      /* get timestamp of last day in the month (by advancing month and, if needed, year, ahead one */
      $aMonth++;
      if ( $aMonth > 12 ) {
         $aMonth = 1;
         $aYear++;
      }
      
      $timeStamp = gmmktime( 0, 0, 0, $aMonth, 1, $aYear ) - ( 3600 * 24 );

      $lastDayNum = gmdate( "w", $timeStamp );
      
      /* compute the distance from the last day of the month to the day desired */
      if ( $aDayNum > $lastDayNum ) {
         $lastDayNum += 7;
      }
      
      return $timeStamp - ( ( $lastDayNum - $aDayNum ) * 3600 * 24 );
   }
   
   /******* dailight savings time methods *******/
   /* a source for some of these: http://www.timeanddate.com/library/abbreviations/timezones/ */
   
   /* United States:
      old method: first sunday in April, last sunday in October.
      After 1/1/2007: second sunday in March, first sunday in November
      (per http://en.wikipedia.org/wiki/Energy_Policy_Act_of_2005)
    */
   function unitedStates( $aLocalStandardTimeStamp ) {
      $isDST = false; /* initialize */
      
      $year = gmdate( "Y", $aLocalStandardTimeStamp );
      if ( $year < 2007 ) {
         $dstStart = $this->FirstOfThisDayInMonth( 0, $year , 4 ) + ( 2 * 3600 );
         if ( $aLocalStandardTimeStamp >= $dstStart ) {
            $dstEnd = $this->LastOfThisDayInMonth( 0, $year, 10 ) + ( ( 2 - $this->dstOffset ) * 3600 );
            if ( $aLocalStandardTimeStamp < $dstEnd ) {
               $isDST = true;
            }
         }
      }
      else { /* greater than 2007, use new rules */
         /* we need second sunday of month, so add a week to it */
         $dstStart = $this->FirstOfThisDayInMonth( 0, $year , 3 ) + ( 2 * 3600 ) + ( 168 * 3600 );
         if ( $aLocalStandardTimeStamp >= $dstStart ) {
            $dstEnd = $this->FirstOfThisDayInMonth( 0, $year, 11 ) + ( ( 2 - $this->dstOffset ) * 3600 );
            if ( $aLocalStandardTimeStamp < $dstEnd ) {
               $isDST = true;
            }
         }
      }
         
      return $isDST;
   }

   /* Eastern European Time (EET, EEST):
      Begins at 3am local on the last Sunday in March and ends at 4am local ST on the last Sunday in October
    */
   function easternEuropeanTime( $aLocalStandardTimeStamp ) {
      $isDST = false; /* initialize */
      
      $year = gmdate( "Y", $aLocalStandardTimeStamp );
      $dstStart = $this->LastOfThisDayInMonth( 0, $year , 3 ) + ( 3 * 3600 );
      if ( $aLocalStandardTimeStamp >= $dstStart ) {
         $dstEnd = $this->LastOfThisDayInMonth( 0, $year, 10 ) + ( ( 4 - $this->dstOffset ) * 3600 );
         if ( $aLocalStandardTimeStamp < $dstEnd ) {
            $isDST = true;
         }
      }
         
      return $isDST;
   }
}

/* abstract class - should be extended in the ancestor */
class TimeZoneDropDown extends PhooDropDown {
   var $name = "TimeZoneDropDown";
   var $dropDownKeyName = "time_zone_id";
   var $tagData;
   var $zoneCount;
   var $currentZone = 0;

   /* read in the timezone xml file */
   function setup( ) {
      parent::setup( );
      
      /* load from the timezones.xml file */
      $tzx = new XML2ArrayParser( "timezones.xml" );
      if ( $tzx->isError( ) ) {
         return;
      }

      $this->setXMLSource( $tzx->tagData );
   }
   
   function setXMLSource( $aTagData ) { 
      $this->tagData = $aTagData;

      $this->zoneCount = count( $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ] );
   }

   /* overrides ancestor, this returns rows from the xml file */
   function getResultRow( ) {
      if ( $this->currentZone >= $this->zoneCount ) {
         return null;
      }

      $row[ "time_zone_id" ] = $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ $this->currentZone ][ "ID" ];
      $row[ "time_zone_name" ] = $this->tagData[ "TIMEZONES" ][0][ "TIMEZONE" ][ $this->currentZone ][ "NAME" ];

      $this->currentZone++;
      return $row;
   }
}

?>
